(*
This file is part of the first order theorem prover Darwin
Copyright (C) 2004, 2005, 2006
              The University of Iowa
              Universitaet Koblenz-Landau 

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*)



type entry_data = 
	{
		flipped: bool;
		subst: Subst.subst;
		literal_position: int;
		term: Term.term;
		clause: Term.clause;
		search_flag: bool ref;
		misc_flag: bool ref;
		redundancy_explanation: State.explanation ref;
	}

let empty_entry_data =
	{
		flipped = false;
		subst = Subst.empty;
		literal_position = 0;
		term = Term.null_term;
		clause = [];
		search_flag = ref false;
		misc_flag = ref false;
		redundancy_explanation = ref State.invalid_explanation;
	}

let create_entry_data 
	?(flipped:bool = false) 
	?(subst:Subst.subst = Subst.empty) 
	?(literal_position:int = 0)
	?(term:Term.term = Term.null_term)
	?(clause:Term.clause = [])
	?(search_flag:(bool ref) = ref false)
	?(misc_flag:(bool ref) = ref false) 
	?(redundancy_explanation:(State.explanation ref) = ref State.invalid_explanation)
	() : entry_data =
	{
		flipped = flipped;
		subst = subst;
		literal_position = literal_position;
		term = term;
		clause = clause;
		search_flag = search_flag;
		misc_flag = misc_flag;
		redundancy_explanation = redundancy_explanation;
	}
	
let get_flipped entry_data =
	entry_data.flipped
	
let get_subst entry_data =
	entry_data.subst
	
let get_literal_position entry_data =
	entry_data.literal_position
	
let get_term entry_data =
	entry_data.term	
	
let get_clause entry_data =
	entry_data.clause
	
let get_search_flag entry_data =
	!(entry_data.search_flag)
	
let get_misc_flag entry_data =
	!(entry_data.misc_flag)
	
let set_search_flag	entry_data bool =
	entry_data.search_flag := bool
	
let set_misc_flag entry_data bool =
	entry_data.misc_flag := bool
	
let is_redundant entry_data =
	State.is_explanation_valid !(entry_data.redundancy_explanation)