(*
This file is part of the first order theorem prover Darwin
Copyright (C) 2004, 2005, 2006
              The University of Iowa
              Universitaet Koblenz-Landau 

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*)

(* Unfortunately the index implementation is designed to be very open/modular
regarding the data being indexed. While this allows the index to store any
type of data, it makes it very hard to modify and expand the operation of
the index. The index does not "know" anything about the data it stores,
and hence it cannot access the data in any meaningful way, as its type and 
structure is unknown to the index module. As a workaround the leaves of the
indexing tree have been modified to contain an [entry_data] record for each
entry. This contains additional information regarding the data stored at the 
entry, and this information can be accessed by the indexing module.
While this may not be the most elegant solution, it can be seen as an expansion
of the workaround already found in the original implementation - here, each
entry also carries a [term], which is used by the [leaf_check] functions of the
index to perform tests during index searches. The information in [term] is
technically redundant, as it could just as well be found in the data stored at
the entry, but since the index cannot look into the data, this additional external
information storage is required. *)



type entry_data

val create_entry_data : ?flipped:bool -> ?subst:Subst.subst -> 
?literal_position:int -> ?term:Term.term -> ?clause:Term.clause -> 
?search_flag:(bool ref) -> ?misc_flag:(bool ref) -> 
?redundancy_explanation:(State.explanation ref) ->
unit -> entry_data

val empty_entry_data : entry_data

val get_flipped : entry_data -> bool

val get_subst : entry_data -> Subst.subst

val get_literal_position : entry_data -> int

val get_term : entry_data -> Term.term

val get_clause : entry_data -> Term.clause

val get_search_flag : entry_data -> bool
	
val get_misc_flag : entry_data -> bool
	
val set_search_flag	: entry_data -> bool -> unit
	
val set_misc_flag : entry_data -> bool -> unit

val is_redundant : entry_data -> bool