(*
This file is part of the first order theorem prover Darwin
Copyright (C) 2006
              The University of Iowa

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*)

exception Found
exception Failed


let symbol_precedence_compare (sym1:Symbol.symbol) (sym2:Symbol.symbol) : int =
	let arity_comp = Pervasives.compare (Symbol.arity sym1) (Symbol.arity sym2)
	in
		if (arity_comp>0) then 1
		else
			(if (arity_comp<0) then (-1)
			else
				(let name_comp =
					Pervasives.compare (Symbol.name sym1) (Symbol.name sym2)
				in
					if (name_comp>0) then 1
					else
						(if (name_comp<0) then (-1)
						else 0)))


let rec lrpo_greater (t1:Term.term) (t2:Term.term) : bool =
	if (Term.term_equal t1 t2) then false
	else
		(match t1, t2 with
			Term.Var _, _ -> false
			|_, Term.Var var -> (Term.term_contains_var t1 var)
			|Term.Const c1, Term.Const c2 ->
				if (symbol_precedence_compare c1 c2)>0 then true
				else false
			|Term.Const _, Term.Func _ -> false
			|Term.Func _, Term.Const _ -> true
			|Term.Func f1, Term.Func f2 ->	
				try
					(Array.iter
						(fun s ->
							if (Term.term_equal s t2)
								||(lrpo_greater s t2)
								then (raise Found))
						f1.Term.subterms;
					match (symbol_precedence_compare f1.Term.symbol f2.Term.symbol) with
						(-1) -> false
						|0 ->
							(let exclusive_1 = ref []
							and exclusive_2 = ref []
							and check_1 = Array.make (Array.length f1.Term.subterms) true
							and check_2 = Array.make (Array.length f1.Term.subterms) true
							in
								Array.iteri
									(fun i1 s1 ->
										if check_1.(i1) then (
											try
												(Array.iteri
													(fun i2 s2 ->
														if Term.term_equal s1 s2 then
															(check_1.(i1)<-false;
															check_2.(i2)<-false;
															raise Failed))
													f2.Term.subterms;
												exclusive_1 := s1 :: !exclusive_1)
											with Failed -> ()))
									f1.Term.subterms;
								Array.iteri
									(fun i2 s2 ->
										if check_2.(i2) then
											(exclusive_2 := s2 :: !exclusive_2))
									f2.Term.subterms;
								if ((List.length !exclusive_1) > 0) then
									(List.for_all
										(fun s2 ->
											List.exists
												(fun s1 ->
													lrpo_greater s1 s2)
												!exclusive_1)
										!exclusive_2)
								else (false))
						|1 -> 
							Array.iter
								(fun s ->
									if (lrpo_greater t1 s) then ()
									else (raise Failed))
								f2.Term.subterms;
							true
						|_ -> raise (Failure "Bad comparison value in LrpOrder.lrpo_greater.")
					false)
				with 
					Found -> true	
					|Failed -> false)
				
let lrpo_greater_var_checked (t1:Term.term) (t2:Term.term) : bool =
	if (Term.term_contains_vars_of_term t1 t2) then
		(lrpo_greater t1 t2)
	else 
		(false)
		
let compare_lrpo_greater_var_checked (t1:Term.term) (t2:Term.term) : int =
	if (Term.term_equal t1 t2) then 0
	else
		(if (lrpo_greater_var_checked t1 t2) then 1
		else
			(if (lrpo_greater_var_checked t2 t1) then (-1)
			else 0))
		
let rec lrpo_greater_ground_like (t1:Term.term) (t2:Term.term) : bool =
	if (Term.term_equal t1 t2) then false
	else
		(match t1, t2 with
			Term.Var v1, Term.Var v2 ->
				(Var.compare v1 v2)=1
			|Term.Var _, _ -> false
			|_, Term.Var var -> true
			|Term.Const c1, Term.Const c2 ->
				if (symbol_precedence_compare c1 c2)>0 then true
				else false
			|Term.Const _, Term.Func _ -> false
			|Term.Func _, Term.Const _ -> true
			|Term.Func f1, Term.Func f2 ->	
				try
					(Array.iter
						(fun s ->
							if (Term.term_equal s t2)
								||(lrpo_greater_ground_like s t2)
								then (raise Found))
						f1.Term.subterms;
					match (symbol_precedence_compare f1.Term.symbol f2.Term.symbol) with
						(-1) -> false
						|0 ->
							(let exclusive_1 = ref []
							and exclusive_2 = ref []
							and check_1 = Array.make (Array.length f1.Term.subterms) true
							and check_2 = Array.make (Array.length f1.Term.subterms) true
							in
								Array.iteri
									(fun i1 s1 ->
										if check_1.(i1) then (
											try
												(Array.iteri
													(fun i2 s2 ->
														if Term.term_equal s1 s2 then
															(check_1.(i1)<-false;
															check_2.(i2)<-false;
															raise Failed))
													f2.Term.subterms;
												exclusive_1 := s1 :: !exclusive_1)
											with Failed -> ()))
									f1.Term.subterms;
								Array.iteri
									(fun i2 s2 ->
										if check_2.(i2) then
											(exclusive_2 := s2 :: !exclusive_2))
									f2.Term.subterms;
								if ((List.length !exclusive_1) > 0) then
									(List.for_all
										(fun s2 ->
											List.exists
												(fun s1 ->
													lrpo_greater_ground_like s1 s2)
												!exclusive_1)
										!exclusive_2)
								else (false))
						|1 -> 
							Array.iter
								(fun s ->
									if (lrpo_greater_ground_like t1 s) then ()
									else (raise Failed))
								f2.Term.subterms;
							true
						|_ -> raise (Failure "Bad comparison value in LrpOrder.lrpo_greater_ground_like.")
					false)
				with 
					Found -> true	
					|Failed -> false)
					
					
let compare_lrpo_greater_ground_like (t1:Term.term) (t2:Term.term) : int =
	if (Term.term_equal t1 t2) then 0
	else
		(if (lrpo_greater_ground_like t1 t2) then 1
		else 0)