(*
This file is part of the first order theorem prover Darwin
Copyright (C) 2004, 2005, 2007
              The University of Iowa
              Universitaet Koblenz-Landau
              Australian National University
              NICTA 

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*)


(** term ordering

  This module provides functions for the ordering of terms, 
  employing a mix of lexicographic/recursive path ordering.
*)


(** [lrpo_greater t1 t2] returns true if [t1] is greater than
[t2] and false otherwise. The formal definition of the RPO is
implemented, and thus variables will be compared. This means
that the function may be unsuitable for comparisons of terms
from different clauses, as their variables are usually 
uncomparable.  *)
val lrpo_greater: Term.term -> Term.term -> bool

(** [lrpo_greater_var_checked t1 t2] works like [lrpo_greater],
but performs a preliminary test whether all variables of [t2]
are contained in t1, a necessary condition for [t2] being smaller
than [t1] if [t2] is not ground.
This is the recommended function for terms that may be non-ground. *)
val lrpo_greater_var_checked:  Term.term -> Term.term -> bool

(** [compare_lrpo_greater_var_checked t1 t2] returns 1 if [t1] is
greater than [t2], -1 if [t2] is greater than [t1], and 0 otherwise.
Comparison is done using [lrpo_greater_var_checked]. *)
val compare_lrpo_greater_var_checked:  Term.term -> Term.term -> int

(** [lrpo_greater_ground_like t1 t2] works like [lrpo_greater],
but treats variables as lower order constants ordered by their 
variable-id number. It is thus total also on non-ground terms, in
the limited sense that it can be used for term sorting within a clause.
It should not be used for comparisons between clauses, as variable 
numbers can change during processing. *)
val lrpo_greater_ground_like: Term.term -> Term.term -> bool

(** Works like [compare_lrpo_greater_var_checked], but uses the
ordering of [lrpo_greater_ground_like]. *) 
val compare_lrpo_greater_ground_like:  Term.term -> Term.term -> int