type token =
  | LEFT_PARENTHESIS
  | RIGHT_PARENTHESIS
  | LEFT_BRACKET
  | RIGHT_BRACKET
  | DOT
  | NEGATION
  | COLON
  | COMMA
  | EQUALITY
  | DISEQUALITY
  | EOI
  | FOF
  | CNF
  | INCLUDE
  | SINGLE_QUOTED of (string)
  | DOLLAR_WORD of (string)
  | DOLLAR_DOLLAR_WORD of (string)
  | DISTINCT_OBJECT of (string)
  | LOWER_WORD of (string)
  | UPPER_WORD of (string)
  | UNSIGNED_INTEGER of (string)
  | SIGNED_INTEGER of (string)
  | REAL of (string)
  | DOLLAR_TRUE
  | DOLLAR_FALSE
  | DOLLAR
  | AND
  | OR
  | FORALL
  | EXISTS
  | BIJECTION
  | LEFT_IMPLICATION
  | RIGHT_IMPLICATION
  | UNKNOWN

open Parsing;;
# 22 "src/parser_tptp.mly"
  
  type var = Var.var
  type symbol = Symbol.symbol
  type term = Term.term
  type literal = Term.literal
  type clause = Term.clause

  (* these are only valid for the current clause
     and have to be invalidated with init_clause for every new clause *)

  (* the variable id counter for the currently read term/clause *)
  let var_id_counter = 
    Counter.create_with 0
      
  (* mapping of the variable names (e.g. "X") of the currently read term/clause
     to variables. *)
  let (var_map: (string * Var.var) list ref) =
    ref []

  (* the literals of the currently read clause *)
  let (literals: literal list ref) =
    ref []


  (* reset everything in order to parse a new term/clause *)
  let init_clause () =
    Counter.set var_id_counter 0;
    var_map := [];
    literals := []


	
  (* gets the variables associated with a string from the variable mapping
     creates a new mapping for a new variable *)
  let get_var (var_name: string) =
    try 
      (* way faster than List.assoc *)
      snd (
	List.find
	  (fun (var_name', _) ->
             var_name = var_name'
	  )
	  !var_map
      )
    with
      | Not_found ->
	  let new_var = 
	    Var.create_universal (Counter.value var_id_counter)
	  in
	    Counter.inc var_id_counter;
	    var_map := (var_name, new_var) :: !var_map;
	    new_var

  (* need to detect if input is fof and contains a conjecture *)
  (* is the input in fof format? *)
  let fof =
    ref false

  (* does the fof input contain a conjecture? *)
  let theorem =
    ref false

# 102 "src/parser_tptp.ml"
let yytransl_const = [|
  257 (* LEFT_PARENTHESIS *);
  258 (* RIGHT_PARENTHESIS *);
  259 (* LEFT_BRACKET *);
  260 (* RIGHT_BRACKET *);
  261 (* DOT *);
  262 (* NEGATION *);
  263 (* COLON *);
  264 (* COMMA *);
  265 (* EQUALITY *);
  266 (* DISEQUALITY *);
  267 (* EOI *);
  268 (* FOF *);
  269 (* CNF *);
  270 (* INCLUDE *);
  280 (* DOLLAR_TRUE *);
  281 (* DOLLAR_FALSE *);
  282 (* DOLLAR *);
  283 (* AND *);
  284 (* OR *);
  285 (* FORALL *);
  286 (* EXISTS *);
  287 (* BIJECTION *);
  288 (* LEFT_IMPLICATION *);
  289 (* RIGHT_IMPLICATION *);
  290 (* UNKNOWN *);
    0|]

let yytransl_block = [|
  271 (* SINGLE_QUOTED *);
  272 (* DOLLAR_WORD *);
  273 (* DOLLAR_DOLLAR_WORD *);
  274 (* DISTINCT_OBJECT *);
  275 (* LOWER_WORD *);
  276 (* UPPER_WORD *);
  277 (* UNSIGNED_INTEGER *);
  278 (* SIGNED_INTEGER *);
  279 (* REAL *);
    0|]

let yylhs = "\255\255\
\001\000\001\000\002\000\002\000\003\000\003\000\004\000\004\000\
\006\000\010\000\010\000\012\000\012\000\014\000\016\000\016\000\
\016\000\016\000\016\000\016\000\015\000\015\000\017\000\019\000\
\019\000\018\000\020\000\020\000\013\000\013\000\013\000\013\000\
\021\000\024\000\024\000\025\000\025\000\022\000\027\000\007\000\
\009\000\011\000\011\000\028\000\028\000\032\000\032\000\033\000\
\033\000\033\000\023\000\023\000\023\000\034\000\038\000\038\000\
\035\000\035\000\035\000\035\000\036\000\039\000\039\000\041\000\
\041\000\041\000\037\000\037\000\042\000\042\000\045\000\046\000\
\043\000\043\000\040\000\040\000\044\000\044\000\050\000\049\000\
\026\000\030\000\031\000\031\000\053\000\005\000\056\000\056\000\
\057\000\057\000\052\000\052\000\052\000\058\000\058\000\058\000\
\058\000\060\000\060\000\059\000\059\000\054\000\054\000\008\000\
\008\000\047\000\047\000\051\000\048\000\048\000\048\000\055\000\
\029\000\000\000"

let yylen = "\002\000\
\002\000\001\000\001\000\002\000\001\000\001\000\001\000\001\000\
\010\000\001\000\001\000\001\000\001\000\003\000\001\000\001\000\
\001\000\001\000\002\000\002\000\001\000\001\000\003\000\001\000\
\003\000\003\000\001\000\003\000\001\000\001\000\003\000\001\000\
\006\000\001\000\001\000\001\000\003\000\002\000\001\000\010\000\
\001\000\001\000\003\000\003\000\001\000\001\000\003\000\003\000\
\001\000\002\000\001\000\001\000\001\000\001\000\001\000\003\000\
\001\000\001\000\003\000\003\000\001\000\001\000\001\000\001\000\
\001\000\001\000\001\000\004\000\001\000\004\000\001\000\001\000\
\001\000\001\000\001\000\004\000\001\000\004\000\001\000\001\000\
\001\000\001\000\002\000\001\000\001\000\006\000\004\000\001\000\
\001\000\003\000\001\000\003\000\001\000\001\000\004\000\001\000\
\001\000\001\000\003\000\002\000\003\000\001\000\003\000\001\000\
\001\000\001\000\001\000\001\000\001\000\001\000\001\000\001\000\
\000\000\002\000"

let yydefred = "\000\000\
\000\000\000\000\002\000\000\000\000\000\000\000\114\000\000\000\
\000\000\005\000\006\000\007\000\008\000\000\000\000\000\000\000\
\001\000\004\000\107\000\106\000\105\000\000\000\104\000\000\000\
\112\000\000\000\000\000\000\000\000\000\088\000\000\000\041\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\086\000\000\000\039\000\108\000\074\000\081\000\111\000\110\000\
\109\000\057\000\058\000\034\000\035\000\000\000\010\000\000\000\
\012\000\013\000\021\000\022\000\029\000\030\000\032\000\000\000\
\063\000\000\000\051\000\052\000\053\000\054\000\000\000\061\000\
\062\000\064\000\065\000\066\000\000\000\000\000\000\000\073\000\
\000\000\000\000\000\000\000\000\000\000\049\000\000\000\045\000\
\000\000\000\000\087\000\000\000\000\000\000\000\042\000\000\000\
\000\000\000\000\015\000\016\000\017\000\018\000\000\000\000\000\
\038\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\050\000\000\000\000\000\090\000\031\000\097\000\000\000\000\000\
\000\000\096\000\082\000\000\000\093\000\000\000\020\000\019\000\
\000\000\026\000\000\000\023\000\014\000\000\000\000\000\059\000\
\069\000\000\000\077\000\000\000\060\000\000\000\000\000\000\000\
\000\000\044\000\048\000\000\000\047\000\100\000\000\000\000\000\
\000\000\084\000\043\000\000\000\000\000\009\000\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\040\000\
\000\000\101\000\083\000\085\000\000\000\000\000\092\000\028\000\
\025\000\000\000\037\000\000\000\000\000\056\000\103\000\000\000\
\095\000\033\000\070\000\078\000\099\000"

let yydgoto = "\002\000\
\007\000\008\000\009\000\010\000\011\000\012\000\013\000\039\000\
\033\000\054\000\094\000\055\000\056\000\057\000\058\000\103\000\
\059\000\060\000\132\000\130\000\061\000\062\000\063\000\064\000\
\134\000\065\000\066\000\087\000\095\000\120\000\155\000\088\000\
\089\000\067\000\068\000\069\000\070\000\142\000\071\000\072\000\
\073\000\074\000\075\000\076\000\077\000\078\000\079\000\080\000\
\081\000\082\000\083\000\151\000\171\000\152\000\026\000\031\000\
\040\000\124\000\125\000\174\000"

let yysindex = "\027\000\
\104\255\000\000\000\000\032\255\052\255\065\255\000\000\062\255\
\006\255\000\000\000\000\000\000\000\000\043\255\043\255\029\255\
\000\000\000\000\000\000\000\000\000\000\075\255\000\000\085\255\
\000\000\089\255\051\255\051\255\017\000\000\000\111\255\000\000\
\119\255\121\255\043\255\114\255\012\000\039\000\126\255\042\000\
\000\000\012\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\128\255\000\000\016\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\135\255\
\000\000\012\000\000\000\000\000\000\000\000\000\090\255\000\000\
\000\000\000\000\000\000\000\000\000\000\141\255\000\000\000\000\
\000\000\142\255\000\000\064\000\075\000\000\000\128\255\000\000\
\117\255\043\255\000\000\144\255\174\255\149\255\000\000\094\255\
\012\000\012\000\000\000\000\000\000\000\000\000\012\000\132\255\
\000\000\054\000\054\000\054\000\054\000\064\000\155\255\014\255\
\000\000\159\255\064\000\000\000\000\000\000\000\165\255\162\255\
\176\255\000\000\000\000\160\255\000\000\168\255\000\000\000\000\
\151\255\000\000\148\255\000\000\000\000\178\255\171\255\000\000\
\000\000\180\255\000\000\190\255\000\000\183\255\182\255\192\255\
\200\255\000\000\000\000\195\255\000\000\000\000\193\255\116\000\
\174\255\000\000\000\000\174\255\174\255\000\000\012\000\012\000\
\203\255\132\255\054\000\054\000\000\000\054\000\000\000\000\000\
\174\255\000\000\000\000\000\000\196\255\209\255\000\000\000\000\
\000\000\012\000\000\000\211\255\215\255\000\000\000\000\174\255\
\000\000\000\000\000\000\000\000\000\000"

let yyrindex = "\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\207\255\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\217\255\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\127\000\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\217\255\000\000\019\255\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\131\255\000\000\122\255\000\000\
\206\255\000\000\197\255\000\000\000\000\000\000\217\255\000\000\
\059\255\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\217\255\
\164\255\000\000\000\000\166\255\000\000\000\000\000\000\000\000\
\063\255\000\000\066\255\000\000\000\000\000\000\218\255\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\219\255\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\167\255\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\239\255\000\000\248\255\000\000\
\000\000\000\000\000\000\000\000\221\255\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000"

let yygindex = "\000\000\
\000\000\217\000\000\000\000\000\000\000\000\000\000\000\111\000\
\199\000\190\000\148\000\000\000\198\255\000\000\000\000\000\000\
\000\000\000\000\076\000\083\000\000\000\000\000\231\255\000\000\
\081\000\163\255\000\000\000\000\240\255\000\000\000\000\187\255\
\189\255\000\000\000\000\000\000\000\000\188\255\172\255\000\000\
\000\000\000\000\000\000\000\000\179\255\184\255\247\255\175\255\
\199\255\204\255\000\000\177\255\000\000\150\255\000\000\000\000\
\154\000\000\000\000\000\062\000"

let yytablesize = 356
let yytable = "\119\000\
\094\000\150\000\091\000\102\000\023\000\023\000\025\000\105\000\
\119\000\030\000\135\000\122\000\086\000\123\000\111\000\147\000\
\112\000\004\000\005\000\006\000\011\000\136\000\141\000\143\000\
\143\000\023\000\011\000\001\000\137\000\137\000\137\000\137\000\
\014\000\138\000\138\000\138\000\138\000\122\000\129\000\131\000\
\144\000\115\000\145\000\019\000\133\000\149\000\172\000\020\000\
\139\000\139\000\139\000\139\000\015\000\140\000\140\000\140\000\
\140\000\019\000\086\000\113\000\046\000\020\000\183\000\021\000\
\027\000\016\000\046\000\024\000\135\000\032\000\027\000\122\000\
\017\000\024\000\122\000\122\000\173\000\175\000\143\000\143\000\
\023\000\143\000\027\000\121\000\086\000\137\000\137\000\122\000\
\137\000\086\000\138\000\138\000\028\000\138\000\180\000\181\000\
\029\000\182\000\106\000\107\000\129\000\131\000\122\000\154\000\
\173\000\139\000\139\000\035\000\139\000\121\000\140\000\140\000\
\036\000\140\000\003\000\004\000\005\000\006\000\041\000\186\000\
\127\000\128\000\072\000\071\000\022\000\024\000\037\000\071\000\
\038\000\071\000\071\000\071\000\067\000\090\000\091\000\093\000\
\067\000\104\000\067\000\069\000\069\000\108\000\109\000\121\000\
\115\000\117\000\121\000\121\000\071\000\071\000\126\000\046\000\
\071\000\071\000\071\000\071\000\146\000\067\000\067\000\121\000\
\148\000\067\000\067\000\067\000\067\000\094\000\157\000\091\000\
\102\000\153\000\094\000\094\000\158\000\091\000\121\000\160\000\
\156\000\159\000\162\000\019\000\163\000\161\000\118\000\020\000\
\165\000\047\000\048\000\049\000\019\000\166\000\164\000\118\000\
\020\000\167\000\047\000\048\000\049\000\079\000\080\000\168\000\
\169\000\147\000\080\000\184\000\080\000\080\000\080\000\075\000\
\170\000\178\000\185\000\075\000\187\000\075\000\077\000\077\000\
\188\000\003\000\113\000\089\000\055\000\036\000\098\000\080\000\
\080\000\018\000\034\000\080\000\080\000\080\000\080\000\092\000\
\075\000\075\000\114\000\177\000\075\000\075\000\075\000\075\000\
\068\000\176\000\179\000\116\000\068\000\189\000\068\000\070\000\
\070\000\076\000\000\000\000\000\000\000\076\000\000\000\076\000\
\078\000\078\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\068\000\068\000\000\000\042\000\068\000\068\000\068\000\
\068\000\043\000\076\000\076\000\000\000\096\000\076\000\076\000\
\076\000\076\000\019\000\000\000\044\000\045\000\020\000\046\000\
\047\000\048\000\049\000\050\000\051\000\000\000\000\000\084\000\
\052\000\053\000\097\000\098\000\085\000\000\000\099\000\100\000\
\101\000\102\000\000\000\000\000\000\000\019\000\000\000\044\000\
\045\000\020\000\046\000\047\000\048\000\049\000\050\000\051\000\
\110\000\000\000\000\000\000\000\019\000\085\000\044\000\045\000\
\020\000\046\000\047\000\048\000\049\000\000\000\019\000\000\000\
\044\000\045\000\020\000\046\000\047\000\048\000\049\000\050\000\
\051\000\019\000\000\000\044\000\045\000\020\000\046\000\047\000\
\048\000\049\000\050\000\051\000"

let yycheck = "\091\000\
\093\000\093\000\093\000\093\000\014\000\015\000\016\000\066\000\
\091\000\026\000\104\000\093\000\038\000\093\000\084\000\002\001\
\084\000\012\001\013\001\014\001\002\001\106\000\107\000\108\000\
\109\000\035\000\008\001\001\000\106\000\107\000\108\000\109\000\
\001\001\106\000\107\000\108\000\109\000\119\000\097\000\098\000\
\109\000\028\001\110\000\015\001\103\000\115\000\153\000\019\001\
\106\000\107\000\108\000\109\000\001\001\106\000\107\000\108\000\
\109\000\015\001\084\000\085\000\002\001\019\001\169\000\021\001\
\002\001\001\001\008\001\002\001\162\000\019\001\008\001\153\000\
\011\001\008\001\156\000\157\000\156\000\157\000\163\000\164\000\
\090\000\166\000\008\001\093\000\110\000\163\000\164\000\169\000\
\166\000\115\000\163\000\164\000\008\001\166\000\163\000\164\000\
\008\001\166\000\009\001\010\001\159\000\160\000\184\000\120\000\
\184\000\163\000\164\000\091\000\166\000\119\000\163\000\164\000\
\002\001\166\000\011\001\012\001\013\001\014\001\005\001\178\000\
\027\001\028\001\001\001\002\001\014\000\015\000\008\001\006\001\
\008\001\008\001\009\001\010\001\002\001\008\001\093\000\008\001\
\006\001\003\001\008\001\009\001\010\001\001\001\001\001\153\000\
\028\001\002\001\156\000\157\000\027\001\028\001\002\001\020\001\
\031\001\032\001\033\001\034\001\002\001\027\001\028\001\169\000\
\002\001\031\001\032\001\033\001\034\001\002\001\007\001\002\001\
\002\001\008\001\007\001\008\001\005\001\008\001\184\000\028\001\
\001\001\027\001\008\001\015\001\001\001\004\001\018\001\019\001\
\002\001\021\001\022\001\023\001\015\001\008\001\001\001\018\001\
\019\001\002\001\021\001\022\001\023\001\001\001\002\001\005\001\
\008\001\002\001\006\001\008\001\008\001\009\001\010\001\002\001\
\093\000\007\001\002\001\006\001\002\001\008\001\009\001\010\001\
\002\001\011\001\002\001\093\000\002\001\004\001\002\001\027\001\
\028\001\009\000\028\000\031\001\032\001\033\001\034\001\042\000\
\027\001\028\001\087\000\160\000\031\001\032\001\033\001\034\001\
\002\001\159\000\162\000\090\000\006\001\184\000\008\001\009\001\
\010\001\002\001\255\255\255\255\255\255\006\001\255\255\008\001\
\009\001\010\001\255\255\255\255\255\255\255\255\255\255\255\255\
\255\255\027\001\028\001\255\255\001\001\031\001\032\001\033\001\
\034\001\006\001\027\001\028\001\255\255\006\001\031\001\032\001\
\033\001\034\001\015\001\255\255\017\001\018\001\019\001\020\001\
\021\001\022\001\023\001\024\001\025\001\255\255\255\255\001\001\
\029\001\030\001\027\001\028\001\006\001\255\255\031\001\032\001\
\033\001\034\001\255\255\255\255\255\255\015\001\255\255\017\001\
\018\001\019\001\020\001\021\001\022\001\023\001\024\001\025\001\
\001\001\255\255\255\255\255\255\015\001\006\001\017\001\018\001\
\019\001\020\001\021\001\022\001\023\001\255\255\015\001\255\255\
\017\001\018\001\019\001\020\001\021\001\022\001\023\001\024\001\
\025\001\015\001\255\255\017\001\018\001\019\001\020\001\021\001\
\022\001\023\001\024\001\025\001"

let yynames_const = "\
  LEFT_PARENTHESIS\000\
  RIGHT_PARENTHESIS\000\
  LEFT_BRACKET\000\
  RIGHT_BRACKET\000\
  DOT\000\
  NEGATION\000\
  COLON\000\
  COMMA\000\
  EQUALITY\000\
  DISEQUALITY\000\
  EOI\000\
  FOF\000\
  CNF\000\
  INCLUDE\000\
  DOLLAR_TRUE\000\
  DOLLAR_FALSE\000\
  DOLLAR\000\
  AND\000\
  OR\000\
  FORALL\000\
  EXISTS\000\
  BIJECTION\000\
  LEFT_IMPLICATION\000\
  RIGHT_IMPLICATION\000\
  UNKNOWN\000\
  "

let yynames_block = "\
  SINGLE_QUOTED\000\
  DOLLAR_WORD\000\
  DOLLAR_DOLLAR_WORD\000\
  DISTINCT_OBJECT\000\
  LOWER_WORD\000\
  UPPER_WORD\000\
  UNSIGNED_INTEGER\000\
  SIGNED_INTEGER\000\
  REAL\000\
  "

let yyact = [|
  (fun _ -> failwith "parser")
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 1 : 'file) in
    Obj.repr(
# 131 "src/parser_tptp.mly"
      (
	let clauses =
          _1
        in
	let is_fof =
	  !fof
	and is_theorem =
	  !theorem
	in
	  (* reset for next parser run *)
	  fof := false;
	  theorem := false;

          if is_fof then
	    raise (Const.FOF is_theorem)

          else
            clauses
      )
# 435 "src/parser_tptp.ml"
               : Term.clause list))
; (fun __caml_parser_env ->
    Obj.repr(
# 152 "src/parser_tptp.mly"
      ( print_endline "empty problem specification"; raise Const.PARSE_ERROR )
# 441 "src/parser_tptp.ml"
               : Term.clause list))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'tptp_input) in
    Obj.repr(
# 161 "src/parser_tptp.mly"
      ( [_1] )
# 448 "src/parser_tptp.ml"
               : 'file))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 1 : 'tptp_input) in
    let _2 = (Parsing.peek_val __caml_parser_env 0 : 'file) in
    Obj.repr(
# 164 "src/parser_tptp.mly"
      ( _1 :: _2 )
# 456 "src/parser_tptp.ml"
               : 'file))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'annotated_formula) in
    Obj.repr(
# 168 "src/parser_tptp.mly"
      ( _1 )
# 463 "src/parser_tptp.ml"
               : 'tptp_input))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'include_) in
    Obj.repr(
# 171 "src/parser_tptp.mly"
      ( _1 )
# 470 "src/parser_tptp.ml"
               : 'tptp_input))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'fof_annotated) in
    Obj.repr(
# 177 "src/parser_tptp.mly"
      ( _1 )
# 477 "src/parser_tptp.ml"
               : 'annotated_formula))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'cnf_annotated) in
    Obj.repr(
# 180 "src/parser_tptp.mly"
      ( _1 )
# 484 "src/parser_tptp.ml"
               : 'annotated_formula))
; (fun __caml_parser_env ->
    let _3 = (Parsing.peek_val __caml_parser_env 7 : 'name) in
    let _5 = (Parsing.peek_val __caml_parser_env 5 : 'formula_role) in
    let _7 = (Parsing.peek_val __caml_parser_env 3 : 'fof_formula) in
    let _8 = (Parsing.peek_val __caml_parser_env 2 : 'annotations) in
    Obj.repr(
# 184 "src/parser_tptp.mly"
    ( fof := true; [] )
# 494 "src/parser_tptp.ml"
               : 'fof_annotated))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'binary_formula) in
    Obj.repr(
# 188 "src/parser_tptp.mly"
    ( "" )
# 501 "src/parser_tptp.ml"
               : 'fof_formula))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'unitary_formula) in
    Obj.repr(
# 191 "src/parser_tptp.mly"
    ( "" )
# 508 "src/parser_tptp.ml"
               : 'fof_formula))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'nonassoc_binary) in
    Obj.repr(
# 196 "src/parser_tptp.mly"
    ( "" )
# 515 "src/parser_tptp.ml"
               : 'binary_formula))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'assoc_binary) in
    Obj.repr(
# 199 "src/parser_tptp.mly"
    ( "" )
# 522 "src/parser_tptp.ml"
               : 'binary_formula))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 2 : 'unitary_formula) in
    let _2 = (Parsing.peek_val __caml_parser_env 1 : 'binary_connective) in
    let _3 = (Parsing.peek_val __caml_parser_env 0 : 'unitary_formula) in
    Obj.repr(
# 203 "src/parser_tptp.mly"
    ( "" )
# 531 "src/parser_tptp.ml"
               : 'nonassoc_binary))
; (fun __caml_parser_env ->
    Obj.repr(
# 207 "src/parser_tptp.mly"
    ( "" )
# 537 "src/parser_tptp.ml"
               : 'binary_connective))
; (fun __caml_parser_env ->
    Obj.repr(
# 209 "src/parser_tptp.mly"
    ( "" )
# 543 "src/parser_tptp.ml"
               : 'binary_connective))
; (fun __caml_parser_env ->
    Obj.repr(
# 211 "src/parser_tptp.mly"
    ( "" )
# 549 "src/parser_tptp.ml"
               : 'binary_connective))
; (fun __caml_parser_env ->
    Obj.repr(
# 213 "src/parser_tptp.mly"
    ( "" )
# 555 "src/parser_tptp.ml"
               : 'binary_connective))
; (fun __caml_parser_env ->
    Obj.repr(
# 215 "src/parser_tptp.mly"
    ( "" )
# 561 "src/parser_tptp.ml"
               : 'binary_connective))
; (fun __caml_parser_env ->
    Obj.repr(
# 217 "src/parser_tptp.mly"
    ( "" )
# 567 "src/parser_tptp.ml"
               : 'binary_connective))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'or_formula) in
    Obj.repr(
# 221 "src/parser_tptp.mly"
    ( "" )
# 574 "src/parser_tptp.ml"
               : 'assoc_binary))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'and_formula) in
    Obj.repr(
# 223 "src/parser_tptp.mly"
    ( "" )
# 581 "src/parser_tptp.ml"
               : 'assoc_binary))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 2 : 'unitary_formula) in
    let _3 = (Parsing.peek_val __caml_parser_env 0 : 'more_or_formula) in
    Obj.repr(
# 227 "src/parser_tptp.mly"
    ( "" )
# 589 "src/parser_tptp.ml"
               : 'or_formula))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'unitary_formula) in
    Obj.repr(
# 231 "src/parser_tptp.mly"
    ( "" )
# 596 "src/parser_tptp.ml"
               : 'more_or_formula))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 2 : 'unitary_formula) in
    let _3 = (Parsing.peek_val __caml_parser_env 0 : 'more_or_formula) in
    Obj.repr(
# 233 "src/parser_tptp.mly"
    ( "" )
# 604 "src/parser_tptp.ml"
               : 'more_or_formula))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 2 : 'unitary_formula) in
    let _3 = (Parsing.peek_val __caml_parser_env 0 : 'more_and_formula) in
    Obj.repr(
# 237 "src/parser_tptp.mly"
    ( "" )
# 612 "src/parser_tptp.ml"
               : 'and_formula))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'unitary_formula) in
    Obj.repr(
# 241 "src/parser_tptp.mly"
    ( "" )
# 619 "src/parser_tptp.ml"
               : 'more_and_formula))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 2 : 'unitary_formula) in
    let _3 = (Parsing.peek_val __caml_parser_env 0 : 'more_and_formula) in
    Obj.repr(
# 243 "src/parser_tptp.mly"
    ( "" )
# 627 "src/parser_tptp.ml"
               : 'more_and_formula))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'quantified_formula) in
    Obj.repr(
# 247 "src/parser_tptp.mly"
    ( "" )
# 634 "src/parser_tptp.ml"
               : 'unitary_formula))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'unary_formula) in
    Obj.repr(
# 249 "src/parser_tptp.mly"
    ( "" )
# 641 "src/parser_tptp.ml"
               : 'unitary_formula))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 1 : 'fof_formula) in
    Obj.repr(
# 251 "src/parser_tptp.mly"
    ( "" )
# 648 "src/parser_tptp.ml"
               : 'unitary_formula))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'atomic_formula) in
    Obj.repr(
# 253 "src/parser_tptp.mly"
    ( "" )
# 655 "src/parser_tptp.ml"
               : 'unitary_formula))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 5 : 'quantifier) in
    let _3 = (Parsing.peek_val __caml_parser_env 3 : 'variable_list) in
    let _6 = (Parsing.peek_val __caml_parser_env 0 : 'unitary_formula) in
    Obj.repr(
# 257 "src/parser_tptp.mly"
    ( "" )
# 664 "src/parser_tptp.ml"
               : 'quantified_formula))
; (fun __caml_parser_env ->
    Obj.repr(
# 261 "src/parser_tptp.mly"
    ( "" )
# 670 "src/parser_tptp.ml"
               : 'quantifier))
; (fun __caml_parser_env ->
    Obj.repr(
# 263 "src/parser_tptp.mly"
    ( "" )
# 676 "src/parser_tptp.ml"
               : 'quantifier))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'variable) in
    Obj.repr(
# 267 "src/parser_tptp.mly"
    ( "" )
# 683 "src/parser_tptp.ml"
               : 'variable_list))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 2 : 'variable) in
    let _3 = (Parsing.peek_val __caml_parser_env 0 : 'variable_list) in
    Obj.repr(
# 269 "src/parser_tptp.mly"
    ( "" )
# 691 "src/parser_tptp.ml"
               : 'variable_list))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 1 : 'unary_connective) in
    let _2 = (Parsing.peek_val __caml_parser_env 0 : 'unitary_formula) in
    Obj.repr(
# 273 "src/parser_tptp.mly"
    ( "" )
# 699 "src/parser_tptp.ml"
               : 'unary_formula))
; (fun __caml_parser_env ->
    Obj.repr(
# 277 "src/parser_tptp.mly"
    ( "" )
# 705 "src/parser_tptp.ml"
               : 'unary_connective))
; (fun __caml_parser_env ->
    let _3 = (Parsing.peek_val __caml_parser_env 7 : 'name) in
    let _5 = (Parsing.peek_val __caml_parser_env 5 : 'formula_role) in
    let _7 = (Parsing.peek_val __caml_parser_env 3 : 'cnf_formula) in
    let _8 = (Parsing.peek_val __caml_parser_env 2 : 'annotations) in
    Obj.repr(
# 283 "src/parser_tptp.mly"
      (
	let clause = 
	  _7
	in
	  init_clause ();
	  clause
      )
# 721 "src/parser_tptp.ml"
               : 'cnf_annotated))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : string) in
    Obj.repr(
# 293 "src/parser_tptp.mly"
    ( let role =
        _1
      in
        if role = "conjecture" then
          theorem := true;

        _1
    )
# 735 "src/parser_tptp.ml"
               : 'formula_role))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'null) in
    Obj.repr(
# 304 "src/parser_tptp.mly"
      ( "" )
# 742 "src/parser_tptp.ml"
               : 'annotations))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 1 : 'source) in
    let _3 = (Parsing.peek_val __caml_parser_env 0 : 'optional_info) in
    Obj.repr(
# 307 "src/parser_tptp.mly"
      ( "" )
# 750 "src/parser_tptp.ml"
               : 'annotations))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 1 : 'disjunction) in
    Obj.repr(
# 313 "src/parser_tptp.mly"
      ( _2 )
# 757 "src/parser_tptp.ml"
               : 'cnf_formula))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'disjunction) in
    Obj.repr(
# 316 "src/parser_tptp.mly"
      ( _1 )
# 764 "src/parser_tptp.ml"
               : 'cnf_formula))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'literal) in
    Obj.repr(
# 320 "src/parser_tptp.mly"
      ( [_1] )
# 771 "src/parser_tptp.ml"
               : 'disjunction))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 2 : 'literal) in
    let _3 = (Parsing.peek_val __caml_parser_env 0 : 'disjunction) in
    Obj.repr(
# 323 "src/parser_tptp.mly"
      ( _1 :: _3 )
# 779 "src/parser_tptp.ml"
               : 'disjunction))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 1 : 'literal) in
    Obj.repr(
# 330 "src/parser_tptp.mly"
     ( _2 )
# 786 "src/parser_tptp.ml"
               : 'literal))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'atomic_formula) in
    Obj.repr(
# 333 "src/parser_tptp.mly"
      ( _1 )
# 793 "src/parser_tptp.ml"
               : 'literal))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 0 : 'atomic_formula) in
    Obj.repr(
# 336 "src/parser_tptp.mly"
      ( Term.request_negated_literal _2 )
# 800 "src/parser_tptp.ml"
               : 'literal))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'plain_atom) in
    Obj.repr(
# 340 "src/parser_tptp.mly"
      ( _1 )
# 807 "src/parser_tptp.ml"
               : 'atomic_formula))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'defined_atom) in
    Obj.repr(
# 343 "src/parser_tptp.mly"
      ( _1 )
# 814 "src/parser_tptp.ml"
               : 'atomic_formula))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'system_atom) in
    Obj.repr(
# 346 "src/parser_tptp.mly"
      ( _1 )
# 821 "src/parser_tptp.ml"
               : 'atomic_formula))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'plain_term_top) in
    Obj.repr(
# 350 "src/parser_tptp.mly"
      ( Term.request_literal true _1 )
# 828 "src/parser_tptp.ml"
               : 'plain_atom))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'term) in
    Obj.repr(
# 354 "src/parser_tptp.mly"
      ( [ _1 ] )
# 835 "src/parser_tptp.ml"
               : 'arguments))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 2 : 'term) in
    let _3 = (Parsing.peek_val __caml_parser_env 0 : 'arguments) in
    Obj.repr(
# 357 "src/parser_tptp.mly"
      ( _1 :: _3 )
# 843 "src/parser_tptp.ml"
               : 'arguments))
; (fun __caml_parser_env ->
    Obj.repr(
# 361 "src/parser_tptp.mly"
      ( Term.true_literal )
# 849 "src/parser_tptp.ml"
               : 'defined_atom))
; (fun __caml_parser_env ->
    Obj.repr(
# 364 "src/parser_tptp.mly"
      ( Term.false_literal )
# 855 "src/parser_tptp.ml"
               : 'defined_atom))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 2 : 'term) in
    let _3 = (Parsing.peek_val __caml_parser_env 0 : 'term) in
    Obj.repr(
# 367 "src/parser_tptp.mly"
      ( Term.request_literal true (Term.request_func (Symbol.equality, [| _1; _3|])) )
# 863 "src/parser_tptp.ml"
               : 'defined_atom))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 2 : 'term) in
    let _3 = (Parsing.peek_val __caml_parser_env 0 : 'term) in
    Obj.repr(
# 370 "src/parser_tptp.mly"
      ( Term.request_literal false (Term.request_func (Symbol.equality, [| _1; _3|])) )
# 871 "src/parser_tptp.ml"
               : 'defined_atom))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'system_term_top) in
    Obj.repr(
# 374 "src/parser_tptp.mly"
      ( Term.request_literal true _1 )
# 878 "src/parser_tptp.ml"
               : 'system_atom))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'function_term) in
    Obj.repr(
# 379 "src/parser_tptp.mly"
      ( _1 )
# 885 "src/parser_tptp.ml"
               : 'term))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'variable) in
    Obj.repr(
# 382 "src/parser_tptp.mly"
      ( Term.request_var _1 )
# 892 "src/parser_tptp.ml"
               : 'term))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'plain_term) in
    Obj.repr(
# 386 "src/parser_tptp.mly"
      ( _1 )
# 899 "src/parser_tptp.ml"
               : 'function_term))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'defined_term) in
    Obj.repr(
# 389 "src/parser_tptp.mly"
      ( _1 )
# 906 "src/parser_tptp.ml"
               : 'function_term))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'system_term) in
    Obj.repr(
# 392 "src/parser_tptp.mly"
      ( _1 )
# 913 "src/parser_tptp.ml"
               : 'function_term))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'constant) in
    Obj.repr(
# 396 "src/parser_tptp.mly"
      ( Term.request_const (Symbol.create_predicate _1 0) )
# 920 "src/parser_tptp.ml"
               : 'plain_term_top))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 3 : 'functor_) in
    let _3 = (Parsing.peek_val __caml_parser_env 1 : 'arguments) in
    Obj.repr(
# 399 "src/parser_tptp.mly"
      ( let subterms = Array.of_list _3 in
	  Term.request_func (Symbol.create_predicate _1 (Array.length subterms), subterms)
      )
# 930 "src/parser_tptp.ml"
               : 'plain_term_top))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'constant) in
    Obj.repr(
# 405 "src/parser_tptp.mly"
      ( Term.request_const (Symbol.create_function _1 0) )
# 937 "src/parser_tptp.ml"
               : 'plain_term))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 3 : 'functor_) in
    let _3 = (Parsing.peek_val __caml_parser_env 1 : 'arguments) in
    Obj.repr(
# 408 "src/parser_tptp.mly"
      ( let subterms = Array.of_list _3 in
	  Term.request_func (Symbol.create_function _1 (Array.length subterms), subterms)
      )
# 947 "src/parser_tptp.ml"
               : 'plain_term))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'atomic_word) in
    Obj.repr(
# 414 "src/parser_tptp.mly"
      ( _1 )
# 954 "src/parser_tptp.ml"
               : 'constant))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'atomic_word) in
    Obj.repr(
# 418 "src/parser_tptp.mly"
      ( _1 )
# 961 "src/parser_tptp.ml"
               : 'functor_))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'number) in
    Obj.repr(
# 422 "src/parser_tptp.mly"
      ( print_endline ("Parser_tptp: <defined_term: number> not supported: " ^ _1); raise Const.PARSE_ERROR )
# 968 "src/parser_tptp.ml"
               : 'defined_term))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : string) in
    Obj.repr(
# 425 "src/parser_tptp.mly"
      ( print_endline ("Parser_tptp: <defined_term: distinct_object> not supported: " ^ _1); raise Const.PARSE_ERROR )
# 975 "src/parser_tptp.ml"
               : 'defined_term))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'system_constant) in
    Obj.repr(
# 429 "src/parser_tptp.mly"
      ( Term.request_const (Symbol.create_predicate _1 0) )
# 982 "src/parser_tptp.ml"
               : 'system_term_top))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 3 : 'system_functor) in
    let _3 = (Parsing.peek_val __caml_parser_env 1 : 'arguments) in
    Obj.repr(
# 432 "src/parser_tptp.mly"
      ( let subterms = Array.of_list _3 in
	  Term.request_func (Symbol.create_predicate _1 (Array.length subterms), subterms)
      )
# 992 "src/parser_tptp.ml"
               : 'system_term_top))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'system_constant) in
    Obj.repr(
# 438 "src/parser_tptp.mly"
      ( Term.request_const (Symbol.create_function _1 0) )
# 999 "src/parser_tptp.ml"
               : 'system_term))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 3 : 'system_functor) in
    let _3 = (Parsing.peek_val __caml_parser_env 1 : 'arguments) in
    Obj.repr(
# 441 "src/parser_tptp.mly"
      ( let subterms = Array.of_list _3 in
	  Term.request_func (Symbol.create_function _1 (Array.length subterms), subterms)
      )
# 1009 "src/parser_tptp.ml"
               : 'system_term))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'atomic_system_word) in
    Obj.repr(
# 447 "src/parser_tptp.mly"
      ( _1 )
# 1016 "src/parser_tptp.ml"
               : 'system_functor))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'atomic_system_word) in
    Obj.repr(
# 451 "src/parser_tptp.mly"
      ( _1 )
# 1023 "src/parser_tptp.ml"
               : 'system_constant))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : string) in
    Obj.repr(
# 457 "src/parser_tptp.mly"
      ( get_var _1 )
# 1030 "src/parser_tptp.ml"
               : 'variable))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'general_term) in
    Obj.repr(
# 463 "src/parser_tptp.mly"
      ( "" )
# 1037 "src/parser_tptp.ml"
               : 'source))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 0 : 'useful_info) in
    Obj.repr(
# 467 "src/parser_tptp.mly"
      ( "" )
# 1044 "src/parser_tptp.ml"
               : 'optional_info))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'null) in
    Obj.repr(
# 470 "src/parser_tptp.mly"
      ( "" )
# 1051 "src/parser_tptp.ml"
               : 'optional_info))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'general_term_list) in
    Obj.repr(
# 474 "src/parser_tptp.mly"
      ( "" )
# 1058 "src/parser_tptp.ml"
               : 'useful_info))
; (fun __caml_parser_env ->
    let _3 = (Parsing.peek_val __caml_parser_env 3 : 'file_name) in
    let _4 = (Parsing.peek_val __caml_parser_env 2 : 'formula_selection) in
    Obj.repr(
# 479 "src/parser_tptp.mly"
      ( print_endline "Parser_tptp: <include> not supported."; raise Const.PARSE_ERROR )
# 1066 "src/parser_tptp.ml"
               : 'include_))
; (fun __caml_parser_env ->
    let _3 = (Parsing.peek_val __caml_parser_env 1 : 'name_list) in
    Obj.repr(
# 483 "src/parser_tptp.mly"
      ( _3 )
# 1073 "src/parser_tptp.ml"
               : 'formula_selection))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'null) in
    Obj.repr(
# 486 "src/parser_tptp.mly"
      ( [] )
# 1080 "src/parser_tptp.ml"
               : 'formula_selection))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'name) in
    Obj.repr(
# 490 "src/parser_tptp.mly"
      ( [_1] )
# 1087 "src/parser_tptp.ml"
               : 'name_list))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 2 : 'name) in
    let _3 = (Parsing.peek_val __caml_parser_env 0 : 'name_list) in
    Obj.repr(
# 493 "src/parser_tptp.mly"
      ( _1 :: _3 )
# 1095 "src/parser_tptp.ml"
               : 'name_list))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'general_data) in
    Obj.repr(
# 499 "src/parser_tptp.mly"
      ( "" )
# 1102 "src/parser_tptp.ml"
               : 'general_term))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 2 : 'general_data) in
    let _3 = (Parsing.peek_val __caml_parser_env 0 : 'general_term) in
    Obj.repr(
# 502 "src/parser_tptp.mly"
      ( "" )
# 1110 "src/parser_tptp.ml"
               : 'general_term))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'general_list) in
    Obj.repr(
# 505 "src/parser_tptp.mly"
      ( "" )
# 1117 "src/parser_tptp.ml"
               : 'general_term))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'atomic_word) in
    Obj.repr(
# 509 "src/parser_tptp.mly"
      ( "" )
# 1124 "src/parser_tptp.ml"
               : 'general_data))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 3 : 'atomic_word) in
    let _3 = (Parsing.peek_val __caml_parser_env 1 : 'general_arguments) in
    Obj.repr(
# 512 "src/parser_tptp.mly"
      ( "" )
# 1132 "src/parser_tptp.ml"
               : 'general_data))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'number) in
    Obj.repr(
# 515 "src/parser_tptp.mly"
      ( "" )
# 1139 "src/parser_tptp.ml"
               : 'general_data))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : string) in
    Obj.repr(
# 518 "src/parser_tptp.mly"
      ( "" )
# 1146 "src/parser_tptp.ml"
               : 'general_data))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'general_term) in
    Obj.repr(
# 522 "src/parser_tptp.mly"
      ( [_1] )
# 1153 "src/parser_tptp.ml"
               : 'general_arguments))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 2 : 'general_term) in
    let _3 = (Parsing.peek_val __caml_parser_env 0 : 'general_arguments) in
    Obj.repr(
# 525 "src/parser_tptp.mly"
      ( _1 :: _3 )
# 1161 "src/parser_tptp.ml"
               : 'general_arguments))
; (fun __caml_parser_env ->
    Obj.repr(
# 529 "src/parser_tptp.mly"
      ( [] )
# 1167 "src/parser_tptp.ml"
               : 'general_list))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 1 : 'general_term_list) in
    Obj.repr(
# 532 "src/parser_tptp.mly"
      ( _2 )
# 1174 "src/parser_tptp.ml"
               : 'general_list))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'general_term) in
    Obj.repr(
# 536 "src/parser_tptp.mly"
      ( [_1] )
# 1181 "src/parser_tptp.ml"
               : 'general_term_list))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 2 : 'general_term) in
    let _3 = (Parsing.peek_val __caml_parser_env 0 : 'general_term_list) in
    Obj.repr(
# 539 "src/parser_tptp.mly"
      ( _1 :: _3 )
# 1189 "src/parser_tptp.ml"
               : 'general_term_list))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'atomic_word) in
    Obj.repr(
# 544 "src/parser_tptp.mly"
      ( _1 )
# 1196 "src/parser_tptp.ml"
               : 'name))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : string) in
    Obj.repr(
# 547 "src/parser_tptp.mly"
      ( _1 )
# 1203 "src/parser_tptp.ml"
               : 'name))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : string) in
    Obj.repr(
# 551 "src/parser_tptp.mly"
      ( _1 )
# 1210 "src/parser_tptp.ml"
               : 'atomic_word))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : string) in
    Obj.repr(
# 554 "src/parser_tptp.mly"
      ( _1 )
# 1217 "src/parser_tptp.ml"
               : 'atomic_word))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : string) in
    Obj.repr(
# 558 "src/parser_tptp.mly"
      ( print_endline ("Parser_tptp: <$$word> not supported: " ^ _1); raise Const.PARSE_ERROR )
# 1224 "src/parser_tptp.ml"
               : 'atomic_system_word))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : string) in
    Obj.repr(
# 562 "src/parser_tptp.mly"
      ( _1 )
# 1231 "src/parser_tptp.ml"
               : 'number))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : string) in
    Obj.repr(
# 565 "src/parser_tptp.mly"
      ( _1 )
# 1238 "src/parser_tptp.ml"
               : 'number))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : string) in
    Obj.repr(
# 568 "src/parser_tptp.mly"
      ( _1 )
# 1245 "src/parser_tptp.ml"
               : 'number))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'atomic_word) in
    Obj.repr(
# 572 "src/parser_tptp.mly"
      ( _1 )
# 1252 "src/parser_tptp.ml"
               : 'file_name))
; (fun __caml_parser_env ->
    Obj.repr(
# 575 "src/parser_tptp.mly"
      ( )
# 1258 "src/parser_tptp.ml"
               : 'null))
(* Entry parse_file *)
; (fun __caml_parser_env -> raise (Parsing.YYexit (Parsing.peek_val __caml_parser_env 0)))
|]
let yytables =
  { Parsing.actions=yyact;
    Parsing.transl_const=yytransl_const;
    Parsing.transl_block=yytransl_block;
    Parsing.lhs=yylhs;
    Parsing.len=yylen;
    Parsing.defred=yydefred;
    Parsing.dgoto=yydgoto;
    Parsing.sindex=yysindex;
    Parsing.rindex=yyrindex;
    Parsing.gindex=yygindex;
    Parsing.tablesize=yytablesize;
    Parsing.table=yytable;
    Parsing.check=yycheck;
    Parsing.error_function=parse_error;
    Parsing.names_const=yynames_const;
    Parsing.names_block=yynames_block }
let parse_file (lexfun : Lexing.lexbuf -> token) (lexbuf : Lexing.lexbuf) =
   (Parsing.yyparse yytables 1 lexfun lexbuf : Term.clause list)
;;
# 578 "src/parser_tptp.mly"




# 1288 "src/parser_tptp.ml"
