(*
This file is part of the first order theorem prover Darwin
Copyright (C) 2004, 2005, 2006
              The University of Iowa
              Universitaet Koblenz-Landau 

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*)


(** selection of (multi-literal) clause candidates

    stores all rewritten clause candidates,
    and selects one for application.
*)

type config = Config.config
type state = State.state
type bound = Bound.bound
type statistic = Statistic.statistic
type selected = Selection_types.selected
type clause = Term.clause
type context = Context.context
type element = Context.element
type candidate_type = State.literal_type
type context_unifier_space = Context_unifier.context_unifier_space
type problem_literals = Problem_literals.problem_literals
type problem_subterms = Problem_subterms.problem_subterms

type candidates

val create : config -> bound -> state -> context -> statistic -> problem_literals 
	-> problem_subterms ->(context_unifier_space Term_indexing.perfect_clause_index) -> candidates
	
val check_exceeding : candidates -> unit	
	
(** Adds a rewritten clause candidate to the repository.
[add ?rewriting candidates element clause], [element] being the rewriting context element
used in the paramodulation inference responsible for creating [clause], and space
being the parent clause. *) 
val add : ?rewriting:Problem_subterms.rewriting -> candidates -> candidate_type 
-> element -> context_unifier_space -> clause -> unit

val select : candidates -> selected option

val backtrack : candidates -> unit

(** tests whether [candidates] contains an empty clause candidate, which can close the derivation branch *)
val has_closing_candidate : candidates -> bool