(*
This file is part of the first order theorem prover Darwin
Copyright (C) 2004, 2005, 2006
              The University of Iowa
              Universitaet Koblenz-Landau 

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*)


(** common types for selection modules *)


type literal = Term.literal
type clause = Term.clause
type choice_point = State.choice_point
type raw_context_unifier = Context_unifier.raw_context_unifier
type context_unifier_space = Context_unifier.context_unifier_space
type element = Context.element
type rewriting = Problem_subterms.rewriting
type explanation = State.explanation

(** a selected candidate *)
type selected = {
  candidate_type: State.literal_type;
  (** what type of candidate? *)

  literal: literal;
  (** the literal to add to the context (null_literal if clause is added). *)
  
  clause: clause;
  (** the clause added to the clause set (empty if literal is added) *)
  
  element: element;
  (** the Context.element used to create the candidate (null_element if literal is added) *)

  raw_context_unifier: raw_context_unifier;
  (** the context unifier from which this candidate was computed. *)
  
  context_unifier_space: context_unifier_space;
  (** the parent clause (null_space if literal is added). *)
  
  rewriting: rewriting;
  (** information about the rewriting leading to this candidate (null_rewriting if not applicable) *)
  
  explanation: explanation;
  (** the explanation behind the clause candidate, resulting from a merger between the element's
  and the context_unifier_space's explanations (State.empty_explanation if not applicable) *)
}
