#!/usr/bin/pl -q -t main -f

:- use_module('script_support').
:- use_module('../contrib/pl/swilib/pretty').

main :-
        main(before(_, Problem), linearize_proof(_, Problem), true).
		
proof_id(node(Id, _, _, _, _), Id) :- !.
proof_id(split(Id, _, _, _), Id) :- !.
proof_id(branch(Id, _, _), Id) :- !.
proof_id(ref(Id), Id) :- %nl,nl,write(Id),nl,nl,
	!.
proof_id(P, _) :-
	( nonvar(P) -> functor(P, F, N) ; true ),
	err('Bad proof term with functor: ~q/~q.', [F,N]).		
	
linearize_proof(szs_satisfiable, Problem) :-
	!,
	nl,
	write('SZS status Satisfiable for '),
	write(Problem),
	nl,nl.
	
linearize_proof(szs_refutation_start, Problem) :-
	!,
	nl,
	write('SZS status Unsatisfiable for '),
	write(Problem),
	nl,nl,
	write('SZS output start CNFRefutation for '),
	write(Problem),
	nl,nl.
	
linearize_proof(szs_refutation_stop, Problem) :-
	!,
	nl,nl,
	write('SZS output stop CNFRefutation for '),
	write(Problem),
	nl.
	
linearize_proof(C, _) :-
	linearize_proof_term([C], [], ProofSteps),
	print_steps(ProofSteps).

inference_input_write([]) :- !.
inference_input_write(L) :-
	write('('),
	write(L),
	write(')').
	
print_steps([]) :- !.
print_steps([(Id, Clause, Type, SubIds)|Rest]) :-
	write(Id),
	write('('),
	pp_clause_line(Clause),
	write(')\tsource: '),
	translate(Type, TranslatedType),
	write(TranslatedType),
	inference_input_write(SubIds),
	nl,
	print_steps(Rest).
	
translate(hyper(_), 'hyperresolution/split'):-!.
translate(ref(_), 'reflexivity'):-!.
translate('sup-left'(_, _), 'superposition-left'):-!.
translate('unit-sup-right'(_, _), 'unit-superposition-right'):-!.
translate(input_clause, 'input'):-!.
translate(input_fact, 'input'):-!.
translate(X, X).	
	
proof_ids([],[]) :- !.
proof_ids([Proof|Rest], [ProofId|RestIds]) :-
		!,
		proof_id(Proof, ProofId),
		proof_ids(Rest, RestIds).
proof_ids(Proof,[ProofId]) :- 
	!,
	proof_id(Proof, ProofId).
	
linearize_proof_term([], Linearized, Linearized) :- !.
linearize_proof_term([node(Id, Clause, Type, Sub1, Sub2)|ToDo], LinearizedSoFar, Linearized) :-%write(Id),write(Type),nl,
	append(Sub1, Sub2, Sub),
	proof_ids(Sub, SubIds),
	append(ToDo, Sub, ToDoNext),	%write(ToDoNext),nl,
	linearize_proof_term(ToDoNext, [(Id, Clause, Type, SubIds)|LinearizedSoFar], Linearized).
	
linearize_proof_term([ref(_Id)|ToDo], LinearizedSoFar, Linearized) :-
	linearize_proof_term(ToDo, LinearizedSoFar, Linearized).
	
linearize_proof_term([split(Id, hyper(Clause), Sub1, Sub2)|ToDo], LinearizedSoFar, Linearized) :-
	append(Sub1, Sub2, Sub),
	proof_ids(Sub, SubIds),
	append(ToDo, Sub, ToDoNext),
	linearize_proof_term(ToDoNext, [(Id, Clause, hyper(Clause), SubIds)|LinearizedSoFar], Linearized).
	
linearize_proof_term([branch(Id, Clause, Sub)|ToDo], LinearizedSoFar, Linearized) :-	%write(Sub),
	proof_ids(Sub, SubIds),			%write(Sub),
	append(ToDo, [Sub], ToDoNext),	%write(ToDo),nl,
	linearize_proof_term(ToDoNext, [(Id, Clause, branch, SubIds)|LinearizedSoFar], Linearized).

before(Options, Problem) :-write(gggggooooooooooooggggggg),
	extract_option_value('-problem=', Options, Problem),write(gggggggggggg).

extract_option_value(Key, Options, Value) :-
	atom_length(Key, L),
	member(Option, Options),
	sub_atom(Option, 0, L, _, Key),
	!,
	sub_atom(Option, L, _, 0, Value).

