#!/usr/bin/pl -G16M -L32M -q -t main -f

:- op(1150, xfy, ::).

:- use_module('script_support').
:- use_module('nf').
:- use_module('../contrib/pl/swilib/pretty').

%%%% 
%%%% Simple magic sets transformation for Horn clauses, using the input
%%%% clause ordering (see Stickel: Upside-Down). Quickly written.
%%%%

ms(C, C1) :-
	tme_clause_to_hb(C, H, B),
	( H = [H1], B \= [] ->
	  ms1(B, H1, C1)
	; C1 = C
	).

abs_intersection([X|Xs], Ys, [Y|Zs]) :-
	member(Y, Ys),
	X == Y,
	!,
	abs_intersection(Xs, Ys, Zs).
abs_intersection([_|Xs], Ys, Zs) :-
	abs_intersection(Xs, Ys, Zs).
abs_intersection([], _, []).
	
gengoal(T, G) :-
	T =.. [F|Args],
	atom_concat('$goal_', F, F1),
	G =.. [F1|Args].


ms1([A1|As], Head, C) :-
	gengoal(Head, GoalHead),
	gengoal(A1, GoalA1),
	( C = (GoalA1 :- GoalHead)
	; free_variables(Head, VsHead),
	  free_variables(A1-As, VsBody),
	  abs_intersection(VsHead, VsBody, VsHead1),
	  functor(Head, HFunctor, _),
	  atom_concat('$cont_', HFunctor, GCK),
	  ms2([A1|As], GoalHead, GCK, [], VsHead1, Head, C)
	).

ms2([A1,A2|As], GC, GCK, VsLeft, VsHead, Head, C) :-
	!,
	free_variables(A2-As, VsRight),
	free_variables(A1-VsLeft, VsLeft1),
	abs_intersection(VsRight, VsLeft1, VsBody),
	append(VsBody, VsHead, VsCont),
	sort(VsCont, VsCont1),
	gensym(GCK, GCF),
	Cont2 =.. [GCF|VsCont1],
	gengoal(A2, GA2),
	( C = (GA2, Cont2 :- GC, A1)
	; ms2([A2|As], Cont2, GCK, VsLeft1, VsHead, Head, C)
	).
ms2([A], GC, _, _, _, Head, C) :-
	C = (Head :- GC, A).

pp_ms(C) :-
	( ms(C, C1),
	  pp_clause_bottom_up(C1),
	  fail
	; true
	).
	
pp_clause_bottom_up((H :- B)) :-
        !,
	numbervars(B-H, 0, _),
	pp_clause((H :- B)).
pp_clause_bottom_up(C) :-
	pp_clause(C).

main :-
	style_check(-atom),
	main(true, pp_ms(_), true).

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


tme_clause_to_hb((H :- B), H1, B1) :-
        !,
        positives(H, H1),
        negatives(B, B1).
tme_clause_to_hb(H, H1, []) :-
        positives(H, H1).

positives(false, []) :-
        !.
positives((false ; Ls), Ls1) :-
        !,
        positives(Ls, Ls1).
positives((L ; Ls), [L|Ls1]) :-
        !,
        positives(Ls, Ls1).
positives(L, [L]).

negatives(true, []) :-
        !.
negatives((true , Ls), Ls1) :-
        !,
        negatives(Ls, Ls1).
negatives((L , Ls), [L|Ls1]) :-
        !,
        negatives(Ls, Ls1).
negatives(L, [L]).

