:- module(nftme,
	  [nf_clause_to_tme/2,
	   tme_clause_to_nf/2,
	   pp_tme_clause_bottom_up/1,
	   m_pp_tme/1]).

:- use_module('../contrib/pl/swilib/pretty').

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%% 
%%%% Conversion between TME Clauses and NF (i.e. CM-NF Transformation)
%%%% Clauses. Only pure CNF clauses are handled, no specials such as
%%%% conjunctions in heads etc.
%%%%
%%%% Literal ordering (among literals with same signs) of input is preserved.
%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

tme_clause_to_nf((H :- B), C) :-
        !,
	positives(H, C1),
	negatives(B, C2),
	append(C1, C2, C).
tme_clause_to_nf(H, C) :-
	positives(H, C).

positives(false, []) :-
	!.
positives((false ; Ls), Ls1) :-
	!,
	positives(Ls, Ls1).
positives((L ; Ls), [L|Ls1]) :-
	!,
	positives(Ls, Ls1).
positives(L, [L]).

negatives(true, []) :-
	!.
negatives((true , Ls), Ls1) :-
	!,
	negatives(Ls, Ls1).
negatives((L , Ls), [~(L)|Ls1]) :-
	!,
	negatives(Ls, Ls1).
negatives(L, [~(L)]).


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

nf_clause_to_tme(Lits, TME) :-
	split_pn(Lits, PLits, NLits),
	list_to_disjunction(PLits, Head),
	( NLits = [] ->
	  TME = Head
	; list_to_conjunction(NLits, Body),
          TME = (Head :- Body)
	).

split_pn([~(A)|Ls], P, [A|N]) :-
	!,
	split_pn(Ls, P, N).
split_pn([A|Ls], [A|P], N) :-
	split_pn(Ls, P, N).
split_pn([], [], []).

list_to_disjunction([], false).
list_to_disjunction([L], L) :-
	!.
list_to_disjunction([L|Ls], (L;Ls1)) :-
	list_to_disjunction(Ls, Ls1).

list_to_conjunction([], true).
list_to_conjunction([L], L) :-
	!.
list_to_conjunction([L|Ls], (L,Ls1)) :-
	list_to_conjunction(Ls, Ls1).

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%% 
%%%% Pretty print a clause, start variable numbering from the body.
%%%% 
pp_tme_clause_bottom_up((H :- B)) :-
        !,
	numbervars(B-H, 0, _),
	pp_clause((H :- B)).
pp_tme_clause_bottom_up(C) :-
	pp_clause(C).

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

m_pp_tme(M) :-
	( member(C, M),
	  nf_clause_to_tme(C, C1),
	  pp_tme_clause_bottom_up(C1),
	  fail
	; true
	).
