#!/usr/bin/pl -G16M -L32M -q -t main -f

% Transforms tme-files into the Magic-Set notation used in TPTP.
:- op(1150,xfy,::).
:- op(100,fx,++).
:- op(100,fx,--).
:- op(400,xfx,'..').
:- use_module('../scripts/script_support').
:- use_module('../scripts/nftme').
:- use_module('../scripts/nf').
:- use_module('../contrib/pl/swilib/pretty').

main:-
	style_check(-atom),
	main(true, process(_), true).


process(Input):-
	tme_clause_to_nf(Input,NF),
	mark(NF, MarkedNF),
	nhms_clause(MarkedNF, MarkedMagicNF),
	unmark(MarkedMagicNF, MagicNF),
	m_pp_tme_mod(MagicNF),
	nl.

% Change the NF to Mark Stickel's positive/negative marking style of literals.
mark([],[]):-!.
mark([~A|Rest], [--A|MarkedRest]):-
	!,
	mark(Rest, MarkedRest).
mark([A|Rest], [++A|MarkedRest]):-
	!,
	mark(Rest, MarkedRest).


% Change the positive/negative marking of literals back to NF.
unmark([],[]):-!.
unmark([A|Rest], [UnmarkedA|UnmarkedRest]):-
	!,
	unmark_sub(A, UnmarkedA),
	unmark(Rest, UnmarkedRest).
unmark_sub([],[]):-!.
unmark_sub([--A|Rest], [~A|MarkedRest]):-
	!,
	unmark_sub(Rest, MarkedRest).
unmark_sub([++A|Rest], [A|MarkedRest]):-
	!,
	unmark_sub(Rest, MarkedRest).
unmark_sub(--A, ~A).
unmark_sub(++A, A).

% Modified version of m_pp_tme/1 from nftme.pl;
% does not skip unit clauses.
m_pp_tme_mod(M) :-
	( member(C, M),
	  m_pp_tme_mod_sub(C),
	  fail
	; true
	).
m_pp_tme_mod_sub(C):-
	nf_clause_to_tme(C, Tme),
	!,
	pp_tme_clause_bottom_up(Tme).
m_pp_tme_mod_sub(C):-
	write(C),
	writeln('.').
	
	
% The following are various predicates from the transform.magic program
% from the tptp2X-utility, adjusted for our purposes.

%----Split literals into positive and negative ones
split_clause([],[],[],[],[]):-
    !.

split_clause([++Atom|RestOfLiterals],[Atom|RestOfPositiveAtoms],
NegativeAtoms,[++Atom|RestOfPositiveLiterals],NegativeLiterals):-
    split_clause(RestOfLiterals,RestOfPositiveAtoms,NegativeAtoms,
RestOfPositiveLiterals,NegativeLiterals).

split_clause([--Atom|RestOfLiterals],PositiveAtoms,
[Atom|RestOfNegativeAtoms],PositiveLiterals,[--Atom|
RestOfNegativeLiterals]):-
    split_clause(RestOfLiterals,PositiveAtoms,RestOfNegativeAtoms,
PositiveLiterals,RestOfNegativeLiterals).

%--------------------------------------------------------------------
%----Convert clauses to sets of magic clauses
%----Positive atoms are not changed
nhms_clause([++Atom],[++Atom]):-!.

%----Other clauses make several clauses
nhms_clause(Literals,MagicClauses):-
%----Split the negative and positive literals apart, and their atoms
    split_clause(Literals,PositiveAtoms,NegativeAtoms,
PositiveLiterals,NegativeLiterals),
%----Make negative goal literals from the positive atoms
    map_list(PositiveAtoms,make_negative_goal_literal,
NegativeGoalLiterals),
%----Make clause for each negative literal
    subgoal_clauses(NegativeGoalLiterals,NegativeAtoms,
MagicLiteralsList),
    append(NegativeLiterals,PositiveLiterals,MagicLiterals1),
    append(NegativeGoalLiterals,MagicLiterals1,MagicLiterals2),
    append(MagicLiteralsList,[MagicLiterals2],MagicClauses).
%--------------------------------------------------------------------
%----Convert each clause and append the output sets
nhms_clauses([],[]):-
    !.

nhms_clauses([FirstClause|RestOfClauses],MagicClauses):-
    nhms_clause(FirstClause,FirstMagicClauses),
    nhms_clauses(RestOfClauses,RestOfMagicClauses),
    append(FirstMagicClauses,RestOfMagicClauses,MagicClauses).
%--------------------------------------------------------------------

%--------------------------------------------------------------------
%----Add "goal_" onto the front of the atom's predicate symbol
make_goal_atom(Atom,Atom1):-
    Atom =.. [PredicateSymbol|Arguments],
    name(PredicateSymbol,PredicateSymbolAscii),
    append("goal_",PredicateSymbolAscii,NewPredicateSymbolAscii),
    name(NewPredicateSymbol,NewPredicateSymbolAscii),
    Atom1 =.. [NewPredicateSymbol|Arguments].
%--------------------------------------------------------------------
%----Make a literal from an atom
make_positive_literal(Atom,++Atom).

make_negative_literal(Atom,--Atom).
%--------------------------------------------------------------------
%----Makes a positive literal from an atom, adding "goal_" to the
%----front of the predicate symbol
make_positive_goal_literal(Atom,PositiveGoalLiteral):-
    make_goal_atom(Atom,GoalAtom),
    make_positive_literal(GoalAtom,PositiveGoalLiteral).
%--------------------------------------------------------------------
%----Makes a negative literal from an atom, adding "goal_" to the
%----front of the predicate symbol
make_negative_goal_literal(Atom,NegativeGoalLiteral):-
    make_goal_atom(Atom,GoalAtom),
    make_negative_literal(GoalAtom,NegativeGoalLiteral).
%--------------------------------------------------------------------
%----Make a clause for each negative literal
subgoal_clauses(_,[],[]):-
    !.

subgoal_clauses(NegativeLiterals,[FirstNegativeAtom|
RestOfNegativeAtoms],[FirstClause|RestOfClauses]):-
    make_positive_goal_literal(FirstNegativeAtom,PositiveLiteral),
    make_negative_literal(FirstNegativeAtom,NegativeLiteral),
    append(NegativeLiterals,[PositiveLiteral],FirstClause),
    append(NegativeLiterals,[NegativeLiteral],NewNegativeLiterals),
    subgoal_clauses(NewNegativeLiterals,RestOfNegativeAtoms,
RestOfClauses).
%--------------------------------------------------------------------

%--------------------------------------------------------------------
%----Apply a relation to a list of first arguments
map_list([],_,[]).

map_list([FirstArgument|RestOfArguments],Mapping,
[MappedFirstArgument|RestOfMappedArguments]):-
    Query =.. [Mapping,FirstArgument,MappedFirstArgument],
    Query,
    map_list(RestOfArguments,Mapping,RestOfMappedArguments).	

