(*
* Copyright (C) 2003 Universitaet Koblenz-Landau
* 
* This file is part of KRHyper.
* 
* KRHyper is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
* 
* KRHyper is distributed in the hope that it will be useful, but WITHOUT ANY
* WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
* FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
* details.
* 
* You should have received a copy of the GNU General Public License along with
* KRHyper; if not, write to the Free Software Foundation, Inc., 59 Temple
* Place, Suite 330, Boston, MA 02111-1307 USA
* 
* Contributor(s): Christoph Wernhard <wernhard@uni-koblenz.de>.
*)

let answer_variable_name = ref "FOCUS"

type cfg = {
    mutable current_directory: string;
    mutable echo_flag: bool;
    mutable debug_flag: bool;
    mutable back_subsumption_flag: bool;
    mutable back_subsumption_within_news_flag: bool;
    mutable sort_heads_flag: bool;
    mutable pretend_range_restricted_input_flag: bool;
    mutable check_disjunctions_are_ground_flag: bool;
    mutable proof_terms_flag: bool;
    mutable level_cut_flag: bool;
    mutable negative_units_flag: bool;
    mutable merge_inf_input_flag: bool;
    mutable wp_semantics_flag: bool;
    mutable show_derived_literals_flag: bool;
    mutable max_weight_initial_param: int;
    mutable max_weight_increment_param: int;
    mutable body_sorting_method_param: int;
    mutable weighing_method_param: int;
    mutable sup_left_flag: bool;
    mutable unit_sup_right_flag: bool;    
    mutable ehyper: bool;
    mutable full_branch_redundancy_flag: bool;
    mutable hyperextension_flag: bool;
    mutable szs_output_flag: bool;
    mutable szs_problem_name: string;
    mutable szs_fof_flag: bool;
    mutable szs_conjecture_flag: bool;
    mutable block_equation_stratification_flag : bool;
    mutable input_type : int;
    mutable previous_input_type : int;
    mutable disjunctive_input : bool;
    mutable non_range_restricted_input : bool;
    mutable tptp_fof_compute_answer_predicate : bool;
    mutable tptp_fof_compute_partial_answers : bool;
    mutable tptp_fof_negate_conjectures : bool;
    mutable reassign_predicates : bool;
    mutable timeout_termination_method : int;
    mutable running_flag : bool;
    mutable prefer_dom_literals_flag:bool;
    mutable finite_model_transformation_flag:bool;
    mutable finite_model_transformation_domain_size:int;
    mutable partitioning_flag:bool;
  }

let get_current_directory cfg =
  cfg.current_directory

let set_current_directory cfg directory =
  cfg.current_directory <- directory

let get_echo_flag cfg = cfg.echo_flag
let set_echo_flag cfg bool = cfg.echo_flag <- bool
let get_debug_flag cfg = cfg.debug_flag
let set_debug_flag cfg bool = cfg.debug_flag <- bool
let get_back_subsumption_flag cfg = cfg.back_subsumption_flag
let set_back_subsumption_flag cfg bool =
  cfg.back_subsumption_flag <- bool
let get_back_subsumption_within_news_flag cfg = 
  cfg.back_subsumption_within_news_flag
let set_back_subsumption_within_news_flag cfg bool =
  cfg.back_subsumption_within_news_flag <- bool
let get_pretend_range_restricted_input_flag cfg = 
	cfg.pretend_range_restricted_input_flag
let set_pretend_range_restricted_input_flag cfg bool = 
	cfg.pretend_range_restricted_input_flag <- bool  
let get_sort_heads_flag cfg = cfg.sort_heads_flag
let set_sort_heads_flag cfg bool = cfg.sort_heads_flag <- bool
let get_check_disjunctions_are_ground_flag cfg =
  cfg.check_disjunctions_are_ground_flag
let set_check_disjunctions_are_ground_flag cfg bool =
  cfg.check_disjunctions_are_ground_flag <- bool
let get_proof_terms_flag cfg = cfg.proof_terms_flag
let set_proof_terms_flag cfg bool = cfg.proof_terms_flag <- bool
let get_level_cut_flag cfg = cfg.level_cut_flag
let set_level_cut_flag cfg bool = cfg.level_cut_flag <- bool
let get_negative_units_flag cfg = cfg.negative_units_flag
let set_negative_units_flag cfg bool = cfg.negative_units_flag <- bool
let get_merge_inf_input_flag cfg = cfg.merge_inf_input_flag
let set_merge_inf_input_flag cfg bool = cfg.merge_inf_input_flag <- bool
let get_wp_semantics_flag cfg = cfg.wp_semantics_flag
let set_wp_semantics_flag cfg bool = cfg.wp_semantics_flag <- bool
let get_show_derived_literals_flag cfg = cfg.show_derived_literals_flag
let set_show_derived_literals_flag cfg bool = cfg.show_derived_literals_flag <- bool
let get_max_weight_initial_param cfg = cfg.max_weight_initial_param
let set_max_weight_initial_param cfg n = cfg.max_weight_initial_param <- n
let get_max_weight_increment_param cfg =
  cfg.max_weight_increment_param
let set_max_weight_increment_param cfg n =
  cfg.max_weight_increment_param <- n
let get_body_sorting_method_param cfg = cfg.body_sorting_method_param
let set_body_sorting_method_param cfg n = cfg.body_sorting_method_param <- n
let get_weighing_method_param cfg = cfg.weighing_method_param
let set_weighing_method_param cfg n = cfg.weighing_method_param <- n
let get_sup_left_flag cfg = cfg.sup_left_flag
let set_sup_left_flag cfg bool = cfg.sup_left_flag <- bool
let get_unit_sup_right_flag cfg = cfg.unit_sup_right_flag
let set_unit_sup_right_flag cfg bool = cfg.unit_sup_right_flag <- bool
let get_ehyper cfg = cfg.ehyper
let set_ehyper cfg bool = cfg.ehyper <- bool
let get_full_branch_redundancy_flag cfg = cfg.full_branch_redundancy_flag
let set_full_branch_redundancy_flag cfg bool = cfg.full_branch_redundancy_flag <- bool
let get_hyperextension_flag cfg = cfg.hyperextension_flag
let set_hyperextension_flag cfg bool = cfg.hyperextension_flag <- bool
let set_szs_output_flag cfg bool = cfg.szs_output_flag <-  bool
let get_szs_output_flag cfg = cfg.szs_output_flag
let set_szs_problem_name cfg name = cfg.szs_problem_name <- name
let get_szs_problem_name cfg = cfg.szs_problem_name
let set_szs_fof_flag cfg bool = cfg.szs_fof_flag <- bool
let get_szs_fof_flag cfg = cfg.szs_fof_flag
let set_szs_conjecture_flag cfg bool = cfg.szs_conjecture_flag <- bool
let get_szs_conjecture_flag cfg = cfg.szs_conjecture_flag
let set_block_equation_stratification_flag cfg bool = cfg.block_equation_stratification_flag <- bool
let get_block_equation_stratification_flag cfg = cfg.block_equation_stratification_flag
let set_input_type cfg input_type = cfg.input_type <- input_type
let get_input_type cfg = cfg.input_type
let set_previous_input_type cfg input_type = cfg.previous_input_type <- input_type
let get_previous_input_type cfg = cfg.previous_input_type
let set_disjunctive_input cfg bool = cfg.disjunctive_input <- bool
let get_disjunctive_input cfg = cfg.disjunctive_input
let set_non_range_restricted_input cfg bool = cfg.non_range_restricted_input <- bool
let get_non_range_restricted_input cfg = cfg.non_range_restricted_input
let set_tptp_fof_compute_answer_predicate cfg bool = cfg.tptp_fof_compute_answer_predicate <- bool
let get_tptp_fof_compute_answer_predicate cfg = cfg.tptp_fof_compute_answer_predicate
let set_tptp_fof_compute_partial_answers cfg bool = cfg.tptp_fof_compute_partial_answers <- bool
let get_tptp_fof_compute_partial_answers cfg = cfg.tptp_fof_compute_partial_answers
let set_tptp_fof_negate_conjectures cfg bool = cfg.tptp_fof_negate_conjectures <- bool
let get_tptp_fof_negate_conjectures cfg = cfg.tptp_fof_negate_conjectures
let set_tptp_fof_answer_variable string = answer_variable_name := string
let get_reassign_predicates cfg = cfg.reassign_predicates
let set_reassign_predicates cfg bool = cfg.reassign_predicates <- bool
let get_timeout_termination_method cfg = cfg.timeout_termination_method
let set_timeout_termination_method cfg int = cfg.timeout_termination_method <- int

let get_running_flag cfg = cfg.running_flag
let set_running_flag cfg bool = cfg.running_flag <- bool
let get_prefer_dom_literals_flag cfg = cfg.prefer_dom_literals_flag
let set_prefer_dom_literals_flag cfg bool = cfg.prefer_dom_literals_flag <- bool
let get_finite_model_transformation_flag cfg = cfg.finite_model_transformation_flag
let set_finite_model_transformation_flag cfg bool = 
	cfg.finite_model_transformation_flag <- bool;
	if bool then
		cfg.ehyper <- true
let get_finite_model_transformation_domain_size cfg = cfg.finite_model_transformation_domain_size
let set_finite_model_transformation_domain_size cfg int = cfg.finite_model_transformation_domain_size <- int		
let get_partitioning_flag cfg = cfg.partitioning_flag
let set_partitioning_flag cfg bool = cfg.partitioning_flag <- bool


let get_tptp_fof_answer_variable = !answer_variable_name


let flags =
  [("back_subsumption",
    ("Use back subsumption - i.e. remove derived or input facts\n\
     if they are subsumed by a newly derived clause. [Works not with\n\
     disjunctions]",
       false,
       get_back_subsumption_flag, set_back_subsumption_flag));
	("tptp_fof_negate_conjectures",
    ("Negate first order formulas in the tptp-format, if they are identified\n\
     as conjectures. Default is true.",
       true,
       get_tptp_fof_negate_conjectures, set_tptp_fof_negate_conjectures));
	("tptp_fof_compute_answer_predicate",
    ("Compute an answer predicate. Normally, a first order conjecture in the\n\
    format is negated, taking the form 'false :- <body>.'\n\
    If a particular variable 'FOCUS' in <body> is of interest,\n\
    then the clause can be transformed into '$answer(FOCUS) :- <body>.' by setting\n\
    this flag to true. Starting the derivation with the command '#(run('$answer'/1)).'\n\
    instead of the regular '#(run).' will then cause the tableau calculation until\n\
    the first derivation of an '$answer'-unit. The value of the answer variable can then be\n\
    obtained with the command '#(print_extent('$answer'/1)).'",
       false,
       get_tptp_fof_compute_answer_predicate, set_tptp_fof_compute_answer_predicate));
       
	("tptp_fof_compute_partial_answers",
    ("This causes the computation of special units which represent failed closing attempts\n\
    on a conjecture clause. These units have the predicate '$partial_answer/3'. The\n\
    first argument is a list of the successfully unified negative literals of the clause.\n\
    The second argument is a list of variable bindings, the unifier computed so far.\n\
    The last argument is a list of the negative literals which have not been unified.\n\
    The first of these is the one where unification failed, causing this closing attempt\n\
    to stop. '$partial_answer/3' is thus a second order predicate and should be treated with\n\
    care. When halting a computation due to a timeout, the partial answers may provide hints\n\
    at the problems in finding a proof.",
       false,
       get_tptp_fof_compute_partial_answers, set_tptp_fof_compute_partial_answers)); 
                             
   ("pretend_range_restricted_input",
    ("Pretend that the input clause set is range restricted,\n\
      thus disabling making it range restricted.",
     false,
     get_pretend_range_restricted_input_flag,
     set_pretend_range_restricted_input_flag));       
       
   ("back_subsumption_within_news",
    ("Use back subsumption within the set of newly derived clauses.",
     false,
     get_back_subsumption_within_news_flag,
     set_back_subsumption_within_news_flag));
   ("sort_heads",
    ("Sort disjunctive heads heuristically.",
     true,
     get_sort_heads_flag, set_sort_heads_flag));
   ("check_disjunctions_are_ground",
    ("Verify at runtime for each derived disjunction that it is ground.",
     false,
     get_check_disjunctions_are_ground_flag,
     set_check_disjunctions_are_ground_flag));
   ("negative_units",
    ("Maintain negative units to detect refutations sooner and enable\n\
      complement splitting.",
      true,
     get_negative_units_flag, set_negative_units_flag));
   ("merge_inf_input",
    ("Merge facts inferred before any disjunctive branching into the input\n\
      facts. These derived facts then serve as lemmas in subsequent\n\
      tasks. This flag can be switched on and off around a call to 'run'\n\
      to generate certain lemmas.",
      false,
     get_merge_inf_input_flag, set_merge_inf_input_flag));
   ("wp_semantics",
    ("[Experimental] Enforce the `weak perfect' KRHyper semantics of\n\
      disjunctions. Overrides effects of other flags conflicting with\n\
      that semantics.",
     false,
     get_wp_semantics_flag, set_wp_semantics_flag));
   ("proof_terms",
    ("Generate proof terms for derived facts.",
     true,
     get_proof_terms_flag, set_proof_terms_flag));
   ("level_cut",
    ("Apply the level cut reduction. This is only effective if proof_terms\n\
      is also set.\n\
      Note: No level cut is applied at a node whose proof involves\n\
      stratified negation as failure or findall.",
     true,
     get_level_cut_flag, set_level_cut_flag));
   ("echo", 
    ("Print each input statement before it is processed.",
     false,
     get_echo_flag, set_echo_flag));
   ("debug",
    ("Debug mode. Print extra information about clause compilation.",
     false,
     get_debug_flag, set_debug_flag));
   ("sup_left_flag",
    ("Use superposition left. If set to false, ordered paramodulation left will be used.",
     true,
     get_sup_left_flag, set_sup_left_flag)); 
   ("unit_sup_right_flag",
    ("Use unit superposition right. If set to false, ordered paramodulation right will be used.",
     true,
     get_unit_sup_right_flag, set_unit_sup_right_flag));          
   ("prefer_dom_literals_flag",
    ("Select domain literals first. Used for finite model computation.\n\
    Experimental! Must be set to true before clauses are entered.\n\
    Should not be changed later, restart E-KRHyper instead.",
     false,
     get_prefer_dom_literals_flag, set_prefer_dom_literals_flag));   
   ("finite_model_transformation_flag",
	("Apply finite model transformation. If set to true, clauses will be range-restricted,\n\
	the domain will be finite and the Herbrand universe will be enumerated.\n\
	The flag affects the entire run of E-KRHyper. If the transformation is to be applied,\n\
	then the flag must be set to true before the first clauses are entered, and the flag\n\
	should not be set to false again later on.",
	false,
	get_finite_model_transformation_flag, set_finite_model_transformation_flag));
   ("full_branch_redundancy_flag",
    ("Use full branch redundancy handling. If set to true, any facts and clauses within the\n
    current branch may be declared redundant, disregarding the E-Hyper calculus rule about\n
    not crossing decision nodes.\n
    If set to false, the calculus will be performed strictly as defined.\n
    Note that there is no efficiency advantage in setting the flag to false, the option is merely\n
    included to allow for a strict calculus implementation. The default value is true.",
     true,
     get_full_branch_redundancy_flag, set_full_branch_redundancy_flag));
   ("hyperextension_flag",
    ("Use hyperextension. If set to true, the hyperextension inference of the (non-equational)\n
    Hyper-tableaux calculus will be used for resolving away non-equational literals, even when\n
    the program is running in E-Hyper mode (i.e. there are equational literals present). The\n
    hyperextension inference is equivalent to a series of sup-left and ref-inferences in the\n
    E-Hyper calculus, so this has no effect on soundness and completeness, and it is faster\n
    than treating all literals as equations. However, to allow for a strict E-Hyper calculus\n
    implementation, usage of the hyperextension can be switched off by setting this flag to false.\n
    In this case, all non-equational literals will be converted to equations, and only the inferences\n
    detailed in the E-Ehyper calculus description will be used.\n
    Note that in this case you should ensure that the input contains no predicates and functions sharing\
    arity and name, because this distinction will be lost during the conversion to equations.",
     true,
     get_hyperextension_flag, set_hyperextension_flag));   
   ("szs_output_flag",
    ("Display SZS-conforming output. If set to true, each result printout will include\n\
      one line displaying the result in SZS-syntax. If the input clause set is found to\n\
      be satisfiable, then the line will read:\n\
      'SZS status Satisfiable'\n\
      Otherwise the line will read:\n\
      'SZS status Unsatisfiable'\n\n\
      Notes: When enumerating models, a 'Closed' result indicates that no further models\n\
      exist. The SZS status may still be satisfiable, provided that at least one model\n\
      has been found.\n\
      The SZS status line is not commented out like other E-KRHyper output, to conform with\n\
      CASC-regulations. The output of E-KRHyper will therefore no longer be Prolog-readable\n\
      if this flag is set to true.",
     false,
     get_szs_output_flag, set_szs_output_flag));
     
   ("szs_fof_flag",
    ("Will be set to true automatically when the input contains first-order formulaic \n\
      TPTP input. This will change the SZS output to that for a fof-problem.\n\ 
      Manual setting is not recommended.",
     false,
     get_szs_fof_flag, set_szs_fof_flag));     
     
     
   ("szs_conjecture_flag",
    ("Will be set to true automatically when the input contains first-order formulaic \n\
      TPTP input with a conjecture. This will change the SZS output to that for a fof-problem.\n\ 
      Manual setting is not recommended.",
     false,
     get_szs_conjecture_flag, set_szs_conjecture_flag));  
     
   ("partitioning_flag",
    ("If set to true, then those rules/clauses which are definitely not required for\n\
    the computation of an answer clause (TPTP conjecture) will be treated as redundant\n\
    and not participate in the tableau derivation.",
     false,
     get_partitioning_flag, set_partitioning_flag));  
             
     
   ("block_equation_stratification_flag",
    ("Prevents the derivation of equations in strata after the first. If set to true,\n\
	  clauses with equations in the head and body literals requiring stratification\n\
	  (like default negation) are not allowed, and their presence will halt the\n\
	  computation.\n\
	  Default setting is true.",
	true,
    get_block_equation_stratification_flag, set_block_equation_stratification_flag));      
    
   ("compute_partial_answers",
    ("Supposed to be used in conjunction with the 'compute_answer_for_tptp_fof_variable'\n\
    command, and will have no effect otherwise.\n\
    When set to true, in addition to the regular answer-units E-KRHyper will compute\n\
    units for partial answers. An answer unit is derived when all body literals of\n\
    the clausified conjecture unify with the extent, and a partial answer unit is\n\
    derived when a subset of the body literals unify with the extent. In the case of\n\
    of a timeout or any other interruption of the tableau derivation process, the\n\
    partial answer units may provide information regarding how close the system was\n\
    to a final answer.\n\
    A partial answer unit uses the partial answer predicate '$partial_answer/1'. This\n\
    is a second order predicate, and its single argument is a list of the unified\n\
    body literals.",
    false,
    get_tptp_fof_compute_partial_answers, set_tptp_fof_compute_partial_answers)); 
    
   ("show_derived_literals",
    ("Show facts as they are derived. May be useful for debugging.\n\
      Output are Prolog readable statements, one per line:\n\n  \
      derived_fact(Fact).  -- Fact has been derived in an extension step.\n  \
      attached_disjunction(DisjunctionId). -- The tableau has been\n      \
          extended  with a disjunction.\n  \
      selected_disjunct(Fact, DisjunctionId). -- Fact has been\n      \
       selected from an attached disjunction.\n\n\
     Notes: DisjunctionIds are currently not implemented.\n\
       Negative facts (see flag negative_units) are currently not considered.",
     false,
     get_show_derived_literals_flag, set_show_derived_literals_flag))]


let parameters =
  [("max_weight_initial", 
    ("The initial weight limit for inferred literals. If -1, weight\n\
      handling is not used. For infinite Herbrand domains, weight handling\n\
      can be required to ensure refutational completeness and\n\
      `finite-branch-model' completeness.",
     20,
     get_max_weight_initial_param, set_max_weight_initial_param));
(*	("tptp_fof_answer_variable", 
    ("The identifier of an answer variable. A variable with this name\n\
    in a tptp first order conjecture can be turned into the focus of the\n\
    derivation (see tptp_fof_compute_answer_predicate). Default is 'FOCUS'.",
     "FOCUS",
     get_tptp_fof_answer_variable, set_tptp_fof_answer_variable));     *)
   ("max_weight_increment", 
    ("The minimum amount by which the weight limit for inferred literals\n\
      is increased in a weight reset step. If max_weight_increment >= 0,\n\
      then the actual number used is the maximum of this number and the\n\
      smallest number by which an additional literal can possibly be\n\
      inferred. If max_weight_increment = -1, max_weight_initial is\n\
      used as weight limit and not incremented, yielding an obviously\n\
      incomplete calculus.",
      1,
      get_max_weight_increment_param,
      set_max_weight_increment_param));
   ("finite_model_transformation_domain_size",
    ("Sets the current domain size parameter for the finite model transformation.\n\
    Setting the parameter to a positive integer will set the domain size to that number.\n\
    Setting the parameter to 0 will set the domain size to the number of constants in\n\
    the current input clause set.\n\
    This parameter has no effect unless the 'finite_model_transformation_flag' has been set.",
    0,
    get_finite_model_transformation_domain_size,
    set_finite_model_transformation_domain_size));
   ("timeout_termination_method",
    ("Determines the behaviour of E-KRHyper whenever a timeout\n\
    induced by a timer (virtual and wallclock) occurs.\n\ 
    The following values can be used:\n\
    0 - E-KRHyper terminates.\n\
    1 - The execution of the current command/statement ends, and E-KRHyper\n\
    returns to the interactive prompt. If the timeout interrupted a tableau derivation,\n\
    then any partial results are discarded, and the most recent input state is restored.\n\
    2 - Analogous to 1, except that partial results are not discarded. Note that while these\n\
    partial results can be accessed like a model ('#(print_extent)' etc.), they most likely do\n\
    not represent a model.",
	0,
    get_timeout_termination_method, 
    set_timeout_termination_method));          
   ("body_sorting_method", 
    ("Determines the heuristics for sorting body literals.\n\
      The following values can be used:\n\
      0 - A heuristics that ensures permissibility for builtins is used.\n\
      1 - The literal order of the input is used, except that delta\n    \
	  predicates are preferred. This may be useful on inputs\n    \
          resulting from program transformations.\n\
      2 - The literal order of the input is used.",
      0, 
      get_body_sorting_method_param,
      set_body_sorting_method_param));
   ("input_type", 
    ("Determines the format of the input.\n\
      E-KRHyper can read two input formats, tme and tptp.\n\
      The default setting is to read all input as tme.\n\
      Exception: files with the extension '.p' or '.ax' will be read as tptp.\n\
      Setting the input parameter manually allows to override the\n\
      default behaviour.\n\
      The following values can be used:\n\
      0 - all input is read as tme, except for tptp-files.\n\
      1 - all input is read as tme.\n\
      2 - all input is read as tptp.\n\
      3 - backward compability mode: all input is read as tme,\n\
      and it must be in canonical Prolog-syntax as ensured by\n\
      the preprocessor scripts.
	  Reading tptp as tme or vice versa may cause parsing errors.\n\
	  Command statements are always parsed correctly, regardless of\n\
	  input type setting.\n\
      Note that a change of input type will only take effect after the current\n\
      file has been read. Having multiple formats within one file is not recommended.",
      0, 
      get_input_type,
      set_input_type));
   ("weighing_method", 
    ("Determines how the weight of a term is computed.\n\
      The following values can be used:\n\
      0 - The size of the term, i.e. the number of occurences of\n    \
	  predicate, function and constant symbols as well as variables\n    \
          in it is used.\n\
      1 - The depth of the term is used.",
     0,
     get_weighing_method_param,
     set_weighing_method_param))]
    
let set_to_defaults cfg =
  List.iter
    (fun (_, (_,default, _, writer)) ->
      (writer cfg default))
    flags;
  List.iter
    (fun (_, (_,default, _, writer)) ->
      (writer cfg default))
    parameters

let make_cfg () =
  let cfg =
    { current_directory=Sys.getcwd ();
      echo_flag=false;
      debug_flag=false;
      back_subsumption_flag=false;
      back_subsumption_within_news_flag=false;
      sort_heads_flag=false;
      pretend_range_restricted_input_flag=false;
      check_disjunctions_are_ground_flag=false;
      negative_units_flag=false;
      merge_inf_input_flag=false;
      proof_terms_flag=false;
      level_cut_flag=false;
      wp_semantics_flag=false;
      show_derived_literals_flag=false;
      sup_left_flag=false;
      unit_sup_right_flag=false;      
      ehyper=false;
      max_weight_initial_param=0;
      max_weight_increment_param=0;
      body_sorting_method_param=0;
      weighing_method_param=0;
      full_branch_redundancy_flag=false;
      hyperextension_flag=false;
      szs_output_flag=false;
      szs_problem_name="";
      szs_fof_flag=false;
      szs_conjecture_flag=false;
      block_equation_stratification_flag=false;
      input_type=0;
      previous_input_type=0;
      disjunctive_input=false;
      non_range_restricted_input=false;
      tptp_fof_negate_conjectures=false;
      tptp_fof_compute_answer_predicate=false;
      tptp_fof_compute_partial_answers=false;
      reassign_predicates=false;
      timeout_termination_method=0;
      running_flag=false;
      prefer_dom_literals_flag=false;
      finite_model_transformation_flag=false;
      finite_model_transformation_domain_size=0;
      partitioning_flag=false;
    } in
  set_to_defaults cfg;
  cfg
   
let print_smallheader x =
  print_string "----------------------------------------";
  print_string "--------------------------------------\n";
  print_string x;
  print_string "\n----------------------------------------";
  print_string "--------------------------------------\n"

let print_settings cfg =
  List.iter
    (fun (flag, (_, _, reader, _)) ->
      print_string "set_flag(";
      print_string flag;
      print_string ", ";
      print_string (if reader cfg then "true" else "false");
      print_endline ").")
    flags;
  List.iter
    (fun (flag, (_, _, reader, _)) ->
      print_string "set_parameter(";
      print_string flag;
      print_string ", ";
      print_int (reader cfg);
      print_endline ").")
    parameters

let print_flags_doc cfg =
  List.iter
    (fun (flag, (doc, default, _, _)) ->
      print_smallheader (Printf.sprintf "Flag: %s, default: %b" flag default);
      Printf.printf "%s\n\n" doc)
    flags;
  flush stdout

let print_parameters_doc cfg =
  List.iter
    (fun (param, (doc, default, _, _)) ->
      print_smallheader (Printf.sprintf "Parameter: %s, default: %d" 
			   param default);
      Printf.printf "%s\n\n" doc)
    parameters;
  flush stdout

let set_flag cfg flag newval =
  try
    let (_, _, _, writer) = List.assoc flag flags in
    writer cfg newval
  with Not_found ->
    raise (Err.Error "Unknown flag specified.")

let set_parameter cfg param newval =
  try
    let (_, _, _, writer) = List.assoc param parameters in
    writer cfg newval
  with Not_found ->
    raise (Err.Error "Unknown parameter specified.")


(****************************************************************************)

type verbosity = int

let current_verbosity = ref 3

let get_verbosity () = !current_verbosity

let set_verbosity n = current_verbosity := n

let current_msg_output = ref stderr

let get_msg_output () = !current_msg_output

let set_msg_output s =
  match s with
    "stdout" -> current_msg_output := stdout
  | "stderr" -> current_msg_output := stderr
  | _ -> (flush stdout; current_msg_output := open_out s)
