(*
* Copyright (C) 2003 Universitaet Koblenz-Landau
* 
* This file is part of KRHyper.
* 
* KRHyper is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
* 
* KRHyper is distributed in the hope that it will be useful, but WITHOUT ANY
* WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
* FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
* details.
* 
* You should have received a copy of the GNU General Public License along with
* KRHyper; if not, write to the Free Software Foundation, Inc., 59 Temple
* Place, Suite 330, Boston, MA 02111-1307 USA
* 
* Contributor(s): Christoph Wernhard <wernhard@uni-koblenz.de>.
*)

(** Configuration. *)

type cfg

val answer_variable_name : string ref

val make_cfg : unit -> cfg

val get_current_directory : cfg -> string

val set_current_directory : cfg -> string -> unit


(** {6 Flags and Paramaeters} *)

val set_flag : cfg -> string -> bool -> unit

val set_parameter : cfg -> string -> int -> unit

val print_settings : cfg -> unit

val print_flags_doc : cfg -> unit

val print_parameters_doc : cfg -> unit

val get_echo_flag : cfg -> bool

val get_debug_flag : cfg -> bool

val get_back_subsumption_flag : cfg -> bool

val get_back_subsumption_within_news_flag : cfg -> bool

val get_sort_heads_flag : cfg -> bool

val get_check_disjunctions_are_ground_flag : cfg -> bool

val get_proof_terms_flag : cfg -> bool

val get_level_cut_flag : cfg -> bool

val get_negative_units_flag : cfg -> bool

val get_merge_inf_input_flag : cfg -> bool

val get_wp_semantics_flag : cfg -> bool

val get_show_derived_literals_flag : cfg -> bool

val get_max_weight_initial_param : cfg -> int

val get_max_weight_increment_param : cfg -> int

val get_body_sorting_method_param : cfg -> int

val get_weighing_method_param : cfg -> int

val get_sup_left_flag : cfg -> bool

val get_unit_sup_right_flag : cfg -> bool

val set_ehyper : cfg -> bool -> unit

val get_ehyper : cfg -> bool

val set_full_branch_redundancy_flag : cfg -> bool -> unit

val get_full_branch_redundancy_flag : cfg -> bool

val set_hyperextension_flag : cfg -> bool -> unit

val get_hyperextension_flag : cfg -> bool

val set_szs_output_flag : cfg -> bool -> unit

val get_szs_output_flag : cfg -> bool

val set_szs_problem_name : cfg -> string -> unit

val get_szs_problem_name : cfg -> string

val set_szs_fof_flag : cfg -> bool -> unit

val get_szs_fof_flag : cfg -> bool

val set_szs_conjecture_flag : cfg -> bool -> unit

val get_szs_conjecture_flag : cfg -> bool

val set_block_equation_stratification_flag : cfg -> bool -> unit

val get_block_equation_stratification_flag : cfg -> bool

val set_input_type : cfg -> int -> unit

val get_input_type : cfg -> int

val set_previous_input_type : cfg -> int -> unit

val get_previous_input_type : cfg -> int

val set_disjunctive_input : cfg -> bool -> unit

val get_disjunctive_input : cfg -> bool

val set_non_range_restricted_input : cfg -> bool -> unit

val get_non_range_restricted_input : cfg -> bool

val set_tptp_fof_compute_answer_predicate : cfg -> bool -> unit

val get_tptp_fof_compute_answer_predicate : cfg -> bool

val set_tptp_fof_compute_partial_answers : cfg -> bool -> unit

val get_tptp_fof_compute_partial_answers : cfg -> bool

val set_tptp_fof_negate_conjectures : cfg -> bool -> unit

val get_tptp_fof_negate_conjectures : cfg -> bool

val set_tptp_fof_answer_variable : string -> unit

val get_reassign_predicates : cfg -> bool

val set_reassign_predicates : cfg -> bool -> unit

val get_timeout_termination_method : cfg -> int

val set_timeout_termination_method : cfg -> int -> unit

val get_running_flag : cfg -> bool

val set_running_flag : cfg -> bool -> unit

val get_prefer_dom_literals_flag : cfg -> bool

val set_prefer_dom_literals_flag : cfg -> bool -> unit

val get_finite_model_transformation_flag : cfg -> bool

val set_finite_model_transformation_flag : cfg -> bool -> unit

val get_finite_model_transformation_domain_size : cfg -> int

val set_finite_model_transformation_domain_size : cfg -> int -> unit

val get_pretend_range_restricted_input_flag : cfg -> bool

val set_pretend_range_restricted_input_flag : cfg -> bool -> unit

val get_partitioning_flag : cfg -> bool

val set_partitioning_flag : cfg -> bool -> unit

(** {6 Verbosity} *) 

(**
   What messages are printed is determined by a global variable
   indicating verbosity by an integer. It is accessed through
   [verbosity] and [set_verbosity].

   Verbosity values are:

    - -1: suppress all messages
    - 0 to 3: print messages with verbosity increasing from 0 to 3
*)
type verbosity = int

(** Return the current verbosity value. *)
val get_verbosity : unit -> verbosity

(** Set the current verbosity to the given value. *)
val set_verbosity : verbosity -> unit

(** Return the output channel where messages are printed. *)
val get_msg_output : unit -> out_channel

(** Set the output channel where messages are printed. If the string
 argument is "stdout", the channel is set to stdout, else it is set to
 stderr.
*)
val set_msg_output : string -> unit
