(*
* Copyright (C) 2007 Universitaet Koblenz-Landau
* 
* This file is part of E-KRHyper.
* 
* E-KRHyper is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
* 
* KRHyper is distributed in the hope that it will be useful, but WITHOUT ANY
* WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
* FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
* details.
* 
* You should have received a copy of the GNU General Public License along with
* KRHyper; if not, write to the Free Software Foundation, Inc., 59 Temple
* Place, Suite 330, Boston, MA 02111-1307 USA
* 
* Contributor(s): Christoph Wernhard <wernhard@uni-koblenz.de>,
*                 Bjoern Pelzer <bpelzer@uni-koblenz.de>.
*
*)

(* provides functions and representations for clausification
of first-order formula input *)

val focus_variable : int ref

(** a temporary representation of first-order formulas used during parsing
and clausification *)
type fof_formula =
	| Assoc_binary_formula of (operator * (fof_formula list))
	| Nonassoc_binary_formula of (operator * fof_formula * fof_formula)
	| Quantified_formula of (operator * (int list) * fof_formula)
	| Atomic_formula of clausify_term
	| Negated_fof_formula of fof_formula
	| Non_formula of fof_formula
	| Conjecture of fof_formula
	
(** a temporary term representation	used during parsing
and clausification *)
and proto_term =
	| Const of string
	| Int of int
	| Float of float
	| Var of (int * string)
	| Compound of (string * (proto_term list))
	
(** a container datatype that consists of a proto_term and its Term.term *)	
and clausify_term
	
and operator =
	| Bijection
	| Left_implication
	| Right_implication
	| Xor
	| And
	| Or
	| Nand
	| Nor
	| Forall
	| Exists
	| True
	| False
	| Negation		

val make_clausify_term : proto_term -> clausify_term

val make_negated_clausify_term : proto_term -> clausify_term
	
val is_negated : clausify_term -> bool
	
val complement_clausify_term : clausify_term -> unit

val complement_fof_formula : fof_formula -> fof_formula

val proto_term_to_string : proto_term -> string

(** Turns a fof formula into a list of clauses. Per default a conjecture is negated
before clausification. This can be turned off by setting the optional parameter
[negate_conjecture] to false. *)
val clausify : ?negate_conjecture:bool -> ?focus:bool 
-> ?partial_answers:bool -> Term.symbol_table -> fof_formula -> (Term.term list)

val get_term : Term.symbol_table -> proto_term -> Term.term