type token =
  | LOWER_WORD of (string)
  | UPPER_WORD of (string)
  | SINGLE_QUOTED of (string)
  | VARIABLE of (string)
  | FLOAT of (string)
  | INTEGER of (string)
  | OR
  | LEFT_PARENTHESIS
  | RIGHT_PARENTHESIS
  | LEFT_BRACE
  | RIGHT_BRACE
  | LEFT_BRACKET
  | RIGHT_BRACKET
  | IMPLICATION
  | CLAUSE_END
  | NEGATION
  | SEMICOLON
  | COMMA
  | EQUALITY
  | EOI
  | META
  | FINDALL
  | NOT_SUBSUMED
  | ARITHMETIC of (string)
  | IS of (string)
  | MULT of (string)
  | PLUS of (string)
  | MINUS of (string)
  | DIV of (string)
  | ABS of (string)
  | FLOAT_OP of (string)
  | SQRT of (string)
  | TRUNCATE of (string)
  | SIN of (string)
  | COS of (string)

open Parsing;;
# 46 "expParser.mly"
  

   (* these are only valid for the current clause
     and have to be invalidated with init_clause for every new clause *)

  (* the variable id counter for the currently read term/clause *)
	    
	let var_id = ref 0
	
	let gen_var_id () =
	  var_id := !var_id + 1;
	  (!var_id-1)
	
	let reset_var_id () =
	  var_id := 0      
    
      
  (* mapping of the variable names (e.g. "X") of the currently read term/clause
     to variables. *)
  let (var_map: (string * int) list ref) =
    ref []



  (* reset everything in order to parse a new term/clause *)
  let init_clause () =
    reset_var_id ();
    var_map := []


	
  (* gets the variables associated with a string from the variable mapping
     creates a new mapping for a new variable *)
  let get_var (var_name: string) =
    try 
      (* way faster than List.assoc *)
      snd (
	List.find
	  (fun (var_name', _) ->
             var_name = var_name'
	  )
	  !var_map
      )
    with
      | Not_found ->
	  let new_var_id = 
	    gen_var_id ()
	  in
	    var_map := (var_name, new_var_id) :: !var_map;
	    new_var_id

  (* need to detect if input is fof and contains a conjecture *)
  (* is the input in fof format? *)
  let fof =
    ref false

  (* does the fof input contain a conjecture? *)
  let theorem =
    ref false


# 102 "expParser.ml"
let yytransl_const = [|
  263 (* OR *);
  264 (* LEFT_PARENTHESIS *);
  265 (* RIGHT_PARENTHESIS *);
  266 (* LEFT_BRACE *);
  267 (* RIGHT_BRACE *);
  268 (* LEFT_BRACKET *);
  269 (* RIGHT_BRACKET *);
  270 (* IMPLICATION *);
  271 (* CLAUSE_END *);
  272 (* NEGATION *);
  273 (* SEMICOLON *);
  274 (* COMMA *);
  275 (* EQUALITY *);
  276 (* EOI *);
  277 (* META *);
  278 (* FINDALL *);
  279 (* NOT_SUBSUMED *);
    0|]

let yytransl_block = [|
  257 (* LOWER_WORD *);
  258 (* UPPER_WORD *);
  259 (* SINGLE_QUOTED *);
  260 (* VARIABLE *);
  261 (* FLOAT *);
  262 (* INTEGER *);
  280 (* ARITHMETIC *);
  281 (* IS *);
  282 (* MULT *);
  283 (* PLUS *);
  284 (* MINUS *);
  285 (* DIV *);
  286 (* ABS *);
  287 (* FLOAT_OP *);
  288 (* SQRT *);
  289 (* TRUNCATE *);
  290 (* SIN *);
  291 (* COS *);
    0|]

let yylhs = "\255\255\
\001\000\001\000\002\000\003\000\004\000\006\000\007\000\007\000\
\008\000\008\000\008\000\005\000\013\000\009\000\009\000\009\000\
\009\000\009\000\009\000\015\000\010\000\010\000\016\000\016\000\
\016\000\016\000\012\000\014\000\014\000\011\000\011\000\011\000\
\011\000\017\000\000\000"

let yylen = "\002\000\
\001\000\001\000\004\000\001\000\001\000\001\000\003\000\003\000\
\004\000\001\000\001\000\001\000\004\000\004\000\001\000\003\000\
\003\000\001\000\001\000\001\000\001\000\001\000\001\000\001\000\
\003\000\003\000\001\000\003\000\001\000\001\000\001\000\002\000\
\002\000\000\000\002\000"

let yydefred = "\000\000\
\000\000\000\000\021\000\002\000\022\000\035\000\001\000\000\000\
\004\000\000\000\012\000\000\000\000\000\027\000\030\000\031\000\
\000\000\000\000\000\000\005\000\006\000\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\018\000\019\000\000\000\
\000\000\000\000\023\000\032\000\033\000\003\000\000\000\000\000\
\000\000\000\000\000\000\000\000\013\000\000\000\000\000\016\000\
\007\000\000\000\010\000\011\000\008\000\000\000\000\000\017\000\
\028\000\026\000\025\000\000\000\009\000\014\000"

let yydgoto = "\002\000\
\006\000\007\000\008\000\019\000\009\000\020\000\021\000\054\000\
\028\000\029\000\030\000\031\000\011\000\055\000\027\000\034\000\
\035\000"

let yysindex = "\009\000\
\002\255\000\000\000\000\000\000\000\000\000\000\000\000\247\254\
\000\000\006\255\000\000\049\255\049\255\000\000\000\000\000\000\
\049\255\003\255\000\255\000\000\000\000\249\254\250\254\010\255\
\000\000\000\000\005\255\017\255\012\255\000\000\000\000\014\255\
\251\254\033\255\000\000\000\000\000\000\000\000\020\255\020\255\
\049\255\049\255\049\255\049\255\000\000\049\255\049\255\000\000\
\000\000\043\255\000\000\000\000\000\000\048\255\053\255\000\000\
\000\000\000\000\000\000\020\255\000\000\000\000"

let yyrindex = "\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\052\255\000\000\000\000\000\000\000\000\000\000\000\000\038\255\
\063\255\074\255\066\255\057\255\051\255\000\000\000\000\000\000\
\055\255\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\052\255\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000"

let yygindex = "\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\004\000\
\251\255\255\255\245\255\248\255\000\000\249\255\000\000\023\000\
\000\000"

let yytablesize = 99
let yytable = "\010\000\
\025\000\046\000\003\000\026\000\012\000\032\000\023\000\036\000\
\037\000\001\000\024\000\033\000\047\000\013\000\038\000\022\000\
\039\000\041\000\040\000\044\000\003\000\004\000\045\000\014\000\
\015\000\016\000\005\000\051\000\051\000\025\000\052\000\052\000\
\026\000\042\000\043\000\057\000\056\000\050\000\050\000\024\000\
\058\000\033\000\049\000\053\000\005\000\048\000\020\000\018\000\
\051\000\003\000\060\000\052\000\014\000\015\000\016\000\020\000\
\061\000\020\000\050\000\020\000\017\000\062\000\020\000\020\000\
\034\000\029\000\020\000\024\000\020\000\059\000\000\000\010\000\
\015\000\005\000\015\000\020\000\018\000\000\000\015\000\020\000\
\018\000\000\000\011\000\015\000\000\000\000\000\010\000\018\000\
\000\000\000\000\015\000\019\000\000\000\000\000\000\000\000\000\
\000\000\011\000\019\000"

let yycheck = "\001\000\
\012\000\007\001\001\001\012\000\014\001\013\000\012\000\005\001\
\006\001\001\000\012\000\017\000\018\001\008\001\015\001\012\000\
\024\001\008\001\025\001\008\001\001\001\020\001\009\001\004\001\
\005\001\006\001\025\001\039\000\040\000\041\000\039\000\040\000\
\041\000\029\001\018\001\043\000\042\000\039\000\040\000\041\000\
\046\000\047\000\039\000\040\000\025\001\013\001\009\001\028\001\
\060\000\001\001\008\001\060\000\004\001\005\001\006\001\018\001\
\009\001\007\001\060\000\009\001\012\001\009\001\025\001\013\001\
\013\001\009\001\029\001\013\001\018\001\047\000\255\255\009\001\
\007\001\025\001\009\001\025\001\028\001\255\255\013\001\029\001\
\018\001\255\255\009\001\018\001\255\255\255\255\024\001\025\001\
\255\255\255\255\025\001\018\001\255\255\255\255\255\255\255\255\
\255\255\024\001\025\001"

let yynames_const = "\
  OR\000\
  LEFT_PARENTHESIS\000\
  RIGHT_PARENTHESIS\000\
  LEFT_BRACE\000\
  RIGHT_BRACE\000\
  LEFT_BRACKET\000\
  RIGHT_BRACKET\000\
  IMPLICATION\000\
  CLAUSE_END\000\
  NEGATION\000\
  SEMICOLON\000\
  COMMA\000\
  EQUALITY\000\
  EOI\000\
  META\000\
  FINDALL\000\
  NOT_SUBSUMED\000\
  "

let yynames_block = "\
  LOWER_WORD\000\
  UPPER_WORD\000\
  SINGLE_QUOTED\000\
  VARIABLE\000\
  FLOAT\000\
  INTEGER\000\
  ARITHMETIC\000\
  IS\000\
  MULT\000\
  PLUS\000\
  MINUS\000\
  DIV\000\
  ABS\000\
  FLOAT_OP\000\
  SQRT\000\
  TRUNCATE\000\
  SIN\000\
  COS\000\
  "

let yyact = [|
  (fun _ -> failwith "parser")
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'clause) in
    Obj.repr(
# 154 "expParser.mly"
      ( init_clause ();
        _1 )
# 278 "expParser.ml"
               : Clausify.proto_term))
; (fun __caml_parser_env ->
    Obj.repr(
# 158 "expParser.mly"
   (raise Err.END_OF_FILE)
# 284 "expParser.ml"
               : Clausify.proto_term))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 3 : 'clause_head) in
    let _3 = (Parsing.peek_val __caml_parser_env 1 : 'clause_body) in
    Obj.repr(
# 163 "expParser.mly"
      ( Clausify.Compound
			(":-", [_1; _3]))
# 293 "expParser.ml"
               : 'clause))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'literal) in
    Obj.repr(
# 168 "expParser.mly"
      ( _1 )
# 300 "expParser.ml"
               : 'clause_head))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'body_literal) in
    Obj.repr(
# 172 "expParser.mly"
      ( _1 )
# 307 "expParser.ml"
               : 'clause_body))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'math_literal) in
    Obj.repr(
# 176 "expParser.mly"
 ( _1 )
# 314 "expParser.ml"
               : 'body_literal))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 2 : 'math_term) in
    let _2 = (Parsing.peek_val __caml_parser_env 1 : string) in
    let _3 = (Parsing.peek_val __caml_parser_env 0 : 'math_term) in
    Obj.repr(
# 180 "expParser.mly"
 ( Clausify.Compound (_2, [_1; _3]) )
# 323 "expParser.ml"
               : 'math_literal))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 2 : 'term) in
    let _2 = (Parsing.peek_val __caml_parser_env 1 : string) in
    let _3 = (Parsing.peek_val __caml_parser_env 0 : 'math_term) in
    Obj.repr(
# 182 "expParser.mly"
 ( Clausify.Compound (_2, [_1; _3]) )
# 332 "expParser.ml"
               : 'math_literal))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 3 : 'string) in
    let _3 = (Parsing.peek_val __caml_parser_env 1 : 'math_term) in
    Obj.repr(
# 187 "expParser.mly"
 ( Clausify.Compound (_1, [_3]) )
# 340 "expParser.ml"
               : 'math_term))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'number) in
    Obj.repr(
# 189 "expParser.mly"
 ( _1 )
# 347 "expParser.ml"
               : 'math_term))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'variable) in
    Obj.repr(
# 191 "expParser.mly"
 ( _1 )
# 354 "expParser.ml"
               : 'math_term))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'predicate_term) in
    Obj.repr(
# 200 "expParser.mly"
      ( _1 )
# 361 "expParser.ml"
               : 'literal))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 3 : 'string) in
    let _3 = (Parsing.peek_val __caml_parser_env 1 : 'arguments) in
    Obj.repr(
# 204 "expParser.mly"
      ( Clausify.Compound (_1, _3) )
# 369 "expParser.ml"
               : 'predicate_term))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 3 : 'string) in
    let _3 = (Parsing.peek_val __caml_parser_env 1 : 'arguments) in
    Obj.repr(
# 208 "expParser.mly"
      ( Clausify.Compound (_1, _3) )
# 377 "expParser.ml"
               : 'term))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'constant) in
    Obj.repr(
# 210 "expParser.mly"
   ( _1 )
# 384 "expParser.ml"
               : 'term))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 1 : 'term_list) in
    Obj.repr(
# 212 "expParser.mly"
 ( _2 )
# 391 "expParser.ml"
               : 'term))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 2 : 'constant) in
    let _2 = (Parsing.peek_val __caml_parser_env 1 : string) in
    let _3 = (Parsing.peek_val __caml_parser_env 0 : 'term) in
    Obj.repr(
# 214 "expParser.mly"
      ( Clausify.Compound (_2, [_1; _3]) )
# 400 "expParser.ml"
               : 'term))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'number) in
    Obj.repr(
# 216 "expParser.mly"
 ( _1 )
# 407 "expParser.ml"
               : 'term))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'variable) in
    Obj.repr(
# 218 "expParser.mly"
 ( _1 )
# 414 "expParser.ml"
               : 'term))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'string) in
    Obj.repr(
# 224 "expParser.mly"
      ( Clausify.Const _1 )
# 421 "expParser.ml"
               : 'constant))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : string) in
    Obj.repr(
# 228 "expParser.mly"
    ( _1 )
# 428 "expParser.ml"
               : 'string))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : string) in
    Obj.repr(
# 230 "expParser.mly"
    ( _1 )
# 435 "expParser.ml"
               : 'string))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'null) in
    Obj.repr(
# 236 "expParser.mly"
     ( Clausify.Const "[]" )
# 442 "expParser.ml"
               : 'term_list))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'term) in
    Obj.repr(
# 238 "expParser.mly"
 ( let nil = Clausify.Const "[]"
		in
			Clausify.Compound
				(".", [_1; nil]) )
# 452 "expParser.ml"
               : 'term_list))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 2 : 'term) in
    let _3 = (Parsing.peek_val __caml_parser_env 0 : 'term_list) in
    Obj.repr(
# 243 "expParser.mly"
 ( Clausify.Compound
		(".", [_1; _3]) )
# 461 "expParser.ml"
               : 'term_list))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 2 : 'term) in
    let _3 = (Parsing.peek_val __caml_parser_env 0 : 'term) in
    Obj.repr(
# 246 "expParser.mly"
 ( Clausify.Compound
		(".", [_1; _3]) )
# 470 "expParser.ml"
               : 'term_list))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : string) in
    Obj.repr(
# 251 "expParser.mly"
      ( Clausify.Var (get_var _1) )
# 477 "expParser.ml"
               : 'variable))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 2 : 'term) in
    let _3 = (Parsing.peek_val __caml_parser_env 0 : 'arguments) in
    Obj.repr(
# 255 "expParser.mly"
      ( _1 :: _3 )
# 485 "expParser.ml"
               : 'arguments))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'term) in
    Obj.repr(
# 257 "expParser.mly"
      ( [_1] )
# 492 "expParser.ml"
               : 'arguments))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : string) in
    Obj.repr(
# 261 "expParser.mly"
 ( Clausify.Float (float_of_string _1) )
# 499 "expParser.ml"
               : 'number))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : string) in
    Obj.repr(
# 263 "expParser.mly"
 ( Clausify.Int (int_of_string _1) )
# 506 "expParser.ml"
               : 'number))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 1 : string) in
    let _2 = (Parsing.peek_val __caml_parser_env 0 : string) in
    Obj.repr(
# 265 "expParser.mly"
 ( Clausify.Float (float_of_string (_1^_2)) )
# 514 "expParser.ml"
               : 'number))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 1 : string) in
    let _2 = (Parsing.peek_val __caml_parser_env 0 : string) in
    Obj.repr(
# 267 "expParser.mly"
 ( Clausify.Int (int_of_string (_1^_2)) )
# 522 "expParser.ml"
               : 'number))
; (fun __caml_parser_env ->
    Obj.repr(
# 270 "expParser.mly"
      ( )
# 528 "expParser.ml"
               : 'null))
(* Entry tme_input *)
; (fun __caml_parser_env -> raise (Parsing.YYexit (Parsing.peek_val __caml_parser_env 0)))
|]
let yytables =
  { Parsing.actions=yyact;
    Parsing.transl_const=yytransl_const;
    Parsing.transl_block=yytransl_block;
    Parsing.lhs=yylhs;
    Parsing.len=yylen;
    Parsing.defred=yydefred;
    Parsing.dgoto=yydgoto;
    Parsing.sindex=yysindex;
    Parsing.rindex=yyrindex;
    Parsing.gindex=yygindex;
    Parsing.tablesize=yytablesize;
    Parsing.table=yytable;
    Parsing.check=yycheck;
    Parsing.error_function=parse_error;
    Parsing.names_const=yynames_const;
    Parsing.names_block=yynames_block }
let tme_input (lexfun : Lexing.lexbuf -> token) (lexbuf : Lexing.lexbuf) =
   (Parsing.yyparse yytables 1 lexfun lexbuf : Clausify.proto_term)
;;
