(*
* Copyright (C) 2003 Universitaet Koblenz-Landau
* 
* This file is part of KRHyper.
* 
* KRHyper is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
* 
* KRHyper is distributed in the hope that it will be useful, but WITHOUT ANY
* WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
* FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
* details.
* 
* You should have received a copy of the GNU General Public License along with
* KRHyper; if not, write to the Free Software Foundation, Inc., 59 Temple
* Place, Suite 330, Boston, MA 02111-1307 USA
* 
* Contributor(s): Christoph Wernhard <wernhard@uni-koblenz.de>.
*)


let push elem listref =
  listref := elem :: !listref

let pushnew elem listref =
  if List.mem elem !listref then
    ()
  else
    listref := elem :: !listref

let pushnewq elem listref =
  if List.memq elem !listref then
    ()
  else
    listref := elem :: !listref

let rec rm_duplicates list =
  match list with
    e1 :: e2 :: list1 ->
      if e1 = e2 then
	rm_duplicates (e2 :: list1)
      else
	e1 :: rm_duplicates (e2 :: list1)
  | _ -> list

let sort_without_duplicates list =
  rm_duplicates (List.fast_sort compare list)

  
let get_sublists list =		
(* gets the sublists of a particular length n *)
	let rec get_sublists_of_length_n prefix number array start =
		if (number < 1) then
			(failwith "get_sublists: bad number of elements");
		let results = ref []
		in
		if (number = 1) then
			(for index = start to ((Array.length array) - 1) do
				(
				results := (array.(index) :: prefix) :: !results)
			done;
			!results)
		else
			(for index = start to ((Array.length array) - number) do
				(
				let extended_prefix = (array.(index)) :: prefix
				in	
					let sublists =	
						get_sublists_of_length_n
							extended_prefix
							(number - 1)
							array
							(index + 1)
					in
						results := 
							List.rev_append
								sublists
								!results)
			done;
			!results)
	and list_as_array = Array.of_list list
	and results = ref []
	in
(* for-loop to get the sublists of all lengths *)
		for number = 1 to (Array.length list_as_array) do
			(let sublists_of_length_number =
				get_sublists_of_length_n
					[]
					number 
					list_as_array
					0
			in
				results := 
					List.rev_append 
						sublists_of_length_number 
						!results)
		done;
		!results
