(*
* Copyright (C) 2003 Universitaet Koblenz-Landau
* 
* This file is part of KRHyper.
* 
* KRHyper is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
* 
* KRHyper is distributed in the hope that it will be useful, but WITHOUT ANY
* WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
* FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
* details.
* 
* You should have received a copy of the GNU General Public License along with
* KRHyper; if not, write to the Free Software Foundation, Inc., 59 Temple
* Place, Suite 330, Boston, MA 02111-1307 USA
* 
* Contributor(s): Christoph Wernhard <wernhard@uni-koblenz.de>.
*)

type entry =
    Fact.entry

type index = 
    (entry list) ref

let make_index () =
  ref []

let clear index =
  index := []

let insert ?(perfect:bool = false) index term entry =
  index := entry :: !index

let iter_all f index =
  List.iter f !index

let iter_all_and_throw_away f index =
  List.iter f !index

let iter_instance_candidates f index term context =
  List.iter f !index

let iter_variant_candidates f index term context =
  List.iter f !index  
  
let perfect_iter_instance_candidates 
	?(substitution:Term.substitution = Term.empty_substitution) f index term context =	
  failwith "NoIndex.perfect_iter_instance_candidates: this function is only provided for discrimination trees." 
  
let iter_generalization_candidates f index term context =
  List.iter f !index

let perfect_iter_generalization_candidates 
	?(substitution:Term.substitution = Term.empty_substitution) f index term context =	
  failwith "NoIndex.perfect_iter_generalization_candidates: this function is only provided for discrimination trees." 
    
  
  
let iter_unification_candidates f index term context =
  List.iter f !index

let rec remove_variant_1 index ?(equation:bool = false) term flipped_term =
  match index with
    entry :: entries ->
    	let entry_term = Fact.get_term entry
    	in
      if (Term.variant term entry_term) 
      || (equation && (Term.variant flipped_term entry_term)) then
	entries
      else
	entry :: remove_variant_1 entries ~equation:equation term flipped_term
  | [] ->
      []

let remove_variant index ?(equation:bool = false) ?(key:Term.term = Term.Unbound) term =
	let flipped_term =
		(if equation then
			(Term.binary_flip term)
		else
			term) 
	in
		index := remove_variant_1 !index ~equation:equation term flipped_term

let size index =
  List.length !index
