type token =
  | LEFT_PARENTHESIS
  | RIGHT_PARENTHESIS
  | LEFT_BRACKET
  | RIGHT_BRACKET
  | DOT
  | NEGATION
  | COLON
  | COMMA
  | EQUALITY
  | DISEQUALITY
  | EOI
  | FOF
  | CNF
  | INCLUDE
  | SINGLE_QUOTED of (string)
  | DOLLAR_WORD of (string)
  | DOLLAR_DOLLAR_WORD of (string)
  | DISTINCT_OBJECT of (string)
  | LOWER_WORD of (string)
  | UPPER_WORD of (string)
  | UNSIGNED_INTEGER of (string)
  | SIGNED_INTEGER of (string)
  | REAL of (string)
  | DOLLAR_TRUE
  | DOLLAR_FALSE
  | DOLLAR
  | AND
  | OR
  | FORALL
  | EXISTS
  | BIJECTION
  | LEFT_IMPLICATION
  | RIGHT_IMPLICATION
  | XOR
  | META
  | SLASH

open Parsing;;
# 45 "tptpParser.mly"
  
  (* these are only valid for the current clause
     and have to be invalidated with init_clause for every new clause *)

  (* the variable id counter for the currently read term/clause *)
	    
	let var_id = ref 0
	
	let gen_var_id () =
	  var_id := !var_id + 1;
	  (!var_id-1)
	
	let reset_var_id () =
	  var_id := 0      
    
      
  (* mapping of the variable names (e.g. "X") of the currently read term/clause
     to variables. *)
  let (var_map: (string * int) list ref) =
    ref []



  (* reset everything in order to parse a new term/clause *)
  let init_clause () =
    reset_var_id ();
    var_map := []


	
  (* gets the variables associated with a string from the variable mapping
     creates a new mapping for a new variable *)
  let get_var (var_name: string) =
    try 
      (* way faster than List.assoc *)
      snd (
	List.find
	  (fun (var_name', _) ->
             var_name = var_name'
	  )
	  !var_map
      )
    with
      | Not_found ->
	  let new_var_id = 
	    gen_var_id ()
	  in
	    var_map := (var_name, new_var_id) :: !var_map;
	    new_var_id

  (* need to detect if input is fof and contains a conjecture *)
  (* is the input in fof format? *)
  let fof =
    ref false
    
  (* does the fof input contain a conjecture? *)
  let conjecture =
    ref false

# 101 "tptpParser.ml"
let yytransl_const = [|
  257 (* LEFT_PARENTHESIS *);
  258 (* RIGHT_PARENTHESIS *);
  259 (* LEFT_BRACKET *);
  260 (* RIGHT_BRACKET *);
  261 (* DOT *);
  262 (* NEGATION *);
  263 (* COLON *);
  264 (* COMMA *);
  265 (* EQUALITY *);
  266 (* DISEQUALITY *);
  267 (* EOI *);
  268 (* FOF *);
  269 (* CNF *);
  270 (* INCLUDE *);
  280 (* DOLLAR_TRUE *);
  281 (* DOLLAR_FALSE *);
  282 (* DOLLAR *);
  283 (* AND *);
  284 (* OR *);
  285 (* FORALL *);
  286 (* EXISTS *);
  287 (* BIJECTION *);
  288 (* LEFT_IMPLICATION *);
  289 (* RIGHT_IMPLICATION *);
  290 (* XOR *);
  291 (* META *);
  292 (* SLASH *);
    0|]

let yytransl_block = [|
  271 (* SINGLE_QUOTED *);
  272 (* DOLLAR_WORD *);
  273 (* DOLLAR_DOLLAR_WORD *);
  274 (* DISTINCT_OBJECT *);
  275 (* LOWER_WORD *);
  276 (* UPPER_WORD *);
  277 (* UNSIGNED_INTEGER *);
  278 (* SIGNED_INTEGER *);
  279 (* REAL *);
    0|]

let yylhs = "\255\255\
\001\000\001\000\001\000\001\000\004\000\002\000\002\000\006\000\
\010\000\010\000\012\000\012\000\014\000\016\000\016\000\016\000\
\016\000\016\000\016\000\015\000\015\000\017\000\019\000\019\000\
\018\000\020\000\020\000\013\000\013\000\013\000\013\000\021\000\
\024\000\024\000\025\000\025\000\022\000\027\000\007\000\009\000\
\011\000\011\000\028\000\032\000\032\000\033\000\033\000\033\000\
\023\000\023\000\023\000\034\000\038\000\038\000\040\000\040\000\
\035\000\035\000\035\000\035\000\036\000\039\000\041\000\041\000\
\041\000\041\000\043\000\043\000\043\000\043\000\037\000\037\000\
\005\000\005\000\005\000\048\000\048\000\048\000\047\000\047\000\
\047\000\047\000\044\000\046\000\049\000\049\000\042\000\042\000\
\050\000\050\000\026\000\030\000\031\000\031\000\054\000\003\000\
\057\000\057\000\058\000\058\000\053\000\053\000\053\000\059\000\
\059\000\059\000\059\000\060\000\060\000\055\000\055\000\061\000\
\061\000\008\000\008\000\051\000\051\000\052\000\045\000\045\000\
\045\000\056\000\056\000\029\000\000\000"

let yylen = "\002\000\
\002\000\002\000\002\000\001\000\004\000\001\000\001\000\009\000\
\001\000\001\000\001\000\001\000\003\000\001\000\001\000\001\000\
\001\000\002\000\002\000\001\000\001\000\003\000\001\000\003\000\
\003\000\001\000\003\000\001\000\001\000\003\000\001\000\006\000\
\001\000\001\000\001\000\003\000\002\000\001\000\011\000\001\000\
\001\000\003\000\001\000\003\000\001\000\003\000\001\000\002\000\
\001\000\001\000\001\000\001\000\001\000\003\000\001\000\003\000\
\001\000\001\000\003\000\003\000\001\000\001\000\001\000\003\000\
\001\000\004\000\003\000\001\000\001\000\001\000\001\000\004\000\
\003\000\001\000\004\000\001\000\001\000\004\000\001\000\001\000\
\003\000\003\000\001\000\001\000\001\000\001\000\001\000\004\000\
\001\000\004\000\001\000\001\000\002\000\001\000\001\000\005\000\
\004\000\001\000\001\000\003\000\001\000\003\000\001\000\001\000\
\004\000\001\000\001\000\001\000\003\000\002\000\003\000\001\000\
\003\000\001\000\001\000\001\000\001\000\001\000\001\000\001\000\
\001\000\001\000\001\000\000\000\002\000"

let yydefred = "\000\000\
\000\000\000\000\004\000\000\000\000\000\000\000\000\000\125\000\
\000\000\000\000\000\000\006\000\007\000\000\000\000\000\000\000\
\000\000\001\000\002\000\003\000\117\000\116\000\115\000\000\000\
\114\000\000\000\123\000\122\000\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\098\000\000\000\005\000\000\000\
\000\000\040\000\000\000\000\000\000\000\096\000\121\000\120\000\
\119\000\073\000\091\000\063\000\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\075\000\000\000\000\000\000\000\
\000\000\000\000\038\000\118\000\086\000\057\000\058\000\033\000\
\034\000\000\000\009\000\000\000\011\000\012\000\020\000\021\000\
\028\000\029\000\031\000\000\000\077\000\000\000\049\000\050\000\
\051\000\052\000\000\000\061\000\062\000\000\000\085\000\000\000\
\068\000\069\000\070\000\000\000\000\000\000\000\097\000\056\000\
\064\000\000\000\000\000\079\000\000\000\076\000\000\000\000\000\
\000\000\000\000\000\000\041\000\000\000\000\000\000\000\014\000\
\015\000\016\000\017\000\000\000\000\000\037\000\000\000\000\000\
\000\000\000\000\000\000\000\000\047\000\000\000\043\000\000\000\
\100\000\066\000\030\000\000\000\000\000\000\000\067\000\000\000\
\000\000\107\000\000\000\106\000\000\000\092\000\103\000\000\000\
\008\000\019\000\018\000\000\000\025\000\000\000\022\000\013\000\
\000\000\000\000\059\000\060\000\000\000\000\000\000\000\000\000\
\048\000\000\000\000\000\081\000\082\000\000\000\000\000\110\000\
\000\000\000\000\000\000\094\000\042\000\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\046\000\000\000\
\044\000\078\000\090\000\000\000\111\000\093\000\095\000\000\000\
\000\000\102\000\027\000\024\000\000\000\036\000\054\000\039\000\
\113\000\000\000\105\000\032\000\109\000"

let yydgoto = "\002\000\
\008\000\009\000\010\000\011\000\030\000\012\000\013\000\059\000\
\043\000\074\000\115\000\075\000\076\000\077\000\078\000\124\000\
\079\000\080\000\159\000\157\000\081\000\082\000\083\000\084\000\
\161\000\085\000\086\000\134\000\108\000\147\000\181\000\135\000\
\136\000\087\000\088\000\089\000\090\000\165\000\091\000\053\000\
\054\000\092\000\093\000\094\000\095\000\096\000\112\000\097\000\
\098\000\099\000\033\000\100\000\177\000\198\000\151\000\029\000\
\038\000\060\000\152\000\201\000\178\000"

let yysindex = "\008\000\
\029\255\000\000\000\000\022\255\032\255\045\255\060\255\000\000\
\066\255\092\255\094\255\000\000\000\000\251\254\251\254\142\255\
\070\255\000\000\000\000\000\000\000\000\000\000\000\000\085\255\
\000\000\093\255\000\000\000\000\095\255\105\255\097\255\143\255\
\000\000\129\255\129\255\147\255\000\000\156\255\000\000\133\255\
\132\255\000\000\158\255\159\255\251\254\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\162\255\167\255\134\255\164\255\
\044\000\180\255\177\255\183\255\000\000\132\255\133\255\132\255\
\044\000\080\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\178\255\000\000\042\255\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\186\255\000\000\044\000\000\000\000\000\
\000\000\000\000\110\255\000\000\000\000\000\000\000\000\192\255\
\000\000\000\000\000\000\193\255\069\000\251\254\000\000\000\000\
\000\000\197\255\198\255\000\000\009\255\000\000\204\255\191\255\
\206\255\033\000\209\255\000\000\113\255\044\000\044\000\000\000\
\000\000\000\000\000\000\044\000\188\255\000\000\080\000\080\000\
\080\000\080\000\002\000\002\000\000\000\210\255\000\000\190\255\
\000\000\000\000\000\000\080\000\080\000\080\000\000\000\080\000\
\091\255\000\000\211\255\000\000\221\255\000\000\000\000\213\255\
\000\000\000\000\000\000\200\255\000\000\201\255\000\000\000\000\
\227\255\220\255\000\000\000\000\233\255\228\255\236\255\237\255\
\000\000\178\255\069\000\000\000\000\000\238\255\239\255\000\000\
\229\255\240\255\247\255\000\000\000\000\033\000\033\000\044\000\
\044\000\246\255\188\255\000\000\080\000\000\000\000\000\243\255\
\000\000\000\000\000\000\033\000\000\000\000\000\000\000\252\255\
\004\000\000\000\000\000\000\000\044\000\000\000\000\000\000\000\
\000\000\033\000\000\000\000\000\000\000"

let yyrindex = "\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\006\000\000\000\011\000\000\000\
\170\255\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\013\000\016\255\000\000\
\000\000\000\000\026\000\000\000\000\000\000\000\000\000\000\000\
\000\000\027\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\006\000\000\000\047\255\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\182\255\000\000\000\000\
\000\000\000\000\000\000\215\255\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\035\000\000\000\000\000\000\000\
\010\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\038\000\
\000\000\000\000\000\000\027\000\000\000\000\000\000\000\000\000\
\000\000\000\000\006\000\000\000\135\255\000\000\000\000\043\255\
\000\000\000\000\000\000\088\255\000\000\090\255\000\000\000\000\
\000\000\042\000\000\000\000\000\000\000\047\000\000\000\000\000\
\000\000\006\000\000\000\000\000\000\000\000\000\000\000\000\000\
\049\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\224\255\000\000\001\000\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\055\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000"

let yygindex = "\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\148\000\
\219\000\249\000\146\000\000\000\176\255\000\000\000\000\000\000\
\000\000\000\000\142\000\149\000\000\000\000\000\187\255\000\000\
\145\000\222\255\000\000\000\000\235\255\000\000\000\000\163\000\
\000\000\000\000\000\000\000\000\000\000\139\255\148\255\251\255\
\000\000\000\000\202\255\244\255\219\255\250\255\195\000\000\000\
\000\000\000\000\242\255\194\255\157\255\000\000\157\000\000\000\
\000\000\235\000\000\000\128\000\156\000"

let yytablesize = 359
let yytable = "\025\000\
\025\000\028\000\050\000\113\000\031\000\126\000\052\000\037\000\
\001\000\021\000\032\000\109\000\167\000\022\000\150\000\023\000\
\140\000\065\000\163\000\164\000\166\000\166\000\014\000\065\000\
\174\000\105\000\175\000\052\000\055\000\052\000\025\000\133\000\
\015\000\166\000\056\000\166\000\141\000\156\000\158\000\003\000\
\004\000\005\000\006\000\160\000\101\000\016\000\101\000\117\000\
\010\000\055\000\101\000\055\000\116\000\110\000\010\000\056\000\
\104\000\056\000\106\000\111\000\017\000\168\000\169\000\007\000\
\113\000\113\000\113\000\113\000\118\000\119\000\018\000\207\000\
\120\000\121\000\122\000\123\000\148\000\113\000\113\000\113\000\
\166\000\113\000\200\000\202\000\021\000\109\000\173\000\025\000\
\022\000\026\000\162\000\023\000\034\000\145\000\176\000\026\000\
\019\000\023\000\020\000\149\000\035\000\133\000\036\000\156\000\
\158\000\021\000\039\000\148\000\146\000\022\000\200\000\047\000\
\048\000\049\000\110\000\110\000\110\000\110\000\127\000\128\000\
\111\000\111\000\111\000\111\000\212\000\180\000\113\000\110\000\
\110\000\110\000\149\000\110\000\040\000\111\000\111\000\111\000\
\104\000\111\000\104\000\154\000\155\000\104\000\104\000\041\000\
\148\000\148\000\021\000\042\000\116\000\045\000\022\000\051\000\
\162\000\047\000\048\000\049\000\021\000\046\000\148\000\027\000\
\022\000\024\000\026\000\061\000\064\000\057\000\058\000\149\000\
\149\000\063\000\084\000\083\000\148\000\083\000\062\000\083\000\
\110\000\083\000\083\000\083\000\101\000\149\000\111\000\071\000\
\102\000\114\000\103\000\071\000\125\000\071\000\076\000\076\000\
\129\000\130\000\143\000\149\000\083\000\083\000\138\000\139\000\
\083\000\083\000\083\000\083\000\142\000\083\000\144\000\051\000\
\071\000\071\000\153\000\170\000\071\000\071\000\071\000\071\000\
\087\000\171\000\179\000\183\000\087\000\182\000\087\000\089\000\
\089\000\072\000\184\000\187\000\185\000\072\000\186\000\072\000\
\078\000\078\000\188\000\189\000\196\000\190\000\191\000\194\000\
\195\000\087\000\087\000\197\000\208\000\087\000\087\000\087\000\
\087\000\145\000\072\000\072\000\205\000\044\000\072\000\072\000\
\072\000\072\000\088\000\210\000\066\000\211\000\088\000\124\000\
\088\000\090\000\090\000\089\000\074\000\089\000\055\000\089\000\
\021\000\089\000\068\000\069\000\022\000\051\000\047\000\048\000\
\049\000\070\000\071\000\088\000\088\000\099\000\124\000\088\000\
\088\000\088\000\088\000\145\000\089\000\089\000\080\000\045\000\
\089\000\089\000\089\000\089\000\065\000\035\000\066\000\021\000\
\053\000\067\000\146\000\022\000\112\000\047\000\048\000\049\000\
\108\000\107\000\021\000\192\000\068\000\069\000\022\000\051\000\
\047\000\048\000\049\000\070\000\071\000\131\000\204\000\066\000\
\072\000\073\000\132\000\206\000\203\000\193\000\172\000\199\000\
\137\000\213\000\066\000\021\000\000\000\068\000\069\000\022\000\
\051\000\047\000\048\000\049\000\070\000\071\000\021\000\209\000\
\068\000\069\000\022\000\051\000\047\000\048\000\049\000"

let yycheck = "\014\000\
\015\000\016\000\040\000\066\000\017\000\086\000\041\000\029\000\
\001\000\015\001\017\000\066\000\130\000\019\001\114\000\021\001\
\008\001\002\001\127\000\128\000\129\000\130\000\001\001\008\001\
\142\000\063\000\144\000\062\000\041\000\064\000\045\000\101\000\
\001\001\142\000\041\000\144\000\028\001\118\000\119\000\011\001\
\012\001\013\001\014\001\124\000\002\001\001\001\004\001\006\001\
\002\001\062\000\008\001\064\000\074\000\066\000\008\001\062\000\
\062\000\064\000\064\000\066\000\001\001\131\000\132\000\035\001\
\127\000\128\000\129\000\130\000\027\001\028\001\005\001\189\000\
\031\001\032\001\033\001\034\001\114\000\140\000\141\000\142\000\
\189\000\144\000\182\000\183\000\015\001\140\000\141\000\102\000\
\019\001\002\001\125\000\002\001\008\001\003\001\004\001\008\001\
\005\001\008\001\005\001\114\000\008\001\171\000\008\001\184\000\
\185\000\015\001\002\001\145\000\018\001\019\001\210\000\021\001\
\022\001\023\001\127\000\128\000\129\000\130\000\009\001\010\001\
\127\000\128\000\129\000\130\000\205\000\147\000\189\000\140\000\
\141\000\142\000\145\000\144\000\036\001\140\000\141\000\142\000\
\002\001\144\000\004\001\027\001\028\001\007\001\008\001\001\001\
\182\000\183\000\015\001\019\001\170\000\003\001\019\001\020\001\
\187\000\021\001\022\001\023\001\015\001\002\001\196\000\018\001\
\019\001\014\000\015\000\002\001\001\001\008\001\008\001\182\000\
\183\000\036\001\001\001\002\001\210\000\004\001\008\001\006\001\
\189\000\008\001\009\001\010\001\001\001\196\000\189\000\002\001\
\008\001\008\001\004\001\006\001\003\001\008\001\009\001\010\001\
\001\001\001\001\004\001\210\000\027\001\028\001\002\001\002\001\
\031\001\032\001\033\001\034\001\001\001\036\001\001\001\020\001\
\027\001\028\001\002\001\002\001\031\001\032\001\033\001\034\001\
\002\001\028\001\008\001\007\001\006\001\001\001\008\001\009\001\
\010\001\002\001\027\001\008\001\028\001\006\001\004\001\008\001\
\009\001\010\001\002\001\008\001\008\001\002\001\002\001\002\001\
\002\001\027\001\028\001\004\001\002\001\031\001\032\001\033\001\
\034\001\003\001\027\001\028\001\007\001\035\000\031\001\032\001\
\033\001\034\001\002\001\008\001\003\001\002\001\006\001\002\001\
\008\001\009\001\010\001\002\001\002\001\004\001\002\001\006\001\
\015\001\008\001\017\001\018\001\019\001\020\001\021\001\022\001\
\023\001\024\001\025\001\027\001\028\001\004\001\004\001\031\001\
\032\001\033\001\034\001\003\001\027\001\028\001\004\001\002\001\
\031\001\032\001\033\001\034\001\001\001\004\001\003\001\015\001\
\002\001\006\001\018\001\019\001\004\001\021\001\022\001\023\001\
\002\001\065\000\015\001\170\000\017\001\018\001\019\001\020\001\
\021\001\022\001\023\001\024\001\025\001\001\001\185\000\003\001\
\029\001\030\001\006\001\187\000\184\000\171\000\140\000\179\000\
\102\000\210\000\003\001\015\001\255\255\017\001\018\001\019\001\
\020\001\021\001\022\001\023\001\024\001\025\001\015\001\196\000\
\017\001\018\001\019\001\020\001\021\001\022\001\023\001"

let yynames_const = "\
  LEFT_PARENTHESIS\000\
  RIGHT_PARENTHESIS\000\
  LEFT_BRACKET\000\
  RIGHT_BRACKET\000\
  DOT\000\
  NEGATION\000\
  COLON\000\
  COMMA\000\
  EQUALITY\000\
  DISEQUALITY\000\
  EOI\000\
  FOF\000\
  CNF\000\
  INCLUDE\000\
  DOLLAR_TRUE\000\
  DOLLAR_FALSE\000\
  DOLLAR\000\
  AND\000\
  OR\000\
  FORALL\000\
  EXISTS\000\
  BIJECTION\000\
  LEFT_IMPLICATION\000\
  RIGHT_IMPLICATION\000\
  XOR\000\
  META\000\
  SLASH\000\
  "

let yynames_block = "\
  SINGLE_QUOTED\000\
  DOLLAR_WORD\000\
  DOLLAR_DOLLAR_WORD\000\
  DISTINCT_OBJECT\000\
  LOWER_WORD\000\
  UPPER_WORD\000\
  UNSIGNED_INTEGER\000\
  SIGNED_INTEGER\000\
  REAL\000\
  "

let yyact = [|
  (fun _ -> failwith "parser")
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 1 : 'annotated_formula) in
    Obj.repr(
# 154 "tptpParser.mly"
      ( _1 )
# 431 "tptpParser.ml"
               : Clausify.fof_formula))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 1 : 'include_) in
    Obj.repr(
# 157 "tptpParser.mly"
      ( _1 )
# 438 "tptpParser.ml"
               : Clausify.fof_formula))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 1 : 'command) in
    Obj.repr(
# 160 "tptpParser.mly"
     ( _1 )
# 445 "tptpParser.ml"
               : Clausify.fof_formula))
; (fun __caml_parser_env ->
    Obj.repr(
# 163 "tptpParser.mly"
     (if !fof || !conjecture then
  	  	(raise (Err.END_OF_FILE_WITH_DATA (!fof, !conjecture)))
  	   else
  	  	(raise Err.END_OF_FILE))
# 454 "tptpParser.ml"
               : Clausify.fof_formula))
; (fun __caml_parser_env ->
    let _3 = (Parsing.peek_val __caml_parser_env 1 : 'command_term_top) in
    Obj.repr(
# 170 "tptpParser.mly"
  (init_clause ();
		Clausify.Non_formula
			(Clausify.Atomic_formula
				(Clausify.make_clausify_term
					(Clausify.Compound
						("#", [_3])))))
# 466 "tptpParser.ml"
               : 'command))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'fof_annotated) in
    Obj.repr(
# 180 "tptpParser.mly"
      ( fof := true;
      	_1 )
# 474 "tptpParser.ml"
               : 'annotated_formula))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'cnf_annotated) in
    Obj.repr(
# 184 "tptpParser.mly"
      ( _1 )
# 481 "tptpParser.ml"
               : 'annotated_formula))
; (fun __caml_parser_env ->
    let _3 = (Parsing.peek_val __caml_parser_env 6 : 'name) in
    let _5 = (Parsing.peek_val __caml_parser_env 4 : 'formula_role) in
    let _7 = (Parsing.peek_val __caml_parser_env 2 : 'fof_formula) in
    let _8 = (Parsing.peek_val __caml_parser_env 1 : 'annotations) in
    Obj.repr(
# 188 "tptpParser.mly"
    ( init_clause ();
      if _5 = "conjecture" then
    	(conjecture := true;
    	Clausify.Conjecture _7)
      else
        (_7) )
# 496 "tptpParser.ml"
               : 'fof_annotated))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'binary_formula) in
    Obj.repr(
# 197 "tptpParser.mly"
    ( _1 )
# 503 "tptpParser.ml"
               : 'fof_formula))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'unitary_formula) in
    Obj.repr(
# 200 "tptpParser.mly"
    ( _1 )
# 510 "tptpParser.ml"
               : 'fof_formula))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'nonassoc_binary) in
    Obj.repr(
# 205 "tptpParser.mly"
    ( _1 )
# 517 "tptpParser.ml"
               : 'binary_formula))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'assoc_binary) in
    Obj.repr(
# 208 "tptpParser.mly"
    ( _1 )
# 524 "tptpParser.ml"
               : 'binary_formula))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 2 : 'unitary_formula) in
    let _2 = (Parsing.peek_val __caml_parser_env 1 : 'binary_connective) in
    let _3 = (Parsing.peek_val __caml_parser_env 0 : 'unitary_formula) in
    Obj.repr(
# 212 "tptpParser.mly"
    ( match _2 with
    	| Clausify.Right_implication ->
    		Clausify.Nonassoc_binary_formula
    			(Clausify.Left_implication, _3, _1)
    	| _ ->
    		Clausify.Nonassoc_binary_formula
    			(_2, _1, _3) )
# 539 "tptpParser.ml"
               : 'nonassoc_binary))
; (fun __caml_parser_env ->
    Obj.repr(
# 222 "tptpParser.mly"
    ( Clausify.Bijection )
# 545 "tptpParser.ml"
               : 'binary_connective))
; (fun __caml_parser_env ->
    Obj.repr(
# 224 "tptpParser.mly"
    ( Clausify.Left_implication )
# 551 "tptpParser.ml"
               : 'binary_connective))
; (fun __caml_parser_env ->
    Obj.repr(
# 226 "tptpParser.mly"
    ( Clausify.Right_implication )
# 557 "tptpParser.ml"
               : 'binary_connective))
; (fun __caml_parser_env ->
    Obj.repr(
# 228 "tptpParser.mly"
    ( Clausify.Xor )
# 563 "tptpParser.ml"
               : 'binary_connective))
; (fun __caml_parser_env ->
    Obj.repr(
# 230 "tptpParser.mly"
    ( Clausify.Nor )
# 569 "tptpParser.ml"
               : 'binary_connective))
; (fun __caml_parser_env ->
    Obj.repr(
# 232 "tptpParser.mly"
    ( Clausify.Nand )
# 575 "tptpParser.ml"
               : 'binary_connective))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'or_formula) in
    Obj.repr(
# 236 "tptpParser.mly"
    ( Clausify.Assoc_binary_formula
    	(Clausify.Or, _1) )
# 583 "tptpParser.ml"
               : 'assoc_binary))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'and_formula) in
    Obj.repr(
# 239 "tptpParser.mly"
    ( Clausify.Assoc_binary_formula
    	(Clausify.And, _1) )
# 591 "tptpParser.ml"
               : 'assoc_binary))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 2 : 'unitary_formula) in
    let _3 = (Parsing.peek_val __caml_parser_env 0 : 'more_or_formula) in
    Obj.repr(
# 244 "tptpParser.mly"
    ( _1 :: _3 )
# 599 "tptpParser.ml"
               : 'or_formula))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'unitary_formula) in
    Obj.repr(
# 248 "tptpParser.mly"
    ( [_1] )
# 606 "tptpParser.ml"
               : 'more_or_formula))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 2 : 'unitary_formula) in
    let _3 = (Parsing.peek_val __caml_parser_env 0 : 'more_or_formula) in
    Obj.repr(
# 250 "tptpParser.mly"
    ( _1 :: _3 )
# 614 "tptpParser.ml"
               : 'more_or_formula))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 2 : 'unitary_formula) in
    let _3 = (Parsing.peek_val __caml_parser_env 0 : 'more_and_formula) in
    Obj.repr(
# 254 "tptpParser.mly"
    ( _1 :: _3 )
# 622 "tptpParser.ml"
               : 'and_formula))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'unitary_formula) in
    Obj.repr(
# 258 "tptpParser.mly"
    ( [_1] )
# 629 "tptpParser.ml"
               : 'more_and_formula))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 2 : 'unitary_formula) in
    let _3 = (Parsing.peek_val __caml_parser_env 0 : 'more_and_formula) in
    Obj.repr(
# 260 "tptpParser.mly"
    ( _1 :: _3 )
# 637 "tptpParser.ml"
               : 'more_and_formula))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'quantified_formula) in
    Obj.repr(
# 264 "tptpParser.mly"
    ( _1 )
# 644 "tptpParser.ml"
               : 'unitary_formula))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'unary_formula) in
    Obj.repr(
# 266 "tptpParser.mly"
    ( _1 )
# 651 "tptpParser.ml"
               : 'unitary_formula))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 1 : 'fof_formula) in
    Obj.repr(
# 268 "tptpParser.mly"
    ( _2 )
# 658 "tptpParser.ml"
               : 'unitary_formula))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'atomic_formula) in
    Obj.repr(
# 270 "tptpParser.mly"
    ( match _1 with
    	| Clausify.Atomic_formula ct ->
    		if (Clausify.is_negated ct) then
	    		(Clausify.complement_clausify_term ct;
	    		Clausify.Negated_fof_formula _1)
	    	else
	    		(_1)
	    | _ -> _1)
# 672 "tptpParser.ml"
               : 'unitary_formula))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 5 : 'quantifier) in
    let _3 = (Parsing.peek_val __caml_parser_env 3 : 'variable_list) in
    let _6 = (Parsing.peek_val __caml_parser_env 0 : 'unitary_formula) in
    Obj.repr(
# 281 "tptpParser.mly"
    ( 	let var_ids =
    		List.map
    			(fun var -> 
    				match var with
    					| Clausify.Var (id, _) ->
    						id
    					| _ -> failwith "TptpParser: bad variable.")
    			_3
    	in
    		Clausify.Quantified_formula (_1, var_ids, _6) )
# 690 "tptpParser.ml"
               : 'quantified_formula))
; (fun __caml_parser_env ->
    Obj.repr(
# 294 "tptpParser.mly"
    ( Clausify.Forall )
# 696 "tptpParser.ml"
               : 'quantifier))
; (fun __caml_parser_env ->
    Obj.repr(
# 296 "tptpParser.mly"
    ( Clausify.Exists )
# 702 "tptpParser.ml"
               : 'quantifier))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'variable) in
    Obj.repr(
# 300 "tptpParser.mly"
    ( [_1] )
# 709 "tptpParser.ml"
               : 'variable_list))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 2 : 'variable) in
    let _3 = (Parsing.peek_val __caml_parser_env 0 : 'variable_list) in
    Obj.repr(
# 302 "tptpParser.mly"
    ( _1 :: _3 )
# 717 "tptpParser.ml"
               : 'variable_list))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 1 : 'unary_connective) in
    let _2 = (Parsing.peek_val __caml_parser_env 0 : 'unitary_formula) in
    Obj.repr(
# 306 "tptpParser.mly"
    ( match _1 with
    	| Clausify.Negation ->
    		(match _2 with
    			| Clausify.Negated_fof_formula fof ->
    				fof
    			| _ -> 
    				Clausify.Negated_fof_formula _2)
    	| _ -> failwith "tptpParser: unsupported unary connective" )
# 732 "tptpParser.ml"
               : 'unary_formula))
; (fun __caml_parser_env ->
    Obj.repr(
# 317 "tptpParser.mly"
    ( Clausify.Negation )
# 738 "tptpParser.ml"
               : 'unary_connective))
; (fun __caml_parser_env ->
    let _3 = (Parsing.peek_val __caml_parser_env 8 : 'name) in
    let _5 = (Parsing.peek_val __caml_parser_env 6 : 'formula_role) in
    let _8 = (Parsing.peek_val __caml_parser_env 3 : 'cnf_formula) in
    let _10 = (Parsing.peek_val __caml_parser_env 1 : 'annotations) in
    Obj.repr(
# 323 "tptpParser.mly"
      (
	let clause = 
	  _8
	in
	  init_clause ();
	  clause
      )
# 754 "tptpParser.ml"
               : 'cnf_annotated))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : string) in
    Obj.repr(
# 333 "tptpParser.mly"
    ( _1 )
# 761 "tptpParser.ml"
               : 'formula_role))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'null) in
    Obj.repr(
# 337 "tptpParser.mly"
      ( "" )
# 768 "tptpParser.ml"
               : 'annotations))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 1 : 'source) in
    let _3 = (Parsing.peek_val __caml_parser_env 0 : 'optional_info) in
    Obj.repr(
# 340 "tptpParser.mly"
      ( "" )
# 776 "tptpParser.ml"
               : 'annotations))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'disjunction) in
    Obj.repr(
# 346 "tptpParser.mly"
      ( _1 )
# 783 "tptpParser.ml"
               : 'cnf_formula))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 2 : 'literal) in
    let _3 = (Parsing.peek_val __caml_parser_env 0 : 'disjunction) in
    Obj.repr(
# 350 "tptpParser.mly"
      ( Clausify.Assoc_binary_formula
      		(Clausify.Or, [_1; _3]) )
# 792 "tptpParser.ml"
               : 'disjunction))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'literal) in
    Obj.repr(
# 354 "tptpParser.mly"
      ( _1 )
# 799 "tptpParser.ml"
               : 'disjunction))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 1 : 'atomic_formula) in
    Obj.repr(
# 360 "tptpParser.mly"
      ( _2 )
# 806 "tptpParser.ml"
               : 'literal))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'atomic_formula) in
    Obj.repr(
# 363 "tptpParser.mly"
      ( _1 )
# 813 "tptpParser.ml"
               : 'literal))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 0 : 'atomic_formula) in
    Obj.repr(
# 366 "tptpParser.mly"
      ( match _2 with
      		| Clausify.Negated_fof_formula fof ->
      			fof
      		| _ -> Clausify.Negated_fof_formula _2 )
# 823 "tptpParser.ml"
               : 'literal))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'plain_atom) in
    Obj.repr(
# 374 "tptpParser.mly"
      ( let ct = _1
      	in
      		if (Clausify.is_negated ct) then
      			(Clausify.complement_clausify_term ct;
      			Clausify.Negated_fof_formula
      				(Clausify.Atomic_formula ct))
      		else
      			(Clausify.Atomic_formula _1) )
# 837 "tptpParser.ml"
               : 'atomic_formula))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'defined_atom) in
    Obj.repr(
# 383 "tptpParser.mly"
      ( let ct = _1
      	in
      		if (Clausify.is_negated ct) then
      			(Clausify.complement_clausify_term ct;
      			Clausify.Negated_fof_formula
      				(Clausify.Atomic_formula ct))
      		else
      			(Clausify.Atomic_formula _1) )
# 851 "tptpParser.ml"
               : 'atomic_formula))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'system_atom) in
    Obj.repr(
# 392 "tptpParser.mly"
      ( let ct = _1
      	in
      		if (Clausify.is_negated ct) then
      			(Clausify.complement_clausify_term ct;
      			Clausify.Negated_fof_formula
      				(Clausify.Atomic_formula ct))
      		else
      			(Clausify.Atomic_formula _1) )
# 865 "tptpParser.ml"
               : 'atomic_formula))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'plain_term_top) in
    Obj.repr(
# 403 "tptpParser.mly"
      ( Clausify.make_clausify_term _1 )
# 872 "tptpParser.ml"
               : 'plain_atom))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'term) in
    Obj.repr(
# 407 "tptpParser.mly"
      ( [ _1 ] )
# 879 "tptpParser.ml"
               : 'arguments))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 2 : 'term) in
    let _3 = (Parsing.peek_val __caml_parser_env 0 : 'arguments) in
    Obj.repr(
# 410 "tptpParser.mly"
      ( _1 :: _3 )
# 887 "tptpParser.ml"
               : 'arguments))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'command_term) in
    Obj.repr(
# 414 "tptpParser.mly"
      ( [ _1 ] )
# 894 "tptpParser.ml"
               : 'command_arguments))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 2 : 'command_term) in
    let _3 = (Parsing.peek_val __caml_parser_env 0 : 'command_arguments) in
    Obj.repr(
# 417 "tptpParser.mly"
      ( _1 :: _3 )
# 902 "tptpParser.ml"
               : 'command_arguments))
; (fun __caml_parser_env ->
    Obj.repr(
# 422 "tptpParser.mly"
      ( Clausify.make_clausify_term 
      		(Clausify.Const "true") )
# 909 "tptpParser.ml"
               : 'defined_atom))
; (fun __caml_parser_env ->
    Obj.repr(
# 426 "tptpParser.mly"
      ( Clausify.make_clausify_term 
      		(Clausify.Const "false") )
# 916 "tptpParser.ml"
               : 'defined_atom))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 2 : 'term) in
    let _3 = (Parsing.peek_val __caml_parser_env 0 : 'term) in
    Obj.repr(
# 430 "tptpParser.mly"
      ( Clausify.make_clausify_term 
      		(Clausify.Compound ("=", [_1; _3])) )
# 925 "tptpParser.ml"
               : 'defined_atom))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 2 : 'term) in
    let _3 = (Parsing.peek_val __caml_parser_env 0 : 'term) in
    Obj.repr(
# 434 "tptpParser.mly"
      ( Clausify.make_negated_clausify_term 
   			(Clausify.Compound ("=", [_1; _3])) )
# 934 "tptpParser.ml"
               : 'defined_atom))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'system_term_top) in
    Obj.repr(
# 439 "tptpParser.mly"
      ( _1 )
# 941 "tptpParser.ml"
               : 'system_atom))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'function_term) in
    Obj.repr(
# 444 "tptpParser.mly"
      ( _1 )
# 948 "tptpParser.ml"
               : 'term))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'variable) in
    Obj.repr(
# 448 "tptpParser.mly"
     ( _1 )
# 955 "tptpParser.ml"
               : 'command_term))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 2 : 'constant) in
    let _3 = (Parsing.peek_val __caml_parser_env 0 : 'number) in
    Obj.repr(
# 451 "tptpParser.mly"
      ( Clausify.Compound ("/", [(Clausify.Const _1); _3]) )
# 963 "tptpParser.ml"
               : 'command_term))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'constant) in
    Obj.repr(
# 454 "tptpParser.mly"
      ( Clausify.Const _1 )
# 970 "tptpParser.ml"
               : 'command_term))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 3 : 'functor_) in
    let _3 = (Parsing.peek_val __caml_parser_env 1 : 'command_arguments) in
    Obj.repr(
# 457 "tptpParser.mly"
      ( 
	  	Clausify.Compound (_1, _3)
      )
# 980 "tptpParser.ml"
               : 'command_term))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 1 : 'function_term_list_arguments) in
    Obj.repr(
# 464 "tptpParser.mly"
     ( _2 )
# 987 "tptpParser.ml"
               : 'function_term))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'plain_term) in
    Obj.repr(
# 467 "tptpParser.mly"
      ( _1 )
# 994 "tptpParser.ml"
               : 'function_term))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'defined_term) in
    Obj.repr(
# 470 "tptpParser.mly"
      ( _1 )
# 1001 "tptpParser.ml"
               : 'function_term))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'system_term) in
    Obj.repr(
# 473 "tptpParser.mly"
      ( _1 )
# 1008 "tptpParser.ml"
               : 'function_term))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'constant) in
    Obj.repr(
# 477 "tptpParser.mly"
      ( Clausify.Const _1 )
# 1015 "tptpParser.ml"
               : 'plain_term_top))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 3 : 'functor_) in
    let _3 = (Parsing.peek_val __caml_parser_env 1 : 'arguments) in
    Obj.repr(
# 480 "tptpParser.mly"
      ( 
	  	Clausify.Compound (_1, _3)
      )
# 1025 "tptpParser.ml"
               : 'plain_term_top))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 2 : 'constant) in
    let _3 = (Parsing.peek_val __caml_parser_env 0 : 'number) in
    Obj.repr(
# 486 "tptpParser.mly"
  ( Clausify.Compound ("/", [(Clausify.Const _1); _3]) )
# 1033 "tptpParser.ml"
               : 'command_term_top))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'constant) in
    Obj.repr(
# 489 "tptpParser.mly"
  ( Clausify.Const _1 )
# 1040 "tptpParser.ml"
               : 'command_term_top))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 3 : 'functor_) in
    let _3 = (Parsing.peek_val __caml_parser_env 1 : 'command_arguments) in
    Obj.repr(
# 492 "tptpParser.mly"
  ( Clausify.Compound (_1, _3) )
# 1048 "tptpParser.ml"
               : 'command_term_top))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'constant) in
    Obj.repr(
# 496 "tptpParser.mly"
      ( Clausify.Const _1 )
# 1055 "tptpParser.ml"
               : 'plain_term))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'variable) in
    Obj.repr(
# 499 "tptpParser.mly"
     ( _1 )
# 1062 "tptpParser.ml"
               : 'plain_term))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 3 : 'functor_) in
    let _3 = (Parsing.peek_val __caml_parser_env 1 : 'arguments) in
    Obj.repr(
# 502 "tptpParser.mly"
      ( 
	  	Clausify.Compound (_1, _3)
      )
# 1072 "tptpParser.ml"
               : 'plain_term))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'null) in
    Obj.repr(
# 508 "tptpParser.mly"
     ( Clausify.Const "[]" )
# 1079 "tptpParser.ml"
               : 'function_term_list_arguments))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'function_term) in
    Obj.repr(
# 511 "tptpParser.mly"
     ( let empty = Clausify.Const "[]"
  	  	in
  	    	Clausify.Compound (".", [_1; empty]) )
# 1088 "tptpParser.ml"
               : 'function_term_list_arguments))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 2 : 'function_term) in
    let _3 = (Parsing.peek_val __caml_parser_env 0 : 'function_term_list_arguments) in
    Obj.repr(
# 516 "tptpParser.mly"
      ( Clausify.Compound (".", [_1; _3]) )
# 1096 "tptpParser.ml"
               : 'function_term_list_arguments))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 2 : 'function_term) in
    let _3 = (Parsing.peek_val __caml_parser_env 0 : 'function_term) in
    Obj.repr(
# 519 "tptpParser.mly"
      ( Clausify.Compound (".", [_1; _3]) )
# 1104 "tptpParser.ml"
               : 'function_term_list_arguments))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'atomic_word) in
    Obj.repr(
# 523 "tptpParser.mly"
      ( _1 )
# 1111 "tptpParser.ml"
               : 'constant))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'atomic_word) in
    Obj.repr(
# 527 "tptpParser.mly"
      ( _1 )
# 1118 "tptpParser.ml"
               : 'functor_))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'number) in
    Obj.repr(
# 531 "tptpParser.mly"
      ( _1 )
# 1125 "tptpParser.ml"
               : 'defined_term))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : string) in
    Obj.repr(
# 534 "tptpParser.mly"
      ( Clausify.Const "<UNSUPPORTED>" )
# 1132 "tptpParser.ml"
               : 'defined_term))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'atomic_system_word) in
    Obj.repr(
# 538 "tptpParser.mly"
      ( Clausify.make_clausify_term 
      		(Clausify.Const _1) )
# 1140 "tptpParser.ml"
               : 'system_term_top))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 3 : 'atomic_system_word) in
    let _3 = (Parsing.peek_val __caml_parser_env 1 : 'arguments) in
    Obj.repr(
# 542 "tptpParser.mly"
      ( Clausify.make_clausify_term 
      		(Clausify.Compound (_1, _3)) )
# 1149 "tptpParser.ml"
               : 'system_term_top))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'atomic_system_word) in
    Obj.repr(
# 550 "tptpParser.mly"
      ( Clausify.Const _1 )
# 1156 "tptpParser.ml"
               : 'system_term))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 3 : 'atomic_system_word) in
    let _3 = (Parsing.peek_val __caml_parser_env 1 : 'arguments) in
    Obj.repr(
# 553 "tptpParser.mly"
      ( Clausify.Compound (_1, _3) )
# 1164 "tptpParser.ml"
               : 'system_term))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : string) in
    Obj.repr(
# 559 "tptpParser.mly"
      ( let var = get_var _1
      	in
      		if _1 = !(Cfg.answer_variable_name) then
      			(Clausify.focus_variable := var)
      		else
      			();
      		Clausify.Var (var, _1) )
# 1177 "tptpParser.ml"
               : 'variable))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'general_term) in
    Obj.repr(
# 571 "tptpParser.mly"
      ( "" )
# 1184 "tptpParser.ml"
               : 'source))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 0 : 'useful_info) in
    Obj.repr(
# 575 "tptpParser.mly"
      ( "" )
# 1191 "tptpParser.ml"
               : 'optional_info))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'null) in
    Obj.repr(
# 578 "tptpParser.mly"
      ( "" )
# 1198 "tptpParser.ml"
               : 'optional_info))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'general_list) in
    Obj.repr(
# 582 "tptpParser.mly"
      ( "" )
# 1205 "tptpParser.ml"
               : 'useful_info))
; (fun __caml_parser_env ->
    let _3 = (Parsing.peek_val __caml_parser_env 2 : 'file_name) in
    let _4 = (Parsing.peek_val __caml_parser_env 1 : 'formula_selection) in
    Obj.repr(
# 587 "tptpParser.mly"
      ( let command =
      		Clausify.Compound
      			("load", [Clausify.Const _3])
      	in
	      	Clausify.Non_formula
				(Clausify.Atomic_formula
					(Clausify.make_clausify_term
						(Clausify.Compound
							("#", [command])))) )
# 1221 "tptpParser.ml"
               : 'include_))
; (fun __caml_parser_env ->
    let _3 = (Parsing.peek_val __caml_parser_env 1 : 'name_list) in
    Obj.repr(
# 599 "tptpParser.mly"
      ( _3 )
# 1228 "tptpParser.ml"
               : 'formula_selection))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'null) in
    Obj.repr(
# 602 "tptpParser.mly"
      ( [] )
# 1235 "tptpParser.ml"
               : 'formula_selection))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'name) in
    Obj.repr(
# 606 "tptpParser.mly"
      ( [_1] )
# 1242 "tptpParser.ml"
               : 'name_list))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 2 : 'name) in
    let _3 = (Parsing.peek_val __caml_parser_env 0 : 'name_list) in
    Obj.repr(
# 609 "tptpParser.mly"
      ( _1 :: _3 )
# 1250 "tptpParser.ml"
               : 'name_list))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'general_data) in
    Obj.repr(
# 615 "tptpParser.mly"
      ( _1 )
# 1257 "tptpParser.ml"
               : 'general_term))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 2 : 'general_data) in
    let _3 = (Parsing.peek_val __caml_parser_env 0 : 'general_term) in
    Obj.repr(
# 618 "tptpParser.mly"
      ( Clausify.Const "<UNSUPPORTED>" )
# 1265 "tptpParser.ml"
               : 'general_term))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'general_list) in
    Obj.repr(
# 621 "tptpParser.mly"
      ( Clausify.Const "<UNSUPPORTED>" )
# 1272 "tptpParser.ml"
               : 'general_term))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'atomic_word) in
    Obj.repr(
# 625 "tptpParser.mly"
      ( Clausify.Const _1 )
# 1279 "tptpParser.ml"
               : 'general_data))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 3 : 'atomic_word) in
    let _3 = (Parsing.peek_val __caml_parser_env 1 : 'general_arguments) in
    Obj.repr(
# 628 "tptpParser.mly"
      ( Clausify.Compound (_1, _3) )
# 1287 "tptpParser.ml"
               : 'general_data))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'number) in
    Obj.repr(
# 630 "tptpParser.mly"
           ( _1 )
# 1294 "tptpParser.ml"
               : 'general_data))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : string) in
    Obj.repr(
# 633 "tptpParser.mly"
      ( Clausify.Const _1 )
# 1301 "tptpParser.ml"
               : 'general_data))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'general_term) in
    Obj.repr(
# 637 "tptpParser.mly"
      ( [_1] )
# 1308 "tptpParser.ml"
               : 'general_arguments))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 2 : 'general_term) in
    let _3 = (Parsing.peek_val __caml_parser_env 0 : 'general_arguments) in
    Obj.repr(
# 640 "tptpParser.mly"
      ( _1 :: _3 )
# 1316 "tptpParser.ml"
               : 'general_arguments))
; (fun __caml_parser_env ->
    Obj.repr(
# 644 "tptpParser.mly"
      ( [] )
# 1322 "tptpParser.ml"
               : 'general_list))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 1 : 'general_term_list) in
    Obj.repr(
# 647 "tptpParser.mly"
      ( _2 )
# 1329 "tptpParser.ml"
               : 'general_list))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'general_term) in
    Obj.repr(
# 651 "tptpParser.mly"
      ( [_1] )
# 1336 "tptpParser.ml"
               : 'general_term_list))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 2 : 'general_term) in
    let _3 = (Parsing.peek_val __caml_parser_env 0 : 'general_term_list) in
    Obj.repr(
# 654 "tptpParser.mly"
      ( _1 :: _3 )
# 1344 "tptpParser.ml"
               : 'general_term_list))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'atomic_word) in
    Obj.repr(
# 659 "tptpParser.mly"
      ( Clausify.Const _1 )
# 1351 "tptpParser.ml"
               : 'name))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : string) in
    Obj.repr(
# 662 "tptpParser.mly"
      ( Clausify.Int (int_of_string _1) )
# 1358 "tptpParser.ml"
               : 'name))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : string) in
    Obj.repr(
# 666 "tptpParser.mly"
      ( _1 )
# 1365 "tptpParser.ml"
               : 'atomic_word))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : string) in
    Obj.repr(
# 669 "tptpParser.mly"
      ( let str = String.sub _1 1 ((String.length _1) - 2)
        in	
        	str )
# 1374 "tptpParser.ml"
               : 'atomic_word))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : string) in
    Obj.repr(
# 675 "tptpParser.mly"
      ( "$$"  )
# 1381 "tptpParser.ml"
               : 'atomic_system_word))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : string) in
    Obj.repr(
# 679 "tptpParser.mly"
      ( Clausify.Float (float_of_string _1) )
# 1388 "tptpParser.ml"
               : 'number))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : string) in
    Obj.repr(
# 682 "tptpParser.mly"
      ( Clausify.Int (int_of_string _1) )
# 1395 "tptpParser.ml"
               : 'number))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : string) in
    Obj.repr(
# 685 "tptpParser.mly"
      ( Clausify.Int (int_of_string _1) )
# 1402 "tptpParser.ml"
               : 'number))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'atomic_word) in
    Obj.repr(
# 689 "tptpParser.mly"
      ( _1 )
# 1409 "tptpParser.ml"
               : 'file_name))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : string) in
    Obj.repr(
# 691 "tptpParser.mly"
     ( let str = String.sub _1 1 ((String.length _1) - 2)
        in	
        	str )
# 1418 "tptpParser.ml"
               : 'file_name))
; (fun __caml_parser_env ->
    Obj.repr(
# 696 "tptpParser.mly"
      ( )
# 1424 "tptpParser.ml"
               : 'null))
(* Entry tptp_input *)
; (fun __caml_parser_env -> raise (Parsing.YYexit (Parsing.peek_val __caml_parser_env 0)))
|]
let yytables =
  { Parsing.actions=yyact;
    Parsing.transl_const=yytransl_const;
    Parsing.transl_block=yytransl_block;
    Parsing.lhs=yylhs;
    Parsing.len=yylen;
    Parsing.defred=yydefred;
    Parsing.dgoto=yydgoto;
    Parsing.sindex=yysindex;
    Parsing.rindex=yyrindex;
    Parsing.gindex=yygindex;
    Parsing.tablesize=yytablesize;
    Parsing.table=yytable;
    Parsing.check=yycheck;
    Parsing.error_function=parse_error;
    Parsing.names_const=yynames_const;
    Parsing.names_block=yynames_block }
let tptp_input (lexfun : Lexing.lexbuf -> token) (lexbuf : Lexing.lexbuf) =
   (Parsing.yyparse yytables 1 lexfun lexbuf : Clausify.fof_formula)
;;
# 719 "tptpParser.mly"




# 1454 "tptpParser.ml"
