module Generate where

import Util
import Form
import List


generateFromTerms :: [Term] -> [[Term]]
generateFromTerms [] = []
generateFromTerms (t:ts) = [ fixVar v t| v <- getVars t] : generateFromTerms ts
--Ex:  f(X,Y,X) becomes [f(X,*,X), f(*,Y,*)]
		
star = (Var ("*" ::: Variable))

fixVar (z ::: Variable) (Var (x ::: Variable))  = 
	if x == z then (Var ("X" ::: Variable)) else star
fixVar z (Fun f ts)  = if and [not (includesVar z t) | t <- ts] then 
	star else Fun f (map (fixVar z) ts)
	
includesVar (z ::: Variable) (Var (x ::: Variable)) = z == x
includesVar z (Fun f ts) = or [includesVar z t | t <- ts]

generateAllX ss n = generate' ss [[star, Var ("X" ::: Variable)]] True n
--generates all terms of function symbols ss of depth n,
--where X occurs AT LEAST once. Subterms with no occurrencies of X are represented by "*".

generate :: [Symbol] -> Int -> [[Term]]
generate ss n = tail $ generate' ss [[star, Var ("X" ::: Variable)]] False n
--generates all terms of function symbols ss of depth n,
--where X occurs EXACTLY once. Subterms with no occurrencies of X are represented by "*".


--Generation of predicates containing at least one "X"-argument,
--all other arguments are represented by "*"
generateP (p ::: (PredArity a)) = 
	[ Atom ( Pred (p ::: (PredArity a)) ts) | ts <- terms a, notallstars ts ]

generate' _ ts _ 0 = ts
generate' ss ts b n =
	let 
		funlist =  [ Fun (f ::: (FunArity a)) terms | (f ::: (FunArity a)) <- ss, 
		 terms <- termLists False a ts ] in
		 	if b then generate' ss (ts ++ [funlist]) b (n-1) else 
			generate' ss (ts ++ [((filter notallstaronex) funlist)]) b (n-1)
		  
			
	
termLists False 1 tts = [[t] | t <- (last tts)]
termLists True  1 tts = [[t] | t <- concat tts]

termLists b arity tts = 
	[(t: terms) | t <- (concat (dropLast tts)), 
		terms <-  termLists b (arity - 1) tts] ++
				
	[(t' : terms') | t' <- (last tts), terms' <- termLists True (arity -1) tts]
	
terms 0 = [[]]
terms n = [ star : t | t <- ts] ++ [(Var ("X" ::: Variable))  : t | t <- ts] 
	where ts = terms (n-1)	 



genRs (Atom (Pred p ts)) = map Atom $ filter xandy [(Pred p ts')| ts' <- (genRs' ts)]
	where
		genRs'   [] = [[]]
		genRs'  ((Var ("X" ::: Variable)):ts) = map  ((Var ("X" ::: Variable)) : )   (genRs'' ts)
		genRs'  ((Var ("*" ::: Variable)):ts) = map  ((Var ("*" ::: Variable)) : )   (genRs' ts)
		genRs'' [] = [[]]
		genRs'' ((Var ("*" ::: Variable)):ts) = map ((Var ("*" ::: Variable)) : )   (genRs'' ts)
		genRs'' ((Var ("X" ::: Variable)):ts) = (map ((Var ("X" ::: Variable)) : ) (genRs'' ts)) ++ 	 				
												 (map ((Var ("Y" ::: Variable)) : ) (genRs'' ts))
										
getPs  [] = []
getPs (p:ps) = generateP p ++ getPs ps


getRs = (filter twox) . getPs   

twox (Atom (Pred p ts)) = foldl (+) 0 [countx t | t <- ts] >= 2

--used for filtering..

notallstars [] = False
notallstars ((Var ("X" ::: Variable)):_) = True
notallstars ((Var ("*" ::: Variable)):ts) = notallstars ts


notallstar t = not (allstar t)
notallstaronex t = (not (allstar t)) && onex t

xandy (Pred p ts) = elem (Var ("X" ::: Variable)) ts &&
	elem (Var ("Y" ::: Variable)) ts

allstar :: Term -> Bool
allstar (Var ("*" ::: Variable)) = True
allstar (Var _) = False
allstar (Fun f ts) = and $ map allstar ts

onex :: Term -> Bool
onex t = countx t == 1

countx (Var ("X" ::: Variable)) = 1
countx (Var _) = 0
countx (Fun f ts) = foldl (+) 0 [countx t | t <- ts]
