 

module Infinox where

import List
import Form
import IO
import System
import System.Time
import Control.Monad
import System.Directory
import qualified Data.Set as Set
import ParseProblem
import Util
import Generate
import Zoom
import Timeout
import Maybe

-------------------------------------------------------------------------------

type ETimeOut = Int

data Inputs = Inputs {ttimeout :: ETimeOut, tzoom :: Bool, tg :: Bool,
				tr :: Bool, tp :: Bool, oni :: Bool, tvb :: Bool,
				 tout :: Maybe FilePath}
-- arguments: 
-- t2: timeout for eprover in seconds, 
-- (b1,b2,b3,b4,b5): boolean values corresponding to use of:
-- b1: zoom, b2: generation of terms, b3: equivalence predicates (r),b4: limiting predicates (p), 
-- b5: True means search for terms that are onto and not injective. False means injective and not onto.
-- b6: prints comments on the progress of the program to stdout
-- output:  output file 
-- ff:		input file.
disprove :: Inputs -> FilePath -> IO ([(Term, Form, Maybe Form)],Int)
disprove (Inputs t b1 b2 b3 b4 b5 b6 output) ff = do
	let
		tempdir = (filter ( (not . (flip elem) ['/','.','-',' '])) ff) ++ "_TEMP"
	createDirectory tempdir
	time1 	<- getClockTime
	p  		<- readProblem ff	
	fs 		<- (if b1 then zoom tempdir (getForms p) 100 else return (getForms p))

	let
		funterms   	= nub $ getFuns $ concat $ map Set.toList $ map subterms fs
		funsymbols 	= nub $ getFunSymbols funterms
		terms' 		= if b2 then generate funsymbols 2 else generateFromTerms funterms
		terms 		= nub $ concat terms'
		preds'		= nub $ concat $ map getPreds' fs		
		preds		= if b3 then preds' else if
						 elem ("=" ::: PredArity 2) preds' then 
						 	[("=" ::: PredArity 2)] else []
		rs 			= getRs preds 
		ps			= if b4 then nub $ deleteEqs $ getPs preds 
								 else [] 								 
	if terms == [] then do	-- if there are no terms - no point to continue!
		time2 <- getClockTime
		let 
			time = tdSec $ diffClockTimes time2 time1
			result = ([]::[(Term, Form, Maybe Form)],time)
		save ff result output
		system $ "rm -r " ++  tempdir
		return result
	
	 else continue ff tempdir fs t (terms,terms) rs ps preds b3 b4 b5 b6 
	 		output time1 Nothing []


--This function needs to be cleaned up!!! (But it works well..)
--It checks each r-p pair and t-p pair ONCE, 
--for each r, reflexivity is checked under all p-predicates.
--If r is found to be reflexive under p, all terms are checked
--to see if they are closed under this p, either by a call to
--eprover (for the first r in the list), or by looking up
--previous results. As soon as a matching t-p-r triple is found,
--it is checked using the desired method. 
--This is to ensure that we don't spend too much time searching
--for triples, and time out before any terms have been checked.
continue :: FilePath -> FilePath -> [Form] -> Int -> ([Term],[Term]) -> 
	[Form] -> [Form] -> [Symbol] -> Bool -> Bool -> Bool -> Bool -> 
	Maybe FilePath -> ClockTime -> Maybe ([(Form,Form)],[Form]) ->
	[(Term,[Form])] ->	IO ([(Term, Form, Maybe Form)],Int)
	
continue file dir _ _ _ [] _ _ _ _ _ _ output time1 _ _ = 
--r-predicate list empty - all tests done.
--calculate time, save output to file if requested,
--return result
 do
 	finish time1 [] file output dir
 

continue file dir fs t ([],ts) rs ps preds b3 b4 b5 b6 output time1 _ fsps = 
--when all terms have been processed for an r-predicate, drop the first
--predicate in the list and start over with the full term list.
	if fsps == [] then 
		finish time1 [] file output dir
	 else continue file dir fs t (ts,ts) (drop 1 rs) ps preds b3 b4 b5 b6 
	 		output time1 Nothing fsps


continue file dir fs t (ts1,ts) rs ps preds b3 b4 b5 b6 output time1 mpsrs mfsps = do
	let
		r	 = head rs
		term = head ts1
		method = if b5 then disproveInjNotOnto else disproveOntoNotInj
	refls <- case mpsrs of
		Just (pr,rs)  -> return rs		--already checked for reflexivity in previous call
											--(same r as in previous call)
		Nothing  -> (if b3 then filterM (isRefl dir fs t b6) [r] else if
					elem ("=" ::: PredArity 2) preds then return [equality] 
						else return [])					
	psrs <- if b4 then case mpsrs of 
		Just (pr,_) -> return pr
		Nothing -> do
			case refls of
				[]	-> do
					psrs' <- getPRPairs dir fs [(p',r') | p' <- ps, r' <- [r] ] t b6			
					return [(p',r') | (p',r) <- psrs', r' <- concat (map genRs [r])] 							
				[r] -> 
					do 
						return [(p',r') | r' <- concat (map genRs [r]), p' <- ps ]
			 else return []	
	let 
		lt = lookup term mfsps			
	fsps <-					
		case lt of
			Just ps'	-> return $ zip (repeat term) ps'
			Nothing 	-> getFPPairs dir fs 
							[(term,p'')|  p'' <- (nubBy eqp ((map fst) psrs))] t b6														
	let
		fsrs = [(term,r') | r' <- concat (map genRs refls)] 
		fsrsps = zippy fsps psrs  
		newfsps = case lt of
			Just ps'	-> mfsps
			Nothing		-> ((term,if fsps == [] then [] else snd (unzip fsps)) : mfsps) 
	mt <- if b4 then (mappy (method dir fs t b6) [(fun,r,Just p) | (fun,r,p) <- fsrsps])
			else mappy (method dir fs t b6) [(fun,r,Nothing) | (fun,r) <- fsrs]	
	case mt of
		[]	-> do
			let ts'' = (drop 1 ts1)
			continue file dir fs t (ts'',ts) rs ps preds b3 b4 b5 b6 
				output time1 (Just (psrs,refls)) newfsps 
		res -> do
			finish time1 mt file output dir


finish time1 result file output dir = do
	time2 <- getClockTime
	let
		time = tdSec $ diffClockTimes time2 time1
	save file (result,time) output
	system $ "rm -r " ++  dir
	return (result,time)

save file result output = case output of
		Nothing 	-> return ()
		Just file'	-> do
			appendFile file' ("\n"++ file ++ ": " ++ (show result))
								 			
pairall :: [Form] -> [Form]
pairall fs = nub [f1 /\ f2 | f1 <- fs, f2 <- fs, f1 /= f2]			

negall fs = [Not f | f <- fs]
		
disproveInjNotOnto :: FilePath -> [Form] -> ETimeOut -> Bool -> (Term,Form, Maybe Form) -> 
 						IO [(Term,Form, Maybe Form)]	

disproveInjNotOnto  dir fs timeout vb (t,r,mp) = do
	if vb then do
			putStrLn $ "\nt: " ++ (show t)
			putStrLn $ "r: " ++ (show r)
		else putStr ""
	let 
		conj = case mp of
			Just p  -> form2conjecture 0 (checkNotInjOntoPR t r p) 
			Nothing -> form2conjecture 0 (checkInjOntoR t r) 
		provefile = dir ++ "/injonto"	
	putStrLn $ if mp == Nothing || not vb then "" else ("p: " ++ show (fromJust mp))
	b <- prove conj fs provefile timeout
	removeFile provefile
	if b then do
				putStrLn "Injective and not onto!"
				return [(t,r,mp)] 
		  else return []


disproveOntoNotInj :: FilePath -> [Form] -> ETimeOut -> Bool -> (Term,Form, Maybe Form) ->  
						IO [(Term,Form, Maybe Form)]	
disproveOntoNotInj dir fs timeout vb (t,r,mp) =  do
	if vb then do
			putStrLn $ "\nt: " ++ (show t)
			putStrLn $ "r: " ++ (show r)
		else putStr ""
	let 
		conj = case mp of 
				Just p 	->	form2conjecture 0 (checkNotInjOntoPR t r p) 
				Nothing ->  form2conjecture 0 (checkNotInjOntoR t r)
		provefile = dir ++ "/notinjonto" 
	putStrLn $ if mp == Nothing || not vb then "" else ("p: " ++ show (fromJust mp))
	b <- prove conj fs provefile timeout 	
	removeFile provefile
	if b then do
		putStrLn "Onto and not injective!"
		return [(t,r,mp)] 
	 else return [] 

 
 
prove conj fs provefile timeout = do   
	h' <- try $ openFile provefile WriteMode			
 	case h' of 
  		Left e -> do 
  			return False	
  		Right h -> do
  			let 
  				s = form2axioms fs 
			hPutStr h (s ++ conj)		
			hClose h		
			eprove provefile timeout
		
		
-------------------------------------------------------------------------------

		
eqp (Atom (Pred p ts)) (Atom (Pred q ts2)) = p == q && ts == ts2
eqp (And ps) (And qs) = Set.size ps == Set.size qs && 
	and [eqp p q | (p,q) <- zip (Set.toList ps) (Set.toList qs)]
eqp _ _ = False

deleteEqs [] = []
deleteEqs ((Atom (Pred ("=" ::: _) _)):fs) = deleteEqs fs
deleteEqs (f:fs) = (f:(deleteEqs fs))	

zippy [] _ = []
zippy ((f,p):fsps) psrs = [(f,r,p') | (p',r) <- psrs, p' == p]  ++ zippy fsps psrs

mappy f []	= return []		--mapping that stops when the result != []
mappy f (x:xs) = do
	r <- f x
	case r of
		[]	-> mappy f xs
		a  -> return a


--get the pairs of p and r predicates where p implies reflexivity of r
getPRPairs :: FilePath -> [Form] -> [(Form,Form)] -> ETimeOut -> Bool -> IO [(Form,Form)]
getPRPairs dir p prs to vb = do
	ms <- mapM (checkPR dir p to vb) prs
	return $ concat ms

--get the pairs of f terms and p predicates where f is closed under p
getFPPairs :: FilePath -> [Form] -> [(Term,Form)] -> ETimeOut -> Bool -> IO [(Term,Form)]
getFPPairs dir p fps to vb = do
	ms <- mapM (checkFP dir p to vb) fps
	return $ concat ms


checkPR :: FilePath -> [Form] -> ETimeOut -> Bool -> (Form,Form) -> IO [(Form,Form)]
checkPR dir problem to vb (p,r)  = do
	let 
		conj = form2conjecture 0 (checkPimpliesRef p r)
		provefile = dir ++ "/checkpr" 	
	putStr (if vb then "Checking " ++ (show r) ++ " reflexive under " ++ (show p) ++ "\n"
				else "")
	b <- prove conj problem provefile to
	putStr (if vb then (show b) ++ "\n" else "")
	removeFile provefile
	if b then return [(p,r)] else return []
	
	
checkFP dir problem to vb (f,p)  = do
	let 
		conj = form2conjecture 0 (checkFClosedUnderP f p)
		provefile = dir ++ "/checkfp"
	putStr (if vb then "Checking " ++ (show f) ++ " closed under " ++ (show p) ++ "\n"
			else "")
	b <- prove conj problem provefile to
	removeFile provefile
	if b then return [(f,p)] else return []
	
				
-------------------------------------------------------------------------------


isRefl dir fs n vb r  = do
	let 
  		rfile = dir ++ "/refl" 
  		conj  = form2conjecture 0 (checkRefl r)
  	putStr (if vb then  ("Checking reflexivity: " ++ (show r) ++ "\n" )
  			else "")
	b <- prove conj fs rfile n
	putStr (if vb then (show b) ++ "\n" else "")
	removeFile rfile
	return b 

checkRefl p@(Atom (Pred r ts)) = exist vs $ forAll ("X" ::: Variable) $
	Atom $ Pred r $ applyStars (subp ts (Var ("Y" ::: Variable)) (Var ("X" ::: Variable))) vs	
	-- Ex:  p(X,*,*,Y) reflexive: 
	-- exists S1, S2 : forAll X p(X,S1,S2,X)
	where
		vs = genVars (predstars p) "S"
		--generate as many fresh variables as there are "*"'s in p,
		--to use for existential quantification
		
-------------------------------------------------------------------------------		
		
isInj :: FilePath -> [Form] -> Term -> Form -> Int -> IO Bool
isInj dir fs t r n = let conj = form2conjecture 0 (checkInj t r) in do
	h <- openFile (dir ++ "/inj")  WriteMode
	hPutStr h $ (form2axioms fs) ++ conj
	hClose h
	b <- eprove (dir ++ "/inj") n 
	removeFile (dir ++ "/inj")
	return b

sortInjTerms :: FilePath -> [Form] -> Int -> Int -> [[Term]]  -> 
				[Form] ->  ([(Term,Form)],[(Term,Form)]) -> 
	 			IO ([(Term,Form)],[(Term,Form)])	 	
sortInjTerms _ _ _ _ [] _ ts = return ts

sortInjTerms dir ps n m (t:tts) rs ts = do
	if (length (fst ts) + length (snd ts)) > m then return ts else  
		let trs = [(tt,rr) | tt <- t, rr <- rs] in do
			sts <- sortInjTerms' dir ps n m trs ts
			sortInjTerms dir ps n m tts rs sts
			
sortInjTerms' dir ps n m [] ts  = return ts	

sortInjTerms' dir ps n m ((fun@(Fun (f ::: _) args),r ):trs) ts = 
	if (length (fst ts) + length (snd ts)) > m then return ts else
		if deep fun && members (args,r) (snd ts) then do
				putStrLn $ "1: " ++ (show fun)
				sortInjTerms' dir ps n m trs (fst ts, (fun,r):(snd ts)) 
			else do
				putStrLn $ "2: " ++ (show fun)
				b <- isInj dir ps fun r n
				putStrLn (show b)
				if b then sortInjTerms' dir ps n m trs ((fun,r): (fst ts), snd ts) else
					sortInjTerms' dir ps n m trs ((fst ts), (fun,r):(snd ts))

sortInjTerms' dir ps n m ((Var _,_):trs) ts = sortInjTerms' dir ps n m trs ts

-------------------------------------------------------------------------------------
	

				
disprove2 :: FilePath -> [Form] -> Int -> Int -> IO [Term]
disprove2 dir fs n m = do
	let
		funterms   	= nub $ getFuns $ concat $ map Set.toList $ map subterms fs
		funs	 	= nub $ getFunSymbols funterms
	mappy (disprove2' dir fs n m) funs
		
			
disprove2' dir p n m f = do
  let allx = generateAllX [f] n in 
  --generate all possible combinations of "X" and "*" as arguments to f
	 let fs = filter (notallstar) (concat $ tail allx) in do
  --remove the ones that don't include "X".
			putStrLn ("Term: " ++ (show f))
			putStrLn (show fs)
			mappy (disprove2'' dir p m) fs
		

disprove2'' dir p m f  = do	
	let
		file = (dir ++ "/hprove")
	h <- openFile file WriteMode
	hSetBuffering h NoBuffering
	hPutStr h $ form2axioms p
	let 
		n = stars f 
		vs = genVars n "T"  
		fun' = applyStar f vs 
		h1 = hterm p fun' in 
			do			
				hPutStr h $ "fof(" ++ "c_0" ++ ", " ++ "conjecture," ++ 
					 (existvs vs) ++ "(("  ++ (show h1) ++ ")"
				hPutTs h p fun'
				b <- eprove file m
				putStrLn $ (show b) ++ "\n"
				hClose h
				removeFile file
				if b then return [f] else return []
	where
		existvs [] = ""
		existvs vs = "? " ++ (show vs) ++ " : "
		

hPutTs h [] f     = do	
	hPutStr h "))."

hPutTs h (t:ts) f = do
	hPutStr h (" & " ++ (show (transform f t)))
	hPutTs h ts f

-- phi -> ( (Exists y . Forall x . t(x) /= y) & phi^t(x) )				
hterm :: [Form] -> Term -> Form
hterm fs fun  = 
 Exists (Bind ("A" ::: Variable)  $
	ForAll (Bind ("X" ::: Variable) $
		 nt $ Atom  $ Pred ("=" ::: (PredArity 2)) [fun,Var ("A" ::: Variable)]))


-------------------------------------------------------------------------------
{-
testform = 
	ForAll (Bind ("X" ::: Variable) $
		Exists (Bind ("Y" ::: Variable) $   
			Atom (Pred ("Q" ::: (PredArity 3)) 
				[Var ("X" ::: Variable), Var ("Y" ::: Variable), 
					Fun ("f" ::: FunArity 2) [Var ("Y" ::: Variable),Fun ("a" ::: FunArity 0) []]])))
					
testterm = (Fun ("g" ::: FunArity 1) [Var ("X" ::: Variable)])
-}	   

transform :: Term -> Form -> Form
transform t f = transform' t [] f

transform' :: Term -> [Symbol] -> Form -> Form
transform'  fun s (Atom (Pred p ts))  	= Atom $ Pred p (map (transformT fun s) ts)
transform'  fun s (And fs) 				= And  $ Set.map (transform' fun s) fs
transform'  fun s (Or fs) 				= Or   $ Set.map (transform' fun s) fs
transform'  fun s (Equiv f1 f2) 		= Equiv ((transform' fun s) f1) ((transform' fun s) f2)
transform'  fun s (Not f) 				= Not $ (transform' fun s) f 
transform'  fun s (Exists (Bind s2 f)) 	= Exists $ Bind s2 ((transform' fun (s2:s)) f) 
transform'  fun s (ForAll (Bind s2 f)) 	= ForAll $ Bind s2 ((transform' fun (s2:s)) f) 

transformT :: Term -> [Symbol] -> Term -> Term
transformT f ss var@(Var x) = 
	if elem x ss then var else
		sub ("X" ::: Variable) (Var x) f 
			
transformT f ss fun@(Fun g ts)  =
	sub ("X" ::: Variable) (Fun g (map (transformT f ss) ts)) f



-------------------------------------------------------------------------------
	
