
module Main where


import IO
import System
import System.Environment
import System.Directory
import Maybe
import Data.List hiding ( stripPrefix )
import Timeout
import Infinox


main = supermain

--tries four different methods in sequence,
--stopping as soon as one is successful.
--default timeout is one hour per method.
--default time limit for eprover is 2 seconds.
--these can be changed using the flags -timeout=n and -elimit=n.
--(n is in seconds in both cases)
--the flag -v will print comments on the progress of the
--program to stdout.
supermain = do
  args <- getArgs
  let 
	file' = getfile args
	t     = gettimeout args
	lim   = getlimit args
	vb   = elem "-verbose" args || elem "-v" args || elem "verbose" args || elem "v" args
  if args == [] then do
	putStrLn "No input file specified!"			
   else do
	 let	
		file = if file' == "" then head args else file'
	 	inputs1 = Inputs lim False False True False True vb Nothing -- r only
		inputs2 = Inputs lim False False True True True vb Nothing  -- r and p
		inputs3 = Inputs lim True False True True True vb Nothing   -- zoom, r and p
		inputs4 = Inputs lim True False True False False vb Nothing   -- r, oni
	 tryAll t file [inputs1,inputs2,inputs3,inputs4]		

		
tryAll _ _ [] = putStrLn "Gave up!"
tryAll t file (x:xs) = do
	result <- mapmap Nothing ((timeout (t*(10^6))) . (disprove x)) [file]
	case result of
		[([],_)]	-> tryAll t file xs
		[(r,_)] 	-> do putStrLn (show r)
                                      putStrLn "+++ NO FINITE (COUNTER-)MODEL"
		
			
mapmap o f [] = return []
mapmap o f (x:xs) = do
	let
		dir = (filter ( (not . (flip elem) ['/','.','-',' '])) x) ++ "_TEMP"
	r <- f x
	case r of
		Nothing		-> do
			maybeAppendFile o ( x ++ ": TIMEOUT!\n")	
			tryRemoveDirectoryRecursive dir
			mapmap o f xs
		Just r		-> do
			rs <- mapmap o f xs
			return (r:rs)
	where
		maybeAppendFile Nothing _  =  return ()
		maybeAppendFile (Just f) x = appendFile f x
		tryRemoveDirectoryRecursive d = do
			rd <- try (removeDirectoryRecursive d)
			case rd of
				Left a -> tryRemoveDirectoryRecursive d
				_	   -> return ()


 
defaultlimit   = 2
defaulttimeout = 3600 


stripPrefix :: Eq a => [a] -> [a] -> Maybe [a]
stripPrefix [] ys = Just ys
stripPrefix (x:xs) (y:ys)
 | x == y = stripPrefix xs ys
stripPrefix _ _ = Nothing
	

getarg xs args = case mm args xs of
	[]		-> ""
	(arg:_)	-> arg
	where
	mm args [] = []
	mm args (x:xs) = mapMaybe (stripPrefix x) args ++
		mm args xs
		
getfile args 	= getarg ["-f=", "f=", "-file=", "file="] args

getlimit args 	= case getarg ["-elimit=", "elimit="] args of
	[]	-> defaultlimit
	lim	-> read lim
	
gettimeout args = case getarg ["-timeout=", "timeout="] args of
	[] -> defaulttimeout
	to -> read to
	
