module Util where

import Form
import System
import IO
import Data.List
import ParseProblem
import qualified Data.Set as Set
import System.Directory
import System.Process



--Call eprover on the input file
eprove :: FilePath  -> Int -> IO Bool
eprove f n  = do
	let 
		limits = "--cpu-limit=" ++ (show n) ++ " " 
		resultfile = f ++ "_result"
	system $ "eprover --tstp-in --tstp-out -tAuto -xAuto --output-level=0 " ++ limits
	 		++ f ++ " > " ++ resultfile	
	h <- openFile resultfile ReadMode	
	r <- hGetContents h					
	let 
		ls = lines r 	
		result = ls !! 1 == "# Proof found!" && ls !! 2 == "# SZS status Theorem" in
	 		if (result == result) then do   -- to close h properly without losing the result.... 
	 			hClose h 
	 			removeFile resultfile
	 			return result
	 		 else error "this never happens" 


--checks if the problem in the input file has a model 
--that can be found by Paradox in less than 2 seconds.
finiteModel :: FilePath -> IO Bool
finiteModel f = do
	system $ "paradox --time 2 " ++ f ++  " > presult" ++ f 
	c <- readFile $ "presult" ++ f 
	putStr $ show (last (lines c))
	system $ "rm presult" ++ f 
	return $ (last (lines c)) /= "+++ RESULT: Timeout"
	
	
-------------------------------------------------------------------------------


-- Conjecture formulas

--check if fun is not injective and surjective, using predicate r
checkNotInjOntoR :: Term -> Form -> Form
checkNotInjOntoR fun r = 

	exist ( vs ++ vs2) (

		(forAll x  reflr)			-- forAll x : r(X,X)
		
	/\
	
		(forAll x (forAll y (		-- forAll x, y : r(X,Y) ==> r(Y,X)
			nt r' \/ ryx
		)
		))
	
	/\
		(exists x (exists y
			( (nt r') /\ (eqfuns r' fun'))))		-- exist X,Y : ~r(X,Y) & r(f(X),f(Y))
	/\
	(
			forAll a (	exists z ( 					-- forAll A exists Z : r(f(Z), A)
				predapply r' fz (Var a)))

	))

			where
			x	 = ("X" ::: Variable)
			y	 = ("Y" ::: Variable)
			z	 = ("Z" ::: Variable)
			a	 = ("A" ::: Variable)
			vs   = genVars n "S"
			vs2  = genVars n2 "T"
			n2   = predstars r
			n    = stars fun
			fun' = applyStar fun vs
			r'   = applyPredStar r vs2
			reflr = subp1 y (Var x)  r' 
			ryx  = subp1 z (Var x) (subp1 x (Var y) (subp1 y (Var z) r' ) ) 
			fz   = sub x (Var z) fun'
	
		
--check if fun is not injective and surjective, using predicate r, and limiting predicate p
checkNotInjOntoPR :: Term -> Form -> Form -> Form
checkNotInjOntoPR fun p r = 
		exist (vs ++ vs2 ++ vs3) (
		(forAll ("X" ::: Variable)				--forAll X : p(X) ==> r(X,X) - ober. av f
			(reflr \/ nt p' ))
	/\
		forAll  ("X" ::: Variable) (			--forAll X : p(X) ==> p(f(X)) - ober. av r
			pfx \/ nt p' )
	/\
		(exists ("X" ::: Variable) (exists ("Y" ::: Variable)
			(p' /\ py /\ nt r' /\ (eqfuns r' fun'))))		-- exist X,Y : ~r(X,Y) & r(f(X),f(Y))
		
	/\
		forAll  ("Y" ::: Variable) (				
			nt py \/ (
		
			exists  ("X" :::	Variable) (	--p(X) /\ p(Y) ==> (r(f(X),f(Y)) ==> r(X,Y))
				p' /\ ( rfxy \/ ryfx)				
	))))
				
	where
		vs    = genVars (stars fun) "S"
		vs2   = genVars (predstars p) "T"
		vs3   = genVars (predstars r) "U"
		fun'  = applyStar fun vs
		r'    = applyPredStar r vs3
		p'    = applyPredStar p vs2
		reflr = subp1 ("Y" ::: Variable) (Var ("X" ::: Variable))  r' 
		py    = subp1 ("X" ::: Variable) (Var ("Y" ::: Variable))  p' 
		pfx   = subp1 ("X" ::: Variable) fun' p' 
		rfxy  = subp1 ("X" ::: Variable) fun' r' 
		ryfx  = subp1 ("X" ::: Variable) (Var ("Y" ::: Variable))  (subp1 ("Y" ::: Variable) fun' r' ) 

	
--check if fun is injective and not surjective, using predicate r	
checkInjOntoR :: Term -> Form -> Form
checkInjOntoR fun r = 
	exist ( vs ++ vs2) (		
		reflr  		
		/\		
		(forAll ("X" ::: Variable) (forAll ("Y" ::: Variable)
			( r' \/ nt (eqfuns r' fun'))))
		/\
		((	exists ("A" ::: Variable)  (forAll ("Z" ::: Variable) ( nt $			
			predapply r' fz (Var ("A" ::: Variable))))
		)
			\/ (	exists ("A" ::: Variable)  (forAll ("Z" ::: Variable) ( nt $			
			predapply r' (Var ("A" ::: Variable)) fz )))))
				
	where
			vs   = genVars n "S"
			vs2  = genVars n2 "T"
			n2   = predstars r
			n    = stars fun
			fun' = applyStar fun vs
			r'	 = applyPredStar r vs2
			reflr = subp1 ("Y" ::: Variable) (Var ("X" ::: Variable))  r' 
			fz   = sub ("X" ::: Variable) (Var ("Z" ::: Variable)) fun'
			
--check if fun is injective and not surjective, using predicate r and limiting predicate p			
checkInjOntoPR :: Term -> Form -> Form  -> Form
checkInjOntoPR fun p r =
	exist (vs ++ vs2 ++ vs3) (
	

		(forAll ("X" ::: Variable)				--forAll X : p(X) ==> r(X,X) - ober. av f
			(reflr \/ nt p' ))
	/\
		forAll  ("X" ::: Variable) (			--forAll X : p(X) ==> p(f(X)) - ober. av r
			pfx \/ nt p' )
	/\
		((exists ("Y" ::: Variable) (
			py /\ forAll  ("X" ::: Variable) (	--Exists y : p(y) /\ forAll x : p(X) ==> ~r(f(X),Y)
				nt rfxy  \/ nt p' )))		
		\/										-- \/
				(exists ("Y" ::: Variable) (	--Exists y : p(y) /\ forAll x : p(X) ==> ~r(Y,f(X))
			py /\ forAll  ("X" ::: Variable) (
				nt ryfx  \/ nt p' ))))
		
	/\
		forAll  ("X" ::: Variable) (				--forAll x : forAll y: 
			forAll  ("Y" :::		 Variable) (	--p(X) /\ p(Y) ==> (r(f(X),f(Y)) ==> r(X,Y))
				(r'  \/ nt rfxfy ) \/ nt ( p' /\ py))))

	where
		vs    = genVars (stars fun) "S"
		vs2   = genVars (predstars p) "T"
		vs3   = genVars (predstars r) "U"
		fun'  = applyStar fun vs
		r'    = applyPredStar r vs3
		p'    = applyPredStar p vs2
		reflr = subp1 ("Y" ::: Variable) (Var ("X" ::: Variable)) r'
		py    = subp1 ("X" ::: Variable) (Var ("Y" ::: Variable)) p'
		pfx   = subp1 ("X" ::: Variable) fun' p'
		rfxy  = subp1 ("X" ::: Variable) fun' r'
		ryfx  = subp1 ("X" ::: Variable) (Var ("Y" ::: Variable))  (subp1 ("Y" ::: Variable) fun' r' ) 
		rfxfy = predapply r' fun' (sub ("X" ::: Variable) (Var ("Y" ::: Variable)) fun')
	
{-
testt = Fun ("f" ::: FunArity 2) [Var ("X" ::: Variable), Var ("*" ::: Variable)]
testp = Atom $ Pred ("p" ::: PredArity 2) [Var ("*" ::: Variable), Var ("X" ::: Variable)]
testr =  Atom $ Pred ("r" ::: PredArity 3) [Var ("*" ::: Variable), Var ("X" ::: Variable), Var ("Y" ::: Variable)]
-}

--forAll X : p(X) ==> r(X,X)
checkPimpliesRef :: Form -> Form -> Form 	
checkPimpliesRef p r = 
	exist (vs ++ vs2) (		
		(forAll ("X" ::: Variable)			
			(reflr \/ nt p' )))
	where
			vs   = genVars (predstars p) "T"
			vs2   = genVars (predstars r) "U"
			p'    = applyPredStar p vs
			r'    = applyPredStar r vs2
			reflr = subp1 ("Y" ::: Variable) (Var ("X" ::: Variable)) r'

--forAll X : p(X) ==> p(f(X))			
checkFClosedUnderP :: Term -> Form -> Form	
checkFClosedUnderP fun p =
	exist (vs ++ vs2) (		
		forAll  ("X" ::: Variable) (		
			pfx \/ nt p' ))
	where
			vs   = genVars (predstars p) "T"
			vs2  = genVars (stars fun) "S"
			p'   = applyPredStar p vs
			fun'  = applyStar fun vs2
			pfx   = subp1 ("X" ::: Variable) fun' p'
			
			
-------------------------------------------------------------------------------
		
checkInj :: Term -> Form -> Form
checkInj fun r = 	exist ( vs ++ vs2) (
		(forAll ("X" ::: Variable) (forAll ("Y" ::: Variable)
			( r' \/ nt (eqfuns r' fun')))))
		where
			vs   = genVars n "S"
			vs2  = genVars n2 "T"
			n2   = predstars r
			n    = stars fun
			fun' = applyStar fun vs
			r'	 = applyPredStar r vs2
			
-------------------------------------------------------------------------------
		
		
predapply r f1 f2 = subpred r (Var ("X" ::: Variable)) (Var ("Y" ::: Variable)) f1 f2		
		
exist [] f = f
exist (x:xs) f = exists x $ exist xs $ f

isVar (x ::: Variable) = True
isVar _ = False
		
--"R(f(x),f(y))"
eqfuns r fun = predapply r fun (sub ("X" ::: Variable) (Var ("Y" ::: Variable)) fun)
			 
stars (Var ("*" ::: Variable))  = 1
stars (Var _)					= 0
stars (Fun f ts) = foldl (+) 0 (map stars ts)

predstars (Atom (Pred p ts)) = foldl (+) 0 (map stars ts) 
predstars (And ps) = foldl (+) 0 (map predstars (Set.toList ps))

		
applyStar (Var ("*" ::: Variable)) (v:vs) = Var v
applyStar (Var x) _ = Var x
applyStar (Fun f vs) ss = Fun f (applyStars vs ss) 

applyPredStar (Atom (Pred p ts)) vs = Atom (Pred p (applyStars ts vs))
applyPredStar (And ps) vs = And $ Set.fromList $ applytoAll (Set.toList ps) vs

applytoAll [] [] = []
applytoAll (x:xs) vs = (applyPredStar x vs : (applytoAll xs (drop (predstars x) vs)))

applyStars [] _ = []
applyStars (v:vs) ss = (applyStar v ss): (applyStars vs (drop (stars v) ss))

genVars 0 s = []
genVars n s = ((s++(show n)) ::: Variable) : genVars (n-1) s


			
dropLast xs = take (length xs -1) xs
	
deep ((Var ("X" ::: Variable) )) = False
deep ((Var ("*" ::: Variable) )) = True
deep ((Fun f ts)) = not $ allVars ts

allVars []			 = True
allVars ((Var _):ts) = allVars ts
allVars _ = False

members :: ([Term],Form) -> [(Term,Form)] -> Bool
members ([],_) _ = True
members ((t:ts),r) terms = (elem (t,r) terms && members (ts,r) terms) || t ==  Var ("*" ::: Variable)

eqterms ([],_) ([],_) = True
eqterms ([],_) _ = False
eqterms _ ([],_) = False
eqterms (((Var x):ts1),s1) ((Var y):ts2,s2) = x == y && s1 == s2 && eqterms (ts1,s1) (ts2,s2)
eqterms ((Var _):_,_s) _ = False 
eqterms ((Fun f1 ts):tts,s1) ((Fun f2 ts2):tts2,s2) = 
	if f1 == f2 && eqterms (ts,s1) (ts2,s2) then eqterms (tts,s1) (tts2,s2) else False
	
eqterms _ _ = False
-------------------------------------------------------------------------------

--translating to fof-form.

form2axioms [] = ""
form2axioms fs = form2axioms' fs 0
form2axioms' [] _ = ""
form2axioms' (f:fs) n = form2axiom f n ++ form2axioms' fs (n+1)

form2axiom f n = 	"fof(" ++ "a_" ++ (show n) ++ ", " ++ "axiom" ++ 
		", (" ++ show f ++ "))."

form2conjecture :: Int -> Form -> String		
form2conjecture n f = 	"fof(" ++ "c_" ++ (show n) ++ ", " ++ "conjecture" ++ 
		", (" ++ show f ++ "))."


-------------------------------------------------------------------------------							 
--substitution
sub x t (Var y) = if x == y then t else Var y
sub x t (Fun s ts) = Fun s (map (sub x t) ts)

subp1 x y (Atom (Pred p ts))  = Atom (Pred p (map (sub x y) ts))
subp1 x y (And ps) = And $ Set.map (subp1 x y) ps
subp1 x y (Not p) = Not (subp1 x y p)

subpred (Atom (Pred p ts)) x y f1 f2 = Atom (Pred p (subp (subp ts y f2) x f1))
--subpred (And ps) x y f1 f2 = And $ Set.map (subpred 
subp [] _ _ = []
subp (t:ts) x y = if t == x then (y:(subp ts x y)) else (t:(subp ts x y))


-------------------------------------------------------------------------------


getForms :: Problem ->  [Form]
getForms [] = []
getForms ((Input k t f):fs) = 
	case k of 
		Conjecture -> (nt f):(getForms fs)
		_		   ->	(f:(getForms fs))
		
getFuns []  = []
getFuns (fun@(Fun f ts):tts)  = if allCons ts then getFuns tts else 
	(fun:(getFuns tts))
	where
		allCons [] = True
		allCons (Var _ : _)         = False
		allCons ((Fun _ ts):fs)     = allCons ts && allCons fs

getFuns (_:ts)  = getFuns ts 

getFunSymbols :: [Term] -> [Symbol]
getFunSymbols [] = []
getFunSymbols ((Fun f ts):tts) = (f:(getFunSymbols tts))

--collects all the variables of a term
getVars :: Term -> [Symbol]
getVars (Var x) = [x]
getVars (Fun _ ts) = nub $ concat $ map getVars ts

equality = Atom (Pred ("=" ::: (PredArity 2)) 
	[Var ("X" ::: Variable), Var ("Y" ::: Variable)])
	
getPreds ps = nub $ concat $ map getPreds' $ getForms ps	

getPreds' (Atom (Pred p ts)) = [p]
getPreds' (And ps) = concatMap getPreds' $ Set.toList ps
getPreds' (Or ps) = concatMap getPreds' $ Set.toList ps
getPreds' (Not p) = getPreds' p
getPreds' (Equiv f1 f2) = getPreds' f1 ++ getPreds' f2
getPreds' (ForAll (Bind _ f)) = getPreds' f
getPreds' (Exists (Bind _ f)) = getPreds' f






