module Zoom where
  

import Form
import IO
import ParseProblem
import qualified Data.Set as Set
import List
import Util
import System
import System.Directory
import Timeout


zoom :: FilePath -> [Form] -> Int -> IO [Form]
zoom dir fs n  = zoom' dir fs (shrink fs) 0 n

zoom' :: FilePath -> [Form] -> [[Form]] -> Int -> Int -> IO [Form]
zoom' dir best [] _ _ = return best
zoom' dir best fs n m = do 
	(f,count) <- smallestUnsat dir best fs n m
	if f == best || count >= m then return f else 
	 zoom' dir f (shrink f) count m

smallestUnsat _ best [] count _ = return (best,count)
smallestUnsat dir best (f:fs) n m = do
	h <- openFile (dir ++ "zoom") WriteMode
	hPutStr h $ (form2axioms f)
	hClose h
	b <- timeout (5*(10^6)) (finiteModel (dir ++ "zoom"))
	removeFile (dir ++ "zoom")
	case b of
		Nothing		-> smallestUnsat dir best fs (n+1) m
		Just True 	-> smallestUnsat dir best fs (n+1) m
		Just False  -> return (f,n+1)


sortforms [] = []
sortforms (f:fs) = insertform f (sortforms fs)

insertform f [] = [f]
insertform f (f':fs) 
	|smallest f f' == f = (f:f':fs) 
	|otherwise = f':(insertform f fs)

smallest x y = if (Set.size (symbols x)) < (Set.size (symbols y)) then x else y

	
shrink :: [a] -> [[a]]
shrink xs = removeChunks xs
 where
  removeChunks xs = rem (length xs) xs
   where
    rem 0 _  = []
    rem 1 _  = [[]]
    rem n xs = xs1
             : xs2
             : ( [ xs1' ++ xs2 | xs1' <- rem n1 xs1, not (null xs1') ]
           `ilv` [ xs1 ++ xs2' | xs2' <- rem n2 xs2, not (null xs2') ]
               )
     where
      n1  = n `div` 2
      xs1 = take n1 xs
      n2  = n - n1
      xs2 = drop n1 xs
  
      []     `ilv` ys     = ys
      xs     `ilv` []     = xs
      (x:xs) `ilv` (y:ys) = x : y : (xs `ilv` ys)




