/*
 Copyright 2006, 2007, 2008 Hao Xu
 ertranne@hotmail.com
 
 This file is part of OSHL-S.

 OSHL-S is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 OSHL-S is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with Foobar; if not, write to the Free Software
 Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

package jfp.datatype;

import java.util.ArrayList;

import jfp.type.TList;
import jfp.type.TVar;
import jfp.type.Util;

/**
 * Use only when all the elements are immutable.
 */
public class List<T> {

	java.util.List<T> l = new java.util.ArrayList<T>();

	private TList type = new TList(new TVar(this.toString()));

	public List(java.util.List<T> l) {
		super();
		this.l = l;
	}

	public List(
	T hd, 
	List<T> tl) {
		this.l = new ArrayList<T>(tl.l);
		l.add(0, hd);
		this.type = new TList(Util.type(hd));
	}

	
	public List() {
	}

	public List(Object componentType, java.util.List<T> name) {
		this(name);
		this.type = new TList(componentType);
	}

	public T hd() {
		return l.get(0);
	}

	public List<T> tl() {
		return new List<T>(l.subList(1, l.size()));
	}

	public boolean isEmpty() {
		return l.isEmpty();
	}

	/**
	 * Use only when all the elements are immutable.
	 */
	public List<T> copy() {
		return this;
		// compute copies for all the elements in list l.
		//
		// return new List<T>(new ArrayList<T>(copy all of l));
	}

	public Object getType() {
		return type;

	}

	public int arity() {
		return 1;
	}

	public Integer size() {
		return l.size();
	}

	@Override
	public int hashCode() {
		final int PRIME = 31;
		int result = 1;
		result = PRIME * result
				+ (this.isEmpty() ? 0 : hd().hashCode() + tl().hashCode());
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		final List other = (List) obj;
		if (this.isEmpty() && other.isEmpty())
			return true;
		else if (this.size() != other.size())
			return false;
		else
			return this.hd().equals(other.hd()) && this.tl().equals(other.tl());
	}

	@Override
	public String toString() {
		if (this.l.isEmpty())
			return "[]";
		StringBuffer obj = new StringBuffer("[");
		obj.append(l.get(0));
		for (int i = 1; i < l.size(); i++) {
			obj.append(", ");
			obj.append(l.get(i));
		}
		obj.append("]");
		return obj.toString();
	}


}
