/*
 Copyright 2006, 2007, 2008 Hao Xu
 ertranne@hotmail.com
 
 This file is part of OSHL-S.

 OSHL-S is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 OSHL-S is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with Foobar; if not, write to the Free Software
 Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
package jfp.datatype;

import jfp.type.Util;

public class Tuple<U, V extends Tuple> {

    private U hd;
    private V tl;
    protected int arity;

    protected Tuple() {
        
    }
    
    public Tuple(U hd, V tl) {
        super();
        this.hd = hd;
        this.tl = tl;
        if (this.tl().arity() == 0) {
            this.arity = 1;
        } else {
            this.arity = tl().arity() + 1;
        }
    }

    public Object[] toArray() {
        Tuple tuple = this;
        Object a[] = new Object[this.arity()];
        for (int i = 0; i < this.arity(); i++) {
            a[i] = tuple.hd();
            tuple = tuple.tl();
        }
        return a;
    }

    public U hd() {
        return hd;
    }

    public V tl() {
        return tl;
    }

    @Override
    public String toString() {
        if (tl().arity() == 0) {
            return "(" + hd().toString() + ")";
        } else {
            return "(" + hd().toString() + "," + tl().toStringTail() + ")";
        }
    }

    public String toStringTail() {
        if (tl().arity() == 0) {
            return hd().toString();
        } else {
            return hd().toString() + "," + tl().toStringTail();
        }
    }

    public Object getType() {
        if (this.tl().arity() != 0) {
            return new Tuple(Util.type(this.hd()), (Tuple) Util.type(this.tl()));
        } else {
            return new Tuple(Util.type(this.hd()), new EmptyTuple());
        }
    }

    public int arity() {
        return arity;
    }

    @Override
    public int hashCode() {
        return this.hd().hashCode() ^ ((this.tl == null) ? 0 : this.tl.hashCode());
    }

    public boolean equals(Object o) {
        return o instanceof Tuple && this.arity() == ((Tuple) o).arity() && this.hd().equals(((Tuple) o).hd()) && (this.tl() == null || this.tl.equals(((Tuple) o).tl()));
    }


}
