/*
 Copyright 2006, 2007, 2008 Hao Xu
 ertranne@hotmail.com
 
 This file is part of OSHL-S.

 OSHL-S is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 OSHL-S is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with Foobar; if not, write to the Free Software
 Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
package jfp.dynamic;

import java.util.Arrays;

import java.util.logging.Level;
import java.util.logging.Logger;
import jfp.dynamic.east.VisitorParameters;
import jfp.environment.GenericEnvironment;

public abstract class ExtensibleAST extends AbstractAST {

    ExtensibleASTDelegate delegate;
    ExtensibleAST ext;
    String[] def;

    public ExtensibleAST(ExtensibleASTDelegate del, String... def) {
        this.delegate = del;
        this.def = def;
    }

    /**
     * auto-loading constructor
     * @param def
     */
    public ExtensibleAST(String... def) {
        try {
            String name = this.getClass().getCanonicalName();
            String delegateName = name + "Delegate";
            ExtensibleASTDelegate ead = (ExtensibleASTDelegate) Class.forName(delegateName).newInstance();
            this.delegate = ead;
            this.def = def;
        } catch (InstantiationException ex) {
            Logger.getLogger(ExtensibleAST.class.getName()).log(Level.SEVERE, null, ex);
        } catch (IllegalAccessException ex) {
            Logger.getLogger(ExtensibleAST.class.getName()).log(Level.SEVERE, null, ex);
        } catch (ClassNotFoundException ex) {
            Logger.getLogger(ExtensibleAST.class.getName()).log(Level.SEVERE, null, ex);
        }
    }

    public Object visitedBy(String visitor, GenericEnvironment<String, Object> env, 
            ExtensibleAST[] extensibleAST, GenericEnvironment<String, Object>[] interface1
            ) throws ASTException {
        return this.visitedBy(visitor, new VisitorParameters(env, extensibleAST, interface1));
    }

    public Object visitedBy(String visitor, GenericEnvironment<String, Object> env) throws ASTException {
        return this.visitedBy(visitor, new VisitorParameters(env));

    }


    public boolean defined(String visitor) {
        return Arrays.asList(def).contains(visitor);
    }

    public Object visitedBy(String visitor, GenericEnvironment<String, Object> env, ExtensibleAST astn, GenericEnvironment<String, Object> envns) throws ASTException {

            return this.visitedBy(visitor, new VisitorParameters(env, astn, envns));
    }


 
    public Object visitedBy(String visitor, VisitorParameters vp) throws ASTException {
        if (!defined(visitor)) {
            return this.ext.visitedBy(visitor, vp);
        } else if (delegate != null) {
            return delegate.visitedBy(visitor, this, vp);
        } else {
            throw new ASTException();
        }
    }

    @Override
    public ExtensibleAST[] getChildNode() {
        return null;
    }

}
