/*
 Copyright 2006, 2007, 2008 Hao Xu
 ertranne@hotmail.com
 
 This file is part of OSHL-S.

 OSHL-S is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 OSHL-S is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with Foobar; if not, write to the Free Software
 Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
package jfp.type;

import jfp.datatype.Tuple;
import jfp.dynamic.ExtensibleTypeAST;

public class TFunction<U,V> extends ExtensibleTypeAST {

	protected U domainType;
	protected V rangeType;

	public TFunction(U domainType, V rangeType) {
		super(new TFunctionDelegate());
		this.domainType = domainType;
		this.rangeType = rangeType;
	}

	public U getDomainType() {
		return domainType;
	}

	public void setDomainType(U domainType) {
		this.domainType = domainType;
	}

	public V getRangeType() {
		return rangeType;
	}

	public void setRangeType(V rangeType) {
		this.rangeType = rangeType;
	}

	public TFunction curry() {
		TFunction type = this;
		if(this.domainType instanceof Tuple) {
			Tuple t = (Tuple) this.domainType;
			Object[] p = t.toArray();
			type = new TFunction(p[p.length-1],this.rangeType);
			for(int i=p.length-2;i>=0;i--) {
				type=new TFunction(p[i], type);
			}
			
		}
		return type;
	}

	@Override
	public String toString() {
		return this.domainType.toString()+" -> "+this.rangeType.toString();
	}
	
	public boolean equals(Object type) {
		if(type instanceof TFunction) {
			TFunction func = (TFunction) type;
			return this.domainType.equals(func.domainType) && this.rangeType.equals(func.rangeType);
		}
		return false;
		
	}
	
	public int hashCode() {
		return this.toString().hashCode();
	}

}
