/*
 Copyright 2006, 2007, 2008 Hao Xu
 ertranne@hotmail.com
 
 This file is part of OSHL-S.

 OSHL-S is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 OSHL-S is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with Foobar; if not, write to the Free Software
 Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
package jfp.type;

import java.io.IOException;
import java.util.*;
import java.util.logging.Level;
import java.util.logging.LogRecord;
import java.util.logging.Logger;

import jfp.datatype.EmptyTuple;
import jfp.datatype.List;
import jfp.datatype.ListTuple;
import jfp.datatype.Pair;
import jfp.datatype.Tuple;
import jfp.dynamic.Term;
import jfp.dynamic.ASTException;
import jfp.dynamic.ExtensibleAST;
import jfp.dynamic.TokenStream;
import jfp.environment.TypeVariableEnvironmentInterface;

public class Util {

    public static Object type(Object value) {
        Object valueType = null;
        if (value instanceof List) {
            valueType = ((List) value).getType();
        } else if (value != null) {
            valueType = value.getClass();
        }
        return valueType;
    }
    public static boolean on = true;

    public static Object getTypeInstance(Object value, TypeVariableEnvironmentInterface env) {
        Object valueType;
        boolean off = !on;
        if (on = true) {
            logger.setLevel(Level.OFF);
            on = false;
        }
        try {
            valueType = ((ExtensibleAST) wrap(value)).visitedBy(
                    Term.INSTANTIATE, env);
        } catch (ASTException e) {
            e.printStackTrace();
            return null;
        }
        if (!off) {
            logger.setLevel(Level.ALL);
            on = true;
        }
        logger.log(new LogRecord(DEBUG, "instantiate: " + value + " => " + valueType));
        return valueType;
    }
    static Logger logger = Logger.getAnonymousLogger();
    public static final Level DEBUG = Level.parse(Integer.toString(Integer.MIN_VALUE));

    public static Object typeInstance(Object val, TypeVariableEnvironmentInterface env) {
        return getTypeInstance(type(val), env);
    }

    public static boolean unifiable(Object domainType, Object valueType,
            TypeVariableEnvironmentInterface env) throws ASTException {
        TypeVariableEnvironmentInterface vp = env;
        TypeVariableEnvironmentInterface vp2 = env;
        return unifiable(domainType, vp, valueType, vp2);

    }

    public static Object parse(String string, Map<String, TVar> vMap)
            throws ClassNotFoundException, IOException {
        TokenStream tk = new TokenStream(string);
        return TypeParser.parseExp(tk, vMap);
    }


    public static java.util.Set<TVar> definedTypeVariables(Object type) {
        java.util.Set<TVar> tvs = new java.util.HashSet<TVar>();
        if (type == null) {
            return tvs;
        } else if (type instanceof TVar) {
            tvs.add((TVar) type);
        } else if (type instanceof TFunction) {
            tvs.addAll(definedTypeVariables(((TFunction) type).domainType));
            tvs.addAll(definedTypeVariables(((TFunction) type).rangeType));
        } else if (type instanceof TList) {
            tvs.addAll(definedTypeVariables(((TList) type).getElementType()));
        } else if (type instanceof Tuple) {
            tvs.addAll(definedTypeVariables(((Tuple) type).hd()));
            tvs.addAll(definedTypeVariables(((Tuple) type).tl()));
        } else if (type instanceof TDisjointUnion) {
            for (Object v : ((TDisjointUnion) type).typeParameters) {
                tvs.addAll(definedTypeVariables(v));
            }
        }
        return tvs;
    }

    public static boolean unifiable(Object domainType,
            TypeVariableEnvironmentInterface vp, Object valueType,
            TypeVariableEnvironmentInterface vp2) throws ASTException {
        logger.log(new LogRecord(DEBUG, "unify: " + domainType + " " + valueType));
        if (domainType == valueType) {
            return true;// only when two objects are the same, even if two
        // variables have the same name, they're treated as
        // different variables;
        } else if (domainType instanceof Class && valueType instanceof TFunction || domainType instanceof TFunction && valueType instanceof Class) {
            return false; // delay type checking to actual function
        // application
        } else if (domainType instanceof Class && valueType instanceof TList || domainType instanceof TList && valueType instanceof Class) {
            return false; // delay type checking to actual function
        // application
        } else if (domainType instanceof TVar) {
            return unifyTVar((TVar) domainType, valueType, vp);
        } else if (valueType instanceof TVar) {
            return unifyTVar((TVar) valueType, domainType, vp);
        } else if (domainType instanceof Class && valueType instanceof Class) {
            return (Boolean) wrap(domainType).visitedBy(Term.UNIFY, vp,
                    wrap(valueType), vp2);
        }
        return (Boolean) wrap(domainType).visitedBy(Term.UNIFY, vp,
                wrap(valueType), vp2);
    // return false;
    }

    public static ExtensibleAST wrap(Object type) {
        if (type instanceof Tuple) {
            return new TTuple((Tuple) type);
        } else {
            return (ExtensibleAST) type;
        }
    }

    public static Object unwrap(ExtensibleAST ast) {
        ExtensibleAST type = ast;
        if (type instanceof TTuple) {
            return ((TTuple) type).getTuple();
        } else {
            return type;
        }

    }

    static boolean unifyTVar(TVar dt, Object vt, TypeVariableEnvironmentInterface vp) throws ASTException {
        return unifyTVarWithOtherType(dt, vp, vt, vp);
    }

    /**
     * currently not in use
     * @param domainType
     * @param vp
     * @param valueType
     * @param vp2
     * @return
     * @throws jfp.dynamic.ASTException
     */
    static boolean unifyTVar(TVar domainType, TypeVariableEnvironmentInterface vp,
            Object valueType, TypeVariableEnvironmentInterface vp2) throws ASTException {
        if (valueType instanceof ExtensibleAST) {
            return (Boolean) domainType.visitedBy(Term.UNIFY, vp,
                    (ExtensibleAST) valueType, vp2);
        } else {
            if (domainType.constant) {
                return false;
            }
            Object deref = domainType.getNonNullReference(vp);
            if (deref.equals(domainType)) {
                Object type = Util.getTypeInstance(valueType, vp);
                domainType.setReference(type, vp);
                return true;
            } else {
                return Util.unifiable(deref, vp,
                        valueType, vp2);
            }

        }

    }

    public static boolean unifyTVarWithOtherType(TVar domainType,
            TypeVariableEnvironmentInterface vp, Object valueType, TypeVariableEnvironmentInterface vp2)
            throws ASTException {
        TVar tvar = (TVar) domainType;
        if (domainType.constant) {
            if (valueType instanceof TVar && ((TVar) valueType).constant) {
                return domainType.equals(valueType);
            } else {
                return false;
            }
        }
        Object deref = tvar.getNonNullReference(vp);
        if (deref.equals(domainType)) {
            Object type = Util.getTypeInstance(valueType, vp2);
            tvar.setReference(type, vp);
            return true;
        } else {
            return Util.unifiable(deref, vp, valueType,
                     vp2);
        }
    }

    public static Object unifyAll(java.util.List<ExtensibleAST> name,
            TypeVariableEnvironmentInterface env) throws ASTException {
        Object prev_type = null, current_type;
        for (ExtensibleAST ea : name) {
            current_type = ea.visitedBy(Term.TYPE, env);
            if (prev_type == null) {
                prev_type = current_type;
            } else {
                if (unifiable(prev_type, current_type, env)) {
                    prev_type = getTypeInstance(current_type, env);
                } else {
                    throw new TypeException();
                }
            }
        }
        if (prev_type == null) {
            prev_type = new TVar(env);
        }
        return prev_type;
    }

    public static Logger getLogger() {

        return logger;
    }

    public static void setLogger(Logger logger) {
        Util.logger = logger;
    }





    public static Object createIntersectionType(Object... param_type) {
        return new TIntersection(Arrays.asList(param_type));
    // return null;
    }
}
