/*
 Copyright 2006, 2007, 2008 Hao Xu
 ertranne@hotmail.com
 
 This file is part of OSHL-S.

 OSHL-S is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 OSHL-S is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with Foobar; if not, write to the Free Software
 Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
package logic.concurrent.data;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.List;
import java.util.SortedSet;

import logic.data.FunctionSymbol;
import logic.data.Term;

public class TermIterator {

	private static final int MAX_TERM_LENGTH = 10;

	private static final Term[] TERM_ARRAY = new Term[] {};

	static List<Term>[] terms = new List[MAX_TERM_LENGTH + 1];

	static List<Term>[] accu_terms = new List[MAX_TERM_LENGTH + 1];

	static int hd = 1;

	// static int d = 1;

	static {
		accu_terms[0] = new ArrayList<Term>();
	}

	// static SortedSet<FunctionSymbol> fs;

	private int ps;

	public TermIterator(int max) {
		super();
		this.max = max;
	}

	public static void addFunctionSymbol(FunctionSymbol f) {
		getFs().add(f);
	}

	public Term next_ml() {
		if (ps == 0)
			while (hd <= max)
				gen();
		if (p < accu_terms[max].size()) {
			return accu_terms[max].get(p++);
		} else {
			return null;
		}
	}

	private void gen() {
		terms[hd] = new ArrayList<Term>();
		accu_terms[hd] = new ArrayList<Term>();
		if (hd == 1) {
			// sort sfs
			sfs = getFs().toArray(new FunctionSymbol[0]);

			Arrays.sort(sfs);
		}
		for (FunctionSymbol f : sfs) {
			int n = f.arity;
			Term[] ts = new Term[n];

			if (n == 0) {
				if (hd == 1) {
					terms[hd].add(new Term(f, ts));
				} else {
					continue;
				}
			} else {
				rec(f, ts, 0, hd - 1);
			}
		}
		accu_terms[hd].addAll(accu_terms[hd - 1]);
		accu_terms[hd].addAll(terms[hd]);
		Collections.sort(accu_terms[hd], new TermComparator());
		hd++;
	}

	private boolean rec(FunctionSymbol fs, Term[] ts, int i, int max) {
		TermIterator ti;
		ti = new TermIterator(max - ts.length + i + 1);
		// each subterm is as least of length 1
		if (ti.max <= 0)
			return false;

		if (i == ts.length - 1) { // tail term
			for (;;) {
				Term t = ti.next();
				if (t != null) {
					ts[i] = t;
					Term[] st = new Term[ts.length];
					System.arraycopy(ts, 0, st, 0, ts.length);
					terms[hd].add(new Term(fs, st));
				} else {
					break;
				}
			}
		} else {
			for (;;) {
				Term t = ti.next_ml();
				if (t != null) {
					ts[i] = t;
					rec(fs, ts, i + 1, max - t.size());
				} else {
					break;
				}
			}
		}
		return true;
	}

	private Term next() {
		if (ps == 0)
			while (hd <= max)
				gen();
		if (ps < terms[max].size()) {
			return terms[max].get(ps++);
		} else {
			return null;
		}
	}

	private FunctionSymbol[] sfs;

	private int p = 0;

	private int max = 1;

	public static void main(String[] args) {
		TermIterator.addFunctionSymbol(new FunctionSymbol("f", 1, 'f'));
		TermIterator.addFunctionSymbol(new FunctionSymbol("g", 2, 'g'));
		TermIterator.addFunctionSymbol(new FunctionSymbol("h", 2, 'h'));
		for (char ch = 'a'; ch < 'f'; ch++)
			TermIterator.addFunctionSymbol(new FunctionSymbol("" + ch, 0, ch));
		TermIterator ti = new TermIterator(5);
		for (int i = 0; i < 100; i++)
			System.out.println(ti.next_ml());
		// System.out.println("The search depth is " + d);
		// for (int i = 0; i < d; i++) {
		// System.out.print(depth_terms[i].size() + "/");
		// }
		// System.out.println("...");
		// System.out.println(terms.size() + " terms generated as follows");
		// System.out.println(terms);
	}

	static SortedSet<FunctionSymbol> getFs() {
		return TermGenerator.fs;
	}
}
