/*
 Copyright 2006, 2007, 2008 Hao Xu
 ertranne@hotmail.com
 
 This file is part of OSHL-S.

 OSHL-S is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 OSHL-S is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with Foobar; if not, write to the Free Software
 Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
package logic.concurrent.data;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Comparator;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.SortedSet;
import java.util.TreeSet;

import jfp.datatype.Monad;
import jfp.datatype.Pair;
import jfp.datatype.Tuple;
import jfp.dynamic.ASTException;
import jfp.dynamic.ExtensibleTypeAST;
import jfp.type.TFunction;
import jfp.type.TVar;

import logic.concurrent.main.data.ProgramParameters;
import logic.concurrent.main.data.ProgramParametersDyn;
import logic.data.FunctionSymbol;
import logic.data.Term;
import logic.semantics.env.TypeEnv;

public class TermTypedIterator implements ProgramParameters {

    // TODO refactor this, currently this should be set to a global TypeEnv by
    // the constructor
    TypeEnv env;

    // static int d = 1;

    private int ps;
    // the reference to generating iterator
    TermTypedGenerator ti;
    private List<Term>[] accu_terms;
    private List<Term>[] terms;

    /**
     * create an iterator that iterates through term of type t
     * @param max the maximum lenght of term
     * @param t the type
     */
    public TermTypedIterator(int max, String t) {
        super();
        this.max = max;

        ti = TermTypedGenerator.tgtMap.get(t);
        accu_terms = ti.accu_terms;
        terms = ti.terms;
        //hd = ti.hd;
        env = ti.env;
        if (max > this.MAX_TERM_LENGTH) {
            System.err.println("ERROR: max term length reached.");
            System.out.println("SZS status GaveUp for "+ProgramParametersDyn.getFileName());
            System.exit(0);
        }

    }

    public Term next_ml() {
        if (p == 0) {
            while (ti.hd <= max) {
                ti.gen();
            }
        }
        if (p < accu_terms[max].size()) {
            return accu_terms[max].get(p++);
        } else {
            return null;
        }
    }

    public Term next() {
        if (ps == 0) {
            while (ti.hd <= max) {
                ti.gen();
            }
        }
//        System.out.println(max);
        if (ps < terms[max].size()) {
            return terms[max].get(ps++);
        } else {
            return null;
        }
    }

    public Term next_term() {
        return next();
    }

    private int p = 0;
    public int max = 1;

    public static void main(String[] args) {
        TypeEnv env = new TypeEnv();
        TVar tv = new TVar("a");
        TVar tvb = new TVar("b");
        TFunction tf2 = new TFunction(new Pair<TVar, TVar>(tv, tv),
                tv);
        TFunction tf = new TFunction(new Monad<TVar>(tvb), tv);
        // register basis tti
        TermTypedGenerator tti = new TermTypedGenerator("a", env);
        TermTypedGenerator ttb = new TermTypedGenerator("b", env);
        // bind environment
        env.bindType("f", tf);
        env.bindType("g", tf2);
        env.bindType("h", tf2);
        // add fs to basis tti
        tti.addFunctionSymbol(new FunctionSymbol("f", 1, 'f'));
        tti.addFunctionSymbol(new FunctionSymbol("g", 2, 'g'));
        tti.addFunctionSymbol(new FunctionSymbol("h", 2, 'h'));
        for (char ch = 'a'; ch < 'f'; ch++) {
            ttb.addFunctionSymbol(new FunctionSymbol("" + ch, 0, ch));
            env.bindType("" + ch, tvb);
        }
        TermTypedIterator ti = new TermTypedIterator(5, "a");
        for (int i = 0; i < 100; i++) {
            System.out.println(ti.next_ml());
        }
    // System.out.println("The search depth is " + d);
    // for (int i = 0; i < d; i++) {
    // System.out.print(depth_terms[i].size() + "/");
    // }
    // System.out.println("...");
    // System.out.println(terms.size() + " terms generated as follows");
    // System.out.println(terms);
    }


    public static HashMap<String, Integer> tlbCache = new HashMap<String, Integer>();
    /**
     * 
     * @param type
     * @return 0 if not bounded, 1 if bounded and length is one
     */
    public static int getTermLengthBound(String type) {
        if(tlbCache.containsKey(type)) {
            return tlbCache.get(type);
        }
        int termLengthBoundRec = getTermLengthBoundRec(type, new HashSet());
        tlbCache.put(type, termLengthBoundRec);
        return termLengthBoundRec;
    }

    public static int getTermLengthBoundRec(String type, Set<String> visited) {

        TermTypedGenerator tti = TermTypedGenerator.getTTIMap().get(type);
        TypeEnv env = tti.env;
        TFunction ftype;
        int max = 1;
        for (FunctionSymbol f : tti.getFs()) {
            if (f.arity > 0) {
                ftype = (TFunction) env.getType(f.name);
                Tuple t = (Tuple) ftype.getDomainType();
                while (t.arity() > 0) {
                    String vtype = ((TVar) t.hd()).getName();
                    if (visited.contains(vtype)) {
                        return Integer.MAX_VALUE;
                    }
                    HashSet<String> hs = new HashSet<String>();
                    hs.addAll(visited);
                    hs.add(type);
                    int l = getTermLengthBoundRec(vtype, hs);
                    if (l == Integer.MAX_VALUE) {
                        return Integer.MAX_VALUE;
                    }
                    max += l;

                    t = t.tl();
                }
            }
        }
        return max;
    }

}
