/*
 Copyright 2006, 2007, 2008 Hao Xu
 ertranne@hotmail.com
 
 This file is part of OSHL-S.

 OSHL-S is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 OSHL-S is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with Foobar; if not, write to the Free Software
 Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
package logic.data;

import java.util.Collection;
import java.util.Iterator;
import logic.syntax.TermParser;

/**
 *
 * @author Ertri
 */
public class EqualityAxioms {

    public static ClauseList gnerateEqualityAxioms(Collection<FunctionSymbol> fs, Collection<PredicateSymbol> ps) {
        ClauseList cl = new ClauseList();


        if (!TermParser.vsmap.containsKey("X")) {

            TermParser.addVariableSymbol("X");
        }
        if (!TermParser.vsmap.containsKey("Y")) {

            TermParser.addVariableSymbol("Y");
        }
        if (!TermParser.vsmap.containsKey("Z")) {

            TermParser.addVariableSymbol("Z");
        }
        PredicateSymbol p = TermParser.getEqualityPredicate();

        Clause c;
        c = new Clause();
        Term[] emptyTree = new Term[0];
        Term X = new Term(TermParser.vsmap.get("X"), emptyTree);
        Term Y = new Term(TermParser.vsmap.get("Y"), emptyTree);
        Term Z = new Term(TermParser.vsmap.get("Z"), emptyTree);
        c.add(new Literal(true, p, new Tree[]{X, X}));
        cl.add(c); // X=X
        c = new Clause();
        c.add(new Literal(false, p, new Tree[]{X, Y}));
        c.add(new Literal(true, p, new Tree[]{Y, X}));
        cl.add(c); // X=Y => Y=X
        c = new Clause();
        c.add(new Literal(false, p, new Tree[]{X, Y}));
        c.add(new Literal(false, p, new Tree[]{Y, Z}));
        c.add(new Literal(true, p, new Tree[]{X, Z}));
        cl.add(c); // X=Y & Y=Z => X=Z

        Term[] Xs, Ys;
        for (Iterator<FunctionSymbol> itr = fs.iterator(); itr.hasNext();) {
            FunctionSymbol f = itr.next();
            c = new Clause();
            if (f.arity > 0) {
                Xs = new Term[f.arity];
                Ys = new Term[f.arity];
                for (int i = 0; i < f.arity; i++) {
                    if (!TermParser.vsmap.containsKey("X" + i)) {

                        TermParser.addVariableSymbol("X" + i);
                    }
                    if (!TermParser.vsmap.containsKey("Y" + i)) {

                        TermParser.addVariableSymbol("Y" + i);
                    }
                    Xs[i] = new Term(TermParser.vsmap.get("X" + i), emptyTree);
                    Ys[i] = new Term(TermParser.vsmap.get("Y" + i), emptyTree);
                    c.add(new Literal(false, p, new Tree[]{Xs[i], Ys[i]}));
                }

                c.add(new Literal(true, p, new Tree[]{new Term(f, Xs), new Term(f, Ys)}));
                cl.add(c);
            }

        }

        for (PredicateSymbol f : ps) {
            if (f.equals(TermParser.getEqualityPredicate())) {
                continue;
            }
            c = new Clause();
            if (f.arity > 0) {
                Xs = new Term[f.arity];
                Ys = new Term[f.arity];
                for (int i = 0; i < f.arity; i++) {
                    if (!TermParser.vsmap.containsKey("X" + i)) {

                        TermParser.addVariableSymbol("X" + i);
                    }
                    if (!TermParser.vsmap.containsKey("Y" + i)) {

                        TermParser.addVariableSymbol("Y" + i);
                    }
                    Xs[i] = new Term(TermParser.vsmap.get("X" + i), emptyTree);
                    Ys[i] = new Term(TermParser.vsmap.get("Y" + i), emptyTree);
                    c.add(new Literal(false, p, new Tree[]{Xs[i], Ys[i]}));
                }

                c.add(new Literal(false, f, Xs));
                c.add(new Literal(true, f, Ys));
                cl.add(c);
            }
        }

        return cl;
    }
}
