/*
 Copyright 2006, 2007, 2008 Hao Xu
 ertranne@hotmail.com
 
 This file is part of OSHL-S.

 OSHL-S is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 OSHL-S is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with Foobar; if not, write to the Free Software
 Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
package logic.data;

import java.util.Arrays;
import java.util.*;
import jfp.datatype.Pair;
import logic.syntax.TermParser;

/**
 *
 * @author Ertri
 */
public class PrefixList {

    public List<Literal> list = new LinkedList<Literal>();
    public int n;
    PrefixNode node;

    public class PrefixNode {

        public PrefixNode[] subtrees;
        public PrefixSkipNode pointer;

        public PrefixNode(PrefixSkipNode pointer) {
            this.pointer = pointer;
            this.subtrees = new PrefixNode[n];
        }

        @Override
        public String toString() {
            return toString("", 0);
        }

        public String toString(String indent, int id) {
            String str = indent + (pointer == null ? "*" : TermParser.sTable[id] + ":" + pointer.l) + "(\n";
            int i = 0;
            for (PrefixNode nd : subtrees) {
                str += nd != null ? nd.toString(indent + " ", i) + "\n" : "";
                i++;
            }
            str += indent + ")";
            return str;
        }
    }


    public boolean addNode(Literal l) {
        PrefixNode n2 = node;
        int[] key = l.getKey();
        int p = 0;
        PrefixSkipNode psn = new PrefixSkipNode(l);
        while (p < key.length - 1 && n2.subtrees[key[p]] != null) {

            n2 = n2.subtrees[key[p]];
            psn.join(n2.pointer, p);
            p++;
        }
        if (p < key.length - 1) {

            psn.startNew(p);
            while (p < key.length - 1) {
                n2 = n2.subtrees[key[p++]] = new PrefixNode(psn);


            }

            list.add(l);
            return true;

        }
        return false;
    }

    public Pair<PrefixSkipNode, Integer> lookup(Literal l) {
        PrefixNode n2 = node;
        int[] key = l.getKey();
        int p = 0;
        while (n2.subtrees[key[p]] != null) {

            n2 = n2.subtrees[key[p++]];
            if (logic.syntax.TermParser.isVariable(key[p]) || p == key.length - 1) {
                return new Pair(n2.pointer, p - 1);
            }

        }
        return null;
    }

    public PrefixList(int n) {
        this.n = n;
        this.node = new PrefixNode(null);
    }

    public class PrefixSkipNode {

        public Literal l;
        public PrefixSkipNode[] next;
        public PrefixSkipNode[] tail;
        public int headStart;

        public PrefixSkipNode(Literal l) {
            this.l = l;
            this.next = new PrefixSkipNode[l.size()-1];
        }

        public void join(PrefixSkipNode n, int level) {
            n.tail[level - n.headStart].next[level] = this;
            n.tail[level - n.headStart] = this;
        }

        public void startNew(int headStart) {
            this.headStart = headStart;
            int len = l.size() - 1 - headStart;
            this.tail = new PrefixSkipNode[len];
            Arrays.fill(tail, this);
        }
    }

    @Override
    public String toString() {
        return list.toString();
    }

    public static void main(String[] args) {
        TermParser.addPredicateSymbol("P", 2);
        TermParser.addFunctionSymbol("f", 1);
        TermParser.addFunctionSymbol("h", 2);
        TermParser.addFunctionSymbol("a", 0);
        TermParser.addFunctionSymbol("b", 0);
        TermParser.addFunctionSymbol("c", 0);
        TermParser.addVariableSymbol("A");
        TermParser.addVariableSymbol("B");
        TermParser.addVariableSymbol("C");
        TermParser.addVariableSymbol("D");

        Literal l1 = TermParser.parseLiteral("P(f(A),B)");
        Literal l11 = TermParser.parseLiteral("P(A,b))");
        Literal l2 = TermParser.parseLiteral("P(f(h(a,c)),b)");
        Literal l3 = TermParser.parseLiteral("P(h(a,c),b)");
        Literal l4 = TermParser.parseLiteral("P(h(a,a),b)");

        PrefixList pl = new PrefixList(10);
        pl.addNode(l2);
        pl.addNode(l3);
        pl.addNode(l4);

        System.out.println(pl.node);

        Pair<PrefixSkipNode, Integer> psni = pl.lookup(l1);
        PrefixList.PrefixSkipNode psn;
        if (psni != null) {
            psn = psni.fst();
            while (psn != null) {
                System.out.println(psn.l);
                psn = psn.next[psni.snd()];
            }
        }





        psni = pl.lookup(l11);
        if (psni != null) {
            psn = psni.fst();
            while (psn != null) {
                System.out.println(psn.l);
                psn = psn.next[psni.snd()];
            }
        }
    }
}
