/*
 Copyright 2006, 2007, 2008 Hao Xu
 ertranne@hotmail.com
 
 This file is part of OSHL-S.

 OSHL-S is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 OSHL-S is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with Foobar; if not, write to the Free Software
 Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
package logic.data;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.HashMap;
import java.util.Map;
import logic.syntax.TermParser;

public class Substitution extends HashMap<Symbol, Term> {

    public Literal sub(Literal t) {

        Term[] sts = new Term[t.subtrees.length];
        for (int i = 0; i < sts.length; i++) {
            sts[i] = sub((Term) t.subtrees[i]);
        }
        return new Literal(t.sign, (PredicateSymbol) t.val, sts);

    }

    public Clause sub(Collection<Literal> t) {

        Clause cl = new Clause();
        for (Literal l : t) {
            cl.add(sub(l));
        }
        return cl;

    }

    public Term sub(Term t) {
        if (t.val instanceof VariableSymbol) {
            if (this.containsKey(t.val)) {
                return this.get(t.val);
            } else {
                return t;
            }
        } else {
            Term[] sts = new Term[t.subtrees.length];
            for (int i = 0; i < sts.length; i++) {
                sts[i] = sub((Term) t.subtrees[i]);
            }
            return new Term(t.val, sts);
        }
    }

    public Substitution add(Symbol s, Term t) {
        Substitution sub = new Substitution();
        Substitution sub2 = new Substitution();
        sub2.put(s, t);
        for (Map.Entry<Symbol, Term> ent : this.entrySet()) {
            sub.put(ent.getKey(), sub2.sub(ent.getValue()));
        }
        sub.put(s, t);
        return sub;
    }

    public Substitution addNoSub(Symbol s, Term t) {
        this.put(s, t);
        return this;
    }

    public Term keySub(Term t) {
        ArrayList<Integer> keyal = new ArrayList<Integer>();
        int[] tkeys = t.getKey();
        for (int i = 0; i < tkeys.length; i++) {
            int k = tkeys[i];
            if (TermParser.isVariable(k)) {
                VariableSymbol v = (VariableSymbol) TermParser.sTable[k];
                Term term = this.get(v);
                int[] termkeys = term.getKey();
                if (term != null) {
                    for (int j = 0; j < termkeys.length; j++) {
                        keyal.add(termkeys[j]);
                    }
                } else {
                    keyal.add(k);
                }

            } else {
                keyal.add(k);
            }
        }
        int[] nkeys = new int[keyal.size()];
        for (int i = 0; i < nkeys.length; i++) {
            nkeys[i] = keyal.get(i);
        }
        KeyTerm kt = new KeyTerm(nkeys);
        return kt;


    }

    public Literal keySub(Literal t) {

        ArrayList<Integer> keyal = new ArrayList<Integer>();
        int[] tkeys = t.getKey();
        for (int i = 0; i < tkeys.length; i++) {
            int k = tkeys[i];
            if (TermParser.isVariable(k)) {
                VariableSymbol v = (VariableSymbol) TermParser.sTable[k];
                Term term = this.get(v);

                if (term != null) {
                    int[] termkeys = term.getKey();
                    for (int j = 0; j < termkeys.length; j++) {
                        keyal.add(termkeys[j]);
                    }
                } else {
                    keyal.add(k);
                }

            } else {
                keyal.add(k);
            // the trailing integer is negative so will go to this branch.
            }
        }
        int[] nkeys = new int[keyal.size()];
        for (int i = 0; i < nkeys.length; i++) {
            nkeys[i] = keyal.get(i);
        }
        KeyLiteral kt = new KeyLiteral(nkeys);
        return kt;

    }

    public Clause keySub(Collection<Literal> t) {

        Clause cl = new Clause();
        for (Literal l : t) {
            cl.add(keySub(l));
        }
        return cl;

    }
}
