/*
 Copyright 2006, 2007, 2008 Hao Xu
 ertranne@hotmail.com
 
 This file is part of OSHL-S.

 OSHL-S is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 OSHL-S is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with Foobar; if not, write to the Free Software
 Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
package logic.data;

import concurrent.SearchFailureException;
import concurrent.Task;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.Collections;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;

import java.util.Stack;
import java.util.logging.Level;
import java.util.logging.Logger;
import jfp.datatype.EmptyTuple;
import jfp.datatype.Pair;
import jfp.datatype.Tuple;
import jfp.type.TFunction;
import jfp.type.TVar;
import logic.concurrent.data.TermComparator;
import logic.concurrent.data.TermTypedGenerator;
import logic.concurrent.data.TermTypedIterator;
import logic.concurrent.main.data.ProgramParameters;
import logic.semantics.*;
import logic.semantics.env.TypeEnv;
import logic.syntax.TermParser;

public class Util {

    public static TypeEnv env;

    public static boolean unify(Literal s, Literal t, Substitution sub) {
        int[] skeys = s.getKey();
        int[] tkeys = t.getKey();
        if (skeys[skeys.length - 1] == tkeys[tkeys.length - 1]) {
            if (skeys[0] != tkeys[0]) {
                return false;
            } else {
                return unify(skeys, tkeys, new int[]{1, 1}, ((PredicateSymbol) TermParser.sTable[skeys[0]]).arity, sub);
            }
        } else {
            return false;
        }
    }

    public static boolean unify(int[] subtrees,
            int[] subtrees2, int[] start, int count, Substitution sub) {
        int n = subtrees.length;
        for (int i = 0; i < count; i++) {
            if (!unify(subtrees, subtrees2, start, sub)) {
                return false;
            }

        }

        return true;
    }

    public static boolean unify(int[] subtrees, int[] subtrees2, int[] start, Substitution sub) {
        Symbol f = (Symbol) TermParser.sTable[subtrees[start[0]]];
        Symbol f2 = (Symbol) TermParser.sTable[subtrees2[start[1]]];
        if (f instanceof FunctionSymbol && f2 instanceof FunctionSymbol) {
            if (f.id != f2.id) {
                return false;
            } else {
                start[0]++;
                start[1]++;
                return unify(subtrees, subtrees2, start, ((FunctionSymbol) f).arity, sub);
            }
        } else if (f instanceof VariableSymbol) {
            Term t = sub.get(f);
            if (t != null) {
                int[] nstart = new int[]{0, start[1]};

                boolean ret = unify(t.getKey(), subtrees2, nstart, sub);
                start[0]++;
                start[1] = nstart[1];
                return ret;
            } else {
                KeyTerm kt = subterm(subtrees2, start[1]);
                sub.addNoSub(f, kt);
                start[0]++;
                start[1] += kt.size();
                return true;
            }
        } /*else if (f2 instanceof VariableSymbol) {
        Term t = sub.get(f2);
        if (t != null) {
        int[] nstart = new int[]{start[0], 0};
        boolean ret = unify(subtrees, t.getKey(), nstart, sub);
        start[0] = nstart[0];
        start[1]++;
        return ret;
        } else {
        KeyTerm kt = subterm(subtrees, start[0]);
        sub.addNoSub(f2, kt);
        start[0] += kt.size();
        start[1]++;
        return true;
        }
        }*/
        return false;
    // f2 can not be a variable symbol because it is always grounded

    }

//    private static void combinations(Trie t, List<Literal> trueLits, List<Substitution> subs) {
//        if (trueLits.isEmpty()) {
//            Substitution sub = new Substitution();
//            subs.add(sub);
//            return;
//        } else {
//            Literal l = trueLits.get(0);
//            List tl = trueLits.subList(1, trueLits.size());
//            List<Substitution> lsubs = t.lookup(l);
//            for (int i = 0; i < lsubs.size(); i++) {
//                List<Literal> ll = new ArrayList<Literal>();
//                ll.addAll(lsubs.get(i).keySub(tl));
//                combinations(t, ll, subs);
//            }
//        }
//    }
//
//    private static void cocombinations(Trie t, List<Literal> trueLits, List<Substitution> comap) {
//        for (Literal l : trueLits) {
//            List<Substitution> lsubs = t.lookup(l);
//            comap.addAll(lsubs);
//        }
//    }
    public static boolean factor(Substitution sub, List<Substitution> comap) {
        List<Substitution> ncomap = new ArrayList<Substitution>();
        ncomap.addAll(comap);
        for (Symbol key : sub.keySet()) {
            Term match = sub.get(key);
            Iterator<Substitution> itr = ncomap.iterator();
//            List<Substitution> ncomap = new ArrayList<Substitution>();
            while (itr.hasNext()) {
                if (match.equals(itr.next().get(key))) {
                    itr.remove();
                }
            }

        }
        return ncomap.isEmpty();
    }

    private static KeyTerm subterm(
            int[] subtrees2, int start) {

        Stack<Integer> aStack = new Stack<Integer>();
        FunctionSymbol fs = (FunctionSymbol) TermParser.sTable[subtrees2[start]];
        aStack.push(fs.arity);
        int end = start + 1;
        while (!aStack.isEmpty()) {
            int i = aStack.pop();
            if (i == 0) {
                continue;
            } else {
                aStack.push(i - 1);
                fs =
                        (FunctionSymbol) TermParser.sTable[subtrees2[end]];
                aStack.push(fs.arity);
                end++;

            }






        }
        int[] nkeys = Arrays.copyOfRange(subtrees2, start, end);
        KeyTerm kt = new KeyTerm(nkeys);
        return kt;
    }

    /**
     * 
     * @param l
     * @param r a ground literal
     * @param fs
     * @return
     */
    public static List<Substitution> xdiffs(Literal l, Literal r) {
        List<Substitution> ret;
        Substitution sub = new Substitution();
        if (unify(l, r, sub)) {
            ret = diffs(l, sub);
        } else {
            ret = new ArrayList<Substitution>();

            ret.add(new Substitution());
        }

        return ret;
    }

    public static List<Substitution> diffs(Literal l, Substitution sub) {

        List<Substitution> dLits = new ArrayList<Substitution>();
        Substitution dLits2;

        dLits2 =
                new Substitution();
        List<String> varnames = new ArrayList<String>();
        variables(l, varnames);
        for (Map.Entry<Symbol, Term> ent : sub.entrySet()) {
            VariableSymbol vs = (VariableSymbol) ent.getKey();
            Term t = ent.getValue();

            List<Term> dTerms = diff(t.getKey(), 0);
            Substitution subst;

            for (Term term : dTerms) {

                subst = new Substitution();
                subst.putAll(dLits2);
                subst.put(vs, term);
                dLits.add(subst);

            }

            dLits2.put(vs, t);
//            System.out.println(vs+","+t);
//            System.out.println(dLits2);
//            System.out.println(dLits);

        }

        return dLits;
    }//    public static List<Literal>[] xdiff(Literal l, Literal r, Collection<FunctionSymbol> fs) {
//        List<Literal>[] ret = new List[2];
//        Substitution sub = new Substitution();
//        if (unify(l, r, sub)) {
//            ret[0] = diff(l, sub, fs);
//            ret[1] = diff(r, sub, fs);
//        } else {
//            ret[0] = new ArrayList<Literal>();
//            ret[1] = new ArrayList<Literal>();
//
//            ret[0].add(l);
//            ret[1].add(r);
//        }
//        return ret;
//    }

//    public static List<Literal> diff(Literal l, Substitution sub, Collection<FunctionSymbol> fs) {
//
//        List<Literal> dLits = new ArrayList<Literal>();
//        List<Literal> dLits2;
//        dLits.add(l);
//        List<String> varnames = new ArrayList<String>();
//        variables(l, varnames);
//        for (Map.Entry<Symbol, Term> ent : sub.entrySet()) {
//            VariableSymbol vs = (VariableSymbol) ent.getKey();
//            if (varnames.contains(vs.name)) {
//                Term t = ent.getValue();
//
//                dLits2 = new ArrayList<Literal>();
//                List<Term> dTerms = diff(t, fs);
//                Substitution subst;
//                for (Term term : dTerms) {
//                    subst = new Substitution();
//                    subst.put(vs, term);
//
//                    for (Literal lit : dLits) {
//                        dLits2.add(subst.sub(lit));
//
//                    }
//                }
//                dLits = dLits2;
//            }
//        }
//        return dLits;
//    }
    public static int tempVarNameId = 0;
    public static final Term[] EMPTY_TERM_ARRAY = new Term[0];

    public static List<Term> diff(int[] keys, int start) {
        List<Term> dTerms = new ArrayList<Term>();
        FunctionSymbol func = (FunctionSymbol) TermParser.sTable[keys[start]];
        Object funcType = env.getType(func.name);
        TVar funcRangeType;

        if (funcType instanceof TVar) {
            funcRangeType = (TVar) funcType;
        } else {
            funcRangeType = (TVar) ((TFunction) funcType).getRangeType();
        }

        TermTypedGenerator tti = TermTypedGenerator.getTTIMap().get(funcRangeType.getName());
        Collection<FunctionSymbol> fs = tti.getFunctionSymbols();

        for (FunctionSymbol f : fs) {
            if (!f.equals(func)) {
                int[] subtrees = new int[f.arity + 1];
                subtrees[0] = f.id;
                if (f.arity > 0) {
                    TFunction ftype = (TFunction) env.getType(f.name);
                    Tuple type = (Tuple) ftype.getDomainType();

                    for (int i = 1; i <
                            f.arity + 1; i++) {
                        String vname = "TempVar" + tempVarNameId++;
                        TermParser.addVariableSymbol(vname);
                        VariableSymbol v = TermParser.vsmap.get(vname);

                        env.bindType(vname, type.hd());
                        type =
                                type.tl();

                        subtrees[i] = v.id;
                    }

                }
                dTerms.add(new KeyTerm(subtrees));
            } else if (f.arity > 0) {
                int start2 = start + 1;
                int[] keyprefix = new int[]{f.id};
                Tuple funcDomainType = (Tuple) ((TFunction) funcType).getDomainType();
                Object[] paramtypes = funcDomainType.toArray();
                for (int j = 0; j <
                        f.arity; j++) {
                    List<Term> terml = diff(keys, start2);
                    for (Term term : terml) {
                        int[] termkeys = term.getKey();
                        int termlength = termkeys.length;
                        int[] subtrees = new int[keyprefix.length + termlength + f.arity - j - 1];
                        System.arraycopy(keyprefix, 0, subtrees, 0, keyprefix.length);
                        System.arraycopy(termkeys, 0, subtrees, keyprefix.length, termlength);
                        for (int i = j + 1; i <
                                f.arity; i++) {
                            String vname = "TempVar" + tempVarNameId++;
                            TermParser.addVariableSymbol(vname);
                            VariableSymbol v = TermParser.vsmap.get(vname);
                            env.bindType(vname, paramtypes[i]);
                            subtrees[keyprefix.length + termlength + i - j - 1] = v.id;
                        }

                        dTerms.add(new KeyTerm(subtrees));
                    }

                    KeyTerm subterm = subterm(keys, start2);
                    int[] subtermkeys = subterm.getKey();
                    start2 +=
                            subtermkeys.length;

                    int[] nkeyprefix = new int[keyprefix.length + subtermkeys.length];
                    System.arraycopy(keyprefix, 0, nkeyprefix, 0, keyprefix.length);
                    System.arraycopy(subtermkeys, 0, nkeyprefix, keyprefix.length, subtermkeys.length);
                    keyprefix =
                            nkeyprefix;

                }

            }
        }
        return dTerms;
    }

    public static void unification(List<Literal> c, int p, Set<Clause> cset, ELInterpretation eluinter) {
        PrefixList pl = new PrefixList(TermParser.fsmap.size() + TermParser.psmap.size());
        for (Literal l : eluinter) {
            pl.addNode(l);
        }

        unification(c, p, cset, pl, eluinter.inter);
    }

    public static void unification(List<Literal> c, int p, Set<Clause> cset, PrefixList pl, Interpretation i) {
        Pair<PrefixList.PrefixSkipNode, Integer> psni;

        if (p < c.size()) {
            Literal literal = c.get(p);
            if (i.interpret(literal)) {
                if ((psni = pl.lookup(literal)) != null) {
                    int level = psni.snd();
                    PrefixList.PrefixSkipNode psn = psni.fst();
                    while (psn != null) {
                        Literal l = psn.l;
                        if (literal.equals(l.negate())) {
                            //continue outer_loop;
                            unification(c, p + 1, cset, pl, i);
                            break;
                        }

                        Substitution sub = new Substitution();
                        if (Util.unify(literal, l.negate(), sub)) {
                            List<Literal> d = new ArrayList<Literal>();
                            for (Literal cl : c) {
                                d.add(sub.keySub(cl));
                            }

                            unification(d, p + 1, cset, pl, i);

                        }

                        psn = psn.next[level];
                    }

                }
            } else {
                unification(c, p + 1, cset, pl, i);

            }
//        }
        } else {
            Clause cl = new Clause();
            cl.addAll(c);
            cset.add(cl);
        }

    }

//    public static Clause partialDisunification(
//            List<Literal> c, Interpretation initial, Trie t) {
//        // partition literals
//        ArrayList<Literal> trueLits = new ArrayList<Literal>();
//        ArrayList<Literal> falseLits = new ArrayList<Literal>();
//        for (Literal l : c) {
//            if (initial.interpret(l)) {
//                trueLits.add(l);
//            } else {
//                falseLits.add(l);
//            }
//
//        }
//        ArrayList<Substitution> subs = new ArrayList<Substitution>();
//        combinations(t, trueLits, subs);
//        Clause falseLitsSub,
//                cSub;
//        List<Substitution> comap = new ArrayList<Substitution>();
//        Clause candidate = null;
//        Task<Pair<List<Literal>, List<Substitution>>, List<Literal>> iipt;
//        for (Substitution sub : subs) {
//            cSub = sub.keySub(c);
//            falseLitsSub =
//                    sub.keySub(falseLits);
//            ArrayList<Literal> ll = new ArrayList<Literal>();
//            ll.addAll(falseLitsSub);
//            comap.clear();
//            cocombinations(t, ll, comap);
//            ArrayList<Literal> llcsub = new ArrayList<Literal>();
//            llcsub.addAll(cSub);
//            InstanceIteratorProcedure_Typed_GUI_PD_RO iip = new InstanceIteratorProcedure_Typed_GUI_PD_RO();
//            iipt =
//                    iip.create();
//            iipt.input(new Pair(llcsub, comap));
//            Clause ncandidate = null;
//            if (iipt.hasNext()) {
//                try {
//                    ncandidate = new Clause();
//                    ncandidate.addAll(iipt.next().run());
//                } catch (SearchFailureException ex) {
//                    Logger.getLogger(Util.class.getName()).log(Level.SEVERE, null, ex);
//                }
//            }
//
//            if (height(ncandidate) < height(candidate)) {
//                candidate = ncandidate;
//            }
//
//        }
//        return candidate;
//    }
    public static int height(Collection<Literal> c) {
        if (c == null) {
            return Integer.MAX_VALUE;
        }

        int height = 0;
        for (Literal l : c) {
            height = Math.max(height, l.key.length - 1);
        }

        return height;
    }

    public static void disunification(List<Literal> c, int p, Set<Clause> cset, ELInterpretation eluinter) {
        PrefixList pl = new PrefixList(TermParser.fsmap.size() + TermParser.psmap.size());
        for (Literal l : eluinter) {
            pl.addNode(l);
        }

        disunification(c, p, cset, pl, eluinter.inter);
    }

    public static void disunification(List<Literal> c, int p, Set<Clause> cset, PrefixList pl, Interpretation inter) {
        Pair<PrefixList.PrefixSkipNode, Integer> psni;

        if (p < c.size()) {
            Literal literal = c.get(p);
            if (inter.interpret(literal)) {
                if ((psni = pl.lookup(literal)) != null) {
                    int level = psni.snd();
                    PrefixList.PrefixSkipNode psn = psni.fst();
                    while (psn != null) {
                        Literal l = psn.l;

                        Substitution sub = new Substitution();
                        if (literal.equals(l.negate())) {
                            //continue outer_loop;
                            disunification(c, p + 1, cset, pl, inter);
                            break;
                        }

                        if (Util.unify(literal, l.negate(), sub)) {

                            List<Literal> d = new ArrayList<Literal>();
                            for (Literal cl : c) {
                                d.add(sub.keySub(cl));
                            }

                            disunification(d, p + 1, cset, pl, inter);

                        }

                        psn = psn.next[level];
                    }

                } else {
                    Set<List<Literal>> clauses = new HashSet();
                    Set<List<Literal>> nclauses = new HashSet();
                    Set<List<Literal>> tclauses;
                    clauses.add(c);
                    for (Literal l : pl.list) {
                        for (List<Literal> d : clauses) {
                            List<Substitution> subs = Util.xdiffs(d.get(p), l);
                            for (Substitution s : subs) {
                                if (subs.isEmpty()) {
                                    nclauses.add(d);
                                } else {
                                    List<Literal> e = new ArrayList<Literal>();
                                    for (Literal cl : d) {
                                        e.add(s.keySub(cl));
                                    }

                                    nclauses.add(e);
                                }

                            }
                        }
                        tclauses = clauses;
                        clauses =
                                nclauses;
                        nclauses =
                                tclauses;
                        nclauses.clear();
                    }

                    for (List<Literal> d : clauses) {
                        disunification(d, p + 1, cset, pl, inter);
                    }

                }
//        }
            } else {
                Clause cl = new Clause();
                cl.addAll(c);
                cset.add(cl);
            }

        }
    }

    public static ClauseList renameVariables(
            ClauseList cl) {
        Set<String> vname = new HashSet<String>();
        ClauseList clist = new ClauseList();
        for (Clause clause : cl) {
            Substitution sub = new Substitution();
            for (Literal l : clause) {
                sub = renameVariables(l, vname, sub);
            }
            clist.add(sub(clause, sub));
            vname.clear();
        }

        return clist;
    }

    private static Clause sub(Clause clause, Substitution sub) {
        Clause d = new Clause();
        for (Literal l : clause) {
            // Literal lit;
            d.add(sub.sub(l));
        // for(int i = 0;i<l.subtrees.length;i++) {
        // st[i] = sub.sub(t)
        // }
        }

        return d;
    }

    private static Substitution renameVariables(Tree<Symbol> l, Set<String> vname,
            Substitution sub) {
        Substitution subst = sub;
        if (l.val instanceof VariableSymbol) {
            if (!vname.contains(l.val.name)) {

                String prefix = l.val.name;
                VariableSymbol vs;

                vs = TermParser.newVar(prefix);
                vname.add(prefix);
                Term v = new Term(vs, new Term[0]);
                subst = subst.add(l.val, v);

            } 

        } else if (l.val instanceof FunctionSymbol || l.val instanceof PredicateSymbol) {
            for (Tree<Symbol> t : l.subtrees) {
                subst = renameVariables(t, vname, subst);
            }

        }
        return subst;
    }

    public static List<String> variables(ClauseList cl) {
        List<String> vname = new ArrayList<String>();
        for (Clause clause : cl) {
            for (Literal l : clause) {
                variables(l, vname);
            }

        }
        return vname;
    }

    public static List<String> variables(Clause clause) {
        List<String> vname = new ArrayList<String>();
        for (Literal l : clause) {
            variables(l, vname);
        }

        return vname;
    }

    private static void variables(Literal l, List<String> vname) {
        int[] keys = l.getKey();
        for (int k : keys) {
            if (TermParser.isVariable(k)) {
                Symbol v = TermParser.sTable[k];
                if (!vname.contains(v.name)) {
                    vname.add(v.name);
                }

            }

        }
    }

    public static Map<VariableSymbol, Integer> occurences(Literal l,
            List<VariableSymbol> fv) {
        Map<VariableSymbol, Integer> omap = new HashMap<VariableSymbol, Integer>();
        for (int i = 0; i <
                fv.size(); i++) {
            omap.put(fv.get(i), 0);
        }

        occurences(l, fv, omap);
        return omap;
    }

    public static void occurences(Literal l, List<VariableSymbol> fv,
            Map<VariableSymbol, Integer> omap) {
        int[] keys = l.getKey();
        for (int i = 0; i <
                keys.length - 1; i++) {
            int k = keys[i];
            if (TermParser.isVariable(k)) {
                Integer integer = omap.get(TermParser.sTable[k]);
                if (integer == null) {
                    integer = 0;
                }

                omap.put((VariableSymbol) TermParser.sTable[k], integer + 1);
            }

        }
    }

    public static List<VariableSymbol> FV(Collection<Literal> c) {
        List<VariableSymbol> list = new ArrayList<VariableSymbol>();
        Set<VariableSymbol> set = new HashSet<VariableSymbol>();
        for (Literal l : c) {
            FV(l, set);

        }

        list.addAll(set);
        return list;
    }

    public static void FV(Literal c, Set<VariableSymbol> vset) {
        int[] keys = c.getKey();
        for (int k : keys) {
            if (TermParser.isVariable(k)) {
                vset.add((VariableSymbol) TermParser.sTable[k]);

            }

        }
    }

    public static void main(String args[]) {
        TermParser.addPredicateSymbol("P", 2);
        TermParser.addFunctionSymbol("f", 1);
        TermParser.addFunctionSymbol("h", 2);
        TermParser.addFunctionSymbol("a", 0);
        TermParser.addFunctionSymbol("b", 0);
        TermParser.addFunctionSymbol("c", 0);
        TermParser.addVariableSymbol("A");
        TermParser.addVariableSymbol("B");
        TermParser.addVariableSymbol("C");
        TermParser.addVariableSymbol("D");

        Collection<FunctionSymbol> fs = TermParser.fsmap.values();
        env =
                new TypeEnv();
        TVar t = new TVar(env);
        for (FunctionSymbol f : fs) {
            if (f.arity == 0) {
                env.bindType(f.name, t);
            } else {
                Tuple dt = new EmptyTuple();

                for (int i = 0; i <
                        f.arity; i++) {
                    dt = new Tuple(t, dt);
                }

                env.bindType(f.name, new TFunction(dt, t));
//                env.bindType(vn, t);
            }

        }

        Literal l1 = TermParser.parseLiteral("~P(f(A),B)");
        Literal l11 = TermParser.parseLiteral("~P(A,b))");
        Literal l2 = TermParser.parseLiteral("P(f(h(a,c)),b)");
        Literal l3 = TermParser.parseLiteral("P(h(a,c),b)");
        Literal l4 = TermParser.parseLiteral("P(h(a,a),b)");
        List<Substitution> xdiffs = xdiffs(l1, l2);

        for (Substitution s : xdiffs) {
            System.out.println(s.keySub(l1));
        }

        List<Literal> cl = new ArrayList<Literal>();
        cl.add(l1);
        cl.add(l11);
        HashSet<Clause> cset = new HashSet<Clause>();
        ELInterpretation eluinter = new ELInterpretation(new TrivialBooleanInterpretation(false));
        eluinter.add(l2);
        eluinter.add(l3);
        eluinter.add(l4);
        disunification(cl, 0, cset, eluinter);
        System.out.println(cset);
    }
}
