/*
 Copyright 2006, 2007, 2008 Hao Xu
 ertranne@hotmail.com
 
 This file is part of OSHL-S.

 OSHL-S is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 OSHL-S is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with Foobar; if not, write to the Free Software
 Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
package logic.data.arithmetics;

public class NatInequality {

    int[] coeffient;
    int bound;
    boolean max;
    boolean non; // sulotion should be in trie.non

    public NatInequality(int[] coeffient, int bound) {
        super();
        this.coeffient = coeffient;
        this.bound = bound;
        this.max = false;
        this.non = false;
    }

    public boolean isMax() {
        return max;
    }

    public NatInequality(int[] coefficient, int bound, boolean max) {
        this(coefficient, bound);
        this.max = max;
        this.non = true;
    }

    public int getBound() {
        return bound;
    }

    public void setBound(int bound) {
        this.bound = bound;
    }

    public String toString() {
        StringBuilder str = new StringBuilder();
        for (int i = 0; i < coeffient.length; i++) {
            str.append(coeffient[i]);
            str.append((char) ('a' + i));
            if (i != coeffient.length - 1) {
                str.append("+");
            } else {
                str.append("<=");
            }
        }
        str.append(bound);
        return str.toString();

    }

    public int[] getRange() {
        int[] range = new int[coeffient.length];
        int sum = 0;
        for (int i = 0; i < coeffient.length; i++) {
            sum += coeffient[i];
        }
        for (int i = 0; i < coeffient.length; i++) {
            if (coeffient[i] != 0) {
                range[i] = 1 + (bound - sum) / coeffient[i];
            } else {
                range[i] = Integer.MAX_VALUE;
            }
        }
        return range;
    }

    public int compute(int[] v) {
        int sum = 0;
        for (int i = 0; i < coeffient.length; i++) {
            sum += coeffient[i] * v[i];
        }
        return sum;
    }
    public boolean satisfy(int[] v) {
        return compute(v) <= bound;

    }

    public boolean tight(int[] v) {
        int sum = 0;
        for (int i = 0; i < coeffient.length; i++) {
            sum += coeffient[i] * v[i];
        }
        return sum == bound;

    }
    public static final int TIGHT = 1,  SATIFY = 0,  FALSIFY = 2;

    public int tightOrSatify(int[] v) {
        int sum = 0;
        for (int i = 0; i < coeffient.length; i++) {
            if (v[i] == Integer.MAX_VALUE) {
                return FALSIFY;
            } else {
                sum += coeffient[i] * v[i];
            }
        }
        if (sum == bound) {
            return TIGHT;
        } else if (sum < bound) {
            return SATIFY;
        } else {
            return FALSIFY;
        }
    }

    public NatInequality substitute(int[] vi, int[] vs) {
        int[] coef = new int[this.coeffient.length - vi.length];
        int bnd = this.bound;
        int off = 0;
        for (int i = 0; i < this.coeffient.length; i++) {
            if (i == vi[off]) {
                off++;
            } else {
                coef[i - off] = this.coeffient[i];
            }
        }

        for (int i = 0; i < vi.length; i++) {
            bnd -= this.coeffient[vi[i]] * vs[i];
        }
        return new NatInequality(coef, bnd);


    }
}
