/*
 Copyright 2006, 2007, 2008 Hao Xu
 ertranne@hotmail.com
 
 This file is part of OSHL-S.

 OSHL-S is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 OSHL-S is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with Foobar; if not, write to the Free Software
 Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
package logic.data.arithmetics;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import logic.data.Trie;

public class NatInequalityGroup {

    int nov;
    List<NatInequality> nelist = new ArrayList<NatInequality>();
    int[] bound;

    public void add(int[] coefficient, int bound) {
        nelist.add(new NatInequality(coefficient, bound));
    }

    public void add(int[] coefficient, int bound, boolean max) {
        nelist.add(new NatInequality(coefficient, bound, max));
    }

    public int[][] solve() {
        int[] range = new int[nov];
        int[] p = new int[nov];
        List<int[]> list = new ArrayList<int[]>();
        for (int k = 0; k < nov; k++) {
            range[k] = bound[k];
        }
        for (NatInequality ni : nelist) {
            int[] range2 = ni.getRange();
            for (int i = 0; i < nov; i++) {
                if (range[i] > range2[i]) {
                    range[i] = range2[i];
                }
            }
        }
        for (int k = 0; k < nov; k++) {
            p[k] = 1;
        }
        for (int k = 0; k < nov; k++) {
            if (range[k] <= 0) {
                return new int[0][nov];
            }
        }
        for (;;) {
            if (p[0] > range[0]) {
                break;
            }
            boolean sat = true;
            for (NatInequality ni : nelist) {
                if (!ni.satisfy(p)) {
                    sat = false;
                    break;
                }
            }
            if (sat) {
                int[] s = Arrays.copyOf(p, 0);
                list.add(s);
            }
            int pp = nov - 1;
            while (pp >= 0) {
                p[pp]++;
                if (p[pp] > range[pp]) {
                    p[pp] = 1;
                    pp--;
                } else {
                    break;
                }
            }
            if (pp < 0) {
                break;
            }
        }
        return list.toArray(new int[0][]);
    }

    public int[][] tightSolve() {
        int[] range = new int[nov];
        int[] p = new int[nov];
        List<int[]> list = new ArrayList<int[]>();
        for (int k = 0; k < nov; k++) {
            range[k] = bound[k];
        }
        for (NatInequality ni : nelist) {
            int[] range2 = ni.getRange();
            for (int i = 0; i < nov; i++) {
                if (range[i] > range2[i]) {
                    range[i] = range2[i];
                }
            }
        }
        for (int k = 0; k < nov; k++) {
            p[k] = range[k] == Integer.MAX_VALUE ? 0 : 1;
        }
        for (int k = 0; k < nov; k++) {
            if (range[k] <= 0) {
                return new int[0][nov];
            }
        }
        for (;;) {
            if (p[0] > range[0]) {
                break;
            }
            boolean sat = true;
            boolean tight = false;
            for (NatInequality ni : nelist) {
                if (ni.tight(p)) {
                    tight = true;
                    continue;
                }
                if (!ni.satisfy(p)) {
                    sat = false;
                    break;
                }
            }
            if (sat && tight) {
                int len = p.length;
                int[] s = Arrays.copyOf(p, len);
                list.add(s);
            }
            int pp = nov - 1;
            while (pp >= 0) {
                p[pp]++;
                if (range[pp] == Integer.MAX_VALUE) {
                    p[pp] = 0;
                    pp--;
                } else if (p[pp] > range[pp]) {
                    p[pp] = 1;
                    pp--;
                } else {
                    break;
                }
            }
            if (pp < 0) {
                break;
            }
        }
        return list.toArray(new int[0][]);
    }

    public int[][] tightSolveQuick() {
        System.out.println(nelist);
        int[] range = new int[nov];
        int[] p = new int[nov];
        List<int[]> list = new ArrayList<int[]>();
        for (int k = 0; k < nov; k++) {
            range[k] = bound[k];
        }
        for (NatInequality ni : nelist) {
            int[] range2 = ni.getRange();
            for (int i = 0; i < nov; i++) {
                if (range[i] > range2[i]) {
                    range[i] = range2[i];
                }
            }
        }
        for (int k = 0; k < nov; k++) {
            p[k] = range[k] == Integer.MAX_VALUE ? 0 : 1;
        }
        for (int k = 0; k < nov; k++) {
            if (range[k] <= 0) {
                return new int[0][nov];
            }
        }
        int spp = nov - 1;
        while (spp >= 0 && range[spp] == Integer.MAX_VALUE) {
            p[spp] = 0;
            spp--;
        }
        for (;;) {
            if (p[0] > range[0]) {
                break;
            }
            boolean sat = true;
            boolean tight = false;
            loop:
            for (NatInequality ni : nelist) {
                switch (ni.tightOrSatify(p)) {
                    case NatInequality.TIGHT:
                        tight = true;
                        sat = true;
                        break;
                    case NatInequality.FALSIFY:
                        tight = false;
                        sat = false;
                        break loop;
                    case NatInequality.SATIFY:
                        sat = true;
                        break;
                }
            }
            if (sat && tight) {
                int len = p.length;
                int[] s = Arrays.copyOf(p, len);
                list.add(s);
            }

            if (!sat) {
                p[spp] = range[spp];
            }
            int pp = spp;
            while (pp >= 0) {
                p[pp]++;
                if (range[pp] == Integer.MAX_VALUE) {
                    p[pp] = 0;
                    pp--;
                } else if (p[pp] > range[pp]) {
                    p[pp] = 1;
                    pp--;
                } else {
                    break;
                }
            }
            if (pp < 0) {
                break;
            }
        }
        return list.toArray(new int[0][]);
    }
//    public boolean allnon;
//
//    public int[][] tightSolveQuick2(Trie trie) {
//        System.out.println(nelist);
//        int[] range = new int[nov];
//        int[] p = new int[nov];
//        List<int[]> list = new ArrayList<int[]>();
//        for (int k = 0; k < nov; k++) {
//            range[k] = bound[k];
//        }
//        allnon = true;
//        for (NatInequality ni : nelist) {
//            if (!ni.non) {
//                allnon = false;
//            }
//            int[] range2 = ni.getRange();
//            for (int i = 0; i < nov; i++) {
//                if (range[i] > range2[i]) {
//                    range[i] = range2[i];
//                }
//            }
//        }
//
//        if (allnon) {
//            System.out.println("all subject to non constraint");
//            for (NatInequality ni : nelist) {
//                if (trie.non[ni.bound] != 0) {
//                    allnon = false;
//                    break;
//                }
//                if(trie.depth < ni.bound)
//            }
//        }
//        if (allnon) {
//            overflow = 
//            return new int[0][];
//            
//        }
//        for (int k = 0; k < nov; k++) {
//            p[k] = range[k] == Integer.MAX_VALUE ? 0 : 1;
//        }
//        for (int k = 0; k < nov; k++) {
//            if (range[k] <= 0) {
//                return new int[0][nov];
//            }
//        }
//        int spp = nov - 1;
//        while (spp >= 0 && range[spp] == Integer.MAX_VALUE) {
//            p[spp] = 0;
//            spp--;
//        }
//        for (;;) {
//            if (p[0] > range[0]) {
//                break;
//            }
//            boolean sat = true;
//            boolean tight = false;
//            loop:
//            for (NatInequality ni : nelist) {
//                switch (ni.tightOrSatify(p)) {
//                    case NatInequality.TIGHT:
//                        tight = true;
//                        sat = true;
//                        break;
//                    case NatInequality.FALSIFY:
//                        tight = false;
//                        sat = false;
//                        break loop;
//                    case NatInequality.SATIFY:
//                        sat = true;
//                        break;
//                }
//            }
//            if (sat && tight) {
//                int len = p.length;
//                int[] s = Arrays.copyOf(p, len);
//                list.add(s);
//            }
//
//            if (!sat) {
//                p[spp] = range[spp];
//            }
//
//            int pp = spp;
//
//            while (pp >= 0) {
//                p[pp]++;
//                if (range[pp] == Integer.MAX_VALUE) {
//                    p[pp] = 0;
//                    pp--;
//                } else if (p[pp] > range[pp]) {
//                    p[pp] = 1;
//                    pp--;
//                } else {
//                    break;
//                }
//            }
//
//            if (pp < 0) {
//                break;
//            }
//        }
//        return list.toArray(new int[0][]);
//    }

    public NatInequalityGroup subtitute(
            int[] vi, int[] vs) {
        NatInequalityGroup nig = new NatInequalityGroup(this.nov - vi.length);
        NatInequality nni;

        for (NatInequality ni : nelist) {
            nni = ni.substitute(vi, vs);
            nig.nelist.add(nni);

        }

        return nig;
    }

    public boolean solvable() {
        int[] p = new int[nov];

        for (int k = 0; k <
                nov; k++) {
            p[k] = 1;
        }

        for (NatInequality ni : nelist) {
            if (!ni.satisfy(p)) {
                return false;
            }

        }

        return true;
    }

    public boolean overflow() {
        for (NatInequality ni : nelist) {
            if (!ni.isMax()) {
                switch (ni.tightOrSatify(bound)) {
                    case NatInequality.FALSIFY:
                        return true;
                    case NatInequality.TIGHT:
                    case NatInequality.SATIFY:
                        break;
                }

            }
        }
        return false;
    }

    @Override
    public String toString() {
        return "NIG:" + nelist;
    }

    public NatInequalityGroup(int nov) {
        super();
        this.nov = nov;
        this.bound = new int[nov];
        Arrays.fill(bound, Integer.MAX_VALUE);
    }

    public void setVarBound(int index, int bound) {
        this.bound[index] = bound;
    }

    public void setBound(int bound) {
        for (NatInequality ne : nelist) {
            ne.setBound(bound);
        }
    }
}
