/*
 Copyright 2006, 2007, 2008 Hao Xu
 ertranne@hotmail.com
 
 This file is part of OSHL-S.

 OSHL-S is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 OSHL-S is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with Foobar; if not, write to the Free Software
 Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
package logic.main.function;

import logic.concurrent.main.data.ProgramParameters;
import chercher.IntegerForest;
import chercher.IntegerTree;
import chercher.IntegerTree.IntTreeNode;
import logic.concurrent.data.*;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import concurrent.AbstractAnalyzerTaskFactory;
import concurrent.SearchFailureException;
import concurrent.Task;

import java.util.Collection;
import java.util.*;
import jfp.datatype.EmptyTuple;
import jfp.datatype.Pair;
import jfp.datatype.Triple;
import jfp.datatype.Tuple;
import jfp.type.TFunction;
import jfp.type.TVar;
import logic.concurrent.data.TermTypedGenerator;
import logic.concurrent.data.TermTypedIterator;
import logic.concurrent.data.TermTypedListML;
import logic.concurrent.function.MFunctionSetProcedure;
import logic.data.*;
import logic.semantics.env.TypeEnv;
import logic.syntax.TermParser;

public class InstanceIteratorProcedure_Typed_PD_RO_Lazy extends AbstractAnalyzerTaskFactory<Triple<List<Literal>, List<Literal>, Trie>, List<Literal>> implements ProgramParameters {

    static boolean verbose;
    public static TypeEnv env;
    private static IntegerForest iforest = new IntegerForest();

    public final class IITask implements
            Task<Triple<List<Literal>, List<Literal>, Trie>, List<Literal>> {

        private boolean alltrueliterals;
        private List<Literal> falseLiterals;
        private List<Literal> trueLiterals;
        private Triple<List<Literal>, List<Literal>, Trie> input;
        private Clause next;
        handlerImpl handlerImpl = new handlerImpl();

        public boolean hasNext() {
            if (next != null) {
                // if next is already set
                return true;
            } else if (fv.isEmpty()) {
                return hasNextGrnd();
            } else {
                // if there are free variables
                if (alltrueliterals) {
                    if (trie.non[max] == 0) {
                        if(maxTrueLit<max) {
                            noMoreSolutions = true;
                        }
                        return false;
                    } else {
                        final int h = max;
                        return trie.lookupCombinationsNoBound(trueLiterals, new SubstitutionHandler() {

                            public boolean handle(Substitution handler) {
                                Clause cls = handler.keySub(C);
                                if (Util.height(cls) == h) {
                                    next = cls;
                                    return true;
                                } else {
                                    return false;
                                }
                            }
                        });
                    }
                } else {
                    if (solutions == null) {
                        solveNIG(C, fv);
                    }
                    while (currentSolution != null) {
                        prepareST();
                        if (trie.lookupCombinations(trueLiterals, lm, handlerImpl)) {
                            return true;
                        }
                        prevSolution = currentSolution;
                        currentSolution = solutions.nextSolution(currentSolution);

                    //System.out.println("clause iterator: next clause " + next);
                    }
                    return false;
//                    }
                }
            }
        }
        List<Literal> C;
        List<VariableSymbol> fv;
        IntegerTree solutions;
        IntegerTree.IntTreeNode currentSolution, prevSolution;
        LengthMap lm = new LengthMap();
        Trie trie = null;

        void prepareST() {
            IntTreeNode solution = currentSolution;
            lm.clear();
            for (int i = fv.size() - 1; i >= 0; i--) {
                VariableSymbol v = fv.get(i);
                // Add terms of length solution[i]. 
                // We need to add 1 back to the bound here.
                lm.put(v.name, solution.value + 1);
                solution = solution.getBacklink();
            }
//            System.out.print("Term size: ");
//            for( int i = 0;i<fv.size() ;i++) {
//                VariableSymbol v = fv.get(i);
//                System.out.print(v.name+": "+lm.get(v.name)+" ");
//            }
//            System.out.println();
        }
        HashMap<Literal, Map<VariableSymbol, Integer>> omap = new HashMap<Literal, Map<VariableSymbol, Integer>>();

        public boolean isNoMoreSolutions() {
            return noMoreSolutions;
        }

        public boolean input(Triple<List<Literal>, List<Literal>, Trie> input) {
            this.input = input;

            trueLiterals = input.fst();
            falseLiterals = input.snd();
            C = new ArrayList<Literal>();
            C.addAll(trueLiterals);
            C.addAll(falseLiterals);
            trie = input.third();
            fv = Util.FV(C);
            next = null;
            maxTrueLit = trie.depth;//+1;

            for (Literal ll : C) {
                omap.put(ll, Util.occurences(ll, fv));
            }

            if (falseLiterals.size() == 0) {
                // all true literal
                alltrueliterals = true;
            }

            return false;
        }

        public concurrent.Task.TaskInstance<List<Literal>> next() {

            final Clause cl = this.next;
            this.next = null;
            return new Task.TaskInstance<List<Literal>>() {

                public List<Literal> run() throws SearchFailureException {
                    ArrayList<Literal> list = new ArrayList<Literal>();
                    list.addAll(cl);
                    //System.out.println("clause iterator: generated clause "
                    //		+ list);
                    return list;
                }
            };
        }

        public Clause peek() {
            return this.next;
        }

        public void reset() {
            noMoreSolutions = false;
            solutions = null;
            next = null;
        }

        public String toString() {
            return "IITask{" + next + "}";
        }

        private boolean hasNextGrnd() {
            // if there is no free variable,
            // then there is only on instance of the clause,
            // which is the clause itself.
            if (isNoMoreSolutions() || Util.height(C) != max) {
                // clause has been generated.
                if(Util.height(C)< max) {
                    noMoreSolutions = true;
                }
                return false;
            } else {
                Clause cl = new Clause();
                cl.addAll(C); // shallow copy clause
                if (maxTrueLit < Util.height(trueLiterals)) {
                    // the true literals of the current clause is longer than the depth of the trie
                    noMoreSolutions = true;
                    return false;
                } else {
                    // clause may be generated as its only instance
                    SubstitutionHandler sh = new SubstitutionHandler() {

                        public boolean handle(Substitution handler) {
                            return true;
                        }
                    };
                    for (Literal tl : trueLiterals) {
//                        System.out.println("looking for "+tl);
                        if (!trie.lookupNoBound(tl, sh)) {
                            noMoreSolutions = true;
                            return false;
                        }
//                        System.out.println("found");
                    }
                    for (Literal tf : falseLiterals) {
                        if (trie.lookupNoBound(tf, sh)) {
                            noMoreSolutions = true;
                            return false;
                        }
                    }
                    next = cl; // set the copy as the next instance.
                    return true;
                }
            }
        }

        public void setNext(Clause next) {
            this.next = next;
        }

//        private void printSolutions() {
//            for (int i = 0; i < solutions.length; i++) {
//                for (int j = 0; j < solutions[i].length; j++) {
//                    System.out.print(solutions[i][j] + " ");
//                }
//                System.out.println();
//            }
//        }
        private IntegerTree.IntTreeNode solveNIG(Collection<Literal> C, List<VariableSymbol> fv) {
            boolean overflow;

            int n = C.size(); // number of inequalities
            int p = fv.size(); // number of variables
            int index = 0; // index of inequalities
            int[][] coeffs = new int[n][];
            int[] bound = new int[n];
            int[] varBound = new int[p];
            boolean[] tightable = new boolean[n];
            for (Literal ll : C) {
                Map<VariableSymbol, Integer> omapl = this.omap.get(ll);
                int[] coefficient = new int[p];
                int sum = 0; // the size take by the variables in the original clause
                for (int i = 0; i <
                        fv.size(); i++) {
                    coefficient[i] = omapl.get(fv.get(i)); // use the occurences of ith fv as the weight of the ith variable
                    sum += coefficient[i];
                }
                coeffs[index] = coefficient;
                // Since IntergerTree solves equation by searching from 0
                // we subtract 1 from the bound for each occurence of a variable
                // and do not add sum to the bound
                if (trueLiterals.contains(ll)) {
                    if (max <= maxTrueLit) {
                        bound[index] = max - ll.getKey().length + 1;
                        tightable[index] = true;
                    } else {
                        bound[index] = maxTrueLit - ll.getKey().length + 1;
                        tightable[index] = false;
                    }
                } else {
                    bound[index] = max - ll.getKey().length + 1;
                    tightable[index] = true;
                }// the rhs constant
                index++;
                if (verbose) {
                    char varch = 'A';
                    for (int i = 0; i < fv.size(); i++) {
                        System.out.print(coefficient[i] + "" + (varch++) + (i != fv.size() - 1 ? "+" : "="));
                    }
                }

            }
//            System.out.println(nig);
            int boundedVars = 0;
            int vbound;
            int vindex;
            for (VariableSymbol v : fv) {
                vindex = fv.indexOf(v);
                String type = ((TVar) env.getType(v.name)).getName();
                vbound = TermTypedIterator.getTermLengthBound(type);
                varBound[vindex] = vbound == Integer.MAX_VALUE?Integer.MAX_VALUE : vbound - 1;
                if (vbound != Integer.MAX_VALUE) {
                    boundedVars++;
                }

            }
//            for( int i = 0;i<fv.size() ;i++) {
//                VariableSymbol v = fv.get(i);
//                System.out.print(v.name+": "+varBound[i]+" ");
//            }
//            System.out.println();
            solutions = iforest.getIntegerTree(coeffs, bound, tightable, varBound);
            overflow = solutions.overflow();
//            printSolutions();
            if (!overflow) {
                // if there is no overflow then no further solutions can be found by increasing height
                noMoreSolutions = true;
            }
            IntTreeNode sol = solutions.nextSolution(currentSolution);
            prevSolution = currentSolution;
            currentSolution = sol;
            return currentSolution;
        // give a tight solution in which at least on literal reaches the bound
        }

        private class handlerImpl implements SubstitutionHandler {

            public boolean handle(Substitution handler) {
                Task<Pair<List<VariableSymbol>, List<List<Term>>>, Pair<List<VariableSymbol>, List<Term>>> st;
                List<List<Term>> terms = new ArrayList<List<Term>>();
                MFunctionSetProcedure<VariableSymbol, Term> fsp = new MFunctionSetProcedure<VariableSymbol, Term>();
                st = fsp.create();
                terms.clear();

                IntTreeNode solution = currentSolution;

                List<VariableSymbol> nfv = new ArrayList<VariableSymbol>();
                for (int i = fv.size() - 1; i >= 0; i--) {
                    VariableSymbol v = fv.get(i);
                    if (!handler.containsKey(v)) {
                        nfv.add(v);
                        // add terms of length solution[i].
                        terms.add(new TermTypedListML(solution.value + 1, ((TVar) env.getType(v.name)).getName()));
                        solution = solution.getBacklink();
                    }
                }
                st.input(new Pair<List<VariableSymbol>, List<List<Term>>>(nfv, terms));
                while (st != null && st.hasNext()) {
                    Pair<List<VariableSymbol>, List<Term>> fun = null;
                    try {
                        fun = st.next().run();
                    } catch (SearchFailureException e) {
                        e.printStackTrace();
                    }

                    Substitution sub = new Substitution();
                    sub.putAll(handler);

                    for (int i = 0; i < fun.fst().size(); i++) {
                        sub = sub.addNoSub(fun.fst().get(i), fun.snd().get(i));
                    }
                    boolean sat = false;
                    for (Literal l : falseLiterals) {
                        if (trie.lookup(l, sub, lm, new SubstitutionHandler() {

                            public boolean handle(Substitution handler) {
                                return true;
                            }
                        })) {
                            sat = true;
                        }
                    }
                    if (!sat) {
                        next = sub.keySub(C);
                        return true;
                    }
                }
                return false;
            }
        }
    }
    public static int max = MAX_CLAUSE_LENGTH;
    private int maxTrueLit = 0;
    private boolean noMoreSolutions = false;

    public static void main(String[] args) {
        TermParser.addPredicateSymbol("P", 2);
        TermParser.addFunctionSymbol("f", 1);
        TermParser.addFunctionSymbol("h", 2);
        TermParser.addFunctionSymbol("a", 0);
        TermParser.addFunctionSymbol("b", 0);
        TermParser.addFunctionSymbol("c", 0);
        TermParser.addVariableSymbol("A");
        TermParser.addVariableSymbol("B");
        TermParser.addVariableSymbol("X");
        TermParser.addVariableSymbol("Y");

        env =
                new TypeEnv();
        TVar vtype;

        vtype =
                new TVar(env);
        TermTypedGenerator tti = new TermTypedGenerator(vtype.getName(),
                env);
        Collection<VariableSymbol> vs = TermParser.vsmap.values();
        for (FunctionSymbol f : TermParser.fsmap.values()) {
            Tuple t = new EmptyTuple();
            for (int i = 0; i <
                    f.arity; i++) {
                t = new Tuple(vtype, t);
            }

            Object type;
            if (f.arity > 0) {
                type = new TFunction(t, vtype);
            } else {
                type = vtype;
            }

            env.bindType(f.name, type); // bind type to env


            tti.addFunctionSymbol(f);
        }

        for (VariableSymbol f : vs) {
            Object type;
            type =
                    vtype;

            env.bindType(f.name, type); // bind type to env

        }

        Literal l1 = TermParser.parseLiteral("P(f(A),B)");
        Literal l12 = TermParser.parseLiteral("P(X,Y)");
        Literal l2 = TermParser.parseLiteral("P(f(h(a,c)),b)");
        Literal l22 = TermParser.parseLiteral("P(h(a,b),b)");

        List<Literal> lt = new ArrayList<Literal>();
        lt.add(l1);

        List<Literal> lf = new ArrayList<Literal>();
        l12.getKey();
        lf.add(l12.negate());

        Trie trie = new Trie(10);
        trie.addNode(l2);
        trie.lookupNoBound(l1, new SubstitutionHandler() {

            public boolean handle(Substitution handler) {
                System.out.println("found: " + handler);
                return true;
            }
        });
        InstanceIteratorProcedure_Typed_PD_RO_Lazy.max = 10;
        InstanceIteratorProcedure_Typed_PD_RO_Lazy ii = new InstanceIteratorProcedure_Typed_PD_RO_Lazy();
        Task<Triple<List<Literal>, List<Literal>, Trie>, List<Literal>> iit = ii.create();
        iit.input(new Triple(lt, lf, trie));
        for (int i = 0; i <
                100; i++) {
            try {
                if (iit.hasNext()) {
                    System.out.println(iit.next().run());
                }

            } catch (SearchFailureException ex) {
            }
        }
    }

    @Override
    public Task<Triple<List<Literal>, List<Literal>, Trie>, List<Literal>> create() {
        return new IITask();
    }
}
