/*
 Copyright 2006, 2007, 2008 Hao Xu
 ertranne@hotmail.com
 
 This file is part of OSHL-S.

 OSHL-S is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 OSHL-S is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with Foobar; if not, write to the Free Software
 Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
package logic.main.function;

import logic.concurrent.main.data.ProgramParameters;
import logic.concurrent.main.data.ProgramParametersDyn;
import java.util.logging.Level;
import java.util.logging.Logger;
import jfp.datatype.Triple;
import logic.concurrent.data.*;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashSet;
import java.util.List;
import java.util.Set;
import java.util.SortedSet;

import jfp.datatype.Pair;
import jfp.datatype.Tuple;
import jfp.type.TFunction;
import jfp.type.TVar;
import logic.concurrent.data.TermGenerator;
import logic.concurrent.data.TermTypedIterator;
import logic.concurrent.function.IterativeSimpProcedure;
import logic.data.Clause;
import logic.data.ClauseList;
import logic.data.FunctionSymbol;
import logic.data.Literal;
import logic.data.Util;
import logic.semantics.ELInterpretation;
import logic.semantics.TrivialBooleanInterpretation;
import logic.semantics.env.TypeEnv;
import logic.semantics.type.TypeInferenceEngine;
import concurrent.AbstractSimpleTaskProcedure;
import concurrent.SearchFailureException;
import java.util.Collections;
import jfp.datatype.EmptyTuple;
import logic.concurrent.data.ClauseComparator;
import logic.concurrent.data.LiteralComparator;
import logic.concurrent.data.TermTypedGenerator;
import logic.data.PredicateSymbol;
import logic.data.Trie;
import logic.data.VariableSymbol;
import logic.semantics.Interpretation;
import logic.syntax.TermParser;

public class ProofProcedure_Typed_PD_RO_Lazy extends AbstractSimpleTaskProcedure<ClauseList, Boolean> implements ProgramParameters {

    private int minLenght;
    public long index = 0;

    @Override
    public Boolean run(ClauseList input) throws SearchFailureException {
        ClauseList list = new ClauseList();
        TypeEnv env = new TypeEnv();
        Collections.sort(input, new ClauseComparator());
        // rename variables
//        System.out.println("renaming variables");
        ClauseList inputDisjoint = renameVariables(input);
//        for (Clause clause : inputDisjoint) {
//            System.out.println(clause);
//        }
        input = inputDisjoint;
        // compute min length
        minLenght = Integer.MAX_VALUE;
        for (Clause clause : inputDisjoint) {
            for (Literal l : clause) {
                l.getKey();
            }
            minLenght = Math.min(Util.height(clause), minLenght);
        }
//        System.out.println("Minimum length of clauses: "+minLenght);
        // set env
        InstanceIteratorProcedure_Typed_PD_RO_Lazy.env = env;
        Util.env = env;
        //reorderGULiterals(input);

        SortedSet<FunctionSymbol> fs = TermGenerator.fs;
        if (ProgramParametersDyn.getTYPE_INFERENCE()) {
//            System.out.println("type inference in progress...");
            typeInference(input, fs, inputDisjoint, env);
        } else {
            monotype(env, fs, inputDisjoint);
        }
        for (Clause c : input) {
            list.add(c);
        }
        ELInterpretation inter = new ELInterpretation(
                new TrivialBooleanInterpretation(ProgramParametersDyn.getInitialInterpretation()));
        ClauseList elclause = new ClauseList();
//        List<InstanceIteratorProcedure_Typed_GUI_PD_RO.IITask>[] q = new List[MAX_CLAUSE_LENGTH];
//        for (Clause c : input) {
//            if (!validInDefaultModel(c)) {
//                InstanceIteratorProcedure_Typed_GUI_PD_RO.IITask t = cic.getClauseMatrix(c);
//                enqueue(q, t);
//            }
//        }
 //       System.out.println("starting proof");
        index = 0;
        int n = TermParser.fsmap.size() + TermParser.psmap.size();
        Trie t = new Trie(n);
        do {

            generateELs(inter, elclause);
    //        System.out.println("ELs: " + inter.size());
            if (verbose) {
                Clause cl = new Clause();
                cl.addAll(inter);
                for (Literal el : cl) {
                    System.out.println("EL: " + el);
                }
            }
  //          System.out.println("running mip");
            Clause mgi;
            mgi = findByPartialDisunification(list, inter.inter, t, env);
            if (mgi == null) {
                return true;
            }

    //        System.out.println("selected instance " + mgi);
            elclause.add(mgi);
            index++;
            if (verbose) {
                for (Clause c : elclause) {
                    System.out.println("Clause to simplify: " + c);
                }
            }
//            System.out.println("running simp");
            Pair<ClauseList, Pair<Set<Literal>, Set<Literal>>> nelclause = IterativeSimpProcedure.simp(elclause);
            if (nelclause.fst().contains(EMPTY_CLAUSE)) {
                break;
            }
            Pair<Set<Literal>, Set<Literal>> elu = nelclause.snd();
            for (Literal l : elu.fst()) {
                if (verbose) {
                    System.out.println("add " + l);
                }
                t.addNode(l);
            }
            for (Literal l : elu.snd()) {
                if (verbose) {
                    System.out.println("delete " + l);
                }
                t.deleteNode(l);
            }
//            System.out.println("EL updates: +" + elu.fst().size() + ", -" + elu.snd().size());
            if (verbose) {
                System.out.println("Trie: " + t);
            }
            elclause = nelclause.fst();

        } while (!elclause.contains(EMPTY_CLAUSE));
        return false;
    }
//    private ClauseInstanceCache cic = new ClauseInstanceCache();
    private Clause findByPartialDisunification(Collection<Clause> clauses, Interpretation initial, Trie t, TypeEnv env) {
        Clause candidate = null;
        InstanceIteratorProcedure_Typed_PD_RO_Lazy iip;

        List<Literal> l2;
//        Clause candidate2;
        ClauseList cl = new ClauseList();
        cl.addAll(clauses);
        InstanceIteratorProcedure_Typed_PD_RO_Lazy.verbose = verbose;
        List<InstanceIteratorProcedure_Typed_PD_RO_Lazy.IITask> iitcol = new ArrayList<InstanceIteratorProcedure_Typed_PD_RO_Lazy.IITask>();
        for (Clause c : cl) {
            ArrayList<Literal> trueLitList = new ArrayList<Literal>();
            ArrayList<Literal> falseLitList = new ArrayList<Literal>();
            for (Literal l : c) {
                if (initial.interpret(l)) {
                    trueLitList.add(l);
                } else {
                    falseLitList.add(l);
                }
            }
            if (verbose) {
                System.out.println("Looking for candidate in instances of " + c + "\n" + "True: " + trueLitList + "\n" + "False: " + falseLitList);
            }
            iip = new InstanceIteratorProcedure_Typed_PD_RO_Lazy();
            InstanceIteratorProcedure_Typed_PD_RO_Lazy.IITask iipt;
            iipt = (InstanceIteratorProcedure_Typed_PD_RO_Lazy.IITask) iip.create();
            iipt.input(new Triple(trueLitList, falseLitList, t));
            iitcol.add(iipt);
        }
        List<InstanceIteratorProcedure_Typed_PD_RO_Lazy.IITask> iitcol2 = new ArrayList<InstanceIteratorProcedure_Typed_PD_RO_Lazy.IITask>();
        for (int max = minLenght; max < MAX_CLAUSE_LENGTH; max++) {
            if (verbose) {
                System.out.println("bound: " + max);
                System.out.println("clauses: " + cl);
            }
            InstanceIteratorProcedure_Typed_PD_RO_Lazy.max = max;

            iitcol2.clear();
            iitcol2.addAll(iitcol);
            for (InstanceIteratorProcedure_Typed_PD_RO_Lazy.IITask iipt : iitcol2) {
                iipt.reset();
                if (iipt.hasNext()) {
                    try {
                        l2 = iipt.next().run();
                        Clause ncandidate = new Clause();
                        ncandidate.addAll(l2);
                        if (verbose) {
                            System.out.println("candiate: " + candidate);
                        }
                        if(candidate == null || ClauseComparator.compareClauses(ncandidate, candidate) < 0)
                        candidate = ncandidate;
                    } catch (SearchFailureException ex) {
                        Logger.getLogger(ProofProcedure_Typed_PD_RO_Lazy.class.getName()).log(Level.SEVERE, null, ex);
                    }
                }
                if (iipt.isNoMoreSolutions()) {
                    iitcol.remove(iipt);
                    if (verbose) {
                        System.out.println("No more solutions: " + iipt.C);
                    }
                }
            }
            if(candidate!=null) {
                return candidate;
            }
            if (iitcol.isEmpty()) {
                return null;
            }
        }
        throw new RuntimeException("ERROR: Max clause length.");
    }

    private void reorderGULiterals(ClauseList input) {

        //extract ground literals
        List<Literal> guLiterals = new ArrayList<Literal>();
        List<Literal> gLiterals = new ArrayList<Literal>();
        for (Clause c : input) {
            if (c.size() == 1 && Util.FV(c).isEmpty()) {
                guLiterals.add(c.first());
            } else if (Util.FV(c).isEmpty()) {
                gLiterals.addAll(c);
            }
        }
        for (Literal l : guLiterals) {
            LiteralComparator.addGroundUnitLiterals(l.sign ? l : l.negate());
            LiteralComparator.addGroundUnitLiterals(!l.sign ? l : l.negate());
        }
        for (Literal l : gLiterals) {
            LiteralComparator.addGroundUnitLiterals(l.sign ? l : l.negate());
            LiteralComparator.addGroundUnitLiterals(!l.sign ? l : l.negate());
        }
    }

    private void generateELs(ELInterpretation inter, ClauseList elclause) {
        inter.clear();
        for (Clause c : elclause) {
            inter.add(c.max());
        }

    }

    private void monotype(TypeEnv env, SortedSet<FunctionSymbol> fs, ClauseList inputDisjoint) {
        TVar vtype;
        vtype = new TVar(env);
        TermTypedGenerator tti = new TermTypedGenerator(vtype.getName(), env);
        List<VariableSymbol> vs = TypeInferenceEngine.variables(inputDisjoint);
        for (FunctionSymbol f : fs) {
            Tuple t = new EmptyTuple();
            for (int i = 0; i < f.arity; i++) {
                t = new Tuple(vtype, t);
            }
            Object type;
            if (f.arity > 0) {
                type = new TFunction(t, vtype);
            } else {
                type = vtype;
            }

            env.bindType(f.name, type); // bind type to env

            tti.addFunctionSymbol(f);
        }

        for (VariableSymbol f : vs) {
            Object type;
            type = vtype;

            env.bindType(f.name, type); // bind type to env
        }
    }

    private void typeInference(ClauseList input, SortedSet<FunctionSymbol> fs, ClauseList inputDisjoint, TypeEnv env) {
        // type inference
        TypeInferenceEngine.typeinference(input, env);
        for (FunctionSymbol f : fs) {
            Object type = env.getType(f.name);
            type = jfp.type.UtilQuick.getTypeInstance(type, env);
            env.bindType(f.name, type); // rebind type to the dereferenced form
            TVar vtype;
            if (type instanceof TFunction) {
                TFunction ftype = (TFunction) type;
                vtype = (TVar) ftype.getRangeType();
            } else {
                vtype = (TVar) type;
            }
            if (TermTypedGenerator.getTTIMap().containsKey(vtype.getName())) {
                TermTypedGenerator.getTTIMap().get(vtype.getName()).addFunctionSymbol(f);
            } else {
                TermTypedGenerator tti = new TermTypedGenerator(vtype.getName(), env);
                tti.addFunctionSymbol(f);
            }
//            System.out.println(f + " : " + type);
        }
        for (PredicateSymbol f : TermParser.psmap.values()) {
            Object type = env.getType(f.name);
            type = jfp.type.UtilQuick.getTypeInstance(type, env);
            env.bindType(f.name, type); // rebind type to the dereferenced form
            TVar vtype;

//            System.out.println(f + " : " + type);
        }
        List<VariableSymbol> vs = TypeInferenceEngine.variables(inputDisjoint);
//        for (VariableSymbol v : vs) {
//            System.out.println(v + " : " + jfp.type.UtilQuick.getTypeInstance(env.getType(v.name), env));
//        }
        // add a constant to those types that is inhabited by no function symbol
        Set<String> usedType = new HashSet<String>();
        for (String ps : Util.variables(input)) {
            Object type = env.getType(ps);
            type = jfp.type.UtilQuick.getTypeInstance(type, env);
            env.bindType(ps, type); // rebind type to the dereferenced form
            try {
                if (type instanceof TFunction) {
                    TFunction ftype = (TFunction) type;
                    Object[] pts = ((Tuple) ftype.getDomainType()).toArray();
                    for (Object t : pts) {
                        usedType.add(((TVar) t).getName());
                    }
                    usedType.add(((TVar) ftype.getRangeType()).getName());
                } else if (type instanceof TVar) {
                    usedType.add(((TVar) type).getName());
                }
            } catch (Exception e) {
//                e.printStackTrace();
            }
        }
//        System.out.println("types: " + usedType);
        for (String t : usedType) {
            if (!TermTypedGenerator.getTTIMap().containsKey(t)) {
                TermTypedGenerator ti = new TermTypedGenerator(t, env);
                TermParser.addFunctionSymbol(t, 0);
                FunctionSymbol f = TermParser.fsmap.get(t);
                ti.addFunctionSymbol(f);
//                System.out.println("symbol " + t + " added as a function symbol of type " + t);
            }
        }
    }

    ClauseList renameVariables(
            ClauseList cl) {
        return Util.renameVariables(cl);
    }
    final static boolean verbose = false;
}
