import os.path
import subprocess
import suspender
import re

def rel_to_abs(path):
	cur=os.path.dirname(__file__)
	return os.path.join(cur, path)	


class Engine(object):
	def get_cmd_line(self, **kwargs):
		pass
	def run(self, **kwargs):
		cmdl=self.get_cmd_line(**kwargs)
		subprocess.call(cmdl)
	def run_into_file(self, output_file, **kwargs):
		cmdl=self.get_cmd_line(**kwargs)
		
		out = open(output_file, 'w')
		try:
			subprocess.call(cmdl, stdout=out)
		finally:
			out.close()

	def get_process(self, output_file, **kwargs):
		cmdl=self.get_cmd_line(**kwargs)
		return suspender.Process(cmdl, output_file)
	def update_proof_axioms(self, output_file, axiom_set_set):
		axs=axiom_set_set.get_axioms()
		names=set(get_matches_groups(output_file, 'file\([^,]*, ([^)]*)\)'))
		for name in names:
			ax=axs.get(name, None)
			if ax:
				axiom_set_set.proof_axioms.add(ax)
	def proof_complete(self,output_file):
		pass
		
	@classmethod
	def get(cls, name):
		if name=='vampire':
			return Vampire()
		elif name=='eprover':
			return EProver()
class Vampire(Engine):
	executable=rel_to_abs('./JUSource/jumpirefix')
	def get_cmd_line(self, problem_file, time):
		return [self.executable, '-t', str(int(time)), '--output_syntax', 'tptp', problem_file]
	def proof_complete(self,output_file):
		return len(get_matches_groups(output_file, ' PROVED ---'))>0
		
class EProver(Engine):
	executable=rel_to_abs('./Source/PROVER/eproof.working')
	def get_cmd_line(self, problem_file, time):
		return [self.executable, '-s', '-tLPO4', '-xAuto','-tAuto','--memory-limit=768', '--tptp3-format', '--cpu-limit=%s' % int(time), problem_file]

	def get_process(self, output_file, **kwargs):
		cmdl=self.get_cmd_line(**kwargs)
		return suspender.Process(cmdl, output_file, handle_subproc=True)
	
	def proof_complete(self,output_file):
		return len(get_matches_groups(output_file, '# SZS status Theorem'))>0
	
	
def get_matches_groups(fname, pattern):
	exp=re.compile(pattern)

	res=[]	
	inp = open(fname)
	try:
		for line in inp:
			res.extend(exp.findall(line))
	finally:
		inp.close()
	return res
	