import tptp
import sets

class SelectionParams(object):
	def __init__(self, select_proof_axioms=False, benevolence=1):
		self.select_proof_axioms=select_proof_axioms
		self.benevolence=benevolence
	def selector_matches(self, selector):
		return self.benevolence==selector.params.benevolence
	def __eq__(self, other):
		return self.select_proof_axioms==other.select_proof_axioms and self.benevolence==other.benevolence
	def __hash__(self):
		return hash(self.select_proof_axioms)^hash(self.benevolence)
	def __str__(self):
		return 'PA: %s, Ben: %s' % (self.select_proof_axioms, self.benevolence)
		


class Selector(object):
	def __init__(self,axioms=None, axiom_set_set=None, params=SelectionParams()):
		self.params=params
		if axioms==None:
			axioms=axiom_set_set.get_axioms()
			
		benevolence=self.params.benevolence
		
		in_degrees={}
		for ax in axioms.itervalues():
			for s in ax.symbols:
				if s in in_degrees:
					in_degrees[s]+=1
				else:
					in_degrees[s]=1
		self.in_degrees=in_degrees
		
		self.axioms_without_symbols=[]
		deps={}
		for ax in axioms.itervalues():
			if not ax.symbols:
				self.axioms_without_symbols.append(ax)
				continue
			
			dsym=[]
			dsym_ind=5000000 #number bigger than any possible degree
			for s in ax.symbols:
				s_ind=in_degrees.get(s,0)
				if dsym_ind>s_ind:
					dsym=[s]
					dsym_ind=s_ind
				elif dsym_ind==s_ind:
					dsym.append(s)
					
			if benevolence>1:
				treshold=dsym_ind*benevolence
				dsym=filter(lambda s:in_degrees.get(s,0)<=treshold, ax.symbols)
			
			for s in dsym:
				if s in deps:
					deps[s].append(ax)
				else:
					deps[s]=[ax]
		self.deps=deps
	
	
	def get_required_symbols(self, symbols):
		deps=self.deps
		res=sets.Set(symbols)
		for s in symbols:
			for ax in deps.get(s,[]):
				res.update(ax.symbols)
		if len(res)==len(symbols):
			return res
		else:
			return self.get_required_symbols(res)
		
	def select_for_symbols(self, symbols):
		deps=self.deps
		rq_syms=self.get_required_symbols(symbols)
		axioms=sets.Set()
		for s in rq_syms:
			for ax in deps.get(s,[]):
				axioms.add(ax)
		axioms.update(self.axioms_without_symbols)
		return axioms
	
	def select_for_problem(self, problem):
		forms=problem.own_formulas
		symbols=sets.Set()
		for f in forms.itervalues():
			symbols.update(f.symbols)
			
		selected=self.select_for_symbols(symbols)
		
		return selected


selectors={}

def get_selector(axss, params):
	import strategy

	if axss not in selectors:
		selectors[axss]=[]
	res=None
	for sel in selectors[axss]:
		if params.selector_matches(sel):
			res=sel
			break
	else:
		print '--creating new selector for %s' % axss
		strategy.Timer.get_instance().start_billing(axss)
		try:
			res=Selector(axiom_set_set=axss, params=params)
			selectors[axss].append(res)
		finally:
			strategy.Timer.get_instance().stop_billing(axss)
	return res

def select_formulas(problem, params=SelectionParams()):
	axss=problem.axiom_sets
	sel=get_selector(axss, params)
	forms=sel.select_for_problem(problem)
	if params.select_proof_axioms:
		forms.update(axss.proof_axioms)
		
	res=list(forms)
	res.extend(problem.own_formulas.itervalues())
	return res

		
