#!/usr/bin/python

import sys
import re
import os
import os.path
import tempfile
import shutil
from optparse import OptionParser

import engines
import tptp
import strategy
import selection
import signal


def load_problems(batch_fname):
	problems=[]
	inp = open(batch_fname)
	try:
		for line in inp:
			line=line.strip()
			if not line:
				continue
			prob_fname, out_fname=line.split(' ')
			problems.append(tptp.Problem(prob_fname, out_fname))
	finally:
		inp.close()
	return problems

def group_problems_by_axioms(problems):
	res={}
	for prb in problems:
		#key=reduce(lambda x,y:x+y, prb.axioms)
		key=prb.axiom_sets
		if not key in res:
			res[key]=set()
		res[key].add(prb)
	return res

def solve_problem_group(axiom_set_set, problems):
	selected_fname=temp_dir+'/req'
	
	selector=selection.Selector(axiom_set_set=axiom_set_set)
	for prb in problems:
		forms=selector.select_for_problem(prb)
		tptp.write_forms_to_file(forms, selected_fname)
		engine.run_into_file(prb.out_fname, problem_file=selected_fname, time=opts.time)


def force_quit(*args):
	shutil.rmtree(temp_dir)
	os._exit(os.EX_OK)

signal.signal(signal.SIGXCPU, force_quit)
signal.signal(signal.SIGALRM, force_quit)

temp_dir=tempfile.mkdtemp()
#temp_dir='tmp'
tptp.fname_dir=temp_dir

opt_parser=OptionParser()
opt_parser.add_option('-t', '--time', type='int', default=0)
opt_parser.add_option('-e', '--engine', type='string', default='vampire')
opt_parser.add_option('-p', '--axiom_prefix', type='string', default='')
opt_parser.add_option('-b', '--bypass', action="store_true")

opts, args=opt_parser.parse_args()

axiom_prefix=opts.axiom_prefix
if not axiom_prefix:
	if 'TPTP' in os.environ:
		axiom_prefix=os.environ['TPTP']
if axiom_prefix:
	tptp.AxiomSet.file_prefix=axiom_prefix

engine=engines.Engine.get(opts.engine)

batch_fname=args[0]
problems=load_problems(batch_fname)		
prb_grps=group_problems_by_axioms(problems)

time=opts.time
if not time:
	time=len(problems)*300

strategy.work(time, prb_grps, engine)


force_quit()