% properties of orderings and successor functions
fof(successor,axiom,
    ! [I] : greater(s(I),I)).
fof(order_transitive,axiom, 
    ! [I,J,K] : (greater(I,J) & greater (J,K) => greater(I,K))). 
fof(order_irreflexive,axiom,  
    ! [I] : ~greater(I,I)). 
fof(order_total,axiom,   
    ! [I,J] : (greater(I,J) | greater(J,I) | I = J)).

% definition of <=
fof(geq,axiom, 
    ! [I,J] : (geq(I,J) <=> greater(I,J) | I = J)). 

% definition of iteration
fof(iteration,axiom,  
    ! [I] : (iteration(I) <=> geq(I,zero) & greater(n,I))). 
fof(loop_counter_limits,axiom,
    geq(n,zero)).
 
% recurrences derived by ALIGATOR
fof(rec_a,axiom,
    ! [I] : a(I) = I).
fof(rec_bc,axiom, 
    ! [I] : plus(b(I),c(I)) = I). 

% initial conditions on b,c
fof(init_b,axiom,b(zero) = zero).
fof(init_c,axiom,c(zero) = zero).

% translation of the guarded commands from the loop
fof(gc1,axiom, 
    ! [I] : (geq(arrA(a(I)),zero) =>
               arrB(s(I),b(I)) = arrA(a(I)) &
               b(s(I)) = s(b(I)) &
               c(s(I)) = c(I))). 
fof(gc2,axiom,  
    ! [I] : (~geq(arrA(a(I)),zero) => 
               arrC(s(I),c(I)) = arrA(a(I)) & 
               c(s(I)) = s(c(I)) & 
               b(s(I)) = b(I))).

% definition of update set of arrays B,C
fof(update_set_B,axiom,  
    ! [I,P] : (updB(I,P) <=> geq(arrA(a(I)),zero) & P = b(I))).
fof(update_set_C,axiom,   
    ! [I,P] : (updC(I,P) <=> ~geq(arrA(a(I)),zero) & P = c(I))). 
fof(update_set_B_def,axiom,    
    ! [P] : (updB(P) <=> ? [I] : updB(I,P))).  
fof(update_set_C_def,axiom,   
    ! [P] : (updC(P) <=> ? [I] : updC(I,P))). 

% monotonicity properties derived from the loop form
fof(monotonic_b,axiom,  
    ! [I,J] : geq(I,J) => geq(b(I),b(J))).  
fof(monotonic_c,axiom, 
    ! [I,J] : geq(I,J) => geq(c(I),c(J))).

% arrays are stable outside of their update sets
fof(stable_B,axiom,    
    ! [P] : (~updB(P) => ! [J] : arrB(J,P) = arrB(zero,P))). 
fof(stable_C,axiom,
    ! [P] : (~updC(P) => ! [J] : arrC(J,P) = arrC(zero,P))).

% property: array B does not change on negative indexes
%fof(stable_B_does_not_change_on_negative,conjecture,
%    ! [P] : (greater(zero,P) => arrB(zero,P) = arrB(n,P))).
% property: array B does not change on indexes greater than or equal to b(n+1)
fof(stable_B_does_not_change_on_large,conjecture, 
    ! [P] : (greater(P,n) => arrB(zero,P) = arrB(n,P))). 
 

