#!/usr/bin/perl -w

use strict;

# global constants
my $DRACULA = '/home/tptp/Systems/Vampire---10.0/dracula';
my $VAMPIRE = '/home/tptp/Systems/Vampire---10.0/vampire';
my $SWIPL = '/home/tptp/Systems/Muscadet---3.0/swi-prolog/bin/i386/pl';
my $TMP = '/tmp';

# declared global since should be cleaned on SIGXCPU
my $prologFile;
my $swiOut;
my $tptpOut;

sub clean {
  unlink $prologFile;
  unlink $swiOut;
  unlink $tptpOut;
}

$SIG{XCPU} = &clean;

my $solved = 0;

# standard code for reaping dead children
use POSIX ":sys_wait_h";
sub REAPER {
  my $child;
  while (($child = waitpid(-1,WNOHANG)) > 0) {
  }
  $SIG{CHLD} = \&REAPER;
}
$SIG{CHLD} = \&REAPER;

sub randomString {
  my ($n) = @_;
  my @chars = ("A" .. "Z", "a" .. "z", 0 .. 9);

  return join "", @chars [map { rand @chars } (1..$n)];
} # randomString

my $exe = $ENV{'_'};

my $SPIDER = 0;
# ids for rule-based executions

if (@ARGV < 2) {
  die "Usage: ltb time_limit file_name <include_dir>\n";
}

my $timeLimit = $ARGV[0];
my $inputFile = $ARGV[1];

my $incDir = $ARGV[2] || $ENV{'TPTP'};
if ($ENV{'TPTP'}) {
  $ENV{'TPTP_DIR'} = "$ENV{'TPTP'}/";
}
my $include = $incDir ? "--include $incDir/" : '';

# end of the time for running
my $start = time();
my $end = $start + $timeLimit;

my @pool = qw(lrs+10_4:3_bas=on_bs=off_ep=on_nwc=1.3_600 lrs+10_10:1_bs=off_fde=none_lcm=predicate_nwc=1.5_600 lrs+13_3:1_bas=on_ep=exp1_600 lrs+1011_2:1_bas=on_bd=off_bs=off_ep=RS_lcm=predicate_nwc=4.5_600 lrs+16_3:1_bas=on_bd=off_bs=off_fde=none_lcm=kinky_nwc=1.6_sp=on_600 lrs+1012_5:4_bas=on_bs=off_ep=on_fde=none_600 lrs+16_7:1_bd=off_bs=off_ep=exp1_nwc=2.0_600 lrs+1010_4:1_bs=off_lcm=predicate_nwc=1.1_600 lrs+11_6_bs=off_ep=exp1_fde=none_lcm=predicate_nwc=1.2_600);

my %times = ();
my %out = ();
my %strat = ();
my %pool = ();
my %problems = ();

sub drakoshaMustWin {
  # reading the input file
  open INP, "<$inputFile" or die "cannot open $inputFile!";
  while (my $line = <INP>) {
    my ($in,$out) = $line =~ m/\A\s*(\S+)\s+(\S+)\s*\Z/;
    if (defined $out) {
      $times{$in} = 0;
      $out{$in} = $out;
      $problems{$in} = 1;
    }
  }
  close INP;

  # finding properties
  foreach my $in (keys %problems) {
    my $drak = "$DRACULA --time_limit $timeLimit --mode profile $include $in";
    my $props = `$drak`;
    print "$in: $props";
    my ($cat,$prop,$atoms) = $props =~ m/\A(\S+)\s+(\d+)\s+(\d+)\s*\Z/;
    my @strat;
    if ($prop == 1) {
      @strat = qw(lrs+10_2:1_bs=off_ep=exp1_fde=none_lcm=predicate_nwc=1.8_600 852 dis+10_41_bas=on_bs=off_ep=RS_lcm=predicate_nwc=2.4_sp=on_600 397 rule 600 dis+1012_128_lcm=predicate_nwc=3.3_600 50 lrs-11_5_bas=on_bs=off_ep=RST_fde=none_nwc=1.7_sos=on_1200 169 dis+11_19_bas=on_bd=off_ep=exp1_lcm=predicate_nwc=2.3_900 130 lrs+1010_4:1_bs=off_lcm=predicate_nwc=1.1_600 273 lrs+16_4:1_bas=on_bs=off_ep=exp1_lcm=kinky_nwc=1.2_600 8 lrs+10_4:3_bas=on_bs=off_ep=on_nwc=1.3_600 48 lrs+17_2:1_bas=on_bs=off_ep=exp1_lcm=kinky_nwc=1.5_600 120 dis+1012_16_bas=on_bs=off_ep=R_fde=none_nwc=2.2_1500 1324 lrs+1013_4_bas=on_bs=off_ep=R_fde=none_lcm=predicate_nwc=1.7_600 339 dis-11_34_bd=off_bs=off_ep=exp1_fsr=off_nwc=1.2_wi=on_900 418 lrs+16_27_bas=on_bd=off_bs=off_ep=exp1_nwc=1.2_600 809 lrs+10_27_bas=on_bs=off_ep=R_nwc=3.2_sos=on_6000 4894 lrs+1012_64_bas=on_bs=off_ep=RST_lcm=predicate_nwc=1.6_stl=600_wi=on_1500 1462); # (total time: 11443)
    }
    else {
      @strat = qw(lrs+10_4:3_bas=on_bs=off_ep=on_nwc=1.3_600 154 lrs+16_4:1_bas=on_bs=off_ep=exp1_lcm=kinky_nwc=1.2_600 107 rule 600 lrs+1010_4:1_bs=off_lcm=predicate_nwc=1.1_600 83 dis+1012_128_lcm=predicate_nwc=3.3_600 53 lrs+16_7:1_bd=off_bs=off_ep=exp1_nwc=2.0_600 62 lrs+11_0_bas=on_bd=off_bs=off_ep=exp1_fde=none_lcm=kinky_nwc=2.3_600 33 lrs+10_2:1_bs=off_ep=exp1_fde=none_lcm=predicate_nwc=1.8_600 238 lrs+8_14_bas=on_bd=preordered_bs=off_lcm=kinky_sp=on_wi=on_600 262 dis+1010_3:2_bd=off_ep=exp1_fde=none_lcm=predicate_nwc=4.0_900 155 lrs+17_0_bas=on_ep=on_fde=none_lcm=predicate_nwc=2.0_600 112 lrs+11_6_bs=off_ep=exp1_fde=none_lcm=predicate_nwc=1.2_600 136 lrs+16_6:1_bas=on_bd=off_bs=off_ep=RS_lcm=predicate_nwc=1.4_600 212 lrs+10_27_bas=on_bs=off_ep=R_nwc=3.2_sos=on_6000 454); # (total time: 2191)
    }

    $strat{$in} = \@strat;
    my @extra = @pool;
    $pool{$in} = \@extra;
  }

  # run all strategies
  while (time() < $end) {
    my @problems = sort {$times{$a} <=> $times{$b}} keys %problems;
    my $problems = @problems;
    print "$problems problems remain\n";
    if (! @problems) {
      return;
    }
    my $prb = $problems[0];
    my $soFar = $times{$prb};
    print "$prb with time $soFar chosen\n";

    my $strats = $strat{$prb};
    if (! @$strats) {
      # remove the problem, TEMPORARY, should be replaced by adding a new strategy
      failed1($prb);
      next;
    }
    my $outFile = $out{$prb};
    open OUT, ">>$outFile" or die "cannot open $outFile!";
    my $strat = shift @$strats;
    my $timeSlice = shift @$strats;
    my $time = time();
    my ($stl) = $strat =~ m/_(\d+)\Z/;
    if ($time + $timeSlice/10 > $end) {
      $timeSlice = ($end-$time)*10;
      $stl = '';
    }
    else {
      # add 20% to stl
      if ($stl) {
	$stl = $stl/10;
	$stl = int($stl * 1.2);
	$stl = "-stl $stl";
      }
      else {
	$stl = '';
      }
      my $extra = int ($timeSlice/60);
      if ($extra < 10) {
	$extra = 10;
      }
      $timeSlice += 10;
    }
    my $sliceLimit = $timeSlice/10;
    # add 20% to slice limit
    $sliceLimit = int($sliceLimit * 1.2);

    print OUT "% Running slice $strat for $timeSlice deciseconds...\n";
    if ($strat eq 'rule') {
      my $RID = randomString(10);
      $prologFile = "$TMP/$RID.pl";
      my $endTime = time() + $sliceLimit;
      my $dracula = "$DRACULA --normalize off --time_limit $sliceLimit --mode rule $prb > $prologFile";
      print "$dracula\n";
      my $draculaResult = `$dracula`;
      print OUT $draculaResult;
      $swiOut = "$TMP/$RID.out";
      my $swipl = "$SWIPL -f $prologFile -g true -t gg > $swiOut";
      print "$swipl\n";
      my $pid = fork();
      unless (defined $pid) {
	die "cannot fork!";
      }
      if ($pid) { # parent
#	print "Child $pid started!\n";
      }
      else { # child
	my $swiResult = `$swipl`;
	exit();
      }

      my $killed;
      my $proved;
      my $proof;
      while (kill 0 => $pid) { # child still alive
#	print "Alive!\n";
	sleep 1;
	if (time() > $endTime) {
	  local $SIG{HUP} = 'IGNORE';
#	  print "Killing child!\n";
	  kill HUP => -$$;
	  $killed = 1;
	  last;
	}
      }
      if (! $killed) {
	if (open SWI, "<$swiOut") {
	  while (my $line = <SWI>) {
	    print OUT $line;
	    print $line;
	    if ($line =~ m/PROVED/) {
	      $proved = 1;
	      ($proof) = $line =~ m/PROVED\s+(\S*)/;
	    }
	  }
	  close SWI;
	  my $spent = time() - $time + 1;
	  $times{$prb} += $spent;
	  if ($proved) {
	    my $collectClauses = "$DRACULA --time_limit 100 --mode output --log_file $proof $prb";
	    print "$collectClauses\n";
	    my $tptp = `$collectClauses`;
	    $tptpOut = "$TMP/$RID.tptp";
	    open TPTP, ">$tptpOut";
	    print TPTP $tptp;
	    close TPTP;

	    my $vamp = "$VAMPIRE --time_limit 60 --mode vampire $tptpOut";
	    print "$vamp\n";
	    my $runResult = `$vamp`;
	    print OUT "%%%%% $runResult";
	    my $total = $spent + $times{$prb};
	    print OUT "%%%%% Total time: $total\n";
	    remove($prb);
	    $solved++;
	    print "$solved problems solved!\n"
	  }
	}
      }
      clean();
    }
    else {
      my $mode = 'vampire';
      my $command = "$VAMPIRE --decode $strat --normalize on --time_limit $sliceLimit $stl --mode $mode $prb";
      print "$command\n";
      my $runResult = `$command`;
      my $spent = time() - $time + 1;
      $times{$prb} += $spent;
      if ($runResult =~ m/Refutation found/s) {
	my $total = $spent + $times{$prb};
	print OUT "%%%%% $runResult",
	          "%%%%% Total time: $total\n";
	remove($prb);
	$solved++;
	print "$solved problems solved!\n"
      }
    }
    close OUT;
  }
  foreach my $problem (keys %problems) {
    failed($problem);
    return;
  }
} # drakoshaMustWin

sub remove {
  my ($prb) = @_;

  print "Problem $prb removed\n";

  delete $problems{$prb};
  delete $out{$prb};
  delete $times{$prb};
  delete $strat{$prb};
}

sub failed1 {
  my ($prb) = @_;
  delete $problems{$prb};
}

sub failed {
  my ($prb) = @_;

  print "Problem $prb failed\n";

  my $outFile = $out{$prb};
  my $total = $times{$prb};

  delete $problems{$prb};
  delete $times{$prb};
  delete $out{$prb};
  delete $strat{$prb};

  open OUT, ">>$outFile";
  print OUT "%%%%% I hate to say that I cannot prove this problem.
%%%%% Why? Well, Geoff set the time limit to prevent me
%%%%% from proving it!
%%%%% SZS status Unknown %%%%%
%%%%% Total time: $total\n";
  close OUT;
}

sub drakoshaNeverGivesUp {
  # run all strategies

  for (;;) {
    my @problems = keys %times;
    last if ! @problems;
    my $problems = @problems;
    my $sliceLimit = int ($end + $problems - 1 - time()) / $problems;
    for my $prb (@problems) {
      my $time = time();
      if ($time >= $end) {
	return;
      }
      print "$prb chosen\n";

      my $strats = $pool{$prb};
      if (! @$strats) {
	failed($prb);
	next;
      }

      my $outFile = $out{$prb};
      open OUT, ">>$outFile" or die "cannot open $outFile!";
      my $strat = shift @$strats;
      my $timeSlice = $sliceLimit * 10;

      print OUT "% Running slice $strat for $timeSlice deciseconds...\n";
      my $mode = 'vampire';
      my $command = "$VAMPIRE --decode $strat --normalize off --time_limit $sliceLimit --mode $mode $prb";
      print "$command\n";
      my $runResult = `$command`;
      my $spent = time() - $time + 1;
      $times{$prb} += $spent;

      if ($runResult =~ m/Refutation found/s) {
	my $total = $spent + $times{$prb};
	print OUT "%%%%% $runResult",
                  "%%%%% Total time: $total\n";
	remove($prb);
	$solved++;
	print "$solved problems solved!\n"
      }
      close OUT;
    }
  }
  foreach my $problem (keys %times) {
    failed($problem);
    return;
  }
} # drakoshaNeverGivesUp

drakoshaMustWin();
drakoshaNeverGivesUp();

