type token =
  | IDENT of (string)
  | STRING of (string)
  | NUM of (string)
  | BANG_
  | PERCENT_
  | AMPER_
  | AMPER_AMPER_
  | LPAREN_
  | LPAREN_RPAREN_
  | RPAREN_
  | STAR_
  | PLUS_
  | PLUS_PLUS_
  | COMMA_
  | DASH_
  | DASH_GT_
  | PERIOD_
  | PERIOD_LPAREN_
  | PERIOD_PERIOD_
  | SLASH_
  | SLASH_BACKSL_
  | COLON_
  | COLON_COLON_
  | COLON_LT_
  | COLON_EQ_
  | COLON_GT_
  | SEMI_
  | LT_
  | LT_DASH_
  | LT_DASH_GT_
  | LT_COLON_
  | LT_EQ_
  | LT_GT_
  | EQ_
  | EQ_GT_
  | EQ_UNDER_D_
  | GT_
  | GT_DASH_GT_
  | GT_EQ_
  | QUEST_
  | QUEST_EQ_
  | AROBAS_
  | LBRACK_
  | BACKSL_SLASH_
  | RBRACK_
  | HAT_
  | LBRACE_
  | BAR_
  | BAR_DASH_
  | BAR_BAR_
  | RBRACE_
  | TILDE_
  | MUSTUSE
  | DEFINITION
  | DEPENDS
  | ELSE
  | END
  | EXISTS
  | FALSE
  | FORALL
  | FUN
  | IF
  | IN
  | INDUCTIVE
  | LET
  | MATCH
  | ON
  | PARAMETER
  | THEN
  | THEOREM
  | TRUE
  | WITH
  | BEGINPROOF
  | BEGINNAME of (string)
  | BEGINHEADER
  | ENDPROOF
  | EOF

open Parsing;;
# 4 "parsecoq.mly"
Version.add "$Id: parsecoq.mly,v 1.13 2006-07-20 13:19:21 doligez Exp $";;

open Printf;;

open Expr;;
open Namespace;;
open Phrase;;

let rec mk_type_string e =
  match e with
  | Evar (s, _) -> s
  | Emeta _ -> assert false
  | Eapp (s, args, _) ->
      List.fold_left (fun s a -> sprintf "%s %s" s (mk_type_string a)) s args
  | _ -> assert false (* FIXME TODO *)
;;

let mk_eapp (s, args) =
  match (s, args) with
  | "and", [e1; e2] -> eand (e1, e2)
  | "or", [e1; e2] -> eor (e1, e2)
  | "not", [e1] -> enot (e1)
  | _ -> eapp (s, args)
;;

let mk_apply (e, l) =
  match e with
  | Eapp (s, args, _) -> mk_eapp (s, args @ l)
  | Evar (s, _) -> mk_eapp (s, l)
  | _ -> raise Parse_error
;;

let mk_all bindings body =
  let f (var, ty) e = eall (evar var, ty, e) in
  List.fold_right f bindings body
;;

let mk_ex bindings body =
  let f (var, ty) e = eex (evar var, ty, e) in
  List.fold_right f bindings body
;;

let mk_lam bindings body =
  let f (var, ty) e = elam (evar var, ty, e) in
  List.fold_right f bindings body
;;

let rec get_params e =
  match e with
  | Elam (v, _, body, _) ->
      let (p, e1) = get_params body in
      (v :: p, e1)
  | _ -> ([], e)
;;

let mk_let id expr body =
  substitute [(evar id, expr)] body
;;

let rec mk_pattern accu l =
  match l with
  | [] -> assert false
  | [c] -> eapp (c, accu)
  | h::t -> mk_pattern (evar h :: accu) t
;;

# 149 "parsecoq.ml"
let yytransl_const = [|
  260 (* BANG_ *);
  261 (* PERCENT_ *);
  262 (* AMPER_ *);
  263 (* AMPER_AMPER_ *);
  264 (* LPAREN_ *);
  265 (* LPAREN_RPAREN_ *);
  266 (* RPAREN_ *);
  267 (* STAR_ *);
  268 (* PLUS_ *);
  269 (* PLUS_PLUS_ *);
  270 (* COMMA_ *);
  271 (* DASH_ *);
  272 (* DASH_GT_ *);
  273 (* PERIOD_ *);
  274 (* PERIOD_LPAREN_ *);
  275 (* PERIOD_PERIOD_ *);
  276 (* SLASH_ *);
  277 (* SLASH_BACKSL_ *);
  278 (* COLON_ *);
  279 (* COLON_COLON_ *);
  280 (* COLON_LT_ *);
  281 (* COLON_EQ_ *);
  282 (* COLON_GT_ *);
  283 (* SEMI_ *);
  284 (* LT_ *);
  285 (* LT_DASH_ *);
  286 (* LT_DASH_GT_ *);
  287 (* LT_COLON_ *);
  288 (* LT_EQ_ *);
  289 (* LT_GT_ *);
  290 (* EQ_ *);
  291 (* EQ_GT_ *);
  292 (* EQ_UNDER_D_ *);
  293 (* GT_ *);
  294 (* GT_DASH_GT_ *);
  295 (* GT_EQ_ *);
  296 (* QUEST_ *);
  297 (* QUEST_EQ_ *);
  298 (* AROBAS_ *);
  299 (* LBRACK_ *);
  300 (* BACKSL_SLASH_ *);
  301 (* RBRACK_ *);
  302 (* HAT_ *);
  303 (* LBRACE_ *);
  304 (* BAR_ *);
  305 (* BAR_DASH_ *);
  306 (* BAR_BAR_ *);
  307 (* RBRACE_ *);
  308 (* TILDE_ *);
  309 (* MUSTUSE *);
  310 (* DEFINITION *);
  311 (* DEPENDS *);
  312 (* ELSE *);
  313 (* END *);
  314 (* EXISTS *);
  315 (* FALSE *);
  316 (* FORALL *);
  317 (* FUN *);
  318 (* IF *);
  319 (* IN *);
  320 (* INDUCTIVE *);
  321 (* LET *);
  322 (* MATCH *);
  323 (* ON *);
  324 (* PARAMETER *);
  325 (* THEN *);
  326 (* THEOREM *);
  327 (* TRUE *);
  328 (* WITH *);
  329 (* BEGINPROOF *);
  331 (* BEGINHEADER *);
  332 (* ENDPROOF *);
    0 (* EOF *);
    0|]

let yytransl_block = [|
  257 (* IDENT *);
  258 (* STRING *);
  259 (* NUM *);
  330 (* BEGINNAME *);
    0|]

let yylhs = "\255\255\
\001\000\001\000\001\000\004\000\004\000\005\000\005\000\003\000\
\003\000\003\000\003\000\003\000\003\000\003\000\003\000\003\000\
\003\000\003\000\003\000\003\000\003\000\003\000\008\000\008\000\
\008\000\008\000\008\000\008\000\009\000\009\000\010\000\010\000\
\007\000\007\000\011\000\011\000\011\000\006\000\006\000\012\000\
\014\000\014\000\013\000\013\000\013\000\015\000\016\000\016\000\
\017\000\017\000\017\000\019\000\019\000\019\000\002\000\002\000\
\018\000\018\000\020\000\020\000\021\000\021\000\000\000"

let yylen = "\002\000\
\007\000\003\000\005\000\004\000\002\000\000\000\002\000\004\000\
\004\000\004\000\006\000\005\000\006\000\003\000\003\000\003\000\
\003\000\003\000\003\000\002\000\002\000\001\000\001\000\001\000\
\004\000\003\000\001\000\001\000\001\000\002\000\002\000\003\000\
\000\000\005\000\003\000\002\000\001\000\001\000\001\000\003\000\
\000\000\002\000\000\000\002\000\004\000\001\000\001\000\001\000\
\005\000\005\000\007\000\003\000\002\000\001\000\002\000\000\000\
\005\000\000\000\001\000\003\000\003\000\001\000\002\000"

let yydefred = "\000\000\
\000\000\000\000\023\000\024\000\000\000\000\000\000\000\000\000\
\000\000\000\000\028\000\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\027\000\000\000\063\000\000\000\000\000\000\000\
\000\000\054\000\000\000\000\000\020\000\053\000\047\000\048\000\
\000\000\000\000\000\000\000\000\000\000\038\000\039\000\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\021\000\055\000\026\000\000\000\000\000\
\000\000\052\000\044\000\042\000\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\007\000\
\000\000\000\000\000\000\000\000\000\000\019\000\018\000\000\000\
\002\000\000\000\030\000\000\000\025\000\000\000\000\000\000\000\
\000\000\040\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\004\000\000\000\000\000\032\000\050\000\000\000\
\045\000\000\000\000\000\000\000\037\000\000\000\000\000\012\000\
\049\000\000\000\003\000\000\000\000\000\000\000\000\000\000\000\
\036\000\000\000\000\000\000\000\051\000\035\000\000\000\001\000\
\000\000\034\000\062\000\000\000\000\000\000\000\000\000\057\000\
\000\000\061\000\060\000"

let yydgoto = "\002\000\
\021\000\022\000\023\000\024\000\049\000\037\000\105\000\025\000\
\060\000\064\000\119\000\038\000\039\000\040\000\098\000\033\000\
\026\000\126\000\027\000\141\000\142\000"

let yysindex = "\031\000\
\106\255\000\000\000\000\000\000\058\255\058\255\038\255\054\255\
\227\254\017\255\000\000\017\255\017\255\058\255\040\255\064\255\
\058\255\054\255\000\000\248\254\000\000\009\255\200\255\058\255\
\004\255\000\000\173\255\009\000\000\000\000\000\000\000\000\000\
\073\255\038\255\017\255\099\255\090\255\000\000\000\000\103\255\
\098\255\087\255\181\255\104\255\114\255\001\255\120\255\248\254\
\067\255\143\255\058\255\058\255\058\255\058\255\058\255\058\255\
\145\000\200\255\004\255\000\000\000\000\000\000\058\255\136\255\
\058\255\000\000\000\000\000\000\099\255\137\255\058\255\058\255\
\058\255\058\255\058\255\150\255\058\255\108\255\058\255\000\000\
\248\254\140\255\017\000\100\255\017\000\000\000\000\000\003\255\
\000\000\081\255\000\000\064\000\000\000\116\001\107\255\017\000\
\017\000\000\000\017\000\017\000\226\255\144\255\119\255\130\255\
\116\255\131\001\000\000\058\255\175\000\000\000\000\000\107\255\
\000\000\058\255\128\255\058\255\000\000\130\255\007\255\000\000\
\000\000\146\001\000\000\017\000\177\255\163\255\017\000\029\255\
\000\000\058\255\183\000\164\255\000\000\000\000\110\001\000\000\
\135\255\000\000\000\000\135\255\128\255\169\255\182\255\000\000\
\135\255\000\000\000\000"

let yyrindex = "\000\000\
\123\255\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\075\255\000\000\075\255\250\254\000\000\000\000\000\000\
\000\000\000\000\000\000\025\255\000\000\000\000\196\000\000\000\
\001\000\000\000\006\000\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\066\255\176\255\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\025\255\
\129\255\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\125\255\058\000\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\176\255\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\146\255\000\000\000\000\
\147\255\000\000\154\000\093\000\184\000\000\000\000\000\128\000\
\000\000\000\000\000\000\194\255\000\000\000\000\005\255\210\000\
\068\255\000\000\240\000\010\001\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\005\255\
\000\000\000\000\202\255\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\040\001\000\000\000\000\066\001\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\146\255\000\000\
\000\000\000\000\000\000\000\000\202\255\254\254\000\000\000\000\
\000\000\000\000\000\000"

let yygindex = "\000\000\
\000\000\242\255\253\255\000\000\218\255\082\000\085\000\251\255\
\158\000\130\000\105\000\188\000\225\255\244\255\000\000\211\000\
\249\255\090\000\000\000\087\000\095\000"

let yytablesize = 702
let yytable = "\030\000\
\022\000\028\000\029\000\067\000\003\000\056\000\004\000\129\000\
\057\000\080\000\043\000\005\000\061\000\046\000\059\000\041\000\
\051\000\035\000\043\000\059\000\058\000\052\000\068\000\052\000\
\036\000\006\000\066\000\006\000\043\000\129\000\053\000\001\000\
\006\000\054\000\055\000\054\000\055\000\034\000\134\000\043\000\
\044\000\130\000\107\000\090\000\056\000\059\000\056\000\083\000\
\084\000\085\000\086\000\087\000\088\000\059\000\031\000\032\000\
\068\000\029\000\003\000\092\000\004\000\094\000\011\000\113\000\
\045\000\005\000\048\000\096\000\097\000\099\000\100\000\101\000\
\078\000\103\000\019\000\106\000\006\000\046\000\050\000\043\000\
\067\000\046\000\006\000\006\000\006\000\006\000\006\000\041\000\
\043\000\006\000\006\000\008\000\017\000\041\000\042\000\006\000\
\041\000\065\000\006\000\069\000\043\000\015\000\046\000\071\000\
\122\000\018\000\003\000\112\000\004\000\006\000\124\000\073\000\
\127\000\005\000\036\000\010\000\011\000\012\000\013\000\014\000\
\052\000\074\000\016\000\017\000\072\000\076\000\135\000\016\000\
\019\000\005\000\117\000\005\000\054\000\055\000\051\000\139\000\
\005\000\118\000\077\000\052\000\081\000\079\000\140\000\082\000\
\089\000\093\000\095\000\006\000\053\000\006\000\102\000\054\000\
\055\000\014\000\006\000\104\000\109\000\006\000\007\000\008\000\
\009\000\108\000\056\000\010\000\011\000\012\000\013\000\014\000\
\115\000\015\000\016\000\017\000\120\000\018\000\123\000\125\000\
\019\000\132\000\020\000\133\000\005\000\116\000\136\000\015\000\
\145\000\137\000\005\000\005\000\005\000\005\000\005\000\146\000\
\056\000\005\000\005\000\056\000\051\000\041\000\006\000\005\000\
\056\000\052\000\033\000\031\000\006\000\006\000\006\000\006\000\
\006\000\009\000\053\000\006\000\006\000\054\000\055\000\051\000\
\091\000\006\000\058\000\138\000\052\000\110\000\128\000\070\000\
\056\000\007\000\008\000\009\000\047\000\053\000\144\000\147\000\
\054\000\055\000\143\000\000\000\015\000\000\000\000\000\008\000\
\018\000\051\000\000\000\056\000\000\000\000\000\052\000\000\000\
\000\000\075\000\000\000\000\000\007\000\008\000\009\000\053\000\
\000\000\000\000\054\000\055\000\000\000\000\000\000\000\015\000\
\000\000\010\000\022\000\018\000\000\000\056\000\022\000\000\000\
\022\000\022\000\062\000\000\000\000\000\022\000\063\000\000\000\
\051\000\114\000\000\000\000\000\000\000\052\000\022\000\000\000\
\051\000\022\000\022\000\022\000\000\000\052\000\053\000\013\000\
\000\000\054\000\055\000\000\000\022\000\000\000\053\000\000\000\
\022\000\054\000\055\000\000\000\056\000\022\000\022\000\022\000\
\022\000\022\000\000\000\000\000\056\000\000\000\000\000\022\000\
\022\000\011\000\000\000\029\000\022\000\022\000\000\000\029\000\
\022\000\029\000\029\000\056\000\022\000\063\000\029\000\051\000\
\000\000\056\000\000\000\000\000\052\000\000\000\000\000\029\000\
\000\000\000\000\029\000\029\000\029\000\053\000\000\000\000\000\
\054\000\055\000\000\000\000\000\000\000\029\000\017\000\000\000\
\000\000\029\000\017\000\056\000\017\000\017\000\029\000\029\000\
\029\000\029\000\029\000\000\000\000\000\000\000\000\000\000\000\
\029\000\029\000\017\000\000\000\000\000\029\000\029\000\017\000\
\000\000\029\000\000\000\000\000\000\000\029\000\000\000\000\000\
\017\000\016\000\000\000\000\000\017\000\016\000\000\000\016\000\
\016\000\017\000\017\000\017\000\017\000\017\000\000\000\000\000\
\000\000\000\000\000\000\017\000\017\000\016\000\000\000\000\000\
\017\000\017\000\016\000\014\000\017\000\000\000\000\000\014\000\
\017\000\000\000\014\000\000\000\000\000\000\000\000\000\016\000\
\000\000\000\000\000\000\000\000\016\000\016\000\016\000\016\000\
\016\000\000\000\000\000\000\000\014\000\000\000\016\000\016\000\
\000\000\015\000\000\000\016\000\016\000\015\000\000\000\016\000\
\015\000\014\000\000\000\016\000\000\000\000\000\014\000\014\000\
\014\000\014\000\014\000\000\000\000\000\000\000\000\000\000\000\
\014\000\014\000\015\000\009\000\000\000\014\000\014\000\009\000\
\000\000\014\000\009\000\000\000\000\000\014\000\000\000\015\000\
\000\000\000\000\000\000\000\000\015\000\015\000\015\000\015\000\
\015\000\000\000\000\000\000\000\009\000\000\000\015\000\015\000\
\000\000\008\000\000\000\015\000\015\000\008\000\000\000\015\000\
\008\000\009\000\000\000\015\000\000\000\000\000\009\000\009\000\
\009\000\009\000\009\000\000\000\000\000\000\000\000\000\000\000\
\009\000\009\000\008\000\010\000\000\000\009\000\009\000\010\000\
\000\000\009\000\010\000\000\000\000\000\009\000\000\000\008\000\
\000\000\000\000\000\000\000\000\008\000\008\000\008\000\008\000\
\008\000\000\000\000\000\000\000\010\000\000\000\008\000\008\000\
\000\000\013\000\000\000\008\000\008\000\013\000\000\000\008\000\
\013\000\010\000\000\000\008\000\000\000\000\000\010\000\010\000\
\010\000\010\000\010\000\000\000\000\000\000\000\000\000\000\000\
\010\000\010\000\013\000\011\000\000\000\010\000\010\000\011\000\
\000\000\010\000\011\000\000\000\000\000\010\000\000\000\013\000\
\000\000\000\000\000\000\000\000\013\000\013\000\013\000\013\000\
\013\000\000\000\000\000\000\000\011\000\000\000\013\000\013\000\
\000\000\000\000\000\000\013\000\013\000\000\000\000\000\013\000\
\000\000\011\000\000\000\013\000\000\000\000\000\011\000\011\000\
\011\000\011\000\011\000\000\000\000\000\051\000\000\000\000\000\
\011\000\011\000\052\000\051\000\111\000\011\000\011\000\000\000\
\052\000\011\000\000\000\053\000\000\000\011\000\054\000\055\000\
\000\000\053\000\051\000\121\000\054\000\055\000\000\000\052\000\
\000\000\056\000\000\000\000\000\000\000\104\000\000\000\056\000\
\053\000\051\000\131\000\054\000\055\000\000\000\052\000\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\056\000\053\000\
\000\000\000\000\054\000\055\000\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\056\000"

let yycheck = "\007\000\
\000\000\005\000\006\000\035\000\001\001\000\000\003\001\001\001\
\023\000\048\000\014\000\008\001\027\000\017\000\017\001\022\001\
\016\001\001\001\014\001\025\000\024\000\021\001\035\000\021\001\
\008\001\001\001\034\000\003\001\035\001\001\001\030\001\001\000\
\008\001\033\001\034\001\033\001\034\001\067\001\010\001\035\001\
\001\001\035\001\081\000\058\000\044\001\048\001\044\001\051\000\
\052\000\053\000\054\000\055\000\056\000\059\000\001\001\002\001\
\069\000\000\000\001\001\063\000\003\001\065\000\059\001\095\000\
\001\001\008\001\075\001\071\000\072\000\073\000\074\000\075\000\
\072\001\077\000\071\001\079\000\052\001\010\001\070\001\014\001\
\112\000\014\001\058\001\059\001\060\001\061\001\062\001\022\001\
\014\001\065\001\066\001\054\001\000\000\012\000\013\000\071\001\
\022\001\025\001\074\001\001\001\035\001\064\001\035\001\014\001\
\108\000\068\001\001\001\001\001\003\001\052\001\114\000\014\001\
\116\000\008\001\008\001\058\001\059\001\060\001\061\001\062\001\
\021\001\035\001\065\001\066\001\022\001\022\001\130\000\000\000\
\071\001\001\001\001\001\003\001\033\001\034\001\016\001\001\001\
\008\001\008\001\025\001\021\001\074\001\022\001\008\001\001\001\
\000\000\010\001\010\001\001\001\030\001\003\001\001\001\033\001\
\034\001\000\000\008\001\048\001\076\001\052\001\053\001\054\001\
\055\001\022\001\044\001\058\001\059\001\060\001\061\001\062\001\
\025\001\064\001\065\001\066\001\057\001\068\001\000\000\048\001\
\071\001\001\001\073\001\017\001\052\001\063\001\000\000\000\000\
\016\001\022\001\058\001\059\001\060\001\061\001\062\001\010\001\
\070\001\065\001\066\001\000\000\016\001\022\001\052\001\071\001\
\076\001\021\001\057\001\010\001\058\001\059\001\060\001\061\001\
\062\001\000\000\030\001\065\001\066\001\033\001\034\001\016\001\
\059\000\071\001\017\001\135\000\021\001\092\000\118\000\036\000\
\044\001\053\001\054\001\055\001\018\000\030\001\141\000\145\000\
\033\001\034\001\140\000\255\255\064\001\255\255\255\255\000\000\
\068\001\016\001\255\255\044\001\255\255\255\255\021\001\255\255\
\255\255\069\001\255\255\255\255\053\001\054\001\055\001\030\001\
\255\255\255\255\033\001\034\001\255\255\255\255\255\255\064\001\
\255\255\000\000\010\001\068\001\255\255\044\001\014\001\255\255\
\016\001\017\001\010\001\255\255\255\255\021\001\014\001\255\255\
\016\001\056\001\255\255\255\255\255\255\021\001\030\001\255\255\
\016\001\033\001\034\001\035\001\255\255\021\001\030\001\000\000\
\255\255\033\001\034\001\255\255\044\001\255\255\030\001\255\255\
\048\001\033\001\034\001\255\255\044\001\053\001\054\001\055\001\
\056\001\057\001\255\255\255\255\044\001\255\255\255\255\063\001\
\064\001\000\000\255\255\010\001\068\001\069\001\255\255\014\001\
\072\001\016\001\017\001\070\001\076\001\014\001\021\001\016\001\
\255\255\076\001\255\255\255\255\021\001\255\255\255\255\030\001\
\255\255\255\255\033\001\034\001\035\001\030\001\255\255\255\255\
\033\001\034\001\255\255\255\255\255\255\044\001\010\001\255\255\
\255\255\048\001\014\001\044\001\016\001\017\001\053\001\054\001\
\055\001\056\001\057\001\255\255\255\255\255\255\255\255\255\255\
\063\001\064\001\030\001\255\255\255\255\068\001\069\001\035\001\
\255\255\072\001\255\255\255\255\255\255\076\001\255\255\255\255\
\044\001\010\001\255\255\255\255\048\001\014\001\255\255\016\001\
\017\001\053\001\054\001\055\001\056\001\057\001\255\255\255\255\
\255\255\255\255\255\255\063\001\064\001\030\001\255\255\255\255\
\068\001\069\001\035\001\010\001\072\001\255\255\255\255\014\001\
\076\001\255\255\017\001\255\255\255\255\255\255\255\255\048\001\
\255\255\255\255\255\255\255\255\053\001\054\001\055\001\056\001\
\057\001\255\255\255\255\255\255\035\001\255\255\063\001\064\001\
\255\255\010\001\255\255\068\001\069\001\014\001\255\255\072\001\
\017\001\048\001\255\255\076\001\255\255\255\255\053\001\054\001\
\055\001\056\001\057\001\255\255\255\255\255\255\255\255\255\255\
\063\001\064\001\035\001\010\001\255\255\068\001\069\001\014\001\
\255\255\072\001\017\001\255\255\255\255\076\001\255\255\048\001\
\255\255\255\255\255\255\255\255\053\001\054\001\055\001\056\001\
\057\001\255\255\255\255\255\255\035\001\255\255\063\001\064\001\
\255\255\010\001\255\255\068\001\069\001\014\001\255\255\072\001\
\017\001\048\001\255\255\076\001\255\255\255\255\053\001\054\001\
\055\001\056\001\057\001\255\255\255\255\255\255\255\255\255\255\
\063\001\064\001\035\001\010\001\255\255\068\001\069\001\014\001\
\255\255\072\001\017\001\255\255\255\255\076\001\255\255\048\001\
\255\255\255\255\255\255\255\255\053\001\054\001\055\001\056\001\
\057\001\255\255\255\255\255\255\035\001\255\255\063\001\064\001\
\255\255\010\001\255\255\068\001\069\001\014\001\255\255\072\001\
\017\001\048\001\255\255\076\001\255\255\255\255\053\001\054\001\
\055\001\056\001\057\001\255\255\255\255\255\255\255\255\255\255\
\063\001\064\001\035\001\010\001\255\255\068\001\069\001\014\001\
\255\255\072\001\017\001\255\255\255\255\076\001\255\255\048\001\
\255\255\255\255\255\255\255\255\053\001\054\001\055\001\056\001\
\057\001\255\255\255\255\255\255\035\001\255\255\063\001\064\001\
\255\255\255\255\255\255\068\001\069\001\255\255\255\255\072\001\
\255\255\048\001\255\255\076\001\255\255\255\255\053\001\054\001\
\055\001\056\001\057\001\255\255\255\255\016\001\255\255\255\255\
\063\001\064\001\021\001\016\001\017\001\068\001\069\001\255\255\
\021\001\072\001\255\255\030\001\255\255\076\001\033\001\034\001\
\255\255\030\001\016\001\017\001\033\001\034\001\255\255\021\001\
\255\255\044\001\255\255\255\255\255\255\048\001\255\255\044\001\
\030\001\016\001\017\001\033\001\034\001\255\255\021\001\255\255\
\255\255\255\255\255\255\255\255\255\255\255\255\044\001\030\001\
\255\255\255\255\033\001\034\001\255\255\255\255\255\255\255\255\
\255\255\255\255\255\255\255\255\255\255\044\001"

let yynames_const = "\
  BANG_\000\
  PERCENT_\000\
  AMPER_\000\
  AMPER_AMPER_\000\
  LPAREN_\000\
  LPAREN_RPAREN_\000\
  RPAREN_\000\
  STAR_\000\
  PLUS_\000\
  PLUS_PLUS_\000\
  COMMA_\000\
  DASH_\000\
  DASH_GT_\000\
  PERIOD_\000\
  PERIOD_LPAREN_\000\
  PERIOD_PERIOD_\000\
  SLASH_\000\
  SLASH_BACKSL_\000\
  COLON_\000\
  COLON_COLON_\000\
  COLON_LT_\000\
  COLON_EQ_\000\
  COLON_GT_\000\
  SEMI_\000\
  LT_\000\
  LT_DASH_\000\
  LT_DASH_GT_\000\
  LT_COLON_\000\
  LT_EQ_\000\
  LT_GT_\000\
  EQ_\000\
  EQ_GT_\000\
  EQ_UNDER_D_\000\
  GT_\000\
  GT_DASH_GT_\000\
  GT_EQ_\000\
  QUEST_\000\
  QUEST_EQ_\000\
  AROBAS_\000\
  LBRACK_\000\
  BACKSL_SLASH_\000\
  RBRACK_\000\
  HAT_\000\
  LBRACE_\000\
  BAR_\000\
  BAR_DASH_\000\
  BAR_BAR_\000\
  RBRACE_\000\
  TILDE_\000\
  MUSTUSE\000\
  DEFINITION\000\
  DEPENDS\000\
  ELSE\000\
  END\000\
  EXISTS\000\
  FALSE\000\
  FORALL\000\
  FUN\000\
  IF\000\
  IN\000\
  INDUCTIVE\000\
  LET\000\
  MATCH\000\
  ON\000\
  PARAMETER\000\
  THEN\000\
  THEOREM\000\
  TRUE\000\
  WITH\000\
  BEGINPROOF\000\
  BEGINHEADER\000\
  ENDPROOF\000\
  EOF\000\
  "

let yynames_block = "\
  IDENT\000\
  STRING\000\
  NUM\000\
  BEGINNAME\000\
  "

let yyact = [|
  (fun _ -> failwith "parser")
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 6 : 'hyp_def_list) in
    let _3 = (Parsing.peek_val __caml_parser_env 4 : string) in
    let _5 = (Parsing.peek_val __caml_parser_env 2 : 'expr) in
    Obj.repr(
# 173 "parsecoq.mly"
      ( (_3, (Hyp (goal_name, enot _5, 0), false) :: _1) )
# 599 "parsecoq.ml"
               : string * (Phrase.phrase * bool) list))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 2 : 'expr) in
    let _2 = (Parsing.peek_val __caml_parser_env 1 : 'hyp_def_list) in
    Obj.repr(
# 175 "parsecoq.mly"
      ( (* Error.warn "deprecated input format"; *)
        ("theorem", (Hyp (goal_name, enot _1, 0), false) :: _2) )
# 608 "parsecoq.ml"
               : string * (Phrase.phrase * bool) list))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 4 : 'proof_head) in
    let _2 = (Parsing.peek_val __caml_parser_env 3 : 'expr) in
    let _3 = (Parsing.peek_val __caml_parser_env 2 : 'hyp_def_list) in
    Obj.repr(
# 178 "parsecoq.mly"
      ( (* Error.warn "deprecated input format"; *)
        (_1, (Hyp (goal_name, enot _2, 0), false) :: _3) )
# 618 "parsecoq.ml"
               : string * (Phrase.phrase * bool) list))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 2 : 'proofheaders) in
    let _3 = (Parsing.peek_val __caml_parser_env 1 : string) in
    let _4 = (Parsing.peek_val __caml_parser_env 0 : 'proofheaders) in
    Obj.repr(
# 184 "parsecoq.mly"
      ( _3 )
# 627 "parsecoq.ml"
               : 'proof_head))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 0 : 'proofheaders) in
    Obj.repr(
# 186 "parsecoq.mly"
      ( "theorem" )
# 634 "parsecoq.ml"
               : 'proof_head))
; (fun __caml_parser_env ->
    Obj.repr(
# 191 "parsecoq.mly"
      ( () )
# 640 "parsecoq.ml"
               : 'proofheaders))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 0 : 'proofheaders) in
    Obj.repr(
# 193 "parsecoq.mly"
      ( () )
# 647 "parsecoq.ml"
               : 'proofheaders))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 2 : 'bindings) in
    let _4 = (Parsing.peek_val __caml_parser_env 0 : 'expr) in
    Obj.repr(
# 198 "parsecoq.mly"
      ( mk_all _2 _4 )
# 655 "parsecoq.ml"
               : 'expr))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 2 : 'bindings) in
    let _4 = (Parsing.peek_val __caml_parser_env 0 : 'expr) in
    Obj.repr(
# 200 "parsecoq.mly"
      ( mk_ex _2 _4 )
# 663 "parsecoq.ml"
               : 'expr))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 2 : 'bindings) in
    let _4 = (Parsing.peek_val __caml_parser_env 0 : 'expr) in
    Obj.repr(
# 203 "parsecoq.mly"
      ( mk_lam _2 _4 )
# 671 "parsecoq.ml"
               : 'expr))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 4 : string) in
    let _4 = (Parsing.peek_val __caml_parser_env 2 : 'expr) in
    let _6 = (Parsing.peek_val __caml_parser_env 0 : 'expr) in
    Obj.repr(
# 206 "parsecoq.mly"
      ( mk_let _2 _4 _6 )
# 680 "parsecoq.ml"
               : 'expr))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 3 : 'expr) in
    let _4 = (Parsing.peek_val __caml_parser_env 1 : 'pat_expr_list) in
    Obj.repr(
# 209 "parsecoq.mly"
      ( eapp ("$match", _2 :: _4) )
# 688 "parsecoq.ml"
               : 'expr))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 4 : 'expr) in
    let _4 = (Parsing.peek_val __caml_parser_env 2 : 'expr) in
    let _6 = (Parsing.peek_val __caml_parser_env 0 : 'expr) in
    Obj.repr(
# 212 "parsecoq.mly"
      ( eapp ("(__g_ifthenelse _)", [_2; _4; _6]) )
# 697 "parsecoq.ml"
               : 'expr))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 2 : 'expr) in
    let _3 = (Parsing.peek_val __caml_parser_env 0 : 'expr) in
    Obj.repr(
# 215 "parsecoq.mly"
      ( eimply (_1, _3) )
# 705 "parsecoq.ml"
               : 'expr))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 2 : 'expr) in
    let _3 = (Parsing.peek_val __caml_parser_env 0 : 'expr) in
    Obj.repr(
# 218 "parsecoq.mly"
      ( eequiv (_1, _3) )
# 713 "parsecoq.ml"
               : 'expr))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 2 : 'expr) in
    let _3 = (Parsing.peek_val __caml_parser_env 0 : 'expr) in
    Obj.repr(
# 221 "parsecoq.mly"
      ( eor (_1, _3) )
# 721 "parsecoq.ml"
               : 'expr))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 2 : 'expr) in
    let _3 = (Parsing.peek_val __caml_parser_env 0 : 'expr) in
    Obj.repr(
# 224 "parsecoq.mly"
      ( eand (_1, _3) )
# 729 "parsecoq.ml"
               : 'expr))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 2 : 'expr) in
    let _3 = (Parsing.peek_val __caml_parser_env 0 : 'expr) in
    Obj.repr(
# 227 "parsecoq.mly"
      ( eapp ("=", [_1; _3]) )
# 737 "parsecoq.ml"
               : 'expr))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 2 : 'expr) in
    let _3 = (Parsing.peek_val __caml_parser_env 0 : 'expr) in
    Obj.repr(
# 229 "parsecoq.mly"
      ( enot (eapp ("=", [_1; _3])) )
# 745 "parsecoq.ml"
               : 'expr))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 0 : 'expr) in
    Obj.repr(
# 232 "parsecoq.mly"
      ( enot (_2) )
# 752 "parsecoq.ml"
               : 'expr))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 1 : 'expr1) in
    let _2 = (Parsing.peek_val __caml_parser_env 0 : 'expr1_list) in
    Obj.repr(
# 235 "parsecoq.mly"
      ( mk_apply (_1, _2) )
# 760 "parsecoq.ml"
               : 'expr))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'expr1) in
    Obj.repr(
# 238 "parsecoq.mly"
      ( _1 )
# 767 "parsecoq.ml"
               : 'expr))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : string) in
    Obj.repr(
# 243 "parsecoq.mly"
      ( evar (_1) )
# 774 "parsecoq.ml"
               : 'expr1))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : string) in
    Obj.repr(
# 245 "parsecoq.mly"
      ( eapp (_1, []) )
# 781 "parsecoq.ml"
               : 'expr1))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 2 : 'expr) in
    let _3 = (Parsing.peek_val __caml_parser_env 1 : 'comma_expr_list) in
    Obj.repr(
# 248 "parsecoq.mly"
      ( mk_apply (evar tuple_name, _2 :: _3) )
# 789 "parsecoq.ml"
               : 'expr1))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 1 : 'expr) in
    Obj.repr(
# 251 "parsecoq.mly"
      ( _2 )
# 796 "parsecoq.ml"
               : 'expr1))
; (fun __caml_parser_env ->
    Obj.repr(
# 253 "parsecoq.mly"
      ( etrue )
# 802 "parsecoq.ml"
               : 'expr1))
; (fun __caml_parser_env ->
    Obj.repr(
# 255 "parsecoq.mly"
      ( efalse )
# 808 "parsecoq.ml"
               : 'expr1))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'expr1) in
    Obj.repr(
# 259 "parsecoq.mly"
                           ( [_1] )
# 815 "parsecoq.ml"
               : 'expr1_list))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 1 : 'expr1) in
    let _2 = (Parsing.peek_val __caml_parser_env 0 : 'expr1_list) in
    Obj.repr(
# 260 "parsecoq.mly"
                           ( _1 :: _2 )
# 823 "parsecoq.ml"
               : 'expr1_list))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 0 : 'expr) in
    Obj.repr(
# 265 "parsecoq.mly"
      ( [_2] )
# 830 "parsecoq.ml"
               : 'comma_expr_list))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 1 : 'expr) in
    let _3 = (Parsing.peek_val __caml_parser_env 0 : 'comma_expr_list) in
    Obj.repr(
# 267 "parsecoq.mly"
      ( _2 :: _3 )
# 838 "parsecoq.ml"
               : 'comma_expr_list))
; (fun __caml_parser_env ->
    Obj.repr(
# 272 "parsecoq.mly"
      ( [] )
# 844 "parsecoq.ml"
               : 'pat_expr_list))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 3 : 'pattern) in
    let _4 = (Parsing.peek_val __caml_parser_env 1 : 'expr) in
    let _5 = (Parsing.peek_val __caml_parser_env 0 : 'pat_expr_list) in
    Obj.repr(
# 274 "parsecoq.mly"
      ( (mk_pattern [] _2) :: _4 :: _5 )
# 853 "parsecoq.ml"
               : 'pat_expr_list))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 1 : 'pattern) in
    Obj.repr(
# 279 "parsecoq.mly"
      ( _2 )
# 860 "parsecoq.ml"
               : 'pattern))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 1 : 'pattern) in
    let _2 = (Parsing.peek_val __caml_parser_env 0 : string) in
    Obj.repr(
# 281 "parsecoq.mly"
      ( _2 :: _1 )
# 868 "parsecoq.ml"
               : 'pattern))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : string) in
    Obj.repr(
# 283 "parsecoq.mly"
      ( [_1] )
# 875 "parsecoq.ml"
               : 'pattern))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'simplebinding) in
    Obj.repr(
# 287 "parsecoq.mly"
                           ( _1 )
# 882 "parsecoq.ml"
               : 'bindings))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'binding_list) in
    Obj.repr(
# 288 "parsecoq.mly"
                           ( _1 )
# 889 "parsecoq.ml"
               : 'bindings))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 2 : 'idlist) in
    let _3 = (Parsing.peek_val __caml_parser_env 0 : 'typ) in
    Obj.repr(
# 292 "parsecoq.mly"
                       ( List.map (fun v -> (v, _3)) _1 )
# 897 "parsecoq.ml"
               : 'simplebinding))
; (fun __caml_parser_env ->
    Obj.repr(
# 296 "parsecoq.mly"
                           ( [] )
# 903 "parsecoq.ml"
               : 'idlist))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 1 : string) in
    let _2 = (Parsing.peek_val __caml_parser_env 0 : 'idlist) in
    Obj.repr(
# 297 "parsecoq.mly"
                           ( _1 :: _2 )
# 911 "parsecoq.ml"
               : 'idlist))
; (fun __caml_parser_env ->
    Obj.repr(
# 302 "parsecoq.mly"
      ( [] )
# 917 "parsecoq.ml"
               : 'binding_list))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 1 : string) in
    let _2 = (Parsing.peek_val __caml_parser_env 0 : 'binding_list) in
    Obj.repr(
# 304 "parsecoq.mly"
      ( (_1, "_") :: _2 )
# 925 "parsecoq.ml"
               : 'binding_list))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 2 : 'simplebinding) in
    let _4 = (Parsing.peek_val __caml_parser_env 0 : 'binding_list) in
    Obj.repr(
# 306 "parsecoq.mly"
      ( _2 @ _4 )
# 933 "parsecoq.ml"
               : 'binding_list))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'expr) in
    Obj.repr(
# 310 "parsecoq.mly"
                           ( mk_type_string _1 )
# 940 "parsecoq.ml"
               : 'typ))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : string) in
    Obj.repr(
# 315 "parsecoq.mly"
           ( _1 )
# 947 "parsecoq.ml"
               : 'id_or_expr))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : string) in
    Obj.repr(
# 316 "parsecoq.mly"
           ( _1 )
# 954 "parsecoq.ml"
               : 'id_or_expr))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 3 : 'id_or_expr) in
    let _4 = (Parsing.peek_val __caml_parser_env 1 : 'expr) in
    Obj.repr(
# 321 "parsecoq.mly"
      ( Hyp (_2, _4, 1) )
# 962 "parsecoq.ml"
               : 'hyp_def))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 3 : 'id_or_expr) in
    let _4 = (Parsing.peek_val __caml_parser_env 1 : 'expr) in
    Obj.repr(
# 323 "parsecoq.mly"
      ( let (params, expr) = get_params _4 in Def (DefReal (_2, params, expr)) )
# 970 "parsecoq.ml"
               : 'hyp_def))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 5 : string) in
    let _4 = (Parsing.peek_val __caml_parser_env 3 : string) in
    let _6 = (Parsing.peek_val __caml_parser_env 1 : 'constr_list) in
    Obj.repr(
# 325 "parsecoq.mly"
      ( (* FIXME should check that $4 = "Set" *)
        Inductive (_2, _6) )
# 980 "parsecoq.ml"
               : 'hyp_def))
; (fun __caml_parser_env ->
    let _3 = (Parsing.peek_val __caml_parser_env 0 : 'hyp_def) in
    Obj.repr(
# 330 "parsecoq.mly"
                               (
      (* FIXME activate this warning at some point.
      Error.warn "use of obsolete \"Depends on\" syntax";
      *)
      (_3, true)
    )
# 992 "parsecoq.ml"
               : 'dep_hyp_def))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 0 : 'hyp_def) in
    Obj.repr(
# 336 "parsecoq.mly"
                               ( (_2, true) )
# 999 "parsecoq.ml"
               : 'dep_hyp_def))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'hyp_def) in
    Obj.repr(
# 337 "parsecoq.mly"
                               ( (_1, false) )
# 1006 "parsecoq.ml"
               : 'dep_hyp_def))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 1 : 'dep_hyp_def) in
    let _2 = (Parsing.peek_val __caml_parser_env 0 : 'hyp_def_list) in
    Obj.repr(
# 341 "parsecoq.mly"
                               ( _1 :: _2 )
# 1014 "parsecoq.ml"
               : 'hyp_def_list))
; (fun __caml_parser_env ->
    Obj.repr(
# 342 "parsecoq.mly"
                               ( [] )
# 1020 "parsecoq.ml"
               : 'hyp_def_list))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 3 : string) in
    let _4 = (Parsing.peek_val __caml_parser_env 1 : 'constr_type) in
    let _5 = (Parsing.peek_val __caml_parser_env 0 : 'constr_list) in
    Obj.repr(
# 347 "parsecoq.mly"
      ( _2 :: _5 )
# 1029 "parsecoq.ml"
               : 'constr_list))
; (fun __caml_parser_env ->
    Obj.repr(
# 349 "parsecoq.mly"
      ( [] )
# 1035 "parsecoq.ml"
               : 'constr_list))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'arg_type) in
    Obj.repr(
# 353 "parsecoq.mly"
                                      ( () )
# 1042 "parsecoq.ml"
               : 'constr_type))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 2 : 'arg_type) in
    let _3 = (Parsing.peek_val __caml_parser_env 0 : 'constr_type) in
    Obj.repr(
# 354 "parsecoq.mly"
                                      ( () )
# 1050 "parsecoq.ml"
               : 'constr_type))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 1 : 'arg_type) in
    Obj.repr(
# 358 "parsecoq.mly"
                                      ( () )
# 1057 "parsecoq.ml"
               : 'arg_type))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : string) in
    Obj.repr(
# 359 "parsecoq.mly"
                                      ( () )
# 1064 "parsecoq.ml"
               : 'arg_type))
(* Entry file *)
; (fun __caml_parser_env -> raise (Parsing.YYexit (Parsing.peek_val __caml_parser_env 0)))
|]
let yytables =
  { Parsing.actions=yyact;
    Parsing.transl_const=yytransl_const;
    Parsing.transl_block=yytransl_block;
    Parsing.lhs=yylhs;
    Parsing.len=yylen;
    Parsing.defred=yydefred;
    Parsing.dgoto=yydgoto;
    Parsing.sindex=yysindex;
    Parsing.rindex=yyrindex;
    Parsing.gindex=yygindex;
    Parsing.tablesize=yytablesize;
    Parsing.table=yytable;
    Parsing.check=yycheck;
    Parsing.error_function=parse_error;
    Parsing.names_const=yynames_const;
    Parsing.names_block=yynames_block }
let file (lexfun : Lexing.lexbuf -> token) (lexbuf : Lexing.lexbuf) =
   (Parsing.yyparse yytables 1 lexfun lexbuf : string * (Phrase.phrase * bool) list)
;;
