(*----------------------------------------------------------------------(C)-*)
(* Copyright (C) 2006-2008 K. Korovin and The University of Manchetser. 
   This file is part of iProver - a theorem prover for first-order logic.

   iProver is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.
   iProver is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  
   See the GNU General Public License for more details.
   You should have received a copy of the GNU General Public License
   along with iProver.  If not, see <http://www.gnu.org/licenses/>.         *)
(*----------------------------------------------------------------------[C]-*)

open Options
open Statistics 

type clause = Clause.clause

let prop_simp clause_list = 
  List.iter 
    Prop_solver_exchange.add_clause_to_solver clause_list;
  (if ((Prop_solver_exchange.solve ()) = PropSolver.Unsat)
  then raise PropSolver.Unsatisfiable
  else ());
  let simplify_clause rest clause = 
    (Prop_solver_exchange.prop_subsumption clause)::rest
  in
  List.fold_left simplify_clause [] clause_list
      
      
let preprocess clause_list =
  let current_list = ref clause_list in
  (if  !current_options.prep_prop_sim 
  then current_list := prop_simp clause_list
  else ());
  (match !current_options.ground_splitting with
  |Split_Input |Split_Full ->    
      let split_result = 
	(Splitting.ground_split_clause_list !current_list) in
      incr_int_stat 
	(Splitting.get_num_of_splits split_result) num_of_splits; 
      incr_int_stat 
	(Splitting.get_num_of_split_atoms split_result) num_of_split_atoms;	
      current_list:=Splitting.get_split_list split_result
  |Split_Off-> ()
    );
  !current_list 
    
