#!/bin/sh
#------------
# File:      randocop.sh
# Version:   1.1
# Date:      15 July 2008
#------------
# Purpose:   Invokes the randoCoP prover
# Usage:     ./randocop.sh <problem file> [<time limit>]
# Authors:   Jens Otten, Thomas Raths
# Web:       www.leancop.de
# Copyright: (c) 2007-2008 by Jens Otten, Thomas Raths
# License:   GNU General Public License
#------------

#-----------
# Parameters

# set ECLiPSe Prolog path
ECLIPSE=/home/voronkov/Eclipse/bin/i386_linux/eclipse

# set randoCoP prover path
PROVERPATH=/home/tptp/Systems/randoCoP---1.1

#----------
# Functions

randocop_sched()
{
 # Input: $FILE, $TIMELIMIT, $OUTPUT
  if [ $OUTPUT == "stdout" ]
    then /home/tptp/Systems/randoCoP---1.1/myulimit $TIMELIMIT "$ECLIPSE -b $PROVERPATH/randocop_sched.pl -e randocop_sched('$FILE',$TIMELIMIT,_)" 
    else /home/tptp/Systems/randoCoP---1.1/myulimit $TIMELIMIT "$ECLIPSE -b $PROVERPATH/randocop_sched.pl -e randocop_sched('$FILE',$TIMELIMIT,_)" > $OUTPUT
  fi
  if [ $? -eq 1 ]; then echo "Timeout for $FILE"; fi
}

#-------------
# Main Program

if [ $# -eq 0 -o $# -gt 2 ]; then
 echo "Usage: $0 <problem file> [<time limit>]"
 echo " or"
 echo "Usage: $0 <problem list file> [<time limit>]"
 echo "       and in <problem list file> each line has the form"
 echo "       <problem file> <output file>"
 exit 2
fi

if [ ! -r "$1" ]; then
 echo "Error: File $1 not found" >&2
 exit 2
fi

case "$2" in (*[!0-9]*)
 echo "Error: Time $2 is not a number" >&2
 exit 2 ;;
esac

if [ $# -eq 1 ]
 then TIMELIMIT=300
 else TIMELIMIT=$2
fi

cat $1 | grep -q '('
if [ $? -eq 0 ]
then
  FILE=$1
  OUTPUT="stdout"
  # OUTPUT=/dev/stdout     # to stdout
  # OUTPUT=$FILE.proof     # to file
  randocop_sched
else
  PROBLEMLIST=$1
  NUMBER_OF_PROBLEMS=`cat $PROBLEMLIST | grep -c ' '`
  TIMELIMIT=$(( $TIMELIMIT / $NUMBER_OF_PROBLEMS ))
  while read LINE
  do
    FILE=`echo $LINE | cut -d ' ' -f1`
    if [ ! -r "$FILE" ]
    then
      echo "Error: File $FILE not found" >&2
    else
      OUTPUT=`echo $LINE | cut -d ' ' -f2`
      randocop_sched
    fi
  done < $PROBLEMLIST
fi
