/*
 Copyright 2006, 2007, 2008 Hao Xu
 ertranne@hotmail.com
 
 This file is part of OSHL-S.

 OSHL-S is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 OSHL-S is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with Foobar; if not, write to the Free Software
 Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
package jfp.type;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import jfp.datatype.ListTuple;
import jfp.datatype.Pair;
import jfp.datatype.Tuple;
import jfp.datatype.TupleFactory;
import jfp.dynamic.Term;
import jfp.dynamic.ASTException;
import jfp.dynamic.ExtensibleAST;
import jfp.dynamic.TypeASTDelegate;
import jfp.environment.TypeVariableEnvironmentInterface;

public class TTupleDelegate extends TypeASTDelegate {

	@Override
	public Object unify_env(ExtensibleAST domainType, TypeVariableEnvironmentInterface env,
			ExtensibleAST valueType, TypeVariableEnvironmentInterface interface1)
			throws ASTException {
		TTuple dt = (TTuple) domainType;
		if (valueType instanceof TTuple) {
			TTuple vt = (TTuple) valueType;
			Tuple tuple = dt.getTuple();
			Tuple tuple2 = vt.getTuple();
			if (tuple.arity() == tuple2.arity()) {
				for (int i = 0; i < tuple.arity(); i++) {
					if (!Util.unifiable(tuple.toArray()[i],
							env, tuple2.toArray()[i],
							interface1)) {
						return false;
					}
				}
				return true;
			}
		}
		return false;
	}

	@Override
	public Object type_varaibles(ExtensibleAST ast, TypeVariableEnvironmentInterface env)
			throws ASTException {
		TTuple ttuple = (TTuple) ast;
		Tuple tuple = ttuple.getTuple();
		if (tuple.arity() == 0)
			return new HashSet<TVar>();
		else {
			Set<TVar> set = new HashSet<TVar>();
			set.addAll(Util.definedTypeVariables(tuple.hd()));
			set.addAll(Util.definedTypeVariables(tuple.tl()));
			return set;
		}
	}

	@Override
	public Object instantiate(ExtensibleAST ast, TypeVariableEnvironmentInterface env)
			throws ASTException {
		TTuple ttuple = (TTuple) ast;
		Tuple tuple = ttuple.getTuple();
		if (tuple.arity() == 0)
			return tuple;
		else
			return new Tuple(Util.getTypeInstance(tuple.hd(), env),
					(Tuple) Util.getTypeInstance(tuple.tl(), env));
	}

	@Override
	public Object inst(ExtensibleAST ast, TypeVariableEnvironmentInterface env)
			throws ASTException {
		TTuple tuple = (TTuple) ast;
		Tuple t = tuple.tuple;
		List<Object> l = new ArrayList<Object>();
		while (t.arity() > 0) {
			l.add(Util.wrap(t.hd()).visitedBy(Term.INST, env));
			t = t.tl();
		}
		return TupleFactory.create(l);
		// new TTuple(TupleFactory.create(l));
	}

}
