/*
 Copyright 2006, 2007, 2008 Hao Xu
 ertranne@hotmail.com
 
 This file is part of OSHL-S.

 OSHL-S is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 OSHL-S is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with Foobar; if not, write to the Free Software
 Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
package jfp.type;

import java.util.ArrayList;
import java.util.List;

import jfp.dynamic.ExtensibleTypeAST;
import jfp.dynamic.Scope;
import jfp.environment.TypeVariableEnvironmentInterface;

public class TVar extends ExtensibleTypeAST {
	private String name;

	TypeVariableEnvironmentInterface env;

	public boolean constant = false;

	public static long ind = 0;

	public TVar(String name) {
		super(new TVarDelegate());
		this.name = name;
	}

	public TVar(TypeVariableEnvironmentInterface env) {
		super(new TVarDelegate());
		this.name = "TVar@" + (ind++);
	}

	public TVar(Scope s, TypeVariableEnvironmentInterface env) {
		super(new TVarDelegate());
		this.name = "TVar@" + (ind++);
	}

	public TVar(boolean constant, Scope s, TypeVariableEnvironmentInterface env) {
		this(s, env);
		this.constant = constant;
	}

	TUnion unionType = new TUnion(new ArrayList());

	public Object getSpecializedType() {
		return unionType;
	}

	public void addSpecializedType(Object type) {
		unionType.addType(type);
		unionType.flattern();
	}

	public String getName() {
		return name;
	}

	public void setName(String name) {
		this.name = name;
	}

	public Object getNonNullReference(TypeVariableEnvironmentInterface vp) {
		Object reference = vp.getType(this.name);
		if (reference == null || reference.equals(this)) {
			return this;
		} else if (reference instanceof TVar) {
			return ((TVar) reference).getNonNullReference(vp);
		} else {
			return reference;
		}
	}

	public void setReference(Object o, TypeVariableEnvironmentInterface env) {
		env.bindType(this.name, o);
	}

	@Override
	public String toString() {
		if (this.name == null)
			return super.toString();
		else
			return name + (constant ? "!" : "");
	}

	public boolean equals(Object type) {
		if (type instanceof TVar) {
			TVar var = (TVar) type;
			if (this.name == null)
				return false;
			else
				return this.name.equals(var.name);
		}
		return false;
	}

	public int hashCode() {
		if (this.name == null) {
			return super.hashCode();
		} else {
			return this.name.hashCode();
		}
	}

	public void setReferenceAtDefinition(Object o) {
		this.env.bindType(this.name, o);
	}

	public void definedAt(TypeVariableEnvironmentInterface environment) {
		this.env = environment;
	}
}
