/*
 Copyright 2006, 2007, 2008 Hao Xu
 ertranne@hotmail.com
 
 This file is part of OSHL-S.

 OSHL-S is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 OSHL-S is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with Foobar; if not, write to the Free Software
 Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
package jfp.type;

import java.io.IOException;
import java.util.*;

import jfp.dynamic.TokenStream;

public class TypeParser {
	private static final String FUNC = "->", OF = "of";

	public static Object parse(TokenStream tk) throws ClassNotFoundException,
			IOException {
		return parseParam(tk, new HashMap<String, TVar>());
	}

	public static Object parse(TokenStream tk, Map<String, TVar> vMap)
			throws ClassNotFoundException, IOException {
		return parseExp(tk, vMap);
	}

	public static Object parseExp(TokenStream tk, Map<String, TVar> vMap)
			throws ClassNotFoundException, IOException {
		Object param = parseParam(tk, vMap);
		if (tk.moreTokens()) {
			String token = tk.nextToken();
			if (FUNC.equals(token)) {
				return new TFunction(param, parseExp(tk, vMap));
			} else if (token.equals(OF)) {
				List params = new ArrayList();
				while (tk.moreTokens()) {
					Object p = parseParam(tk, vMap);
					if (p != null)
						params.add(p);
					else
						break;
				}
				return new TDisjointUnion(
						new TGenericType((String) param, params.size()), params
								.toArray());
			} else {
				tk.push();
				return param;
			}
		} else
			return param;

	}

	private static Object parseParam(TokenStream tk, Map<String, TVar> vMap)
			throws ClassNotFoundException, IOException {
		Object func;
		String token = tk.nextToken();
		if (token.equals("(")) {
			func = parseExp(tk, vMap);
			tk.nextToken();
		} else if (token.equals("[")) {
			func = new TList(parseExp(tk, vMap));
			tk.nextToken();
			// tk.nextToken();
		} else if (token.equals("<")) {
			ArrayList list = new ArrayList();
			if (!tk.nextToken().equals(">")) {
				tk.push();
				do {
					list.add(TypeParser.parseExp(tk, vMap));
				} while (!tk.nextToken().equals(">"));
			}
			func = jfp.datatype.TupleFactory.create(list);
			// tk.nextToken();
			// tk.nextToken();
		} else if (vMap.containsKey(token)) {
			func = vMap.get(token);
		} else if (token.startsWith("!")) {
			func = new TVar(token.substring(1));
		} else if (token.contains(".")) {
			func = Class.forName(token);
		} else if (token.equals("]") || token.equals(")") || token.equals("|")
				|| token.equals(";") || token.equals(">")) {
			tk.push();
			func = null;
		} else if (tk.tokenType() != -1) {
			func = token;
		} else
			func = null;
		return func;
	}

	// private ExtensibleAST parseArray(StringTokenizer tk, Map<String, TVar>
	// env) throws ClassNotFoundException {
	// List params = new ArrayList();
	// while (tk.hasMoreTokens()) {
	// Object param = parseParam(tk, env);
	// if (param != null)
	// params.add(param);
	// else
	// break;
	// if(tk.nextToken().equals("]"))
	// break;
	// }
	// return new ASTList(params);
	// }

}
