/*
 Copyright 2006, 2007, 2008 Hao Xu
 ertranne@hotmail.com
 
 This file is part of OSHL-S.

 OSHL-S is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 OSHL-S is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with Foobar; if not, write to the Free Software
 Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
package jfp.type;

import java.io.IOException;
import java.util.*;
import java.util.logging.Level;
import java.util.logging.LogRecord;
import java.util.logging.Logger;

import jfp.datatype.EmptyTuple;
import jfp.datatype.List;
import jfp.datatype.ListTuple;
import jfp.datatype.Pair;
import jfp.datatype.Tuple;
import jfp.datatype.TupleFactory;
import jfp.dynamic.Term;
import jfp.dynamic.ASTException;
import jfp.dynamic.ExtensibleAST;
import jfp.dynamic.Scope;
import jfp.dynamic.TokenStream;
import jfp.environment.TypeVariableEnvironmentInterface;

public class UtilQuick {

    public static boolean on = true;
    static Logger logger = Logger.getAnonymousLogger();
    public static final Level DEBUG = Level.parse(Integer.toString(Integer.MIN_VALUE));

    public static Object type(Object value) {
        Object valueType = null;
        if (value instanceof List) {
            valueType = ((List) value).getType();
        } else if (value != null) {
            valueType = value.getClass();
        }
        return valueType;
    }

    public static Object getTypeInstance(Object domainType, TypeVariableEnvironmentInterface vp) {

        if (domainType instanceof TFunction) {
            TFunction dft;
            dft = (TFunction) domainType;
            return new TFunction(getTypeInstance(dft.domainType, vp), getTypeInstance(dft.rangeType, vp));
        } else if (domainType instanceof Tuple) {
            Tuple dtt;
            dtt = (Tuple) domainType;
            ArrayList<Object> l = new ArrayList<Object>();

            while (dtt.arity() > 0) {
                l.add(getTypeInstance(dtt.hd(), vp));
                dtt = dtt.tl();
            }

            return TupleFactory.create(l);
        } else if (domainType instanceof TVar) {
            TVar tvar = (TVar) domainType;
            return tvar.getNonNullReference(vp);
        }


        return null;
    }

    public static Object typeInstance(Object val, TypeVariableEnvironmentInterface env) {
        return getTypeInstance(type(val), env);
    }

    public static Object parse(String string, Map<String, TVar> vMap)
            throws ClassNotFoundException, IOException {
        TokenStream tk = new TokenStream(string);
        return TypeParser.parseExp(tk, vMap);
    }

    public static java.util.Set<TVar> definedTypeVariables(Object type) {
        java.util.Set<TVar> tvs = new java.util.HashSet<TVar>();
        if (type == null) {
            return tvs;
        } else if (type instanceof TVar) {
            tvs.add((TVar) type);
        } else if (type instanceof TFunction) {
            tvs.addAll(definedTypeVariables(((TFunction) type).domainType));
            tvs.addAll(definedTypeVariables(((TFunction) type).rangeType));
        } else if (type instanceof Tuple) {
            tvs.addAll(definedTypeVariables(((Tuple) type).hd()));
            tvs.addAll(definedTypeVariables(((Tuple) type).tl()));
        }
        return tvs;
    }

    public static boolean unifiable(Object domainType,
            Object valueType,
            TypeVariableEnvironmentInterface vp) throws ASTException {

        if (domainType == valueType) {
            return true;
        } else if (domainType instanceof TFunction && valueType instanceof TFunction) {
            TFunction dft;
            TFunction vft;
            dft = (TFunction) domainType;
            vft = (TFunction) valueType;
            return unifiable(dft.domainType, vft.domainType, vp) && unifiable(dft.rangeType, vft.rangeType, vp);
        } else if (domainType instanceof Tuple && valueType instanceof Tuple) {
            Tuple dtt;
            Tuple vtt;
            dtt = (Tuple) domainType;
            vtt = (Tuple) valueType;
            if (dtt.arity() == vtt.arity()) {
                int n = dtt.arity();
                for (int i = 0; i < n; i++) {
                    if (!unifiable(dtt.hd(), vtt.hd(), vp)) {
                        return false;
                    }
                    dtt = dtt.tl();
                    vtt = vtt.tl();
                }
                return true;
            }
            return false;
        } else if (domainType instanceof TVar) {
            return unifyTVar((TVar) domainType, valueType, vp);
        } else if (valueType instanceof TVar) {
            return unifyTVar((TVar) valueType, domainType, vp);
        }
        return false;
    }

    static boolean unifyTVar(TVar domainType, Object valueType, TypeVariableEnvironmentInterface vp) throws ASTException {
        TVar tvar = domainType;

        Object deref = tvar.getNonNullReference(vp);
        if (deref.equals(domainType)) {
            Object type = UtilQuick.getTypeInstance(valueType, vp);
            if (!domainType.equals(type)) {
                tvar.setReference(type, vp);
            }
            return true;
        } else {
            return UtilQuick.unifiable(deref, valueType,
                    vp);
        }
    }

    public static Object unifyAll(java.util.List<ExtensibleAST> name,
            TypeVariableEnvironmentInterface env) throws ASTException {
        Object prev_type = null, current_type;
        for (ExtensibleAST ea : name) {
            current_type = ea.visitedBy(Term.TYPE, env);
            if (prev_type == null) {
                prev_type = current_type;
            } else {
                if (unifiable(prev_type, current_type, env)) {
                    prev_type = getTypeInstance(current_type, env);
                } else {
                    throw new TypeException();
                }
            }
        }
        if (prev_type == null) {
            prev_type = new TVar(env);
        }
        return prev_type;
    }

    public static Logger getLogger() {

        return logger;
    }

    public static void setLogger(Logger logger) {
        UtilQuick.logger = logger;
    }

    public static Object createIntersectionType(Object... param_type) {
        return new TIntersection(Arrays.asList(param_type));
    // return null;
    }
}
