/*
 Copyright 2006, 2007, 2008 Hao Xu
 ertranne@hotmail.com
 
 This file is part of OSHL-S.

 OSHL-S is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 OSHL-S is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with Foobar; if not, write to the Free Software
 Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
package logic.concurrent.data;

import java.util.ArrayList;
import java.util.Comparator;
import java.util.List;
import java.util.SortedSet;
import java.util.TreeSet;

import jfp.datatype.Pair;
import logic.concurrent.function.FunctionSetProcedure;
import logic.data.FunctionSymbol;
import logic.data.Term;
import concurrent.SearchFailureException;
import concurrent.Task;
import concurrent.Task.TaskInstance;

public class TermGenerator {

	private static final int MAX_TERM_LENGTH = 10;

	private static final Term[] TERM_ARRAY = new Term[] {};

	static SortedSet<Term>[] terms = new SortedSet[MAX_TERM_LENGTH];

	static List<Term>[] depth_terms = new ArrayList[MAX_TERM_LENGTH];

	static int hd = 1;

	static int d = 1;

	static {
		terms[0] = new TreeSet<Term>(new TermComparator());
		depth_terms[0] = new ArrayList<Term>();
	}

	public static SortedSet<FunctionSymbol> fs = new TreeSet<FunctionSymbol>(
			new Comparator<FunctionSymbol>() {

				public int compare(FunctionSymbol arg0, FunctionSymbol arg1) {
					return arg0.id - arg1.id;
				}

			});

	public static void generate(List<Term> list, int max) {
		if (hd >= max) {
			for (int i = 0; i < max; i++) {
				if (terms[i] != null)
					list.addAll(terms[i]);
			}
		}
		else {
			while (hd < max) {
				int min = Integer.MAX_VALUE;
				d++;
				depth_terms[d - 1] = new ArrayList<Term>();
				for (FunctionSymbol f : fs) {
					if (f.arity > 0) {
						FunctionSetProcedure<Integer, Term> fsp = new FunctionSetProcedure<Integer, Term>();
						Task<Pair<List<Integer>, List<Term>>, Pair<List<Integer>, List<Term>>> task = fsp
								.create();

						List<Integer> int_list = new ArrayList<Integer>();
						for (int i = 0; i < f.arity; i++) {
							int_list.add(i);
						}
						task.input(new Pair<List<Integer>, List<Term>>(
								int_list, depth_terms[d - 2]));
						while (task.hasNext()) {
							TaskInstance<Pair<List<Integer>, List<Term>>> ti = task
									.next();

							Term t = null;
							try {
								Term[] subterms = ti.run().snd().toArray(
										TERM_ARRAY);
								t = new Term(f, subterms);
							} catch (SearchFailureException e) {
								// TODO add error handling
								e.printStackTrace();
							}
							if (t.size() <= MAX_TERM_LENGTH) {
								depth_terms[d - 1].add(t);
								int s = t.size() - 1;
								if (terms[s] == null) {
									terms[s] = new TreeSet<Term>(
											new TermComparator());
								}
								terms[s].add(t);
								if (s + 1 < min)
									min = s + 1;
							}

						}

					}

				}
				hd = min;
			}
			for (int i = 0; i < max; i++)
				if (terms[i] != null)
					list.addAll(terms[i]);

		}

	}

	public static void addFunctionSymbol(FunctionSymbol f) {
		fs.add(f);
		if (f.arity == 0) {
			Term t = new Term(f, new Term[0]);
			terms[0].add(t);
			depth_terms[0].clear();
			depth_terms[0].addAll(terms[0]);
		}
	}

	public static void main(String[] args) {
		TermGenerator.addFunctionSymbol(new FunctionSymbol("f", 1, 'f'));
		TermGenerator.addFunctionSymbol(new FunctionSymbol("g", 2, 'g'));
		TermGenerator.addFunctionSymbol(new FunctionSymbol("h", 2, 'h'));
		for (char ch = 'a'; ch < 'f'; ch++)
			TermGenerator.addFunctionSymbol(new FunctionSymbol("" + ch, 0, ch));
		ArrayList<Term> terms = new ArrayList<Term>();
		TermGenerator.generate(terms, 3);
		System.out.println("The search depth is " + d);
		for (int i = 0; i < d; i++) {
			System.out.print(depth_terms[i].size() + "/");
		}
		System.out.println("...");
		System.out.println(terms.size() + " terms generated as follows");
		System.out.println(terms);
	}
}
