/*
 Copyright 2006, 2007, 2008 Hao Xu
 ertranne@hotmail.com
 
 This file is part of OSHL-S.

 OSHL-S is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 OSHL-S is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with Foobar; if not, write to the Free Software
 Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
package logic.concurrent.data;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Comparator;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.SortedSet;
import java.util.TreeSet;

import jfp.datatype.Monad;
import jfp.datatype.Pair;
import jfp.datatype.Tuple;
import jfp.dynamic.ASTException;
import jfp.dynamic.ExtensibleTypeAST;
import jfp.type.TFunction;
import jfp.type.TVar;

import logic.concurrent.main.data.ProgramParameters;
import logic.data.FunctionSymbol;
import logic.data.Term;
import logic.semantics.env.TypeEnv;

public class TermTypedGenerator implements ProgramParameters {

    // tgtMap maps a type to the GENERATING ITERATOR for the type.	
    static Map<String, TermTypedGenerator> tgtMap = new HashMap<String, TermTypedGenerator>();
    //private static final Term[] TERM_ARRAY = new Term[] {};

    // TODO refactor this, currently this should be set to a global TypeEnv by
    // the constructor
    TypeEnv env;
    // The following variables should refer to the same arrays as the GENERATING ITERATOR.
    List<Term>[] terms = new List[MAX_TERM_LENGTH + 1];
    List<Term>[] accu_terms = new List[MAX_TERM_LENGTH + 1];
    // The following variables should be used by the generator only.
    int hd = 1;

    // static int d = 1;
    /**
     * constructs a term generator and
     * set this object as the default term generator of type t
     * @param t the type
     * @param env the type environment
     */
    public TermTypedGenerator(String t, TypeEnv env) {
        this.env = env;
        accu_terms[0] = new ArrayList<Term>();
        tgtMap.put(t, this);
    }    // The following set is used by the GENERATING ITERATOR only.
    SortedSet<FunctionSymbol> fs = new TreeSet<FunctionSymbol>(
            new Comparator<FunctionSymbol>() {

                public int compare(FunctionSymbol arg0, FunctionSymbol arg1) {
                    return arg0.id - arg1.id;
                }
            });

    /**
     * Add a function symbol. This method should be invoked only on the default term generator.
     * @param f the funtion symbol
     */
    public void addFunctionSymbol(FunctionSymbol f) {
        getFs().add(f);
    }

    public Set<FunctionSymbol> getFunctionSymbols() {
        return this.fs;
    }

    // The following method is invoked on the GENERATING ITERATOR only.
    public void gen() {
        terms[hd] = new ArrayList<Term>();
        accu_terms[hd] = new ArrayList<Term>();
        if (sfs == null) {
            // sort sfs
            sfs = getFs().toArray(new FunctionSymbol[0]);

            Arrays.sort(sfs);
        }
        for (FunctionSymbol f : sfs) {
            int n = f.arity;
            Term[] ts = new Term[n];

            if (n == 0) {
                if (hd == 1) {
                    terms[hd].add(new Term(f, ts));
                } else {
                    continue;
                }
            } else {
                rec(f, ts, 0, hd - 1);
            }
        }
        accu_terms[hd].addAll(accu_terms[hd - 1]);
        accu_terms[hd].addAll(terms[hd]);

        // Collections.sort(accu_terms[hd], new TermComparator());
        // The list accu_terms[hd] should be in the sorted order. Therefore, there is no need to sort it.
        hdadv();
    }
    // The following method is invoked on the GENERATING ITERATOR only.
    private void hdadv() {
        hd++;
    }

    private boolean rec(FunctionSymbol fs, Term[] ts, int i, int max) {
        TermTypedIterator ti;
        ExtensibleTypeAST tf = null;
        try {
            tf = (ExtensibleTypeAST) env.get(fs.name);
        } catch (ASTException e) {
            e.printStackTrace();
        }
        String tstr = ((TVar) ((Tuple) ((TFunction) tf).getDomainType()).toArray()[i]).getName();
        ti = new TermTypedIterator(max - ts.length + i + 1, tstr);
        // each subterm is as least of length 1
        if (ti.max <= 0) {
            return false;
        }

        if (i == ts.length - 1) { // tail term
            for (;;) {
                Term t = ti.next();
                if (t != null) {
                    ts[i] = t;
                    Term[] st = new Term[ts.length];
                    System.arraycopy(ts, 0, st, 0, ts.length);
                    terms[hd].add(new Term(fs, st));
                } else {
                    break;
                }
            }
        } else {
            for (;;) {
                Term t = ti.next_ml();
                if (t != null) {
                    ts[i] = t;
                    rec(fs, ts, i + 1, max - t.size());
                } else {
                    break;
                }
            }
        }
        return true;
    }    // The following variable is used by the GENERATING ITERATOR only.
    private FunctionSymbol[] sfs;

    public static void main(String[] args) {
        TypeEnv env = new TypeEnv();
        TVar tv = new TVar("a");
        TVar tvb = new TVar("b");
        TFunction tf2 = new TFunction(new Pair<TVar, TVar>(tv, tv),
                tv);
        TFunction tf = new TFunction(new Monad<TVar>(tvb), tv);
        // register basis tti
        TermTypedGenerator tti = new TermTypedGenerator("a", env);
        TermTypedGenerator ttb = new TermTypedGenerator("b", env);
        // bind environment
        env.bindType("f", tf);
        env.bindType("g", tf2);
        env.bindType("h", tf2);
        // add fs to basis tti
        tti.addFunctionSymbol(new FunctionSymbol("f", 1, 'f'));
        tti.addFunctionSymbol(new FunctionSymbol("g", 2, 'g'));
        tti.addFunctionSymbol(new FunctionSymbol("h", 2, 'h'));
        for (char ch = 'a'; ch < 'f'; ch++) {
            ttb.addFunctionSymbol(new FunctionSymbol("" + ch, 0, ch));
            env.bindType("" + ch, tvb);
        }
        TermTypedIterator ti = new TermTypedIterator(5, "a");
        for (int i = 0; i < 100; i++) {
            System.out.println(ti.next_ml());
        }
    // System.out.println("The search depth is " + d);
    // for (int i = 0; i < d; i++) {
    // System.out.print(depth_terms[i].size() + "/");
    // }
    // System.out.println("...");
    // System.out.println(terms.size() + " terms generated as follows");
    // System.out.println(terms);
    }

    SortedSet<FunctionSymbol> getFs() {
        return fs;
    }
    public static HashMap<String, Integer> tlbCache = new HashMap<String, Integer>();

    /**
     * 
     * @param type
     * @return 0 if not bounded, 1 if bounded and length is one
     */
    public static int getTermLengthBound(String type) {
        if (tlbCache.containsKey(type)) {
            return tlbCache.get(type);
        }
        int termLengthBoundRec = getTermLengthBoundRec(type, new HashSet());
        tlbCache.put(type, termLengthBoundRec);
        return termLengthBoundRec;
    }

    public static int getTermLengthBoundRec(String type, Set<String> visited) {

        TermTypedGenerator tti = getTTIMap().get(type);
        TypeEnv env = tti.env;
        TFunction ftype;
        int max = 1;
        for (FunctionSymbol f : tti.getFs()) {
            if (f.arity > 0) {
                ftype = (TFunction) env.getType(f.name);
                Tuple t = (Tuple) ftype.getDomainType();
                while (t.arity() > 0) {
                    String vtype = ((TVar) t.hd()).getName();
                    if (visited.contains(vtype)) {
                        return Integer.MAX_VALUE;
                    }
                    HashSet<String> hs = new HashSet<String>();
                    hs.addAll(visited);
                    hs.add(type);
                    int l = getTermLengthBoundRec(vtype, hs);
                    if (l == Integer.MAX_VALUE) {
                        return Integer.MAX_VALUE;
                    }
                    max += l;

                    t = t.tl();
                }
            }
        }
        return max;
    }

    public static Map<String, TermTypedGenerator> getTTIMap() {
        return tgtMap;
    }
}
