/*
 Copyright 2006, 2007, 2008 Hao Xu
 ertranne@hotmail.com
 
 This file is part of OSHL-S.

 OSHL-S is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 OSHL-S is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with Foobar; if not, write to the Free Software
 Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
package logic.concurrent.function;

import java.util.ArrayList;
import java.util.List;

import jfp.datatype.Pair;
import concurrent.AbstractSimpleTaskProcedure;
import concurrent.SearchFailureException;
import java.util.HashSet;
import java.util.Set;
import logic.main.function.ProofProcedure_Typed_PD_RO_Lazy;
import logic.data.Clause;
import logic.data.ClauseList;
import logic.data.Literal;

public class IterativeSimpProcedure extends AbstractSimpleTaskProcedure<ClauseList, Pair<ClauseList, Pair<Set<Literal>, Set<Literal>>>> {

    public Pair<ClauseList, Pair<Set<Literal>, Set<Literal>>> run(ClauseList input) throws SearchFailureException {
        ClauseList cl = new ClauseList();
        cl.addAll(input);
        return simp(cl);
    }

    public static Pair<ClauseList, Pair<Set<Literal>, Set<Literal>>> simp(ClauseList cl) {
        Set<Literal> deleteList;
        Set<Literal> addList;
        deleteList = new HashSet<Literal>();
        addList = new HashSet<Literal>();
        addList.add(cl.get(cl.size() - 1).max());// Cn is a newly added clause, therefore contributing to one added literal

        if (cl.contains(ProofProcedure_Typed_PD_RO_Lazy.EMPTY_CLAUSE)) {
            cl.clear(); // destructive update
            cl.add(ProofProcedure_Typed_PD_RO_Lazy.EMPTY_CLAUSE);
            return new Pair(cl, new Pair(addList, deleteList));
        }
        Clause.LiteralLSCptr lc = Clause.LiteralLSCptr.instance;
        while (true) {
            if (cl.size() < 2) {

                return new Pair(cl, new Pair(addList, deleteList));
            } else {
                Clause Cn = cl.get(cl.size() - 2);
                Clause D = cl.get(cl.size() - 1);

                if (complementary(Cn.max(), D.max())) {

                    Clause nD = a_res(Cn, D);
                    cl.remove(cl.size() - 1);
                    cl.remove(cl.size() - 1);

                    cl.add(nD);

                    addList.clear();
                    if(!nD.isEmpty())
                    addList.add(nD.max());
                    deleteList.add(Cn.max());

                    if (nD.size() == 0) {

                        return new Pair(cl, new Pair(addList, deleteList));
                    }

                } else if (lc.compare(Cn.max(), D.max()) > 0) {
                    cl.remove(cl.size() - 2);
                    deleteList.add(Cn.max());

                } else {
                    return new Pair(cl, new Pair(addList, deleteList));
                }
            }
        }


    }

    private static void updateList(List<Literal> list, Literal literal) {
        Literal neg = literal.negate();
        if (list.contains(neg)) {
            list.remove(neg);
        } else {
            list.add(literal);
        }

    }

    private static boolean complementary(Literal literal, Literal literal2) {
        return literal.complementary(literal2);
    }

    private static Clause a_res(Clause cn, Clause d) {
        Clause nd = new Clause();
        Clause nc = new Clause();
        nc.addAll(cn);
        nc.remove(cn.max());
        nd.addAll(d);
        nd.remove(d.max());
        nd.addAll(nc);
        return nd;
    }
}
