/*
 Copyright 2006, 2007, 2008 Hao Xu
 ertranne@hotmail.com
 
 This file is part of OSHL-S.

 OSHL-S is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 OSHL-S is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with Foobar; if not, write to the Free Software
 Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
package logic.data;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import jfp.datatype.Pair;
import logic.data.QuantifiedFormula.Quantifier;

/**
 * A single (possibly negative) literal formula, not an atom.
 * @author ertri
 */
public class AtomicFormula extends Formula {

    public Literal literal;

    public AtomicFormula(Literal lit) {
        this.literal = lit;
    }

    @Override
    public Formula pushNegation(boolean neg) {
        if (neg) {
            return this;
        } else {
            return new AtomicFormula(literal.negate());
        }
    }

    @Override
    public Pair<List<Pair<Quantifier, VariableSymbol>>, Formula> extractQuantifier() {
        List<Pair<Quantifier, VariableSymbol>> list = new ArrayList<Pair<QuantifiedFormula.Quantifier, VariableSymbol>>();

        return new Pair(list, this);
    }

    @Override
    public Formula replace(VariableSymbol snd, Term nv) {
        Substitution sub = new Substitution();
        sub.addNoSub(snd, nv);
        return new AtomicFormula(sub.sub(literal));
    }

    @Override
    public ConnectedFormula flatten() {
        return new ConnectedFormula(ConnectedFormula.Connective.AND, new ConnectedFormula(ConnectedFormula.Connective.OR, this));
    }

    @Override
    public String toString() {
        return literal.toString();
    }

    @Override
    public List<VariableSymbol> fv() {
        List<VariableSymbol> fvl = new ArrayList<VariableSymbol>();
        HashSet<VariableSymbol> vset = new HashSet<VariableSymbol>();
        Util.FV(literal, vset);
        fvl.addAll(vset);
        return fvl;
    }
}
