/*
 Copyright 2006, 2007, 2008 Hao Xu
 ertranne@hotmail.com
 
 This file is part of OSHL-S.

 OSHL-S is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 OSHL-S is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with Foobar; if not, write to the Free Software
 Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
package logic.data;

import java.util.*;
import logic.syntax.TermParser;

/**
 *
 * @author Ertri
 */
public class Trie {

    public List<Literal> list = new LinkedList<Literal>();
    public int n;
    public int depth = 0;
    PrefixNode node;
    public int[] non = new int[1024];

    public static class SubtreeMap extends HashMap<Integer, PrefixNode> {
    }

    public class PrefixNode {

        public SubtreeMap subtrees;
        public Literal pointer;
        public int mul;

        public PrefixNode(Literal l) {
            this.pointer = l;
            this.subtrees = new SubtreeMap();
            this.mul = l == null ? 0 : 1;
        }

        @Override
        public String toString() {
            return toString("", 0);
        }

        public String toString(String indent, int id) {
            String str = indent + (TermParser.sTable[id] + ":" + (pointer == null ? "*" : pointer)) + ", " + mul + " (\n";
            for (Map.Entry<Integer, PrefixNode> nd : subtrees.entrySet()) {
                str += nd != null ? nd.getValue().toString(indent + "|", nd.getKey()) + "\n" : "";
            }
            str += indent + ")";
            return str;
        }
    }

    /** the node must be in the trie, otherwise there is an error */
    public void deleteNode(Literal l) {
        PrefixNode n2 = node;
        int[] key = l.getKey();
        non[key.length - 1]--;
        int p = 0;
        PrefixNode on2 = null;
        while (p < key.length - 1) {
            n2.mul--;
            on2 = n2;
            n2 = n2.subtrees.get(key[p]);
            if (n2.mul == 1) {
                on2.subtrees.remove(key[p]);
                return;
            }
            p++;
        }
    }

    public boolean addNode(Literal l) {
        PrefixNode n2 = node;
        int[] key = l.getKey();
        non[key.length - 1]++;
        if (key.length - 1 > this.depth) {
            this.depth = key.length - 1;
        }
        int p = 0;
        while (p < key.length - 1 && n2.subtrees.containsKey(key[p])) {
            n2.mul++;
            n2 = n2.subtrees.get(key[p]);
            p++;
        }
        if (p < key.length - 1) {

            while (p < key.length - 2) {
                n2.mul++;
                Trie.PrefixNode subNode = new PrefixNode(null);
                n2.subtrees.put(key[p++], subNode);
                n2 = subNode;
            }
            n2.mul++;
            Trie.PrefixNode subNode = new PrefixNode(l);
            n2.subtrees.put(key[p++], subNode);
            n2 = subNode;

            return true;

        }
        return false;
    }

    public boolean lookup(Literal l, LengthMap vb, SubstitutionHandler cont) {
        Substitution sub = new Substitution();
        return lookup(l, sub, vb, cont);
    }

    public boolean lookup(Literal l, Substitution sub, LengthMap vb, SubstitutionHandler cont) {
        return lookup(l, 0, node, vb, cont, sub);
    }

    public boolean lookupCombinations(List<Literal> l, LengthMap vb, SubstitutionHandler cont) {
        Substitution sub = new Substitution();

        return lookupCombinations(l, sub, vb, cont);
    }

    public boolean lookupCombinations(final List<Literal> l, final Substitution sub, final LengthMap vb, final SubstitutionHandler cont) {
        if (l.isEmpty()) {
            return cont.handle(sub);
        } else {
            return lookup(l.get(0), sub, vb, new SubstitutionHandler() {

                public boolean handle(Substitution handler) {
                    return lookupCombinations(l.subList(1, l.size()), handler, vb, cont);
                }
            });

        }
    }

    public boolean lookupCombinationsNoBound(List<Literal> l, SubstitutionHandler cont) {
        Substitution sub = new Substitution();

        return lookupCombinationsNoBound(l, sub, cont);
    }

    public boolean lookupCombinationsNoBound(final List<Literal> l, final Substitution sub, final SubstitutionHandler cont) {
        if (l.isEmpty()) {
            return cont.handle(sub);
        } else {
            return lookupNoBound(l.get(0), sub, new SubstitutionHandler() {

                public boolean handle(Substitution handler) {
                    return lookupCombinationsNoBound(l.subList(1, l.size()), handler, cont);
                }
            });

        }
    }

    public boolean lookup(Literal l, int index, PrefixNode nd, LengthMap vb, SubstitutionHandler cont, Substitution sub) {
        PrefixNode n2 = nd;
        int[] key = l.getKey();
        int p = index;
        while (p < key.length - 1) { // discard sign 
            if (logic.syntax.TermParser.isVariable(key[p])) {
                //deal with variable
                VariableSymbol var = (VariableSymbol) logic.syntax.TermParser.sTable[key[p]];

                if (sub.containsKey(var)) {// has substitution
                    Term t = sub.get(var);
                    int[] keyTerm = t.getKey();
                    for (int k : keyTerm) {
                        n2 = n2.subtrees.get(k);
                        if (n2 == null) {
                            return false;
                        }
                    }
                    p++;
                    continue;
                } else { // don't have substitution
                    Integer len = vb.get(var.name);
                    if (len == null) {
                        System.out.println("error");
                    }
                    return lookup(l, p, n2, var, new ArrayList<Integer>(), 1, len, vb, cont, sub);
                }
            } else if (n2.subtrees.containsKey(key[p])) {
                n2 = n2.subtrees.get(key[p++]);
            } else {
                return false;
            }
        }
        return cont.handle(sub);

    }

    public boolean lookup(Literal l, int index, PrefixNode nd, VariableSymbol var, List<Integer> fsl, int jump, int len, LengthMap vb, SubstitutionHandler cont, Substitution sub) {
        PrefixNode n2 = nd;
        if (len < jump) {
            return false;
        }
        for (Integer i : n2.subtrees.keySet()) {
            FunctionSymbol fs = (FunctionSymbol) TermParser.sTable[i];
            int njump = jump - 1 + fs.arity;
            if (njump == 0) {
                if (len > 1) {
                    continue;
                    //return false;
                }
                ArrayList<Integer> nfsl = new ArrayList<Integer>(fsl);
                nfsl.add(i);
                int[] key = new int[nfsl.size()];
                for (int ki = 0; ki < key.length; ki++) {
                    key[ki] = nfsl.get(ki);
                }

                KeyTerm kt = new KeyTerm(key);
                Substitution nsub = new Substitution();
                nsub.putAll(sub);
                nsub.put(var, kt);
                if (lookup(l, index + 1, n2.subtrees.get(i), vb, cont, nsub)) {
                    return true;
                }
            } else {
                ArrayList<Integer> nfsl = new ArrayList<Integer>(fsl);
                nfsl.add(i);
                if (lookup(l, index, n2.subtrees.get(i), var, nfsl, njump, len - 1, vb, cont, sub)) {
                    return true;
                }
            }
        }
        return false;
    }

    public boolean lookupNoBound(Literal l, SubstitutionHandler cont) {
        Substitution sub = new Substitution();
        return lookupNoBound(l, sub, cont);
    }

    public boolean lookupNoBound(Literal l, Substitution sub, SubstitutionHandler cont) {
        return lookupNoBound(l, 0, node, cont, sub);
    }

//    public boolean lookupCombinations(List<Literal> l, LengthMap vb, SubstitutionHandler cont) {
//        Substitution sub = new Substitution();
//
//        return lookupCombinations(l, sub, vb, cont);
//    }
//
//    public boolean lookupCombinations(final List<Literal> l, final Substitution sub, final LengthMap vb, final SubstitutionHandler cont) {
//        if (l.isEmpty()) {
//            return cont.handle(sub);
//        } else {
//            return lookup(l.get(0), sub, vb, new SubstitutionHandler() {
//
//                public boolean handle(Substitution handler) {
//                    return lookupCombinations(l.subList(1, l.size()), handler, vb, cont);
//                }
//            });
//
//        }
//    }
    public boolean lookupNoBound(Literal l, int index, PrefixNode nd, SubstitutionHandler cont, Substitution sub) {
        PrefixNode n2 = nd;
        int[] key = l.getKey();
        int p = index;
        while (p < key.length - 1) { // discard sign 
            if (logic.syntax.TermParser.isVariable(key[p])) {
                //deal with variable
                VariableSymbol var = (VariableSymbol) logic.syntax.TermParser.sTable[key[p]];

                if (sub.containsKey(var)) {// has substitution
                    Term t = sub.get(var);
                    int[] keyTerm = t.getKey();
                    for (int k : keyTerm) {
                        n2 = n2.subtrees.get(k);
                        if (n2 == null) {
                            return false;
                        }
                    }
                    p++;
                    continue;
                } else { // don't have substitution
                    return lookupNoBound(l, p, n2, var, new ArrayList<Integer>(), 1, cont, sub);
                }
            } else if (n2.subtrees.containsKey(key[p])) {
                n2 = n2.subtrees.get(key[p++]);
            } else {
                return false;
            }
        }
        return cont.handle(sub);

    }

    public boolean lookupNoBound(Literal l, int index, PrefixNode nd, VariableSymbol var, List<Integer> fsl, int jump, SubstitutionHandler cont, Substitution sub) {
        PrefixNode n2 = nd;
        for (Integer i : n2.subtrees.keySet()) {
            FunctionSymbol fs = (FunctionSymbol) TermParser.sTable[i];
            int njump = jump - 1 + fs.arity;
            if (njump == 0) {
                ArrayList<Integer> nfsl = new ArrayList<Integer>(fsl);
                nfsl.add(i);
                int[] key = new int[nfsl.size()];
                for (int ki = 0; ki < key.length; ki++) {
                    key[ki] = nfsl.get(ki);
                }

                KeyTerm kt = new KeyTerm(key);
                Substitution nsub = new Substitution();
                nsub.putAll(sub);
                nsub.put(var, kt);
                if (lookupNoBound(l, index + 1, n2.subtrees.get(i), cont, nsub)) {
                    return true;
                }
            } else {
                ArrayList<Integer> nfsl = new ArrayList<Integer>(fsl);
                nfsl.add(i);
                if (lookupNoBound(l, index, n2.subtrees.get(i), var, nfsl, njump, cont, sub)) {
                    return true;
                }
            }
        }
        return false;
    }

    public Trie(int n) {
        this.n = n;
        this.node = new PrefixNode(null);
    }

    public class PrefixSkipNode {

        public Literal l;

        public PrefixSkipNode(Literal l) {
            this.l = l;
        }
    }

    @Override
    public String toString() {
        return node.toString();
    }

    public static void main(String[] args) {
        TermParser.addPredicateSymbol("P", 2);
        TermParser.addFunctionSymbol("f", 1);
        TermParser.addFunctionSymbol("h", 2);
        TermParser.addFunctionSymbol("a", 0);
        TermParser.addFunctionSymbol("b", 0);
        TermParser.addFunctionSymbol("c", 0);
        TermParser.addVariableSymbol("A");
        TermParser.addVariableSymbol("B");
        TermParser.addVariableSymbol("C");
        TermParser.addVariableSymbol("D");

        Literal[] ll = {
            TermParser.parseLiteral("P(f(A),B)"),
            TermParser.parseLiteral("P(A,b))"),
            TermParser.parseLiteral("P(h(A,A),b))"),
            TermParser.parseLiteral("P(f(h(A,A)),b))")
        };
        Literal l2 = TermParser.parseLiteral("P(f(h(a,c)),b)");
        Literal l3 = TermParser.parseLiteral("P(h(a,c),b)");
        Literal l4 = TermParser.parseLiteral("P(h(a,a),b)");

        Trie pl = new Trie(10);
        System.out.println(pl.node);
        pl.addNode(l2);
        System.out.println(pl.node);
        pl.addNode(l3);
        System.out.println(pl.node);
        pl.addNode(l4);
        System.out.println(pl.node);
        pl.deleteNode(l4);

        System.out.println(pl.node);
        LengthMap lm = new LengthMap();
        lm.put("A", 4);
        lm.put("B", 4);
        for (final Literal l : ll) {
            pl.lookup(l, lm, new SubstitutionHandler() {

                public boolean handle(Substitution handler) {
                    System.out.println(handler.keySub(l));
                    return false;
                }
            });
        }





    }
}
