/*
Copyright 2006, 2007, 2008 Hao Xu
ertranne@hotmail.com

This file is part of OSHL-S.

OSHL-S is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

OSHL-S is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Foobar; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
package logic.main.function;

import logic.concurrent.main.data.ProgramParameters;
import logic.concurrent.main.data.ProgramParametersDyn;
import logic.concurrent.function.syntax.ParserProcedure;
import java.io.BufferedReader;
import java.io.FileReader;
import java.io.IOException;
import java.io.InputStreamReader;


import concurrent.SearchFailureException;
import java.io.File;
import logic.data.ClauseList;
import logic.data.EqualityAxioms;
import logic.syntax.TermParser;

public class Main implements ProgramParameters {

    public static boolean finished = false;

    public static void main(String[] args) throws SearchFailureException,
            IOException {
        String filename;
        if (args.length > 0) {
            filename = args[0];
        } else {
            BufferedReader reader = new BufferedReader(new InputStreamReader(
                    System.in));
            String fn = reader.readLine();

            filename = fn.isEmpty() ? "input.txt" : fn;
        }
        final String relativeFilename = new File(filename).getName();
        ProgramParametersDyn.setFileName(relativeFilename.substring(0, relativeFilename.indexOf('.')));
        String ai;
        String command;
        String param;
        String[] comp;

        if (args.length > 1) {
            for (int i = 1; i < args.length; i++) {
                ai = args[i];
                comp = ai.split("=");
                command = comp[0];
                param = comp[1];
                if (command.equals("timeout")) {
                    ProgramParametersDyn.setTimeout(Integer.parseInt(param));
                } else if (command.equals("sortinference")) {
                    ProgramParametersDyn.setTypeInference(Boolean.parseBoolean(param));
                } else if (command.equals("verbose")) {
                    ProgramParametersDyn.setVerbose(Boolean.parseBoolean(param));
                } else if (command.equals("initial")) {
                    ProgramParametersDyn.setInitialInterpretation(Boolean.parseBoolean(param));
                }
            }
        // commented for TPTP
        // println("Using parameters: timeout = " + ProgramParametersDyn.getTIMEOUT() + ", type inferecne = " + ProgramParametersDyn.getTYPE_INFERENCE() + ", verbose = " + ProgramParametersDyn.isVerbose() + ", initial = " + ProgramParametersDyn.getInitialInterpretation());
        }

//        String msg = "INFO: reading input from " + filename;
        // commented for TPTP
//        println(msg);
        final FileReader fr = new FileReader(filename);
        final ParserProcedure parser = new ParserProcedure();
        final ProofProcedure_Typed_PD_RO_Lazy proof = new ProofProcedure_Typed_PD_RO_Lazy();
        final Object lock = new Object();

        class ProofProcedureThread extends Thread {

            public void run() {
                // commented for TPTP
//                long startTime = 0;


                try {
                    // commented for TPTP
//                    startTime = System.currentTimeMillis();
                    ClauseList cl = parser.run(fr);
                    if (ProgramParametersDyn.isEquality()) {
                        ClauseList eas = EqualityAxioms.gnerateEqualityAxioms(TermParser.fsmap.values(), TermParser.psmap.values());
                        // commented for TPTP
/*                      println("adding equality axioms: ");
                        for(Clause cc: eas) {
                        System.out.println(cc);
                        } */
                        cl.addAll(eas);
                    }

                    Boolean run = proof.run(cl);
                    // commented for TPTP
                    //                  long finishTime = System.currentTimeMillis();
                    synchronized (lock) {
                        finished = true;
                        // commented for TPTP
//                        double time = (finishTime - startTime) / 1000.0;
/*          println("INFO: proof time " + time);
                        println("INFO: " + proof.index + "cls\n" +
                        "INFO: " + proof.index / time + "c/s");
                         */ if (run) {
                            //println("INFO: satisfiable");
                            if (ProgramParametersDyn.isHasConjecture()) {
                                System.out.println("SZS status CounterSatisfiable for " + ProgramParametersDyn.getFileName());
                            } else {
                                System.out.println("SZS status Satisfiable for " + ProgramParametersDyn.getFileName());
                            }
                        } else {
                            //println("INFO: unsatisfiable");
                            if (ProgramParametersDyn.isHasConjecture()) {
                                System.out.println("SZS status Theorem for " + ProgramParametersDyn.getFileName());
                            } else {
                                System.out.println("SZS status Unsatisfiable for " + ProgramParametersDyn.getFileName());
                            }
                        }
                    // commented for TPTP
//                       println("INFO: solutions cache: " + IntegerForest.getAccesses() + " access " + IntegerForest.getHits() + " hits");*/
                    }
                } catch (SearchFailureException e) {
                    synchronized (lock) {
                        finished = true;
                        // commented for TPTP
//                        long finishTime = System.currentTimeMillis();
//                        double time = (finishTime - startTime) / 1000.0;
//                        println("INFO: search failure " + e.getMessage());
//                        println("INFO: proof time " + time);
//                        println("INFO: " + proof.index + "cls\n" +
//                        "INFO: " + proof.index / time + "c/s");
//                        println("INFO: solutions cache: " + IntegerForest.getAccesses() + " access " + IntegerForest.getHits() + " hits");
                        System.out.println("SZS status GaveUp for " + ProgramParametersDyn.getFileName());

                    }

                } catch (Throwable e) {
                    synchronized (lock) {
                        finished = true;
                        // commented for TPTP
//                        long finishTime = System.currentTimeMillis();
                        //                      double time = (finishTime - startTime) / 1000.0;
//                        println("INFO: search failure with exception " + e.getMessage());
//                        e.printStackTrace();
//                        println("INFO: proof time " + time);
//                        println("INFO: " + proof.index + "cls\n" +
//                        "INFO: " + proof.index / time + "c/s");
//                        println("INFO: solutions cache: " + IntegerForest.getAccesses() + " access " + IntegerForest.getHits() + " hits");
                        System.out.println("SZS status GaveUp for " + ProgramParametersDyn.getFileName());



                    }
                }
                System.exit(0);
            }
        }
        ProofProcedureThread th = new ProofProcedureThread();
        th.start();
        try {
            Thread.sleep(ProgramParametersDyn.getTIMEOUT());
        } catch (InterruptedException e) {
            e.printStackTrace();
        }
        synchronized (lock) {
            if (!finished) {
                th.interrupt();
                // commented for TPTP
//                double time = ProgramParametersDyn.getTIMEOUT() / 1000.0;
                /*                println("INFO: proof timed out");
                println("INFO: " + proof.index + "cls\n" +
                "INFO: " + proof.index / time + "c/s");
                println("INFO: solutions cache: " + IntegerForest.getAccesses() + " access " + IntegerForest.getHits() + " hits");
                 */
                System.out.println("SZS status GaveUp for " + ProgramParametersDyn.getFileName());
                System.exit(0);
            }
        }
    }

    private static void println(String msg) {
        System.out.println(msg);
        System.err.println(msg);
    }
}
