/*
 Copyright 2006, 2007, 2008 Hao Xu
 ertranne@hotmail.com
 
 This file is part of OSHL-S.

 OSHL-S is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 OSHL-S is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with Foobar; if not, write to the Free Software
 Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
package logic.semantics.type;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;

import java.util.Set;
import jfp.datatype.Tuple;
import jfp.datatype.TupleFactory;
import jfp.dynamic.ASTException;
import jfp.dynamic.ExtensibleTypeAST;
import jfp.type.TFunction;
import jfp.type.TTuple;
import jfp.type.TVar;
import jfp.type.UtilQuick;
import logic.data.Clause;
import logic.data.ClauseList;
import logic.data.FunctionSymbol;
import logic.data.Literal;
import logic.data.PredicateSymbol;
import logic.data.Symbol;
import logic.data.Tree;
import logic.data.VariableSymbol;
import logic.semantics.env.TypeEnv;

public class TypeInferenceEngine {

    public static void typeinference(ClauseList cl, TypeEnv pfc) {
        // assume that the free variables in cl are renamed to be disjoint
        for (int i = 0; i < cl.size(); i++) {
            ti(cl.get(i), pfc);
        }
    }

    private static void ti(Clause clause, TypeEnv pfc) {
        for (Literal l : clause) {
            ti(l, pfc);
        }
    }

    private static Object ti(Tree<Symbol> l, TypeEnv pfc) {
        // assume that predicates and functions are in the same namespace
        List list = new ArrayList();
        if (l.val instanceof PredicateSymbol) {
            PredicateSymbol ps = (PredicateSymbol) l.val;

            for (int i = 0; i < ps.arity; i++) {
                list.add(ti(l.subtrees[i], pfc));
            }
            Object t = TupleFactory.create(list);
            Object ot = null;
            if ((ot = pfc.getType(ps.name)) != null) {
                try {
                    UtilQuick.unifiable(t, ot, pfc);
                    t = (Tuple) UtilQuick.getTypeInstance(t, pfc);
                    pfc.bindType(ps.name, t// = new TTuple((Tuple) Util
                            // .getTypeInstance(t, pfc))
                            );
                } catch (ASTException e) {
                    e.printStackTrace();
                }
            } else {
                pfc.bindType(ps.name, t);
            }
            return t;

        } else if (l.val instanceof FunctionSymbol) {
            FunctionSymbol fs = (FunctionSymbol) l.val;

            for (int i = 0; i < fs.arity; i++) {
                list.add(ti(l.subtrees[i], pfc));
            }
            ExtensibleTypeAST t = fs.arity == 0 ? new TVar(getId())
                    : new TFunction((TupleFactory.create(list)), new TVar(
                    getId()));
            ExtensibleTypeAST ot = null;
            if ((ot = (ExtensibleTypeAST) pfc.getType(fs.name)) != null) {
                try {
                    UtilQuick.unifiable(t, ot, pfc);
                    pfc.bindType(fs.name, t = (ExtensibleTypeAST) UtilQuick.getTypeInstance(t, pfc));
                } catch (ASTException e) {
                    e.printStackTrace();
                }
            } else {

                pfc.bindType(fs.name, t);
            }
            return fs.arity == 0 ? t : (ExtensibleTypeAST) ((TFunction) t).getRangeType();

        } else if (l.val instanceof VariableSymbol) {
            VariableSymbol fs = (VariableSymbol) l.val;

            ExtensibleTypeAST t = new TVar(fs.name);
            ExtensibleTypeAST ot = null;
            if ((ot = (ExtensibleTypeAST) pfc.getType(fs.name)) != null) {
                try {
                    UtilQuick.unifiable(t, ot, pfc);
                    pfc.bindType(fs.name, t = (ExtensibleTypeAST) UtilQuick.getTypeInstance(t, pfc));
                } catch (ASTException e) {
                    e.printStackTrace();
                }
            } else {

                pfc.bindType(fs.name, t);
            }
            return t;

        }
        return null;

    }

    public static List<VariableSymbol> variables(ClauseList cl) {
        // assume that the free variables in cl are renamed to be disjoint
        HashSet<VariableSymbol> list = new HashSet<VariableSymbol>();
        for (int i = 0; i < cl.size(); i++) {
            for (Literal l : cl.get(i)) {
                variables(l, list);
            }
        }
        return new ArrayList<VariableSymbol>(list);
    }

    public static void variables(Tree<Symbol> l, Set<VariableSymbol> list) {
        // assume that predicates and functions are in the same namespace
        if (l.val instanceof PredicateSymbol) {
            PredicateSymbol ps = (PredicateSymbol) l.val;

            for (int i = 0; i < ps.arity; i++) {
                variables(l.subtrees[i], list);
            }


        } else if (l.val instanceof FunctionSymbol) {
            FunctionSymbol fs = (FunctionSymbol) l.val;

            for (int i = 0; i < fs.arity; i++) {
                variables(l.subtrees[i], list);
            }

        } else if (l.val instanceof VariableSymbol) {
            
                list.add((VariableSymbol) l.val);
            

        }

    }
    static int id = 0;

    private static String getId() {
        return "tv" + id++;
    }
}
